/*
 * csir_ims.h: CSIR Incident Management System
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2002  Holger Schurig <h.schurig@mn-logistik.de>
 * Copyright (C) 2002  Jeff Sutherland <jeffs@accelent.com>
 * Copyright (C) 2003  Abraham vd Merwe <abz@4dllc.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef BLOB_ARCH_CSIR_IMS_H
#define BLOB_ARCH_CSIR_IMS_H

#define USE_SERIAL1
#define TERMINAL_SPEED baud_115200

/* the base address were BLOB is loaded by the first stage loader */
#define BLOB_ABS_BASE_ADDR	(0xa0200400)

/* where do various parts live in RAM */
#define BOOT_PARAMS			(0xa0000100)
#define BLOB_RAM_BASE		(0xa0100000)
#define PARAM_RAM_BASE		(0xa0200000)
#define KERNEL_RAM_BASE		(0xa0800000)
#define RAMDISK_RAM_BASE	(0xa1000000)

/* and where do they live in flash */
#define BLOB_FLASH_BASE		(0x00000000)
#define BLOB_FLASH_LEN		(128 * 1024)
#define PARAM_FLASH_BASE	(BLOB_FLASH_BASE + BLOB_FLASH_LEN)
#define PARAM_FLASH_LEN		(128 * 1024)
#define KERNEL_FLASH_BASE	(PARAM_FLASH_BASE + PARAM_FLASH_LEN)
#define KERNEL_FLASH_LEN	(1024 * 1024)
#define RAMDISK_FLASH_BASE	(KERNEL_FLASH_BASE + KERNEL_FLASH_LEN)
#define RAMDISK_FLASH_LEN	(4 * 1024 * 1024)

/* this needs to be defined if you want parameter block support */
#define PARAM_START PARAM_FLASH_BASE
#define PARAM_LEN PARAM_FLASH_LEN

/* load ramdisk into ram */
#define LOAD_RAMDISK		1

/* the size (in kbytes) to which the compressed ramdisk expands */
#define RAMDISK_SIZE		(4 * 1024)

#if 0
#define MSC0_VALUE      0x
#define MSC1_VALUE      0x
#define MSC2_VALUE      0x
#define MECR_VALUE      0x
#define MCMEM0_VALUE    0x
#define MCMEM1_VALUE    0x
#define MCATT0_VALUE    0x
#define MCATT1_VALUE    0x
#define MCIO0_VALUE     0x
#define MCIO1_VALUE     0x
#endif

#define MDREFR_VALUE    0x01018013
#define MDCNFG_VALUE    0x000019c9
#define MDMRS_VALUE     0x00020002

/* GPIO configuration */
#define GPIO0_VALUE		GPIO_OUT_LO
#define GPIO1_VALUE		GPIO_OUT_LO
#define GPIO2_VALUE		GPIO_OUT_LO
#define GPIO3_VALUE		GPIO_INPUT						/* eth_wakeup			*/
#define GPIO4_VALUE		GPIO_INPUT						/* bank_switch_int		*/
#define GPIO5_VALUE		GPIO_INPUT						/* eth_link_status		*/
#define GPIO6_VALUE		GPIO_OUT_LO
#define GPIO7_VALUE		GPIO_OUT_LO						/* cpld_clk				*/
#define GPIO8_VALUE		GPIO_OUT_LO
#define GPIO9_VALUE		GPIO_INPUT						/* eth_int				*/
#define GPIO10_VALUE	GPIO_OUT_HI						/* eth_reset			*/
#define GPIO11_VALUE	GPIO_OUT_LO
#define GPIO12_VALUE	GPIO_OUT_LO						/* watchdog_strobe		*/
#define GPIO13_VALUE	GPIO_INPUT						/* cpld_hw_reset		*/
#define GPIO14_VALUE	GPIO_INPUT						/* cpld_??				*/
#define GPIO15_VALUE	(GPIO_OUT_HI | GPIO_ALT_FN2)	/* nCS1 [img_buffer]	*/
#define GPIO16_VALUE	GPIO_OUT_LO
#define GPIO17_VALUE	GPIO_INPUT						/* cpld_??				*/
#define GPIO18_VALUE	GPIO_INPUT						/* vlio_ready_signal	*/
#define GPIO19_VALUE	GPIO_OUT_LO
#define GPIO20_VALUE	GPIO_OUT_LO
#define GPIO21_VALUE	GPIO_OUT_LO
#define GPIO22_VALUE	GPIO_OUT_LO
#define GPIO23_VALUE	GPIO_OUT_LO
#define GPIO24_VALUE	GPIO_OUT_LO
#define GPIO25_VALUE	GPIO_OUT_HI						/* LED [debug]			*/
#define GPIO26_VALUE	GPIO_OUT_LO
#define GPIO27_VALUE	GPIO_OUT_LO
#define GPIO28_VALUE	GPIO_OUT_LO
#define GPIO29_VALUE	GPIO_OUT_LO
#define GPIO30_VALUE	GPIO_OUT_LO
#define GPIO31_VALUE	GPIO_OUT_LO
#define GPIO32_VALUE	GPIO_OUT_LO
#define GPIO33_VALUE	GPIO_OUT_HI						/* nCS5 [unused]		*/
#define GPIO34_VALUE	(GPIO_INPUT | GPIO_ALT_FN1)		/* FFRXD				*/
#define GPIO35_VALUE	GPIO_OUT_LO
#define GPIO36_VALUE	GPIO_OUT_LO
#define GPIO37_VALUE	GPIO_OUT_LO
#define GPIO38_VALUE	GPIO_OUT_LO
#define GPIO39_VALUE	(GPIO_OUT_LO | GPIO_ALT_FN2)	/* FFTXD				*/
#define GPIO40_VALUE	GPIO_OUT_LO
#define GPIO41_VALUE	GPIO_OUT_LO
#define GPIO42_VALUE	GPIO_OUT_LO
#define GPIO43_VALUE	GPIO_OUT_LO
#define GPIO44_VALUE	GPIO_OUT_LO
#define GPIO45_VALUE	GPIO_OUT_LO
#define GPIO46_VALUE	GPIO_OUT_LO
#define GPIO47_VALUE	GPIO_OUT_LO
#define GPIO48_VALUE	GPIO_OUT_LO
#define GPIO49_VALUE	GPIO_OUT_HI						/* cpld_pcmcia_pwe		*/
#define GPIO50_VALUE	GPIO_OUT_LO
#define GPIO51_VALUE	GPIO_OUT_LO
#define GPIO52_VALUE	GPIO_OUT_LO
#define GPIO53_VALUE	GPIO_OUT_LO
#define GPIO54_VALUE	GPIO_OUT_LO
#define GPIO55_VALUE	GPIO_OUT_LO
#define GPIO56_VALUE	GPIO_INPUT
#define GPIO57_VALUE	GPIO_INPUT
#define GPIO58_VALUE	GPIO_OUT_LO
#define GPIO59_VALUE	GPIO_OUT_LO
#define GPIO60_VALUE	GPIO_OUT_LO
#define GPIO61_VALUE	GPIO_OUT_LO
#define GPIO62_VALUE	GPIO_OUT_LO
#define GPIO63_VALUE	GPIO_OUT_LO
#define GPIO64_VALUE	GPIO_OUT_LO
#define GPIO65_VALUE	GPIO_OUT_LO
#define GPIO66_VALUE	GPIO_OUT_LO
#define GPIO67_VALUE	GPIO_OUT_LO
#define GPIO68_VALUE	GPIO_OUT_LO
#define GPIO69_VALUE	GPIO_OUT_LO
#define GPIO70_VALUE	GPIO_OUT_LO
#define GPIO71_VALUE	GPIO_OUT_LO
#define GPIO72_VALUE	GPIO_OUT_LO
#define GPIO73_VALUE	GPIO_OUT_LO
#define GPIO74_VALUE	GPIO_OUT_LO
#define GPIO75_VALUE	GPIO_OUT_LO
#define GPIO76_VALUE	GPIO_OUT_LO
#define GPIO77_VALUE	GPIO_OUT_LO
#define GPIO78_VALUE	(GPIO_OUT_HI | GPIO_ALT_FN2)	/* nCS2 [img_buffer]	*/
#define GPIO79_VALUE	(GPIO_OUT_HI | GPIO_ALT_FN2)	/* nCS3 [eth]			*/
#define GPIO80_VALUE	GPIO_OUT_HI						/* nCS4 [unused]		*/
#define GPIO81_VALUE	GPIO_OUT_LO
#define GPIO82_VALUE	GPIO_OUT_LO
#define GPIO83_VALUE	GPIO_OUT_LO
#define GPIO84_VALUE	GPIO_OUT_LO

#endif	/* #ifndef BLOB_ARCH_CSIR_IMS_H */
