#ifndef _I2C_H_
#define _I2C_H_

#include <blob/types.h>

struct i2c_bus {
	int (*init_bus)(struct i2c_bus *);
	int (*set_scl)(struct i2c_bus *, unsigned sense);
	int (*set_sda)(struct i2c_bus *, unsigned sense);
	int (*get_scl)(struct i2c_bus *);
	int (*get_sda)(struct i2c_bus *);

	void *private;
};

////////////////////////////////////////////////////////////////////////

/* high-level operators */
extern int i2c_init(struct i2c_bus *);
extern int i2c_read_device_byte(struct i2c_bus *, u8);
extern int i2c_read_device_byte_at(struct i2c_bus *bus, u8, u8);
extern int i2c_read_device_bytes(struct i2c_bus *, u8, u8, u8 *, unsigned long);
extern int i2c_write_device_byte_at(struct i2c_bus *, u8, u8, u8);
extern int i2c_write_device_bytes_at(struct i2c_bus *, u8, u8, const u8 *, unsigned long);
extern int i2c_probe(struct i2c_bus *bus, u8);
extern int i2c_scan(struct i2c_bus *bus, u8 map[256]);

/* low-level operators */
extern int i2c_start(struct i2c_bus *);
extern int i2c_stop(struct i2c_bus *);
extern int i2c_send_ack(struct i2c_bus *);
extern int i2c_get_ack(struct i2c_bus *);
extern int i2c_get_ack_quietly(struct i2c_bus *);
extern int i2c_write_byte(struct i2c_bus *, u8);
extern int i2c_read_byte(struct i2c_bus *);

#endif
