/* nand.h
 *
 * Created by stanley cai (stanley.cai@intel.com)
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 *
 * Note:
 *   This is a private file which will NOT be release without any modification. The file
 * may include some source codes from other open source. There is no warranty to the code.
 *   -stanley
 */
#if !defined(__LINUX_NAND_H)
#define __LINUX_NAND_H

#define NAND_BUSWIDTH_16         0x1

#define  ALLOW_ERASE_BAD_DEBUG 0

#define CONFIG_MTD_NAND_ECC  /* enable ECC */
#define CONFIG_MTD_NAND_ECC_JFFS2

#define CONFIG_MAX_NAND_DEVICE  1

#define NAND_MAX_CHIPS          1 /* Only for one chip now.-stanley */

#define NAND_CHIPID_UNKNOWN     0

/* bits for nand_rw() `cmd'; or together as needed */
#define NANDRW_READ             0x01
#define NANDRW_WRITE            0x00
#define NANDRW_JFFS2            0x02
#define NANDRW_JFFS2_SKIP       0x04

/*
 * Standard NAND flash commands
 */
#define NAND_CMD_READ0          0
#define NAND_CMD_READ1          1
#define NAND_CMD_PAGEPROG       0x10
#define NAND_CMD_READOOB        0x50
#define NAND_CMD_ERASE1         0x60
#define NAND_CMD_STATUS         0x70
#define NAND_CMD_SEQIN          0x80
#define NAND_CMD_READID         0x90
#define NAND_CMD_ERASE2         0xd0
#define NAND_CMD_RESET          0xff

/*
 * NAND Private Flash Chip Data
 *
 * Structure overview:
 *
 *  IO_ADDR - address to access the 8 I/O lines of the flash device
 *
 *  hwcontrol - hardwarespecific function for accesing control-lines
 *
 *  dev_ready - hardwarespecific function for accesing device ready/busy line
 *
 *  chip_lock - spinlock used to protect access to this structure
 *
 *  wq - wait queue to sleep on if a NAND operation is in progress
 *
 *  state - give the current state of the NAND device
 *
 *  page_shift - number of address bits in a page (column address bits)
 *
 *  data_buf - data buffer passed to/from MTD user modules
 *
 *  data_cache - data cache for redundant page access and shadow for
 *               ECC failure
 *
 *  ecc_code_buf - used only for holding calculated or read ECCs for
 *                 a page read or written when ECC is in use
 *
 *  reserved - padding to make structure fall on word boundary if
 *             when ECC is in use
 */
struct Nand {
        char chip;
        unsigned long curadr;
        unsigned char curmode;
        /* Also some erase/write/pipeline info when we get that far */
};

struct nand_chip {
        int             page_shift;
        u8              *data_buf;
        u8              *data_cache;
        int             cache_page;
        u8              ecc_code_buf[6];
        u8              reserved[2];
        char chipID; /* Type of DiskOnChip */
        struct Nand *chips;
        int chipshift;
        char* chips_name;
        unsigned long erasesize;
        unsigned long mfr; /* Flash IDs - only one type of flash per device */
        unsigned long id;
        char* name;
        int numchips;
        char page256;
        char pageadrlen;
        u32 IO_ADDR;  /* address to access the 8 I/O lines to the flash device */
        unsigned long totlen;
        u32 oobblock;  /* Size of OOB blocks (e.g. 512) */
        u32 oobsize;   /* Amount of OOB data per block (e.g. 16) */
        u32 eccsize;
};

/*
 * NAND Flash Manufacturer ID Codes
 */
#define NAND_MFR_TOSHIBA	0x98
#define NAND_MFR_SAMSUNG	0xec
#define NAND_MFR_FUJITSU	0x04
#define NAND_MFR_NATIONAL	0x8f

/*
 * NAND Flash Device ID Structure
 *
 * Structure overview:
 *
 *  name - Complete name of device
 *
 *  manufacture_id - manufacturer ID code of device.
 *
 *  model_id - model ID code of device.
 *
 *  chipshift - total number of address bits for the device which
 *              is used to calculate address offsets and the total
 *              number of bytes the device is capable of.
 *
 *  page256 - denotes if flash device has 256 byte pages or not.
 *
 *  pageadrlen - number of bytes minus one needed to hold the
 *               complete address into the flash array. Keep in
 *               mind that when a read or write is done to a
 *               specific address, the address is input serially
 *               8 bits at a time. This structure member is used
 *               by the read/write routines as a loop index for
 *               shifting the address out 8 bits at a time.
 *
 *  erasesize - size of an erase block in the flash device.
 */
struct nand_flash_dev {
        char * name;
        int manufacture_id;
        int model_id;
        int chipshift;
        char page256;
        char pageadrlen;
        unsigned long erasesize;
};

/*
 * Constants for oob configuration
 */
#define NAND_NOOB_ECCPOS0               0
#define NAND_NOOB_ECCPOS1               1
#define NAND_NOOB_ECCPOS2               2
#define NAND_NOOB_ECCPOS3               3
#define NAND_NOOB_ECCPOS4               6
#define NAND_NOOB_ECCPOS5               7
#define NAND_NOOB_BADBPOS               -1
#define NAND_NOOB_ECCVPOS               -1

#define NAND_JFFS2_OOB_ECCPOS0          0
#define NAND_JFFS2_OOB_ECCPOS1          1
#define NAND_JFFS2_OOB_ECCPOS2          2
#define NAND_JFFS2_OOB_ECCPOS3          3
#define NAND_JFFS2_OOB_ECCPOS4          6
#define NAND_JFFS2_OOB_ECCPOS5          7
#define NAND_JFFS2_OOB_BADBPOS          5
#define NAND_JFFS2_OOB_ECCVPOS          4

#define NAND_JFFS2_OOB8_FSDAPOS         6
#define NAND_JFFS2_OOB16_FSDAPOS        8
#define NAND_JFFS2_OOB8_FSDALEN         2
#define NAND_JFFS2_OOB16_FSDALEN        8

extern struct nand_flash_dev nand_flash_ids[];
extern struct nand_manufacturers nand_manuf_ids[];

struct nand_bbt_descr {
	int	options;
	int	pages[NAND_MAX_CHIPS];
	int	offs;
	int	veroffs;
	u8	version[NAND_MAX_CHIPS];
	int	len;
	int	maxblocks;
	int	reserved_block_code;
	u8	*pattern;
};

#define NAND_BBT_NRBITS_MSK	0xf
#define NAND_BBT_1BIT		0x1
#define NAND_BBT_2BIT		0x2
#define NAND_BBT_4BIT		0x4
#define NAND_BBT_8BIT		0x8

#define NAND_BBT_LASTBLOCK	0x10
#define NAND_BBT_ABSPAGE	0x20
#define NAND_BBT_SEARCH		0x40
#define NAND_BBT_PERCHIP	0x80
#define NAND_BBT_VERSION	0x100
#define NAND_BBT_CREATE		0x200
#define NAND_BBT_SCANALLPAGES	0x400
#define NAND_BBT_SCANEMPTY	0x800
#define NAND_BBT_WRITE		0x1000
#define NAND_BBT_SAVECONTENT	0x2000
#define NAND_BBT_SCAN2NDPAGE	0x4000

#define NAND_BBT_SCAN_MAXBLOCKS	4

/*
* Constants for oob configuration
*/
#define NAND_SMALL_BADBLOCK_POS		5
#define NAND_BADBLOCK_POS		5
#define NAND_LARGE_BADBLOCK_POS		0

#define NAND_NONE_OOB			0
#define NAND_JFFS2_OOB			1
#define NAND_YAFFS_OOB			2

#define NAND_NOOB_ECCPOS0		0
#define NAND_NOOB_ECCPOS1		1
#define NAND_NOOB_ECCPOS2		2
#define NAND_NOOB_ECCPOS3		3
#define NAND_NOOB_ECCPOS4		6
#define NAND_NOOB_ECCPOS5		7

#define NAND_JFFS2_OOB_ECCPOS0		0
#define NAND_JFFS2_OOB_ECCPOS1		1
#define NAND_JFFS2_OOB_ECCPOS2		2
#define NAND_JFFS2_OOB_ECCPOS3		3
#define NAND_JFFS2_OOB_ECCPOS4		6
#define NAND_JFFS2_OOB_ECCPOS5		7

#define NAND_YAFFS_OOB_ECCPOS0		8
#define NAND_YAFFS_OOB_ECCPOS1		9
#define NAND_YAFFS_OOB_ECCPOS2		10
#define NAND_YAFFS_OOB_ECCPOS3		13
#define NAND_YAFFS_OOB_ECCPOS4		14
#define NAND_YAFFS_OOB_ECCPOS5		15

#define NAND_JFFS2_OOB8_FSDAPOS		6
#define NAND_JFFS2_OOB16_FSDAPOS	8
#define NAND_JFFS2_OOB8_FSDALEN		2
#define NAND_JFFS2_OOB16_FSDALEN	8

#define NAND_RELOC_MAX 127

struct nand_rel_table{
	u16 header;
	u16 total;
	struct {
		u16 from;
		u16 to;
	} rel[NAND_RELOC_MAX];
};
#endif /* __LINUX_NAND_H */
