/*
 * pcmcia.h
 *
 * Copyright (C) 2001, Brad Parker (brad@heeltoe.com)
 * Copyright (C) 2002, Stefan Eletzhofer <stefan.eletzhofer@eletztrick.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef BLOB_PCMCIA_H
#define BLOB_PCMCIA_H 1

/* defines for PCMCIA configuration information */
#define PCMCIA_MAX_TUPEL_SZ		512
#define PCMCIA_MAX_FEATURES		4

#define PCMCIA_MAX_IDENT_CHARS	64
#define PCMCIA_MAX_IDENT_FIELDS	4
#define PCMCIA_MAX_FEATURES		4

#define CISTPL_VERS_1           0x15
#define CISTPL_FUNCID           0x21
#define CISTPL_FUNCE            0x22
#define CISTPL_CONFIG           0x1a

/*
 * CIS Function ID codes
 */
#define CISTPL_FUNCID_MULTI     0x00
#define CISTPL_FUNCID_MEMORY    0x01
#define CISTPL_FUNCID_SERIAL    0x02
#define CISTPL_FUNCID_PARALLEL  0x03
#define CISTPL_FUNCID_FIXED     0x04
#define CISTPL_FUNCID_VIDEO     0x05
#define CISTPL_FUNCID_NETWORK   0x06
#define CISTPL_FUNCID_AIMS      0x07
#define CISTPL_FUNCID_SCSI      0x08

/*
 * Fixed Disk FUNCE codes
 */
#define CISTPL_IDE_INTERFACE    0x01

#define CISTPL_FUNCE_IDE_IFACE  0x01
#define CISTPL_FUNCE_IDE_MASTER 0x02
#define CISTPL_FUNCE_IDE_SLAVE  0x03

/* First feature byte */
#define CISTPL_IDE_SILICON      0x04
#define CISTPL_IDE_UNIQUE       0x08
#define CISTPL_IDE_DUAL         0x10

/* Second feature byte */
#define CISTPL_IDE_HAS_SLEEP    0x01
#define CISTPL_IDE_HAS_STANDBY  0x02
#define CISTPL_IDE_HAS_IDLE     0x04
#define CISTPL_IDE_LOW_POWER    0x08
#define CISTPL_IDE_REG_INHIBIT  0x10
#define CISTPL_IDE_HAS_INDEX    0x20
#define CISTPL_IDE_IOIS16       0x40

/**********************************************************************
 * Arch dependent defines (should be pretty much the same everywhere,
 * tough)
 */
#define PCMCIA_MAX_SLOTS		2

#if defined(BADGE4) || defined(IDR) || defined(PT_SYSTEM3) || defined(SHANNON) || \
    defined(PT_DAFIT)
#	define PCMCIA_S0_BASE		0x20000000
#	define PCMCIA_S0_IO_BASE	0x20000000
#	define PCMCIA_S0_ATTR_BASE	0x28000000
#	define PCMCIA_S0_MEM_BASE	0x2C000000
#	define PCMCIA_S1_BASE		0x30000000
#	define PCMCIA_S1_IO_BASE	0x30000000
#	define PCMCIA_S1_ATTR_BASE	0x38000000
#	define PCMCIA_S1_MEM_BASE	0x3C000000
#else
#	error "Please add PCMCIA HW defines for your board here"
#endif

/* some slot stati */
#define PCMCIA_SS_DETECT		(1<<0)
#define PCMCIA_SS_ENABLED		(1<<1)
#define PCMCIA_SS_POWER			(1<<2)
#define PCMCIA_SS_IDENT			(1<<3)

/**********************************************************************
 * the following symbols can be overwritten by arch dependent code
 * (weak symbols)
 */
/* do whatever is needed to get pcmcia initialized */
int pcmcia_init();

/* set pcmcia slot voltage levels */
int pcmcia_voltage_set( int slot, int vcc, int vpp );

/* detect slot */
int pcmcia_slot_detect( int slot );

/* enable slot */
int pcmcia_slot_enable( int slot );

/* disable slot */
int pcmcia_slot_disable( int slot );

/* reset pcmcia slot */
int pcmcia_slot_reset(int slot);

/* dump slot stati */
int pcmcia_dump_stati( void );

/**********************************************************************
 * following symbols cannot be overwritten
 */
/* set debug verbosity */
void pcmcia_dbg_set( int level );

/* identify pcmcia card */
int pcmcia_identify( volatile unsigned char *p );

/* parse cis tuples of card in slot <slot> */
int pcmcia_cis_parse( int slot );

/* get base adress and attribute space address of slot <slot> */
int pcmcia_slot_address_get( int slot, u32 *base, u32 *attr );

/* get config register */
int pcmcia_slot_cfg_reg_get( int slot, u16 *cfg_base );

/* read/write attribute space */
int pcmcia_slot_attr_read( int slot, u16 offset, u8 *value );
int pcmcia_slot_attr_write( int slot, u16 offset, u8 value );
#endif
