/*
 * proc/sa11x0.h - processor specific defines
 * 
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef BLOB_SA11X0_PROC_H
#define BLOB_SA11X0_PROC_H

/* memory start and end */
#define MEMORY_START		(0xc0000000)
#define MEMORY_END		(0xe0000000)

/*
 * CPU frequency table.
 * 
 * See Section 8.2 (Core Clock Configuration Register) SA-1110 Developers
 * Manual for more information.
 *
 * f_cpu = 16*f_osc + n*4*f_osc
 *       = 4*(n+4)*f_osc
 *
 * We have 3.6864-MHz oscillator.
 *
 *	CCF[4:0]	3.6864-MHz Crystal Oscillator	3.5795-MHz Crystal Oscillator
 *	-------------------------------------------------------------------------
 *	00			59.0							57.3
 * 	01			73.7							71.6
 * 	02			88.5							85.9
 * 	03			103.2							100.2
 * 	04			118.0							114.5
 * 	05			132.7							128.9
 * 	06			147.5							143.2
 * 	07			162.2							157.5
 * 	08			176.9							171.8
 * 	09			191.7							186.1
 * 	0a			206.4							200.5
 * 	0b			221.2							214.8
 *
 * these are undocumented.
 *
 *  0c			235.9							??
 *  0d			250.7							??
 *  0e			265.4							??
 *  0f			280.2							??
 */
#define CPU_CORE_SPEED_59mhz	0x00
#define CPU_CORE_SPEED_73mhz	0x01
#define CPU_CORE_SPEED_88mhz	0x02
#define CPU_CORE_SPEED_103mhz	0x03
#define CPU_CORE_SPEED_118mhz	0x04
#define CPU_CORE_SPEED_132mhz	0x05
#define CPU_CORE_SPEED_147mhz	0x06
#define CPU_CORE_SPEED_162mhz	0x07
#define CPU_CORE_SPEED_176mhz	0x08
#define CPU_CORE_SPEED_191mhz	0x09
#define CPU_CORE_SPEED_206mhz	0x0a
#define CPU_CORE_SPEED_221mhz	0x0b
#define CPU_CORE_SPEED_235mhz	0x0c
#define CPU_CORE_SPEED_250mhz	0x0d
#define CPU_CORE_SPEED_265mhz	0x0e
#define CPU_CORE_SPEED_280mhz	0x0f

#define io_p2v(x) (x)
#define __REG(x)      (*((volatile u32 *)io_p2v(x)))

/* Tell SA-1100.h to shut up; we're including it anyway. Nyah nyah ;-) */
#define __ASM_ARCH_HARDWARE_H
#include <asm-arm/arch-sa1100/SA-1100.h>

#endif
