/*-------------------------------------------------------------------------
 * Filename:      ether.h
 * Version:       $Id: ether.h,v 1.5 2003/09/04 18:34:26 russd Exp $
 * Copyright:     Copyright (C) 1999, Erik Mouw
 * Author:        Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Description:   Header file for ether.c
 * Created at:    Tue Aug 24 20:25:00 1999
 * Modified by:   Russ Dill <Russ.Dill@asu.edu>
 * Modified at:   Wed, Aug 27 2003
 *-----------------------------------------------------------------------*/
/*
 * ether.h: Ethernet driver header for blob
 *
 * Copyright (C) 1999 2002  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2003  Russ Dill (Russ.Dill@asu.edu)
 * Copyright 1996-2001 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: ether.h,v 1.5 2003/09/04 18:34:26 russd Exp $"

#ifndef _BLOB_ETHER_H
#define _BLOB_ETHER_H 1

#include <blob/types.h>

struct ether_hdr {
	unsigned char dest[6];
	unsigned char src[6];
	unsigned short proto;
} __attribute__ ((packed));

typedef int (*ether_init_func_t)(u32 base);
typedef int (*ether_rcv_func_t)(u8 *buf, int size);
typedef int (*ether_snd_func_t)(u8 *buf, int size);
typedef int (*ether_set_addr_func_t)(u8 hw_addr[6]);
typedef int (*ether_get_addr_func_t)(u8 hw_addr[6]);


typedef struct {
	ether_init_func_t init;

	ether_rcv_func_t rcv;
	ether_snd_func_t snd;

	ether_set_addr_func_t set_addr;
	ether_get_addr_func_t get_addr;

	u32 base;
	char *name;

} ether_driver_t;

#define HTON( s )	( ((s&0xff00)>>8 ) | ((s&0xff)<<8) )

/* implemented etheret drivers */
extern ether_driver_t s3c2500_ether_driver;
extern ether_driver_t smc9196_ether_driver;
extern ether_driver_t smc91x_ether_driver;
extern ether_driver_t pxausb_ether_driver;
extern ether_driver_t bvdusb_ether_driver;
extern ether_driver_t mhnu2d_ether_driver;
extern ether_driver_t mvu2o_ether_driver;

/* should be filled out by the architecture dependent files */
extern ether_driver_t *ether_driver;

/* exported functions */
void ether_init(void);
int ether_rcv(u8 *buf, int size);
int ether_snd(u8 *buf, int size);
int ether_set_addr(u8 hw_addr[6]);
int ether_get_addr(u8 hw_addr[6]);
char *ether_get_name(void);

/* exported variables */
extern u8 hwaddress[6];

#endif
