/*-------------------------------------------------------------------------
 * Filename:      main.c
 * Version:       $Id: main.c,v 1.1.1.1 2005/03/04 09:36:18 swcai Exp $
 * Copyright:     Copyright (C) 1999, Jan-Derk Bakker
 * Author:        Jan-Derk Bakker <J.D.Bakker@its.tudelft.nl>
 * Description:   Main file for the trivially simple bootloader for the 
 *                LART boards
 * Created at:    Mon Aug 23 20:00:00 1999
 * Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Modified at:   Sat Mar 25 14:31:16 2000
 *-----------------------------------------------------------------------*/
/*
 * main.c: main file for the blob bootloader
 *
 * Copyright (C) 1999 2000 2001
 *   Jan-Derk Bakker (J.D.Bakker@its.tudelft.nl) and
 *   Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: main.c,v 1.1.1.1 2005/03/04 09:36:18 swcai Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/debug.h>
#include <blob/command.h>
#include <blob/errno.h>
#include <blob/error.h>
#include <blob/init.h>
#include <blob/main.h>
#include <blob/md5.h>
#include <blob/md5support.h>
#include <blob/memory.h>
#include <blob/param_block.h>
#include <blob/serial.h>
#include <blob/time.h>
#include <blob/util.h>
#include <blob/load_kernel.h>
#include <blob/partition.h>
#include <blob/download.h>

#define DDR64M_OFFSET   ( 1<<0 )
#define DDR128M_OFFSET  ( 1<<1 )
#define DDR256M_OFFSET  ( 1<<2 )
#define DDR512M_OFFSET  ( 1<<3 )
#define FTM_BOOTUP_REASON (1<<21)
#define FTM_COMBIN_REASON_LOW (1<<23)
#define FTM_COMBIN_REASON_HIGH (1<<24)

typedef enum {
	DDR64M = 0,
	DDR128M,
	DDR256M,
	DDR512M,
	MAXDDR
}ddr_size_t;

#ifdef BLOB_DEBUG
char default_cmdline[] = "console=ttyS0,115200 init=/init android comm_v75 \
initcall_debug uart_dma panic=3 console_loglevel=7";
#else
char default_cmdline[] = "console=ttyS0,115200 init=/init android comm_v75 \
initcall_debug uart_dma panic=3 log_buf_len=4k console_loglevel=1";
#endif

//static void __attribute__((unused)) print_elf_sections(void);
//static void __attribute__((unused)) *find_parameters(void);

blob_status_t blob_status;
static char *version_str = PACKAGE " version " VERSION " for " BOARD_NAME "\n";

void blob_set_cmdline(void);
extern void set_powerup_reason( unsigned int new );
extern int blob_nand_erase( unsigned int flash_ofs, unsigned int size, char *type );

extern inline unsigned get_boot_info( void );
extern int nor_i2c_read( unsigned char addr, unsigned char* data );
extern int nor_i2c_write( unsigned char addr, unsigned char* data );
extern void init_product(void);
extern int BLOB_BOOT_DELAY_TIME;
extern int mtdparts[];


int serial_received_str[10] = {0};
int serial_received_str_num;
int main(void)
{
	int numRead = 0;
	char commandline[MAX_COMMANDLINE_LENGTH];
	u32 __attribute__((unused)) conf = 0;
	void __attribute__((unused)) *params;
	u32 bootup_reason;

    serial_received_str_num = 0;
	/* initialise status */
	blob_status.paramType = fromFlash;
	blob_status.kernelType = fromFlash;
	blob_status.ramdiskType = fromFlash;
	blob_status.downloadSpeed = baud_115200;
	blob_status.terminalSpeed = TERMINAL_SPEED;
	blob_status.load_ramdisk = LOAD_RAMDISK;
	blob_status.cmdline[0] = '\0';
	blob_status.boot_delay = BLOB_BOOT_DELAY_TIME;

        serial_driver = &mhn_serial_driver;
        serial_init(TERMINAL_SPEED);
 
	/* call subsystems (blob_status.* may change) */
	init_subsystems();

#if 0
	/* find parameter partition, if it exists */
	params = find_parameters();

	/* parse the core tag, for critical things like terminal speed */
	if (params)
		parse_ptag(params, &conf);
#endif

	init_product();
	bootup_reason = get_boot_info();
	/* Print the required GPL string */
	printf("\nConsider yourself BLOBed!\n\n");
	printf("%sCopyright (C) 1999 2000 2001 2002 2003 "
	       "Jan-Derk Bakker and Erik Mouw\n", version_str);
	printf(PACKAGE " comes with ABSOLUTELY NO WARRANTY; "
	       "read the GNU GPL for details.\n");
	printf("This is free software, and you are welcome "
	       "to redistribute it\n");
	printf("under certain conditions; read the GNU GPL for details.\n");

#ifdef NOISE_WORKAROUND_ENABLE   //Workaround for noise 
	{
		unsigned char data;
		unsigned char reg;

		nor_i2c_read( 0x12, &data );
		data &= 0xF7;                   //Buck3_enable = 0
		nor_i2c_write( 0x12, data );
	}
#endif  //NOISE_WORKAROUND_ENABLE

	/* get the amount of memory */
	/* FIXME : get_memory_map doesn't work always */
	/* get_memory_map(); */
	//print_elf_sections();

#ifdef BLOB_DEBUG
	{
//		register unsigned int stackptr;
//		asm("mov %0, sp": "=r" (stackptr));
//		printf("Current stack pointer: 0x%08x\n", stackptr);
	}
#endif

#if 0
	do_reload("kernel");
	do_reload("ramdisk");

	/* Load remaining parameters */
	if (params)
		parse_ptags(params, &conf);
#endif 

#if 1
    	blob_set_cmdline();
	/* Load kernel and ramdisk from flash to RAM */
	//do_reload("blob");
	//do_reload("kernel");
	if(blob_status.load_ramdisk)
		parse_command("nramdisk");	//do_reload("ramdisk");

	if (blob_status.boot_delay > -1) {
		int i;
		int retval = 0;

		/* wait boot_delay seconds before starting autoboot */
		printf("Autoboot (%i seconds) in progress, press any key to stop ",
		       blob_status.boot_delay);


		if (blob_status.boot_delay == 0) {
			retval = SerialInputBlock(commandline, 1, 0); 
		} else {
			for(i = 0; i < blob_status.boot_delay; i++) {
				serial_write('.');

				retval =SerialInputString(commandline, 2, 1); 
				
				if(retval > 0)
					break;
			}
		}

		/* no key was pressed, so proceed booting the kernel */
#ifdef BLOB_DEBUG
		if ( commandline[0] != 0x20 ) {
#else
		if (1) {
#endif
//		if(retval == 0) {
			commandline[0] = '\0';
			if (bootup_reason & FTM_BOOTUP_REASON){
				parse_command("nftmkernel");
			}else{
				parse_command("nkernel");
			}
			commandline[0] = '\0';
			parse_command("boot");
		}

		printf("\nAutoboot aborted\n");
		printf("Type \"help\" to get a list of commands\n");
	}

#endif
	/* the command loop. endless, of course */
	for(;;) {
		DisplayPrompt(NULL);

		/* wait 10 minutes for a command */
		numRead = GetCommand(commandline, MAX_COMMANDLINE_LENGTH, 600);

		if(numRead > 0) {
			int retval;

			if((retval = parse_command(commandline)) < 0 )
				printerror(retval, NULL);
		}
	}

	return 0;
} /* main */



static int PrintStatus(int __attribute__((unused)) argc,
			char __attribute__((unused)) *argv[])
{
	printf(version_str);

	printf("Download speed      : ");
	PrintSerialSpeed(blob_status.downloadSpeed);
	printf(" baud\n");

	printf("Terminal speed      : ");
	PrintSerialSpeed(blob_status.terminalSpeed);
	printf(" baud\n");

	printf("blob    (0x%08x): ", BLOB_FLASH_BASE);
	if(blob_status.blobType == fromFlash) {
		printf("from flash\n");
	} else {
		printf("downloaded at 0x%08x, %i bytes\n",
		       BLOB_RAM_BASE, blob_status.blobSize);
#ifdef CONFIG_MD5_SUPPORT
		printf("                      MD5: ");
		print_md5_digest(blob_status.blob_md5_digest);
		printf("\n");
#endif
	}

#ifdef PARAM_FLASH_BASE
	printf("param  (0x%08x): ", PARAM_FLASH_BASE);
	if(blob_status.paramType == fromFlash) {
		printf("from flash\n");
	} else {
		printf("downloaded at 0x%08x, %i bytes\n",
		       PARAM_RAM_BASE, blob_status.paramSize);
#ifdef CONFIG_MD5_SUPPORT
		printf("                      MD5: ");
		print_md5_digest(blob_status.param_md5_digest);
		printf("\n");
#endif
	}
#else
	printf("param               : Not available\n");
#endif

#ifdef KERNEL_FLASH_BASE
	printf("kernel  (0x%08x): ", KERNEL_FLASH_BASE);
	if(blob_status.kernelType == fromFlash) {
		printf("from flash\n");
	} else {
		printf("downloaded at 0x%08x, %i bytes\n",
		       KERNEL_RAM_BASE, blob_status.kernelSize);
#ifdef CONFIG_MD5_SUPPORT
		printf("                      MD5: ");
		print_md5_digest(blob_status.kernel_md5_digest);
		printf("\n");
#endif
	}
#endif

#ifdef RAMDISK_FLASH_BASE
	printf("ramdisk (0x%08x): ", RAMDISK_FLASH_BASE);
	if(blob_status.ramdiskType == fromFlash) {
		printf("from flash\n");
	} else {
		printf("downloaded at 0x%08x, %i bytes\n",
		       RAMDISK_RAM_BASE, blob_status.ramdiskSize);
#ifdef CONFIG_MD5_SUPPORT
		printf("                      MD5: ");
		print_md5_digest(blob_status.ramdisk_md5_digest);
		printf("\n");
#endif
	}
#endif
	
	return 0;
}

static char statushelp[] = "status\n"
"Display current status\n";

__commandlist(PrintStatus, "status", statushelp);

#ifdef SIM_WORKAROUND_ENABLE

#define LDO4_ON         1
#define LDO4_OFF        0
static void Micco_LDO4_on_off( int on_off )
{
	unsigned char data;

	if( on_off == LDO4_ON ){//on
		nor_i2c_read( 0x12, &data );
		data |= 0x40;           //LDO4 on
		nor_i2c_write( 0x12, data );
	}
	else{//off
		nor_i2c_read( 0x12, &data );
		data &= 0xBF;           //LDO4 off
		nor_i2c_write( 0x12, data );
	}
}

#define SIM_1P8V         1
#define SIM_2P9V         0
static void Micco_SIM_1P8_2P9_V( int V_1p8_2p9 )
{
	unsigned char data;

	if( V_1p8_2p9 == SIM_1P8V ){//1.8V
		nor_i2c_read( 0x13, &data );
		data &= 0xEF;           //1.8V
		nor_i2c_write( 0x13, data );
	}
	else{//V2.9
		nor_i2c_read( 0x13, &data );
		data |= 0x10;           //2.9V
		nor_i2c_write( 0x13, data );
	}
}

static int cmd_MiccoSIM2P9(int argc, char *argv[])
{
	Micco_SIM_1P8_2P9_V( SIM_2P9V );
        Micco_LDO4_on_off( LDO4_ON );

	return 0;
}

static char MiccoSIM2P9help[] = "MiccoSIM2P9\n"
"Set SIM 2.9V\n";
__commandlist(cmd_MiccoSIM2P9, "MiccoSIM2P9", MiccoSIM2P9help);
 
#endif //SIM_WORKAROUND_ENABLE



static int Reload(int argc, char *argv[])
{
	if(argc < 2)
		return -ENOPARAMS;

	return do_reload(argv[1]);
}

static char reloadhelp[] = "reload {blob|param|kernel|ramdisk}\n"
"Reload <argument> from flash to RAM\n";

__commandlist(Reload, "reload", reloadhelp);



#if 0
static void print_elf_sections(void)
{
#ifdef BLOB_DEBUG
	extern u32 __text_start, __text_end;
	extern u32 __rodata_start, __rodata_end;
	extern u32 __data_start, __data_end;
	extern u32 __got_start, __got_end;
	extern u32 __commandlist_start, __commandlist_end;
	extern u32 __initlist_start, __initlist_end;
	extern u32 __exitlist_start, __exitlist_end;
	extern u32 __ptagtable_begin, __ptagtable_end;
	extern u32 __bss_start, __bss_end;
	extern u32 __stack_start, __stack_end;

	printf("ELF sections layout:\n");

#define PRINT_ELF_SECTION(from, to, name) \
	printf("  0x%08x - 0x%08x  %s\n", \
	       (unsigned int)&(from), \
	       (unsigned int)&(to), \
	       name);

	PRINT_ELF_SECTION(__text_start, __text_end, ".text");
	PRINT_ELF_SECTION(__rodata_start, __rodata_end, ".rodata");
	PRINT_ELF_SECTION(__data_start, __data_end, ".data");
	PRINT_ELF_SECTION(__got_start, __got_end, ".got");
	PRINT_ELF_SECTION(__commandlist_start, __commandlist_end, ".commandlist");
	PRINT_ELF_SECTION(__initlist_start, __initlist_end, ".initlist");
	PRINT_ELF_SECTION(__exitlist_start, __exitlist_end, ".exitlist");
	PRINT_ELF_SECTION(__ptagtable_begin, __ptagtable_end, ".ptaglist");
	PRINT_ELF_SECTION(__bss_start, __bss_end, ".bss");
	PRINT_ELF_SECTION(__stack_start, __stack_end, ".stack (in .bss)");
#endif
}

static void *find_parameters(void) 
{
	const blob_partition_t *p = pt_find_by_name("param");

	return p ? pt_flash_start(p) : NULL;
}
#endif

static int cmd_blob_serial_received_char(int __attribute__((unused)) argc, char __attribute__((unused)) *argv[])
{
    int i;
	
	printf( "serial_received_str_num: %x \n", serial_received_str_num );

    for ( i = 0; i <= serial_received_str_num; i++ )
    {
        printf( "serial_received_str[%d] = 0x%x \n", i, serial_received_str[i] );
    }

	return 0;
}


static char cmd_blob_serial_received_char_help[] = "print received char from serial\n";
__commandlist( cmd_blob_serial_received_char, "blob_test", cmd_blob_serial_received_char_help );

void write_to_memory(unsigned long *start_addr, int num,
		unsigned long value )
{
	unsigned long *addr;
	int i;

	for(addr = start_addr, i = 0; i < num; i++, addr++){
		*addr = value;
	}
}

unsigned long read_from_memory(unsigned long *start_addr, int num,
		unsigned long value )
{
	unsigned long *addr, ret;
	int i;

	for(addr = start_addr, i = 0; i < num; i++, addr++){
		ret = *addr;
		if (ret != value){
			break;
		}
	}

	return ret;
}

static int cmd_memdetect(int __attribute__((unused)) argc,
		char __attribute__((unused)) *argv[])
{
/*
 * if ddr address at 0x80000000 is 128Mddr set retval low 16 bits 
 * 0000000000000100, if ddr is 256M set retval low 16 bits 
 * 0000000000001000
 * if ddr address at 0xc0000000 is 128M ddr set retval high 16 bits
 * 0000000000000100, if ddr is 256M set reval higt 16 bits 
 * 0000000000001000 
 * */
#define CHECK_NUM (20)
	unsigned long *addr;
	unsigned long w1, w2;
	unsigned long pattern[] = {0xaaaaaaaa, 0x55555555}, backup[CHECK_NUM];
	int i;
	int retval = 0;

	write_to_memory((unsigned long *)(0x88000000-0x100000), 
			CHECK_NUM, pattern[0]);
	write_to_memory((unsigned long *)(0x90000000-0x100000), 
			CHECK_NUM, pattern[1]);

	write_to_memory((unsigned long *)(0xc0000000), CHECK_NUM, pattern[0]);
	write_to_memory((unsigned long *)(0xc8000000), CHECK_NUM, pattern[1]);


	w1 = read_from_memory((unsigned long *)(0x88000000-0x100000), 
			CHECK_NUM, pattern[0]);
	w2 = read_from_memory((unsigned long *)(0x90000000-0x100000), 
			CHECK_NUM, pattern[1]);

	if ( w1 == pattern[0] && w2 == pattern[1]){
		for(addr = (unsigned long *)(0x88000000-CHECK_NUM/2), i = 0; 
				i < CHECK_NUM; i++, addr++){
			backup[i] = *addr;
			*addr = pattern[0];
		}

		w1 = read_from_memory((unsigned long *)(0x88000000 - CHECK_NUM/2),
			   	CHECK_NUM, pattern[0]);
		
		if (w1 != pattern[0]){
			retval |= DDR128M_OFFSET;
		}else{
			retval |= DDR256M_OFFSET;
		}
		
		for(addr = (unsigned long *)(0x88000000-CHECK_NUM/2), i = 0; 
				i < CHECK_NUM; i++, addr++){
			*addr = backup[i];
		}
	}else if (w1 == pattern[1] && w2 == pattern[1]){
		retval |= DDR128M_OFFSET;
	}

//	write_to_memory((unsigned long *)(0xc0000000), CHECK_NUM, pattern[0]);
//	write_to_memory((unsigned long *)(0xc8000000), CHECK_NUM, pattern[1]);
	w1 = read_from_memory((unsigned long *)(0xc0000000), 
			CHECK_NUM, pattern[0]);
	w2 = read_from_memory((unsigned long *)(0xc8000000),
		   	CHECK_NUM, pattern[1]);

	if ( w1 == pattern[0] && w2 == pattern[1]){
		for(addr = (unsigned long *)(0xc8000000-CHECK_NUM/2), i = 0;
				i < CHECK_NUM; i++, addr++){
			backup[i] = *addr;
			*addr = pattern[0];
		}
		w1 = read_from_memory((unsigned long *)(0xc8000000-CHECK_NUM/2),
			   	CHECK_NUM, pattern[0]);

		if (w1 != pattern[0]){
			retval |= DDR128M_OFFSET;
		}else{
			retval |= DDR256M_OFFSET;
		}
		
		for(addr = (unsigned long *)(0xc8000000-CHECK_NUM/2), i = 0;
				i < CHECK_NUM; i++, addr++){
			*addr = backup[i];
		}
	}else if (w1 == pattern[1] &&  w2 == pattern[1]){
		retval |= (DDR128M_OFFSET<<16);
	}

	return retval;
}
//static char memdetecthelp[] = "detect memory \n";
//__commandlist(cmd_memdetect, "memdetect", memdetecthelp);

void modify_mtdparts(char *des, char *src)
{
	int i, j;

	for (i = 0, j = 0; src[i] != 0; i++, j++){
		if ('r' == src[i] && 'o' == src[i+1] && ')' == src[i-1] && 
				('T' != src[i-2] && 'B' != src[i-3] && 'B' != src[i-4])){
			i += 2;
		}

		des[j] = src[i];
	}
	des[j] = 0;
}

void blob_set_cmdline(void)
{
	int ret;
	int i, size, pmem=0;
	char *cmdline;
	u32	bootup_reason;
	char commandline[MAX_COMMANDLINE_LENGTH];
	char *mtd_partition;
	char *p;

typedef enum {
	PU_REASON_HARDWARE_RESET,
	PU_REASON_WATCHDOG,
	PU_REASON_RTC_ALARM,
	PU_REASON_PWR_KEY_PRESS,
	PU_REASON_USB_CABLE,
	PU_REASON_USB_FACTORY,
	PU_REASON_USB_CHARGER,
	PU_REASON_REBOOT_NORMAL,
	PU_REASON_REBOOT_FACTORY,
	PU_REASON_REBOOT_RECOVERY,
	PU_REASON_REBOOT_FOTA,
	PU_REASON_REBOOT_CRITICAL,
	PU_REASON_LOW_PWR_RESET,
	PU_REASON_MAX
} powerup_reason_t;


	memcpy( blob_status.cmdline, default_cmdline, sizeof(default_cmdline));
	cmdline = (char *)&blob_status.cmdline;

	bootup_reason = get_boot_info();

	mtd_partition = mtdparts;

	bootup_reason = bootup_reason & 0x0000ffff;

	if (PU_REASON_REBOOT_RECOVERY == bootup_reason){
		p = cmdline+strlen(cmdline);
		*p= ' ';
		p++;
		modify_mtdparts( p, mtd_partition);
	} else {
		snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE, " ");
		snprintf(cmdline + strlen(cmdline), 
				COMMAND_LINE_SIZE, mtd_partition);
	}

	pmem = 1;
	size = 512 - 16 - 12;
	ret = 0;

    //	printf("0x80000000 pmem = %d, mem = %dM\n", pmem, size);
	snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE,
			" pmem=%d mem=%dM@0xa0000000", pmem, size);
//	printf("0x8000000 cmdline = %s\n", cmdline);

	if ( (ret >> 16) > 0)
	{
		for (i = 0, size = 64; i < MAXDDR; i++ ){
			if ( (ret>>16) & (1<<i))
			{
				break;
			}
			size = size*2;
		}
		if ( i < MAXDDR ){
		snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE,
				" mem=%dM@0xc0000000", size);
		}

//	printf("0xc0000000 mem = %dM\n", size);
	}

#ifdef CONFIG_SECURITY_SUPPORT
    parse_command("setcmdline -a security_chip");
#endif
}



static int cmd_setcmdline(int __attribute__((unused)) argc,
		char __attribute__((unused)) *argv[])
{
	int i, j;
	char temp_cmdline[COMMAND_LINE_SIZE];
	char *cmdline;
	int temp_len;
	
	cmdline = (char *)&blob_status.cmdline;

	if (!strncmp("-a", argv[1], 2)&&(3<=argc)){//add new command
 		for (i = 2; i < argc; i++ ){
 			snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE, " ");
 			snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE, argv[i]);
 		}
 	} else if (!strncmp("-r", argv[1], 2) && (4<=argc)){// replace command
 		temp_len = strlen(argv[2]);
 
		for (i = 0, j = 0; (cmdline[i] != 0 || i < COMMAND_LINE_SIZE); 
				i++ ){
 			if (!strncmp(cmdline + i, argv[2], temp_len)){
 				i = i + temp_len;
 				for(; cmdline[i] != 0;i++){
 					if (0x20 == cmdline[i]){
 						i++;
 						break;
 					}
 				}
 			}
 			temp_cmdline[j++] = cmdline[i];
 		}
		if ( i == j ){
			printf("can`t find : %s\n", argv[2]);
			return 0;
		}else{
	  		cmdline[0] = 0;
	  		memcpy(cmdline, temp_cmdline, sizeof(temp_cmdline));
	  		snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE, " ");
	  		snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE, argv[3]);
		}
   	} else if (!strncmp("-d", argv[1], 2)&&(3<=argc)){// delete command
		temp_len = strlen(argv[2]);
		for (i = 0, j = 0; (cmdline[i] != 0 || i < COMMAND_LINE_SIZE); 
				i++ ){
			if (!strncmp(cmdline + i, argv[2], temp_len)){
				i = i + temp_len;
				for(; cmdline[i] != 0;i++){
					if (0x20 == cmdline[i]){
						i++;
						break;
					}
				}
			}
			temp_cmdline[j++] = cmdline[i];
		}
		if ( i == j ){
			printf("can`t find : %s\n", argv[2]);
			return 0;
		}else{
			cmdline[0] = 0;
			memcpy(cmdline, temp_cmdline, sizeof(temp_cmdline));
		}
   	} else if (!strncmp("-n", argv[1], 2)&&(2<=argc)){
			// new command line, will delete default command line
		cmdline[0] = 0;
		if (3<=argc){
			for (i = 2; i < argc; i++ ){
				snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE,
					   	argv[i]);
				snprintf(cmdline + strlen(cmdline), COMMAND_LINE_SIZE, " ");
			}
		}
	}	       

	printf("argc = %d\n",argc);
	printf("cmdline :%s\n", cmdline);
	return 0;
}
static char setcmdlinehelp[] = " setcmdline -a cmd1 cmd2 ......[add new command at the end of the current command line\n"
"setcmdline -r [replaced cmd] [new cmd]\n"
"setcmdline -d delete a cmd\n"
"setcmdline -n write a new command line\n";
__commandlist(cmd_setcmdline, "setcmdline", setcmdlinehelp);


