/*
 * mainstone.c: Mainstone specific stuff
 *
 * Copyright (C) 2003 Intel Corporation (yu.tang@intel.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/serial_usb.h>
#include <blob/led.h>
#include <blob/time.h>
#include <net/ether.h>
#include <net/net.h>
#include <blob/strata.h>
#include <blob/command.h>




static struct flash_chip flash_chips[2];
  

#undef  FLASH_SYNC_MODE_SUPPORT
//#define  FLASH_SYNC_MODE_SUPPORT 1

/* Flash sync-mode enabling */
#ifdef FLASH_SYNC_MODE_SUPPORT

#undef  SXCNFG_VAL

#define SXCNFG_VAL  	0x00006001  	/* Synch. Static Mem. Config. Reg. (Burst-of-8 reads, CL=4, SDCLK return latching) */
#define RCR_VAL		0x000000c2

static void flash_sync_mode(struct flash_chip *chip)
{
	u32 L= CCCR & 0x1f;
	u32 rcr_val;
	u8 lc;
	volatile u16 *rcr16;//, *base16;
	volatile u32 *rcr32;//, *base32;
	unsigned long  msc0_val, memclk, clkcfg_val, M, mdrefr_val;
	unsigned long sxcnfg_val = SXCNFG_VAL;
	float memclk_period;
	int we_clk_pulse;
	u8 rrr_val;

	u8 LC_vals[]={
		0,	// L=0x0 ; placeholder
		0,	// L=0x1 ; placeholder
		1,	// L=0x2 ; * close
		2,	// L=0x3 
		3,	// L=0x4
		2,	// L=0x5
		2,	// L=0x6
		3,	// L=0x7
		3,      // L=0x8 ; * close
		2,	// L=0x9
		2,	// L=0xA 10
		2,	// L=0xB 11
		2,	// L=0xC 12
		3,      // L=0xD 13* close
		3,	// L=0xE 14
		3,	// L=0xF 15
		3,      // L=0x10 16 close
		/* 0x11-0x1C - de-featured */
		0,	//L=0x11 
		0,	//L=0x12
		0,	//L=0x13
		0,	//L=0x14
		0,	//L=0x15
		0,	//L=0x16
		0,	//L=0x17
		0,	//L=0x18
		0,	//L=0x19
		0,	//L=0x1A
		0,	//L=0x1B
		0,	//L=0x1C

		3,	//L=0x1D 29
		3,	//L=0x1E 30
		3,	//L=0x1F 31
	};
	
	u8 rdf_converter[31] = {  //incompatible with bulverde A0
		0, //invalid can't do zero clks
		0,
		1,
		2,
		3,
		4,
		5,
		6,
		7,
		8,
		9,
		10,
		11, //12 clks for we_ low
		15, //13 clks rounds to 
		15, //14 clk rounds to
		15, // 15 clks = 15 clks
		20, // 16 clks rounds to 20
		20, // 17 clks rounds to 20
		20, // 18 clks rounds to 20
		20, // 19 clks rounds to 20
		20, // 20 clks = 20 clks
		26, //21
		26, //22
		26, //23
		26, //24
		26, //25
		26, //26 = 26
		30, //27
		30, //28
		30, //29
		30  //30
	};  

	msc0_val 	= MSC0;
	
	/*Retrive the clkcfg_val from coprocessor*/
	asm volatile ("mrc p14, 0, %0, c6, c0, 0":"=r"(clkcfg_val)); 
	
	/*calculate L and M (bulverde clock values)*/
	if ( CCCR_A & CCCR )  {
		/* MEM clock frequency = System Bus Frequence */
		if (CLKCFG_B & clkcfg_val) memclk = (L*13);
		else memclk = (L*13)/2;
	}
	else {
		if (L<=10) M = 1;
		else if (L<=20) M = 2;
		else M = 4;

		memclk = (L*13)/M;
	}


	/*Set SDCLK0 (flash clock) for tyax*/
	if ( (chip->type ==STRATA_L3) || (chip->type == STRATA_L18) ) {		
		
		/*Fetch MDREFR*/
		mdrefr_val	= MDREFR;
	
		if (memclk <= 52) {
			/* SDCLK0 = MEMCLK */
			mdrefr_val &= ~(MDREFR_K0DB2 | MDREFR_K0DB4);
		}
		else if (memclk <= 104) {
			/* SDCLK0 = MEMCLK/2 */
			mdrefr_val &= ~MDREFR_K0DB4;
			mdrefr_val |= MDREFR_K0DB2;
		}else{
			/* SDCLK0 = MEMCLK/4 */
			mdrefr_val |= MDREFR_K0DB4;
		}
		MDREFR = mdrefr_val;
		__asm__ ( "ldr r0, [%0]" ::"r"(&MDREFR):"r0");
	}	


	/*Only proceed if we know what kind of flash this is*/
	if ( (chip->type != STRATA_K3) && (chip->type != STRATA_K18) && (chip->type != STRATA_L3) && (chip->type != STRATA_L18) ) return;

 	/*Look to see if we know how to translate the L to an LC count*/
	if( L >= sizeof(LC_vals) ) return; /* leave in asyn mode */
	
	/* Look up lc count for from the LC_vals table*/	
	lc = LC_vals[L];

	/* Adjust RCR for optimal timing - Mainstone */
	rcr_val = (RCR_VAL & 0x87ff) | (lc << 11);
	chip->rcr = rcr_val;

	switch(chip->buswidth){
	case 2:
		rcr_val |= 0x3; /* Turning on burst of 16 */
		
		rcr16 = (volatile u16*) (chip->base + (rcr_val << 1));
		
		*rcr16 = 0x00ff;

		*rcr16 = 0x0060;

		*rcr16 = 0x0003;

		*rcr16 = 0x00ff;
		
		sxcnfg_val |= 0x3000; //forcing burst of 16 to the SXCNFG register
		break;
	case 4:
		/* 32 bit, reflecting << 2 */
		rcr32  = (volatile u32*) (chip->base + (rcr_val << 2));
		
		/* read-mode */
		*rcr32 = 0x00ff00ff;

		*rcr32 = 0x00600060;
 
		*rcr32 = 0x00030003;

		/* read-mode */
		*rcr32 = 0x00ff00ff;

		break;
	default:
		/* won't sync */
		return;
	}

 	/* Adjust and set SXCNFG */
	SXCNFG = (sxcnfg_val & (~0x801C)) | ((lc & 0x7) << 2 ) | ((lc & 0x8) << 12 );
	__asm__ ("ldr r0, [%0]"::"r"(&SXCNFG):"r0");
	
	/*Calculate the memclk period*/
	memclk_period = (float) (1000 / memclk); //period is nanoseconds memclk is mhz
	
	#define L_WE_CLK 50 /* 48 +2 nanoseconds */
	#define L_RRR_CLK 20 /* 20 nanoseconds */
	
	/*If we have tyax we can tune the we# clock pulse length to speed up write cycles*/
	if ( (chip->type ==STRATA_L3) || (chip->type == STRATA_L18) ) {
		we_clk_pulse = (int) ((L_WE_CLK)/ memclk_period); 
		/*If there is a remainder above round up*/
		if( (float)(L_WE_CLK)!=(((float)we_clk_pulse)*memclk_period) ) we_clk_pulse++;
		/*Change the RDF value in the MSCO value*/
		msc0_val = ( (msc0_val & (~0x00f000f0)) | ((rdf_converter[we_clk_pulse] << 4)|(rdf_converter[we_clk_pulse] << 20)) );
		
		/*calculate rrr_val*/
		rrr_val = (u8) ((float)(L_RRR_CLK)/ memclk_period); 
		/*If there is a remainder above round up*/
		if( (float)(L_RRR_CLK)!=(((float)rrr_val)*memclk_period) ) rrr_val++;
		
		/* rrr = 2*X+1 memclks */
		if( (float)(L_RRR_CLK) != (float)((rrr_val-1)/2)*memclk_period )
		{
			rrr_val=(rrr_val-1)/2;
			rrr_val++;
		}
		else
		{
			rrr_val=(rrr_val-1)/2;
			rrr_val++;
		}
		
		
		/*update the msc0 value with our new rrr*/
		msc0_val = ( (msc0_val & (~0x70007000)) | ((rrr_val << 12)|(rrr_val << 28)) );

		/*write the value out the register then read it back because thats what everybody else does*/
		MSC0 = msc0_val;
		__asm__ ("ldr r0, [%0]"::"r"(&MSC0):"r0");
		 
	}
	
}

#endif

static void init_mainstone_flash_driver(void)
{
	int i = 0;
	flash_descriptors = NULL;
 	flash_driver = NULL;
 
 	/* Two banks */
 	for (i=0; i<2; i++) {
 		struct flash_chip *chip = flash_chips + i;

 		chip->base = i * 0x04000000;
 
 		if (BOOT_DEF & 0x1) chip->buswidth = 2;
 		else chip->buswidth = 4;

		flash_probe(chip);
	}

#ifdef FLASH_SYNC_MODE_SUPPORT
	/* BOOT ROM sync-mode */
	flash_sync_mode(&flash_chips[0]);
#endif
}
  
__initlist(init_mainstone_flash_driver, INIT_LEVEL_DRIVER_SELECTION);


static void mainstone_init_hardware(void)
{
#ifdef CONFIG_NETWORK_SUPPORT
	unsigned char serverip[4]={192,168,1,100};
	unsigned char clientip[4]={192,168,1,101};

	/* set default ip */
	memcpy(serveripaddress, serverip, sizeof(serverip));
	memcpy(clientipaddress, clientip, sizeof(clientip));
#endif
	/* light HEX Led */
	MST_LEDDAT1 = 0xb10bb10b;

#ifdef USE_SERIAL2
	/* enable BTUART */
	MST_MSCWR1 |= (0x3<<7);
	
	/* setup BTUART GPIOs (BTRXD, BTTXD, BTCTS, BTRTS) */
	GPDR1 &= ~0x00003A00;
	GPDR1 |= 0x00002800;
	GAFR1_L &= ~0x0FF00000;
	GAFR1_L |= 0x09900000;
#endif

	/* select drivers */
	reboot_driver = &pxa_reboot_driver;
	serial_driver = &pxa_serial_driver;

#ifdef CONFIG_USBSERIAL_SUPPORT	
	/* This is USB client softconnect control bit 
	   in Mainstone II board register MST_MSCWR2.
	*/
	MST_MSCWR2 &= ~MST_USBC_SC;
	usb_serial_driver= &bvd_usb_serial_driver;

#endif 

	led_driver = &mmap_led_driver;
	timer_driver = &intelarm_timer_driver;
#ifdef CONFIG_USBDNET_SUPPORT
	/* This is USB client softconnect control bit 
	   in Mainstone II board register MST_MSCWR2.
	*/
	MST_MSCWR2 &= ~MST_USBC_SC;

	ether_driver = &bvdusb_ether_driver;
#else
	ether_driver = &smc91x_ether_driver;
#endif
}


__initlist(mainstone_init_hardware, INIT_LEVEL_DRIVER_SELECTION);

/*********************************************************************
 * cmd_flash_write
 *
 * AUTOR:	SELETZ
 * REVISED:
 *
 * Command wrapper for low-level flash write access
 *
 */
static int cmd_flash_write( int argc, char *argv[] )
{
	int ret = 0;
	u32	src	= 0L;
	u32	dest	= 0L;
	u32	len	= 0L;
	struct flash_chip *chip=NULL;

	if ( argc < 4 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[1], &src );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[2], &dest );
	if ( ret < 0 )	{
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[3], &len );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}

	if (dest<0x04000000) chip = &flash_chips[0];
	else chip = &flash_chips[1];

	ret = flash_write( chip, dest, (u8*) src, len); 

DONE:
	return ret;
}


static char flashwritehelp[] = "fwrite srcadr destadr size(bytes)\n"
"flash a memory region\n";
__commandlist( cmd_flash_write, "fwrite", flashwritehelp );

/*********************************************************************
 * cmd_flash_erase
 *
 * AUTOR:	SELETZ
 * REVISED:
 *
 * Command wrapper for low-level flash erasing
 *
 */
static int cmd_flash_erase( int argc, char *argv[] )
{
	int ret = 0;
	u32	dest	= 0L;
	u32	len	= 0L;
	struct flash_chip *chip=NULL;

	if ( argc < 3 ) {
		ret = -1;
		goto DONE;
	}

	ret = strtou32( argv[1], &dest );
	if ( ret < 0 ) {
		ret = -1;
		goto DONE;
	}	

	ret = strtou32( argv[2], &len );
	if ( ret < 0 ){
		ret = -1;
		goto DONE;
	}

	if(dest < 0x04000000) chip = &flash_chips[0];
	else chip = &flash_chips[1];

	ret = flash_erase(chip, dest, len);

DONE:
	return ret;
}

static char flasherasehelp[] = "ferase adr size(bytes)\n"
"erase a flash region\n";
__commandlist( cmd_flash_erase, "ferase", flasherasehelp );

/*********************************************************************
 * cmd_clock_info
 *
 * AUTOR:	yu tang, rusty geldmacher
 * REVISED:
 *
 * prints out clock information. taken from Yu's kernel patch
 *
 */
#define BASE_CLK 13000000
static int cmd_clock_info(int argc, char *argv[])
{
	unsigned long cccr_val, ccsr_val, clkcfg_val;
	unsigned int L, M, N2;
	unsigned int r_clk, t_clk, s_clk, m_clk;
	int k0db, k1db;
  
	cccr_val = CCCR;
	ccsr_val = CCSR;
	asm( "mrc p14, 0, %0, c6, c0, 0" : "=r" (clkcfg_val) );
  
	L  =  ccsr_val & 0x1f;
	N2 =  (ccsr_val>>7) & 0x7;
  
	if (L <= 10) M = 1;
	else if (L <=20 ) M =2;
	else M=4;
  
	r_clk = (L * BASE_CLK);
	t_clk = (L * BASE_CLK * N2) / 2;
  
	/* Fast-bus Mode ? */
	if (clkcfg_val & 0x8) 
		s_clk = r_clk;
	else
		s_clk = r_clk/2;
  
	/* Alternate setting? */
	if (cccr_val & 0x2000000)
		m_clk = s_clk;
	else
		m_clk = r_clk / M;
  
	if (MDREFR & MDREFR_K0DB4) 
		k0db = 4;
	else if (MDREFR & MDREFR_K0DB2) 
		k0db = 2;
	else 
		k0db = 1;
  
	if (MDREFR & MDREFR_K1DB2) 
		k1db = 2;
	else 
		k1db = 1;
  
	printf("Run Mode clock: %d.%02dMHz (*%d)\n",
	 	r_clk / 1000000, (r_clk % 1000000) / 10000, L );
	printf("Turbo Mode clock: %d.%02dMHz (*%d.%d, %sactive)\n",
		t_clk / 1000000, (t_clk % 1000000) / 10000, N2 / 2, (N2 % 2) * 5,
		(clkcfg_val & 0x1) ? "" : "in" );
	printf("Memory clock: %d.%02dMHz (Alt=%d, SDCLK[0]=/%d, SDCLK[1]=/%d)\n",
		m_clk / 1000000, (m_clk % 1000000) / 10000,(int)(cccr_val>>25)&1,
		k0db, k1db );
	printf("System bus clock: %d.%02dMHz \n",
		s_clk / 1000000, (s_clk % 1000000) / 10000 );

#ifdef FLASH_SYNC_MODE_SUPPORT
	printf("----------\n");
	printf("MEMC_CTL : MDREFR  = 0x%08X, MSC0 = 0x%08X, SXCNFG = 0x%08X\n", MDREFR, MSC0, SXCNFG);
	printf("BOOT_ROM : RCR_VAL = 0x%08X, LC   = 0x%08x\n",  flash_chips[0].rcr, (flash_chips[0].rcr >> 11) & 0x7 ); 
	printf("----------\n");
#endif
}

static char clockinfohelp[] = "clockinfo\n"
"print out processor clock information\n";
__commandlist(cmd_clock_info, "clockinfo", clockinfohelp);


/*********************************************************************
 * cmd_go
 *
 * AUTOR:	Yu Tang
 * REVISED:
 *
 * Start execution at indicated address
 *
 */
static int cmd_go(int argc, char *argv[])
{
	void (*myfunc)() = NULL;
	int ret;

	if (argc != 2) return -1;

	ret = strtou32( argv[1], &myfunc);
	if (ret < 0) return -1;

	printf("\nStarting at 0x%08x...\n\n", (u32)myfunc);
	myfunc();
	printf("\nDone.\n");

	return 0;
}

static char gohelp[] = "go address\n"
"Start execution at $address\n";
__commandlist(cmd_go, "go", gohelp);
