/*
 * memory.c: memory test routines for the blob bootloader
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: memory.c,v 1.5 2002/02/13 00:09:51 erikm Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/debug.h>
#include <blob/memory.h>
#include <blob/util.h>

/* test in 1MB chunks */
#define TEST_BLOCK_SIZE (1024 * 1024)




/* from testmem2.S */
extern int testram(u32 addr);




memory_area_t memory_map[NUM_MEM_AREAS];




void get_memory_map(void)
{
	u32 addr;
	int i;

	/* init */
	for(i = 0; i < NUM_MEM_AREAS; i++)
		memory_map[i].used = 0;

	/* first write a 0 to all memory locations */
	for(addr = MEMORY_START; addr < MEMORY_END; addr += TEST_BLOCK_SIZE)
		* (u32 *)addr = 0;

	/* scan memory in blocks */
	i = 0;
	for(addr = MEMORY_START; addr < MEMORY_END; addr += TEST_BLOCK_SIZE) {
		if(testram(addr) == 0) {
			/* yes, memory */
			if(* (u32 *)addr != 0) { /* alias? */
				dprintf("detected alias at 0x%08x, "
					"aliased from 0x%08x\n",
				        addr, * (u32 *)addr);

				if(memory_map[i].used)
					i++;

				continue;
			}

			/* not an alias, write the current address */
			* (u32 *)addr = addr;

			dprintf("detected memory at 0x%08x\n", addr);

			/* does this start a new block? */
			if(memory_map[i].used == 0) {
				memory_map[i].start = addr;
				memory_map[i].len = TEST_BLOCK_SIZE;
				memory_map[i].used = 1;
			} else {
				memory_map[i].len += TEST_BLOCK_SIZE;
			}
		} else {
			/* no memory here */
			if(memory_map[i].used == 1)
				i++;
		}
	}
	printf("Memory map:\n");
	for(i = 0; i < NUM_MEM_AREAS; i++) {
		if(memory_map[i].used) {
			printf("  0x%08x @ 0x%08x (%d MB)\n",
			       memory_map[i].len,
			       memory_map[i].start,
			       memory_map[i].len / (1024 * 1024));
		}
	}
}
