/*
 * pleb.c: PLEB specific stuff
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: pleb.c,v 1.9 2003/09/04 17:39:00 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/flash.h>
#include <blob/init.h>
#include <blob/led.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/time.h>




/* the LED should always be on or otherwise the serial transmitter
 * doesn't work
 */
static void lock_pleb_led(void)
{
	led_unlock();
	led_on();
	led_lock();
}


/* do this just after the initial hardware has been initialised */
__initlist(lock_pleb_led, INIT_LEVEL_INITIAL_HARDWARE + 1);




/* flash descriptor for PLEB flash. */
static const flash_descriptor_t pleb_flash_descriptors[] =
{
#warning "Please add PLEB flash layout"
	{
		/* NULL block */
	},
};




static void init_pleb_flash_driver(void)
{
	flash_descriptors = pleb_flash_descriptors;
#warning "Please add a proper PLEB flash driver"
	flash_driver = &null_flash_driver;
}

__initlist(init_pleb_flash_driver, INIT_LEVEL_DRIVER_SELECTION);




static void pleb_init_hardware(void)
{
	/* select drivers */
	reboot_driver = &sa11x0_reboot_driver;
	serial_driver = &sa11x0_serial_driver;
	led_driver = &sa11x0_gpio_led_driver;
	timer_driver = &intelarm_timer_driver;
}

__initlist(pleb_init_hardware, INIT_LEVEL_DRIVER_SELECTION);
