/*
 * littleton_fvscaler.c: Intel Monahans specific stuff
 *
 * Modified from src/blob/lubbock.c
 *
 * Copyright (C) 2006 Intel Corporation (jack.ren@intel.com)
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/time.h>
#include <blob/command.h>
#include <blob/util.h>
#include <net/ether.h>
#include <net/net.h>
#include <blob/strata.h>
#include <blob/command.h>
#include <blob/errno.h>
#include <blob/error.h>
#include "xllp_clkmgr.h"
#include "xllp_ost.h"
#include "xllp_i2c.h"

#define FREQ_CORE(xl, xn) ((xl)*(xn)*13)

#define FREQ_SRAM(sflfs) (((sflfs) == 0x0)?104:\
						  ((sflfs) == 0x1)?156:\
						  ((sflfs) == 0x2)?208:312)
						  
#define FREO_STMM(smcfs) (((smcfs) == 0x0)?78:\
						  ((smcfs) == 0x2)?104:\
						  ((smcfs) == 0x5)?208:0)
						  
#define FREQ_DDR(dmcfs)	 (((dmcfs) == 0x0)?26:\
				((dmcfs) == 0x2)?208:\
				((dmcfs) == 0x3)?260:0)
					      
#define FREQ_HSS(hss)    (((hss) == 0x0)?104:\
						  ((hss) == 0x1)?156:\
					 	  ((hss) == 0x2)?208:0)

#define FREQ_DFCLK(smcfs, df_clkdiv)    (((df_clkdiv) == 0x1)?FREO_STMM((smcfs)):\
						  ((df_clkdiv) == 0x2)?FREO_STMM((smcfs))/2:\
					 	  ((df_clkdiv) == 0x3)?FREO_STMM((smcfs))/4:0)

#define FREQ_EMPICLK(smcfs, empi_clkdiv)    (((empi_clkdiv) == 0x1)?FREO_STMM((smcfs)):\
						  ((empi_clkdiv) == 0x2)?FREO_STMM((smcfs))/2:\
					 	  ((empi_clkdiv) == 0x3)?FREO_STMM((smcfs))/4:0)

#define ACCR_G_XL(accr)			((accr) & 0x1F)
#define ACCR_G_XN(accr)			(((accr) >> 8) & 0x7)
#define ACCR_G_SMCFS(accr)		(((accr) >> 23)  & 0x7)
#define ACCR_G_SFLFS(accr)		(((accr) >> 18)  & 0x3)
#define ACCR_G_HSS(accr)		(((accr) >> 14)  & 0x3)
#define ACCR_G_DMCFS(accr)		(((accr) >> 12)  & 0x3)
#define ACCR_G_XSPCLK(accr)		(((accr) >> 16)  & 0x3)
#define ACCR_G_D0CS(accr)          (((accr) >> 26)  & 0x1)
#define ACCR_G_SPDIS(accr)          (((accr) >> 30)  & 0x1)
#define ACCR_G_XPDIS(accr)          (((accr) >> 31)  & 0x1)
#define ACCR_G_D0CS(accr)          (((accr) >> 26)  & 0x1)
#define MEMCLKCFG_G_DFCLKDIV(memclkcfg)   (((memclkcfg) >> 16)  & 0x7)
#define MEMCLKCFG_G_EMPICLKDIV(memclkcfg) (((memclkcfg) >> 0)  & 0x7)

#define ACCR_XL_MASK    		0x0000001f
#define ACCR_XN_MASK    		0x00000700
#define ACCR_PCCE_MASK 			0x00000800
#define ACCR_DMCFS_MASK      	0x00003000
#define ACCR_HSS_MASK 			0x0000c000
#define ACCR_XSPCLK_MASK   	    0x00030000
#define ACCR_SFLFS_MASK         0x000c0000
#define ACCR_13MEND2_MASK       0x00200000
#define ACCR_SMCFS_MASK         0x03800000
#define ACCR_D0CS_MASK          0x04000000
#define ACCR_13MEND1_MASK       0x08000000
#define ACCR_SPDIS_MASK         0x40000000
#define ACCR_XPDIS_MASK         0x80000000
#define MEMCLKCFG_DFCLKDIV_MASK    0x00070000
#define MEMCLKCFG_EMPICLKDIV_MASK  0x00000007
#define ACCR_MASK (ACCR_XL_MASK | ACCR_XN_MASK | ACCR_DMCFS_MASK |\
                   ACCR_HSS_MASK | ACCR_XSPCLK_MASK | ACCR_SFLFS_MASK |\
                   ACCR_13MEND2_MASK | ACCR_SMCFS_MASK | ACCR_PCCE_MASK |\
                   ACCR_D0CS_MASK | ACCR_13MEND1_MASK | ACCR_SPDIS_MASK |\
                   ACCR_XPDIS_MASK)

#define ACCR_XL_OFFSET   		0
#define ACCR_XN_OFFSET    		8
#define ACCR_PCCE_OFFSET      	11
#define ACCR_DMCFS_OFFSET      	12
#define ACCR_HSS_OFFSET 	    14
#define ACCR_XSPCLK_OFFSET   	16
#define ACCR_SFLFS_OFFSET       18
#define ACCR_13MEND2_OFFSET     21
#define ACCR_SMCFS_OFFSET       23
#define ACCR_D0CS_OFFSET        26
#define ACCR_13MEND1_OFFSET     27
#define ACCR_SPDIS_OFFSET       30
#define ACCR_XPDIS_OFFSET       31
#define MEMCLKCFG_DFCLKDIV_OFFSET    16
#define MEMCLKCFG_EMPICLKDIV_OFFSET  0

extern XLLP_STATUS_T XllpPmAravaSetVccSram(XLLP_UINT32_T mv);
extern XLLP_STATUS_T XllpPmAravaSetVccCore(XLLP_UINT32_T mv);
extern XLLP_STATUS_T XllpPmAravaGetVccSram(XLLP_UINT32_T *pmv);
extern XLLP_STATUS_T XllpPmAravaGetVccCore(XLLP_UINT32_T *pmv);
extern XLLP_STATUS_T XllpPmMiccoSetVccSram(XLLP_UINT32_T mv);
extern XLLP_STATUS_T XllpPmMiccoSetVccCore(XLLP_UINT32_T mv);
extern XLLP_STATUS_T XllpPmMiccoGetVccSram(XLLP_UINT32_T *pmv);
extern XLLP_STATUS_T XllpPmMiccoGetVccCore(XLLP_UINT32_T *pmv);
extern void udelay(unsigned int usec);

static int fvscaler_fCore(char *param);
static int fvscaler_fSmc(char *param);
static int fvscaler_fSram(char *param);
static int fvscaler_fHss(char *param);
static int fvscaler_fCore(char *param);
static int fvscaler_fDmc(char *param);
static int fvscaler_fDfClk(char *param);
static int fvscaler_fEmpiClk(char *param);
static int fvscaler_vCore(char *param);
static int fvscaler_vSram(char *param);
static int fvscaler_clkASet(char *param);
static int fvscaler_clkBSet(char *param);
static int fvscaler_clkAClear(char *param);
static int fvscaler_clkBClear(char *param);
static int fvscaler_i2cmode(char *param);
int fvscaler_init(char *param);
int detect_micco(void);

/*
 * 0 -- Standard I2C operation style,
 * 1 -- Power I2C with software style,
 * 2 -- Power I2c with hardware style,
 */
static int i2c_mode = 0;	/* default is normal I2C operation */

struct fv_scaler_handler {
	char *name;
	int (*fv_scaler)(char *param);
};

static struct fv_scaler_handler fv_scaler_tbl[]= {
	{"fCore", fvscaler_fCore},
	{"fSmc", fvscaler_fSmc},
	{"fSram", fvscaler_fSram},
	{"fHss", fvscaler_fHss},
	{"fCore", fvscaler_fCore},
	{"fDmc", fvscaler_fDmc},
	{"fDfClk", fvscaler_fDfClk},
	{"fEmpiClk",fvscaler_fEmpiClk},
	{"vCore", fvscaler_vCore},
	{"vSram", fvscaler_vSram},
	{"clkASet", fvscaler_clkASet},
	{"clkBSet", fvscaler_clkBSet},
	{"clkAClear", fvscaler_clkAClear},
	{"clkBClear", fvscaler_clkBClear},		
	{"mode", fvscaler_i2cmode},
	{"init", fvscaler_init},
	{NULL, NULL},
};

#define D0CKEN_A (*(volatile unsigned int*)0x4134000c)
#define D0CKEN_B (*(volatile unsigned int*)0x41340010)

static int fvscaler_clkASet(char *param)
{
	u32 clk;
	volatile unsigned int d0cken;

	if (strtou32(param, &clk)){
		printerror(ENAN, param);
		return -1;
	}

	if (clk > 31) {
		printf("clk:%d out of range\n", clk);
		return -1;
	}

	D0CKEN_A |= (0x1UL << clk);
	d0cken = D0CKEN_A;
	
	return 0;
}

static int fvscaler_clkBSet(char *param)
{
	u32 clk;
	volatile unsigned int d0cken;

	if (strtou32(param, &clk)){
		printerror(ENAN, param);
		return -1;
	}

	if (clk > 31) {
		printerror(ENAN, param);
		printf("clk:%d\n", clk);
		return -1;
	}

	D0CKEN_B |= (0x1UL << clk);
	d0cken = D0CKEN_B;
	
	return 0;
}

static int fvscaler_clkAClear(char *param)
{
	u32 clk;
	volatile unsigned int d0cken;

	if (strtou32(param, &clk)){
		printerror(ENAN, param);
		return -1;
	}

	if (clk > 31) {
		printerror(ENAN, param);
		printf("clk:%d\n", clk);
		return -1;
	}

	D0CKEN_A &= ~(0x1UL << clk);
	d0cken = D0CKEN_A;
	
	return 0;
}

static int fvscaler_clkBClear(char *param)
{
	u32 clk;
	volatile unsigned int d0cken;
	
	if (strtou32(param, &clk)){
		printerror(ENAN, param);
		return -1;
	}

	if (clk > 31) {
		printerror(ENAN, param);
		printf("clk:%d\n", clk);
		return -1;
	}

	D0CKEN_B &= ~(0x1UL << clk);
	d0cken = D0CKEN_B;
	
	return 0;
}

static int get_cpuid(void)
{
	 unsigned int    cpuid;
         /* read CPU ID */
         __asm__ (
                 "mrc p15, 0, %0, c0, c0, 0\n"
                 : "=r" (cpuid)
         );

	 return cpuid;
}

static int pwr_i2c_init(int level)
{
	unsigned int reg;
	unsigned int cpuid;

	cpuid = get_cpuid();

	if (((cpuid & 0xFFFF0000) != 0x69050000) &&
        ((cpuid & 0xFFFF0000) != 0x56050000)){
		/* It's not xscale chip. */
		return -1;
	}
	cpuid &= 0x0000FFFF;

	if (level && ((cpuid & 0xFFF0) == 0x6930)) {/* TavorPV */
		/* initialize power i2c with hardware style */

		/* disable interrupt after freq change, power mode sequence, temperatue */
		AICSR &= ~0x1F;

		/* set FVE, PVE and TVE bit */
		PVCR = 0xE0500034;

		/* set AVCR and SVCR */
		/* AVCR:
		 *	level 0: 1250mv, 0x15
		 *	level 1: 1250mv, 0x15
		 *	level 2: 1250mv, 0x15
		 *	level 3: 1250mv, 0x15
		 */
		reg = AVCR;
		reg &= 0xF0F0F0F0;
		reg |= (0x15 << 24);
		reg |= (0x15 << 16);
		reg |= (0x15 << 8);
		reg |= 0x15;
		AVCR = reg;
		reg = AVCR;
	} else if (level) { /* Monahans, TavorP */
		/* initialize power i2c with hardware style */

		/* disable interrupt after freq change, power mode sequence, temperatue */
		AICSR &= ~0x1F;

		/* set FVE, PVE and TVE bit */
		PVCR = 0xE0500034;

		/* set AVCR and SVCR */
		/* AVCR:
		 *	level 0: 1000mv, 0x0b
		 *	level 1: 1100mv, 0x0f
		 *	level 2: 1375mv, 0x1a
		 *	level 3: 1400mv, 0x1b
		 */
		reg = AVCR;
		reg &= 0xF0F0F0F0;
		reg |= (0x1b << 24);
		reg |= (0x1a << 16);
		reg |= (0x0f << 8);
		reg |= 0x0b;
		AVCR = reg;
		reg = AVCR;

		/* SVCR:
		 * 	level 0: 1100mv, 0x0f
		 * 	level 1: 1200mv, 0x13
		 * 	level 2: 1400mv, 0x1b
		 * 	level 3: 1400mv, 0x1b
		 */
		reg = SVCR;
		reg &= 0xF0F0F0F0;
		reg |= (0x1b << 24);
		reg |= (0x1b << 16);
		reg |= (0x13 << 8);
		reg |= 0x0f;
		SVCR = reg;
		reg = SVCR;

		/* set delay */
		reg = PCFR;
		reg &= 0x000FFFFF;
		reg |= 0xCCF00000;
		/* Disable pullup/pulldown resistor in PWR_SCL and PWR_SDA */
		reg |= 0x04;
		PCFR = reg;
		reg = PCFR;

	} else {
		/* initialize power i2c with software style */

		/* Disable pullup/pulldown resistor in PWR_SCL and PWR_SDA */
		PCFR |= 0x04;

		/* Polling VCSA bit */
		while (PVCR & 0x00004000) {}
		reg = PVCR;
		/* clear read pointer */
		reg &= 0xfe0fffff;
		/* clear delay timer */
		reg &= 0xfffff07f;
		/* set delay timer */
		reg |= 0x400;
		/* set FVC bit */
		reg |= 0x10000000;
		PVCR = reg;
		
		/* disable interrupt after freq change, power mode sequence, temperatue */
		AICSR &= ~0x1F;
	}
	return 0;
}

/* It's only used in software style */
static int pwr_i2c_send(unsigned char addr, unsigned char data)
{
	unsigned int reg;
	unsigned int __attribute__((unused)) tmp = 0;

	/* Polling VCSA bit */
	while (PVCR & 0x00004000) {}

	/* reset the read pointer in PVCR */
	PVCR &= 0xfe0fffff;

	/* fill PCMD[0] with reg address */
	reg = addr & 0xff;
	/* set DCE, MBC bit */
	reg |= 0x1800;
	PCMD(0) = reg;

	/* fill PCMD[1] with reg data */
	reg = data & 0xff;
	/* set DCE bit */
	reg |= 0x800;
	PCMD(1) = reg;

	/* fill PCMD[2] with 0x20 (VCC1) address) */
	reg = 0x20;
	/* set DCE, MBC and LC bit */
	reg |= 0x1c00;
	PCMD(2) = reg;

	/* fill PCMD[3] with 0x33 */
	reg = 0x33;
	/* set DCE and LC bit */
	reg |= 0xc00;
	PCMD(3) = reg;

	/* set 'V' bit in PWRMODE */
	__asm__ (
		"mrc	p14, 0, %0, c7, c0, 0\n"
		"orr	%0, %0, #0x08\n"
		"mcr	p14, 0, %0, c7, c0, 0\n"
		:
		: "r"(tmp)
	);

	/* Polling VCSA bit */
	while (PVCR & 0x00004000) {}
	return 0;
}

static int pll_gb(int level)
{
	volatile unsigned int reg;
	int timeout = 100;
	int turnoff = 0;

	if (level) {
		/* turn on GB PLL */

		for (;;) {
			/* clear OSCC[GPRM] */
			reg = OSCC;
			reg &= ~(1 << 18);
			OSCC = reg;
			reg = OSCC;

			do {
				/* set AGENP[GBPLL_CTRL] and AGENP[GBPLL_ST] */
				reg = AGENP;
				reg |= (3 << 28);
				AGENP = reg;
				reg = AGENP;
				if (--timeout == 0)
					break;
			} while (!(OSCC & (1 << 17))); /* check OSCC[GPRL] */
			if (timeout)
				break;
		}
	} else {
		/* turn off GB PLL */

		for (;;) {
			/* clear AGENP[GBPLL_CTRL] and AGENP[GBPLL_ST] */
			reg = AGENP;
			if (reg & (1 << 28)) {
				turnoff = 1;
				reg &= ~(3 << 28);
				reg |= (2 << 28);
				AGENP = reg;
				reg = AGENP;
				udelay(204);

				/* check OSCC[GPRL] */
				while (OSCC & (1 << 17)) {
					if (--timeout)
						break;;
				}
			}
			if (timeout)
				break;
		}

		if (turnoff) {
			/* set OSCC[GPRM] */
			reg = OSCC;
			reg |= (1 << 18);
			OSCC = reg;
			reg = OSCC;
		}
	}
	//udelay(504);
	return 0;
}

static int verify_ddr_freq(int xl, int xn)
{
	int ddr_freq;

	/* ddr_freq:	2, increase it to 260.
	 * 		1, decrease it to 208.
	 * 		0, don't change ddr freq.
	 */
	ddr_freq = 0;
	if (((xl == 0x0c) && (xn == 1))
		&& ((ACSR & 0x71F) != 0x10C))
		ddr_freq = 1;
	if (((xl != 0x0c) && (xl > 0))
		&& ((ACSR & 0x71f) == 0x10c))
		ddr_freq = 2;

	return ddr_freq;
}

static int query_smc(int xl, int xn)
{
	int smc = 0, sum;
	sum = 13 * xl * xn;
	switch (sum) {
	case 624:
		smc = 208;
		break;
	case 416:
	case 208:
	case 156:
		smc = 104;
		break;
	case 104:
		smc = 78;
		break;
	}
	return smc;
}

static int set_df(int smc)
{
	unsigned int reg;

	reg = MEMCLKCFG;
	reg &= ~((7 << 16) | 7);
	if (smc == 208) {
		/* set DFI clock divider as 4 */
		reg |= (3 << 16);
		/* set EMPI clock divider as 4 */
		reg |= 3;
	} else if (smc == 104) {
		/* set DFI clock divider as 2 */
		reg |= (2 << 16);
		/* set EMPI clock divider as 2 */
		reg |= 2;
	}
	MEMCLKCFG = reg;
	reg = MEMCLKCFG;
	return 0;
}

static int set_smc(int xl, int xn)
{
	int smc = 0, sum;
	unsigned int accr;

	sum = 13 * xl * xn;
	switch (sum) {
	case 624:
		/* 208M */
		smc = 5;
		break;
	case 416:
	case 208:
	case 156:
		/* 104M */
		smc = 2;
		break;
	case 104:
		/* 78M */
		smc = 0;
		break;
	}

	accr = ACCR;
	accr &= (ACCR_MASK & (~ACCR_SMCFS_MASK));
	accr |= (smc << ACCR_SMCFS_OFFSET);
	ACCR = accr;
	accr = ACCR;

	while ((ACSR & ACCR_SMCFS_MASK) != (accr & ACCR_SMCFS_MASK));	
	return 0;
}

static int set_sfl(int xl, int xn)
{
	int sfl=0, sum;
	unsigned int reg, cpuid;

	sum = 13 * xl * xn;
	switch (sum) {
	case 624:
		cpuid = get_cpuid();
		if ((cpuid & 0xfff0) == 0x6930)
		{
			/* 208M - this is a workaroung for TAVOR PV to prevent the use of 312Mhz SRAM*/
			sfl = 2;
		}
		else
		{
		   /* 312M */
			sfl = 3;
		}
		break;
	case 416:
		/* 208M */
		sfl = 2;
		break;
	case 208:
	case 156:
		/* 156M */
		sfl = 1;
		break;
	case 104:
		/* 104M */
		sfl = 0;
		break;
	}

	reg = ACCR;
	reg &= ~(3 << 18);
	reg |= (sfl << 18);
	ACCR = reg;
	reg = ACCR;
	udelay(1);

	while ((ACCR & (3 << 18)) != (ACSR & (3 << 18)));
	return 0;
}

#define LCCR3          __REG(0x4400000C)  /* LCD Controller Control Register 3 */
#define LCCR4           __REG(0x44000010)  /* LCD Controller Control Register 4 */

#define LCCR3_PCD       Fld (8, 0)      /* Pixel Clock Divisor */
#define LCCR3_PixClkDiv(Div)            /* Pixel Clock Divisor */ \
                        (((Div) << FShft (LCCR3_PCD)))
#define LCCR4_PCDDIV    (1 << 31)

// PCD values for LPJ032L001B
// FIX ME: Update these values according real visual effect!
// Pixel clock of LPJ032L001B at 45HZ should be 7.692 MHz
// In theory, PCD_HSS_104M = 104/(7.692*2) - 1 = 6

#define PCD_HSS_104M    6
#define PCD_HSS_156M    9
#define PCD_HSS_208M    14
#define PCD_HSS_60M     4

static int pxafb_adjust_pcd(int hss)
{
        unsigned int pcd, lccr3;

        switch (hss) {
        case 104:
		pcd = PCD_HSS_104M;
		break;
        case 156:
		pcd = PCD_HSS_156M;
		break;
        case 208:
		pcd = PCD_HSS_208M;
		break;
        case 60:
                /* D0CS mode, hss: 60M */
                pcd = PCD_HSS_60M;
                break;
        default:
                /* otherwise, we just skip adjusting PCD */
                return 0;
        }

        lccr3 = (LCCR3 & ~0xff) | LCCR3_PixClkDiv(pcd);
        if (lccr3 != LCCR3) {
                LCCR3 = lccr3;
        }

        return 0;
}

static int set_hss(int xl, int xn)
{
	int hss = 0, sum;
	unsigned int reg;
	int hss_pcd=0;

	sum = 13 * xl * xn;
	switch (sum) {
	case 624:
		/* 208M */
		hss = 1;
		hss_pcd=156;
		break;
	case 416:
		/* 156M */
		hss_pcd=156;
		hss = 1;
		break;
	case 208:
	case 156:
	case 104:
		/* 104M */
		hss = 0;
		hss_pcd=104;
		break;
	}

	reg = ACCR;
	reg &= ~(3 << 14);
	reg |= (hss << 14);
	ACCR = reg;
	reg = ACCR;

	udelay(1);
	while ((ACCR & (3 << 14)) != (ACSR & (3 << 14)));

	if( hss_pcd != 0)
		pxafb_adjust_pcd(hss_pcd);
	return 0;
}

static int set_dmc(int xl, int xn)
{
	int dmc = 0, sum;
	unsigned int reg;

	sum = 13 * xl * xn;
	switch (sum) {
	case 156:
		dmc = 2;
		break;
	default:
		dmc = 3;
		break;
	}
	reg = ACCR;
	reg &= ~(3 << 12);
	reg |= (dmc << 12);
	ACCR = reg;
	reg = ACCR;
	udelay(1);
	while ((ACCR & (3 << 12)) != (ACSR & (3 << 12))) {};
	return 0;
}

static int fvscaler_fCore(char *param)
{
	int xl, xn, cpuid, ddr_freq, timeout;
	unsigned int reg;
	volatile int accr, tmp;

	/* read CPU ID */
	__asm__ (
		"mrc p15, 0, %0, c0, c0, 0\n"
		: "=r" (cpuid)
	);
	if (((cpuid & 0xFFFF0000) != 0x69050000) &&
		((cpuid & 0xFFFF0000) != 0x56050000)) {
		/* It's not xscale chip. */
		return -1;
	}
	cpuid &= 0x0000FFFF;

	if (!strncmp("104Mr", param, 5)){
		xl = 0x08;
		xn = 0x01;
	} else if (!strncmp("156Mr", param, 5)) {
		xl = 0x0c;
		xn = 0x01;
		if (cpuid != 0x6835 && cpuid != 0x6837 && cpuid != 0x6838 && cpuid != 0x6931) {
			/* It's not Tavor B0, B1 or B2 */
			printf("156Mr can't be supported\n");
			return -1;
		}
	} else if (!strncmp("208Mr", param, 5)){
		xl = 0x10;
		xn = 0x01;
	} else if (!strncmp("416Mt", param, 5)){
		xl = 0x10;
		xn = 0x02;
	} else if (!strncmp("312Mr", param, 5)){
		xl = 0x18;
		xn = 0x01;
	} else if (!strncmp("624Mt", param, 5)){
		xl = 0x18;
		xn = 0x02;
	} else if (!strncmp("403Mr", param, 5)){
		xl = 0x1f;
		xn = 0x01;
	} else if (!strncmp("806Mt", param, 5)){
		xl = 0x1f;
		xn = 0x02;
	} else {
		printf("Wrong frequency is specified\n");
		return -1;
	}

	/* keep DDR frequency is consistent with core frequency */
	ddr_freq = verify_ddr_freq(xl, xn);
	if (ddr_freq == 2)
		pll_gb(1);


	if ((xn == 2) & (ACCR_G_XN(ACCR) == 1)) {
		/* set MTS */
		reg = ASCR;
		reg &= 0xFFFF8FFF;
		reg |= (xn << 12);
		ASCR = reg;
		reg = ASCR;

		do {
			tmp = (ASCR >> 8) & 0x07;
		} while (xn != tmp);
	}


	accr = ACCR;
	accr &= (ACCR_MASK & (~(ACCR_XL_MASK | ACCR_XN_MASK)));
	accr |= (xl << ACCR_XL_OFFSET | xn << ACCR_XN_OFFSET);
	accr |= (3 << ACCR_XSPCLK_OFFSET);
	ACCR = accr;
	accr = ACCR;

	/* wait 2 cycles of 13MHz clock delay */
	udelay(1);

	if ((xn == 1) && (ACCR_G_XN(ACCR) == 2)) {
		reg = ASCR;
		reg &= 0xFFFF8FFF;
		reg |= (xn << 12);
		ASCR = reg;
		do {
			tmp = (ASCR >> 8) & 0x07;
		} while (xn != tmp);
	}

	XllpWriteClkCfg(3);

	while ((ACSR & (ACCR_XL_MASK | ACCR_XN_MASK)) != 
		(ACCR & (ACCR_XL_MASK | ACCR_XN_MASK)));
	
	if (ddr_freq == 2) {
		/* up to 260MHz */
		reg = MDCNFG;
		reg &= ~(3 << 28);
		reg |= (2 << 28);
		MDCNFG = reg;

		for (;;) {
			reg = MDCNFG >> 28;
			reg &= 3;
			if (reg == 2)
				break;
		}

		reg = DDR_HCAL;
		reg |= (1 << 31);
		DDR_HCAL = reg;
		reg = DDR_HCAL;
	} else if (ddr_freq == 1) {
		/* down to 208MHz */
		reg = MDCNFG;
		reg &= ~(3 << 28);
		reg |= (3 << 28);
		MDCNFG = reg;

		for (;;) {
			reg = MDCNFG >> 28;
			reg &= 3;
			if (reg == 3)
				break;
		}

		reg = DDR_HCAL;
		reg |= (1 << 31);
		DDR_HCAL = reg;
		reg = DDR_HCAL;
	}

	/* set DFI clock divider */
	reg = query_smc(xl, xn);
	if (reg == 208)
		set_df(reg);
	/* set SMCFS */
	set_smc(xl, xn);
	/* set SFLFS */
	set_sfl(xl, xn);
	/* set HSS */
	set_hss(xl, xn);
	/* set DMC */
	set_dmc(xl, xn);

	timeout = 10;
	while (MDCNFG & (3 << 28)) {
		udelay(1);
		if (--timeout == 0) {
			//printf("MDCNFG timeout\n");
			break;
		}
	};

	/* set DFI clock divider */
	if (reg == 104)
		set_df(reg);

	if (ddr_freq == 1)
		pll_gb(0);
		
	return 0;
}

static int fvscaler_fSmc(char *param)
{
	int smcfs;
	volatile int accr;

	if (!strncmp("78M",param,3)){
		smcfs = 0x0;
	} else if (!strncmp("104M", param, 4)){
		smcfs = 0x2;
	} else if (!strncmp("208M", param, 4)){
		smcfs = 0x5;
	} else
		return -1;

	accr = ACCR;
	accr &= (ACCR_MASK & (~ACCR_SMCFS_MASK));
	accr |= (smcfs << ACCR_SMCFS_OFFSET);
	ACCR = accr;
	accr = ACCR;

	while ((ACSR & ACCR_SMCFS_MASK) != (ACCR & ACCR_SMCFS_MASK));	

	return 0;
}

static int fvscaler_fSram(char *param)
{
	int sflfs;
	volatile int accr;

	if (!strncmp("104M",param, 4)){
		sflfs = 0x0;
	} else if (!strncmp("156M", param, 4)){
		sflfs = 0x1;
	} else if (!strncmp("208M", param, 4)){
		sflfs = 0x2;
	} else if (!strncmp("312M", param, 4)){
		sflfs = 0x3;
	} else
		return -1;

	accr = ACCR;
	accr &= (ACCR_MASK & (~ACCR_SFLFS_MASK));
	accr |= (sflfs << ACCR_SFLFS_OFFSET);
	ACCR = accr;
	accr = ACCR;

	while ((ACSR & ACCR_SFLFS_MASK) != (ACCR & ACCR_SFLFS_MASK));	

	return 0;
}

static int fvscaler_fHss(char *param)
{
	int hss;
	volatile int accr;

	if (!strncmp("104M",param, 4)){
		hss = 0x0;
	} else if (!strncmp("156M", param, 4)){
		hss = 0x1;
	} else if (!strncmp("208M", param, 4)){
		hss = 0x2;
	} else
		return -1;

	accr = ACCR;
	accr &= (ACCR_MASK & (~ACCR_HSS_MASK));
	accr |= (hss << ACCR_HSS_OFFSET);
	ACCR = accr;
	accr = ACCR;

	while ((ACSR & ACCR_HSS_MASK) != (ACCR & ACCR_HSS_MASK));	

	return 0;
}

static int fvscaler_fDmc(char *param)
{
	int dmcfs;
	volatile int accr;

	if (!strncmp("260M", param, 4)){
		dmcfs = 0x3;
	} else
		return -1;

	accr = ACCR;
	accr &= (ACCR_MASK & (~ACCR_DMCFS_MASK));
	accr |= (dmcfs << ACCR_DMCFS_OFFSET);
	ACCR = accr;
	accr = ACCR;

	while ((ACSR & ACCR_DMCFS_MASK) != (ACCR & ACCR_DMCFS_MASK));	

	return 0;
}

static int fvscaler_fDfClk(char *param)
{
	int df_clkdiv;
	volatile unsigned int memclkcfg;
	int smc_freq;
	int dfclk_freq;
	volatile unsigned int smcfs;

	if (!strncmp("26M",param, 3)){
		dfclk_freq = 26;
	} else if (!strncmp("52M", param, 3)){
		dfclk_freq = 52;
	} else
		return -1;

	smcfs = ((ACCR >> 23) & 0x7);
	if (smcfs == 0x0){
		smc_freq = 78;
	} else if (smcfs == 0x2) {
		smc_freq = 104;
	} else if (smcfs == 0x5) {
		smc_freq = 208;
	} else
		return -1;

	if (dfclk_freq == smc_freq){
		df_clkdiv = 0x1;
	} else if (dfclk_freq*2 == smc_freq){
		df_clkdiv = 0x2;
	} else if (dfclk_freq*4 == smc_freq){
		df_clkdiv = 0x3;
	} else
		return -1;

	memclkcfg = MEMCLKCFG;
	memclkcfg &= ~MEMCLKCFG_DFCLKDIV_MASK;
	memclkcfg |= (df_clkdiv << MEMCLKCFG_DFCLKDIV_OFFSET);
	MEMCLKCFG = memclkcfg;
	
	return 0;
}

static int fvscaler_fEmpiClk(char *param)
{
	int empi_clkdiv;
	volatile unsigned int memclkcfg;
	int smc_freq;
	int empiclk_freq;
	volatile unsigned int smcfs;

	if (!strncmp("26M",param, 3)){
		empiclk_freq = 26;
	} else if (!strncmp("52M", param, 3)){
		empiclk_freq = 52;
	} else
		return -1;

	smcfs = ((ACCR >> 23) & 0x7);
	if (smcfs == 0x0){
		smc_freq = 78;
	} else if (smcfs == 0x2) {
		smc_freq = 104;
	} else if (smcfs == 0x5) {
		smc_freq = 208;
	} else
		return -1;

	if (empiclk_freq == smc_freq){
		empi_clkdiv = 0x1;
	} else if (empiclk_freq*2 == smc_freq){
		empi_clkdiv = 0x2;
	} else if (empiclk_freq*4 == smc_freq){
		empi_clkdiv = 0x3;
	} else
		return -1;

	memclkcfg = MEMCLKCFG;
	memclkcfg &= ~MEMCLKCFG_EMPICLKDIV_MASK;
	memclkcfg |= (empi_clkdiv << MEMCLKCFG_EMPICLKDIV_OFFSET);
	MEMCLKCFG = memclkcfg;

	return 0;
}


/* The voltage rage of core and sram is same */
static int core_mv(int mv)
{
	int ret;
	if ((mv < 725) || (mv > 1500))
		return -1;

	ret = (mv - 725) / 25;
	return ret;
}

static int fvscaler_vCore(char *param)
{
	int str_len, ret;
	u32 mv;
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;

	str_len = strlen(param);

	if ((param[str_len - 2] != 'm') || (param[str_len - 1] != 'v'))
		return -1;

	param[str_len - 2] = '\0';
	if (strtou32(param, &mv)){
		printerror(ENAN, param);
		return -1;
	} else {
		switch (i2c_mode) {
		case 1:
			pwr_i2c_init(0);
			ret = core_mv(mv);
			if (ret < 0)
				return -1;
			/* operate 0x24 register (ADTV2) */
			pwr_i2c_send(0x24, ret);
			break;
		case 0:
		default:
			if (detect_micco())
				status = XllpPmMiccoSetVccCore(mv);
			else
				status = XllpPmAravaSetVccCore(mv);
			break;
		}
		if (status != XLLP_STATUS_SUCCESS)
			return -1;
	}
	return 0;

}

static int fvscaler_vSram(char *param)
{	
	int str_len, ret;
	u32 mv;
	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;

	str_len = strlen(param);

	if ((param[str_len - 2] != 'm') || (param[str_len - 1] != 'v'))
		return -1;

	param[str_len - 2] = '\0';
	if (strtou32(param, &mv)){
		printerror(ENAN, param);
		return -1;
	} else {
		switch (i2c_mode) {
		case 1:
			pwr_i2c_init(0);
			ret = core_mv(mv);
			if (ret < 0)
				return -1;
			/* operate 0x2A register (SDTV2) */
			pwr_i2c_send(0x2A, ret);
			break;
		case 0:
		default:
			if (detect_micco())
				status = XllpPmMiccoSetVccSram(mv);
			else
				status = XllpPmAravaSetVccSram(mv);
			break;
		}
		if (status != XLLP_STATUS_SUCCESS)
			return -1;
	}
	return 0;
}

static int fvscaler_i2cmode(char *param)
{
	int str_len;
	u32 mode;

	str_len = strlen(param);

	if (strtou32(param, &mode)) {
		printerror(ENAN, param);
		return -1;
	} else {
		if (mode > 2)
			mode = 0;
		i2c_mode = mode;

		/* if micco doesn't exist, i2c_mode can only be normal */
		if (!detect_micco()) {
			printf("Can't detect micco. Set PMIC as normal I2C mode.\n");
			i2c_mode = 0;
		}

		switch (i2c_mode) {
		case 2:
			pwr_i2c_init(1);
			break;
		case 1:
		case 0:
		default:
			/* clear FVE, PVE, TVE and FVC bit */
			PVCR &= 0x0fffffff;
			break;
		}
	}
	return 0;
}

int fvscaler_init(char *param)
{
	unsigned int cpuid;

	__asm__ (
		"mrc p15, 0, %0, c0, c0, 0\n"
		: "=r" (cpuid)
	);
	if (((cpuid & 0xFFFF0000) != 0x69050000) && ((cpuid & 0xFFFF0000) != 0x56050000)) {
		/* It's not xscale chip. */
		printerror(ENAN, param);
		return -1;
	}
	cpuid &= 0x0000FFFF;
	/* set the I2C mode as PowerI2C FVE mode */
	fvscaler_i2cmode("2");

	if (cpuid == 0x6835 || cpuid == 0x6837 || cpuid == 0x6838 || cpuid == 0x6931) {
		/* Tavor B0, B1 or B2*/
		/* Because mobm set DDR as 208MHz.
		 * Set DDR as 260MHz by enter/exit single PLL mode.
		 */
		fvscaler_fCore("156Mr");
	}

	fvscaler_fCore("624Mt");
	//fvscaler_fCore("416Mt");
	fvscaler_fHss("156M");
	/* After set HSS, should call pxafb_adjust_pcd to set lcd pcd
 	 * Since we have set it in fvscaler_fCore, and this call does
 	 * not change it, ignore this time.
	 */
	if ((cpuid == 0x6891) || (cpuid == 0x6881)){
		/* Monahans LV A1 or L A1*/
		/* workaround for NAND */
		fvscaler_vCore("1500mv");
	}
	return 0;
}


static int fvscaler_info()
{
	int acsr = ACSR;
	int memclkcfg = MEMCLKCFG;
	XLLP_UINT32_T sram_vol = 0;
	XLLP_UINT32_T core_vol = 0;

	if (detect_micco()) {
		if (XLLP_STATUS_SUCCESS != XllpPmMiccoGetVccSram(&sram_vol))
			sram_vol = -1;
		if (XLLP_STATUS_SUCCESS != XllpPmMiccoGetVccCore(&core_vol))	
			core_vol = -1;
	} else {
		if (XLLP_STATUS_SUCCESS != XllpPmAravaGetVccSram(&sram_vol))
			sram_vol = -1;
		if (XLLP_STATUS_SUCCESS != XllpPmAravaGetVccCore(&core_vol))
			core_vol = -1;
	}
	
	printf( "   fCore:%dMhz(%s)\n"
			"   fSmc:%dMhz\n"
			"   fSram:%dMhz\n"
			"   fHss:%dMhz\n"
			"   fDmc(%s):%dMhz\n"
			"   fDfClk:%dMhz\n"
			"   fEmpiClk:%dMhz\n"
			"   vCore:%dmv\n"
			"   vSram:%dmv\n"
			"   D0CKEN_A:0x%08x\n"
			"   D0CKEN_B:0x%08x\n"
			"   i2c mode:%s\n",
			FREQ_CORE(ACCR_G_XL(acsr), ACCR_G_XN(acsr)),
			(ACCR_G_XN(acsr) != 0x1)? "Turbo Mode" : "Run Mode",
			FREO_STMM(ACCR_G_SMCFS(acsr)),
			FREQ_SRAM(ACCR_G_SFLFS(acsr)),
			FREQ_HSS(ACCR_G_HSS(acsr)),
			((MDCNFG & 0x4) == 0) ? "x32" : "x16",
			FREQ_DDR(ACCR_G_DMCFS(acsr)),
			FREQ_DFCLK(ACCR_G_SMCFS(acsr), MEMCLKCFG_G_DFCLKDIV(memclkcfg)),
			FREQ_EMPICLK(ACCR_G_SMCFS(acsr), MEMCLKCFG_G_EMPICLKDIV(memclkcfg)),
			(int)core_vol,
			(int)sram_vol,
			D0CKEN_A,
			D0CKEN_B,
			(i2c_mode == 2) ? "Power I2C hardware" :	\
			((i2c_mode == 1) ? "Power I2C software" : "Standard I2C")
			);
	return 0;
}

static int fvscaler(int argc, char *argv[])
{
	struct fv_scaler_handler *scaler = fv_scaler_tbl;
	
	if (argc < 2) {
		fvscaler_info();
	} else {
		while (scaler->fv_scaler) {
			if (!strncmp(scaler->name, argv[1], strlen(scaler->name)))
				break;
			scaler++;
		}

		if (scaler->fv_scaler) {
			if (argc > 2) {
				if (scaler->fv_scaler(argv[2])) {
					return -ENOPARAMS;
				} else
					return 0;
			} else {
				if (scaler->fv_scaler(NULL)) {
					return -1;
				} else
					return 0;
			}
		} else
			return -ENOPARAMS;
	}
	return 0;
}

static char fvscalerhelp[] = "fvscaler\n"
"usage: fvscaler [fCore|fSmc|fSram|fHss|fDmc|fDfClk|fEmpiClk|vCore|vSram|mode] [freq|vol]\n"
"    OR fvscaler clkASet 0~31\n"
"    OR fvscaler clkBSet 0~31\n"
"    OR fvscaler clkAClear 0~31\n"
"    OR fvscaler clkBClear 0~31\n"
"    OR fvscaler mode\n"
"    OR fvscaler init\n"
"   fCore:104Mr,208Mr,416Mt,312Mr,624Mt,403Mr,806Mt\n"
"   fSmc:78M,104M,208M\n"
"   fSram:104M,156M,208M,312M\n"
"   fHss:104M,156M,208M\n"
"   fDmc:260M\n"
"   fDfClk:26M,52M\n"
"   fEmpiClk:26M,52M\n"
"   vCore:850mv,875mv,900mv,925mv,...,1600mv,1625mv\n"
"   vSram:1100mv,1150mv,1200mv,...,2600mv,2650mv\n"
"   mode:0,1,2\n"
"   init\n"
"Frequency/Voltage Scaler\n";

__commandlist(fvscaler, "fvscaler", fvscalerhelp);


/* if micco is detected, return 1.
 * Otherwise, return 0. At this time, we believe other PMIC is installed.
 */
int detect_micco(void)
{
	int micco_flag, tmp;

	/*
	 * Polling 0x34 address (micco address) in standard I2C bus.
	 * Required: I2C bus and SDA,SCL pins are initialized.
	 */
	/* enable ITEIE, BEIE */
	ICR = 0x500;
	/* enable I2C unit and SCL */
	ICR |= 0x60;

	/* write operation */
	IDBR = (0x34 << 1);
	/* set ICR[start], ICR[TB], clear ICR[STOP] */
	ICR |= 0x9;

	/* wait ITE */
	while (!(ISR & 0x40)) {}
	if (ISR & 0x400) {
		/* not detect micco
		 * I2C interface will auto-send stop
		 */
		micco_flag = 0;
	} else {
		/* detect micco */
		micco_flag = 1;
	}
	/* clear ISR */
	tmp = ISR;
	ISR = tmp;

	if (micco_flag) {
		/* set ICR[stop], ICR[TB], clear ICR[START] */
		tmp = ICR;
		tmp &= ~0x01;
		tmp |= 0x0a;
		ICR = tmp;

		while (!(ISR & 0x40)) {}

		/* clear ISR */
		tmp = ISR;
		ISR = tmp;

		/* clear ICR[stop] */
		ICR &= ~0x02;
	}

	msleep(1);
	return micco_flag;
}

#if 0
/* This function is used to test memory speed */
static int ddr(int argc, char *argv[])
{
	int mem_a[256], mem_b[256];
	int i, j, tmp;
	unsigned int start, end, time;
	unsigned int sum = 0;

	/* time = (OSCR delta << 2) / 13, time is us */
	for (i = 0; i < 500000; i++) {
		/* flush icache */
		__asm__ (
			"mcr	p15, 0, %0, c7, c5, 0\n"
			:
			: "r" (tmp)
		);
		start = OSCR;
		memset(mem_a, i, 256);
		for (j = 0; j < 256; j++) {
			mem_b[j] = mem_a[j];
		}
		end = OSCR;
		time = (end > start) ? (end - start) :
			(0xffffffff - start + end);
		sum += time;
	}
	sum = (sum << 2) / 13;
	printf("summary time: %dms\n", sum / 1000);
	return 0;
}
static char ddrhelp[] = "ddr\n"
"usage: ddr\n";
__commandlist(ddr, "ddr", ddrhelp);

static int singlepll(int argc, char *argv[])
{
	int i = 10, j;

	if (argc > 1) {
		if (strtou32(argv[1], &i)){
			i = 10;
		}
	}

	printf("entry/exit single PLL mode for %d times\n", i);
	for (j = 0; j < i; j++) {
		fvscaler_fCore("156Mr");
		fvscaler_fCore("624Mt");
		printf("cycle %d\n", j);
	}
	return 0;
}

static char singlehelp[] = "singlepll\n"
"usage: signlepll [num]\n";
__commandlist(singlepll, "singlepll", singlehelp);

static int stress(int argc, char *argv[])
{
	char op1[10], op2[10];
	int i = 0;

	if (argc < 2) {
		printf("set op1, op2 as default op. op1:208, op2:624\n");
		strcpy(op1, "208Mr");
		strcpy(op2, "624Mt");
	} else {
	}

	for (;;) {
		fvscaler_fCore(op1);
		printf("%d times, op:%s, ACSR:0x%x\n", ++i, op1, ACSR);
		fvscaler_fCore(op2);
		printf("%d times, op:%s, ACSR:0x%x\n", ++i, op2, ACSR);
	}
	return 0;
}

static char stresshelp[] = "stress\n"
"usage: stress [op1] [op2]\n";
__commandlist(stress, "stress", stresshelp);
#endif

