static inline int is_buf_blank(u8 * buf, int size);

/* Move to Monahans native NAND driver -- yfw */
static XLLP_DFC_MODE_T DFC_mode = {
	.enableDMA = 0,
	.enableECC = 1,
	.enableSpareArea = 1,
	.chipSelect = 0,
};

/* Move to Monahans native NAND driver -- yfw */
static XLLP_DFC_CONTEXT_T DFC_context = {
	.pDFCReg = (P_XLLP_DFC_REG_T)(0x43100000),
	.pDFCMode = &DFC_mode,
	.pDma = NULL,
	.pMFPBase = (P_XLLP_VUINT32_T)NULL,
	.pMFPRmDb = (P_XLLP_MFP_RM_DB_ID_T)NULL,
	.pOSTReg = (P_XLLP_OST_T)(0x40A00000),
	.pDmacRegsBase = NULL,
	.pFlashInfo = NULL,
};

static unsigned int flash_config = XLLP_DFC_FLASH_NULL;

/* Move to native NAND driver. We need always enable HW ECC on Onenand -- yfw */
static void enable_hw_ecc(int enable, struct nand_info *info)
{
	if (!enable) {
		NDCR &= ~NDCR_ECC_EN;
		info->pcontext->pDFCMode->enableECC = 0;
	} else {
		NDCR |= NDCR_ECC_EN;
		info->pcontext->pDFCMode->enableECC = 1;
	}
} 

/* read one page */
/* Need make this function can work with native NAND and Onenand -- yfw */
static int mhn_read_one_page(struct nand_info * info, u8 *buf, u32 page, int option)
{
	int block, reloc_block;
	int pagesize, pages_shift;
	int len;
	XLLP_UINT32_T eccpage = 0xFFFFFFFF;	/* ecc error page number */
	XLLP_STATUS_T status;
	int hwecc = !(option == READ_OOB);

	/* page means the page number
	 * If current block is a bad block, relocate the page
	 * to a correct one in a relocated block
	 */
	pages_shift = info->phys_erase_shift - info->page_shift;
	block = page >> pages_shift;
	if (option != READ_RAW && is_bad_block(block, info))
		return -EBADBLOCK;

	reloc_block = search_rel_block(block, info);
	if (option != READ_RAW && reloc_block != block) {
		page = (reloc_block << pages_shift) + (page
			& (1 << pages_shift) - 1);
	}
	pagesize = 1 << info->page_shift;
	memset(buf, 0xFF, pagesize + info->oobsize);
	
	enable_hw_ecc(hwecc, info);
	/* In XLLP code, len is checked restrictly.
	 * So it must be set accurately.
	 */
	if (hwecc)
		len = pagesize + info->oobsize - info->eccbytes;
	else
		len = pagesize + info->oobsize;
	status = XllpFlashDfcRead(info->pcontext, buf, len, page, 1, NAND_CMD_TIMEOUT,
					&eccpage);
	enable_hw_ecc(1, info);
	
	//PRINT_BUF(buf, pagesize + info->oobsize);
	//
	//Workaround for read the erased page, when read the whole 0xFF page, func return error
	//however, this is bearable.
	if ((status != XLLP_STATUS_SUCCESS) && ((eccpage != page) || 
		!is_buf_blank(buf, pagesize))) {
		DPRINTF("monahans_read::%d, 0x%x, %x\n", status, page,
			eccpage);
		return -ECMDFAIL;
	}
	return 0;
}

static int mhn_erase_direct(struct nand_info *info, unsigned int start_block,
		int block_num, int *badblock)
{
	return XllpFlashDfcErase(info->pcontext, start_block, 1, NAND_CMD_TIMEOUT,
			&badblock);
}

/* erase one block, buf length must be larger than info->pagesize
 * + info->oobsize
 */
/* Need make this function can work with native NAND and Onenand -- yfw */
int mhn_erase_one_block(struct nand_info * info, u32 block)
{
	int reloc_block, ret;
	XLLP_STATUS_T status;
	XLLP_UINT32_T badblock = 0xFFFFFFFF;
	u32 start;
	
	if (is_bad_block(block, info))
		return -EBADBLOCK;

	reloc_block = search_rel_block(block, info);
	if (reloc_block)
		start = reloc_block;
	else
		start = block;

	status = XllpFlashDfcErase(info->pcontext, start, 1, NAND_CMD_TIMEOUT, &badblock);

	if (status != XLLP_STATUS_SUCCESS)
		printf("monahans_erase::%d, 0x%x, %x\n", status, 
			block << info->phys_erase_shift, badblock);
	if (badblock == start) {
		/* If there's an relocated block, return 0.
		 * Otherwise, return error.
		 */
		return mark_badblock(block, info);
	}

	if (status != XLLP_STATUS_SUCCESS)
		return  -ECMDFAIL;

	return 0;
}

/* Need make this function can work with native NAND and Onenand -- yfw */
int mhn_write_one_page(struct nand_info *info, u8 * buf, u32 page, int hwecc)
{
	int reloc_block, block;
	int pages_shift, ret;
	XLLP_STATUS_T status;
	XLLP_UINT32_T badblock = 0xFFFFFFFF;
	
	pages_shift = info->phys_erase_shift - info->page_shift;
	/* calculate the page address with relocated block, if exists */
	block = page >> pages_shift;

	if (is_bad_block(block, info))
		return -EBADBLOCK;

	reloc_block = search_rel_block(block, info);
	if (reloc_block != block) {
		//DPRINTF("relocate page %x ", page);
		page = (reloc_block << pages_shift) + (page
			& (1 << pages_shift) - 1);
		//DPRINTF(" to %x\n", page);
	}

	enable_hw_ecc(hwecc, info);
	status = XllpFlashDfcWrite(info->pcontext, buf, page, 1, NAND_CMD_TIMEOUT, &badblock);
	enable_hw_ecc(1, info);

	if (status != XLLP_STATUS_SUCCESS)
		printf("monahans_write::%d, 0x%x, %x\n", status, 
			page << info->page_shift, badblock);
	if (badblock == reloc_block) {
		/* If there's an relocated block, return 0.
		 * Otherwise, return error.
		 */
		 return mark_badblock(block, info);
	}

	if (status != XLLP_STATUS_SUCCESS)
		return -ECMDFAIL;

	return 0;
}

/* check whether a page and spare area is empty
 * size should be sum of page size and spare area length
 */
static inline int is_buf_blank(u8 * buf, int size)
{
	int i = 0;
	while (i < size) {
		if (*((unsigned long *)(buf + i)) != 0xFFFFFFFF)
			return 0;
		i += 4;
	}
	if (i > size) {
		i -= 4;
		while (i < size) {
			if (*(buf + i) != 0xFF)
				return 0;
			i++;
		}
	}
	return 1;
}

static int unlock_nand(struct nand_info *info)
{
	return XllpFlashDfcUnlock(info->pcontext, 0, 
		info->chipsize + (nand_reloc_max << info->phys_erase_shift) - 1);
}

static void nand_driver_init(void)
{
	XLLP_STATUS_T status;
	XLLP_UINT8_T manufacturer, device;
	P_XLLP_DFC_FLASH_SPECS_T pFlashInfo = NULL;
        P_XLLP_CLKMGR_T pCLKMGR_REGS = (P_XLLP_CLKMGR_T)(0x41340000);
	int i = XLLP_DFC_FLASH_NULL + 1, id;
	//unsigned char buf[BUFLEN];

	monahans_nand.pcontext = &DFC_context;
	memset(buf, 0, BUFLEN);

	/* enable NAND clock, default is on */
	CKENA |= CKENA_4_DFC;
	while(i < XLLP_DFC_FLASH_END) {
		status = XllpFlashDfcInitNOGPIO(monahans_nand.pcontext, i);
		if (status != XLLP_STATUS_SUCCESS) {
			printf("XllpFlashDfcInitNoGPIO() failed.\n");
			return;
		}
		
		XllpFlashDfcReadID(monahans_nand.pcontext, &manufacturer,
					&device);
		if (manufacturer == (monahans_nand.pcontext->pFlashInfo->chipID & 0xFF)
		       	&& device == ((monahans_nand.pcontext->pFlashInfo->chipID >> 8) & 0xFF)){
			flash_config = i;
			break;
		}
		i++;
	}
	if (i == XLLP_DFC_FLASH_END) {
		printf("Flash can not be recognized. Man:0x%x, Dev:0x%x\n",
			manufacturer, device);
		return;
	}
	
	printf("\nNAND flash(Manu=0x%x Device=0x%x) detected!\n",
		manufacturer, device);

	pFlashInfo = monahans_nand.pcontext->pFlashInfo;
	monahans_nand.page_shift = fls(pFlashInfo->pageSize);
	monahans_nand.phys_erase_shift = fls(pFlashInfo->pagePerBlock
					* pFlashInfo->pageSize);
	nand_reloc_max = (pFlashInfo->numBlocks / 100) * 2;
	monahans_nand.chipsize = (pFlashInfo->numBlocks - nand_reloc_max)
					<< monahans_nand.phys_erase_shift;
	/* Only two format of pages are supported: 512 bytes and 2048 bytes */
	if (pFlashInfo->pageSize > 512) {
		monahans_nand.oobsize = 64;
		monahans_nand.eccbytes = 24;
		monahans_nand.hwecc = 24;
		monahans_nand.badblockpos = 0;
		monahans_nand.badblockbytes = 2;
		monahans_bbt_main.veroffs = 6;
		monahans_bbt_main.offs = 2;
		monahans_bbt_mirror.veroffs = 6;
		monahans_bbt_mirror.offs = 2;
	} else {
		monahans_nand.oobsize = 16;
		monahans_nand.eccbytes = 6;
		monahans_nand.hwecc = 6;
		monahans_nand.badblockpos = 5;
		monahans_nand.badblockbytes = 1;
		monahans_bbt_main.veroffs = 12;
		monahans_bbt_main.offs = 8;
		monahans_bbt_mirror.veroffs = 12;
		monahans_bbt_mirror.offs = 8;
	}
	monahans_nand.td = &monahans_bbt_main;
	/* mirror */
	monahans_nand.md = &monahans_bbt_mirror;
	
	monahans_nand.maf_id = manufacturer;
	monahans_nand.dev_id = device;
	scan_nand(&monahans_nand);
	
	return;
}

