/*
 * smdk2500.c: smdk2500 specific stuff
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2003  Russ Dill (Russ.Dill@asu.edu)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: smdk2500.c,v 1.2 2003/09/04 18:34:25 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/flash.h>
#include <blob/init.h>
#include <blob/reboot.h>
#include <blob/serial.h>
#include <blob/led.h>
#include <blob/time.h>
#include <blob/partition.h>
#include <blob/arch.h>
#include <net/ether.h>


/* flash descriptor for smdk2500 flash */
/* 1x Am29LV160DB (2MB) */
static const flash_descriptor_t smdk2500_flash_descriptors[] =
{
	{
		size: 16 * 1024,
		num: 1,
		lockable: 1
	},
	{
		size: 8 * 1024,
		num: 2,
		lockable: 1
	},
	{
		size: 32 * 1024,
		num: 1,
		lockable: 1
	},
	{
		size: 64 * 1024,
		num: 31,
		lockable: 1
	},
	{
		/* NULL block */
	},
};


/* default partition table for the SMDK2500 */
static const blob_partition_t smdk2500_default_partition_table[] = {
	{
		/* start of table */
		magic:	BLOB_DEFAULT_PART_TABLE_MAGIC,
		next:	sizeof(blob_partition_t),
		offset:	0x00000000,	/* absolute base address */
		size:	2 * 1024 * 1024 /* total size of the flash */
	},
	{
		/* blob itself */
		magic:		BLOB_PART_VALID_MAGIC,
		next:		sizeof(blob_partition_t),
		offset:		BLOB_FLASH_BASE,
		size:		BLOB_FLASH_LEN,
		name:		"blob",
		mem_base:	BLOB_RAM_BASE
	},
	{
		/* parameter block */
		magic:	BLOB_PART_VALID_MAGIC,
		next:	sizeof(blob_partition_t),
		offset:	PARAM_FLASH_BASE,
		size:	PARAM_FLASH_LEN,
		name:	"param",
		flags:	BLOB_PART_FLAG_PTABLE
	},
	{
		/* kernel */
		magic:		BLOB_PART_VALID_MAGIC,
		next:		sizeof(blob_partition_t),
		offset: 	KERNEL_FLASH_BASE,
		size:		KERNEL_FLASH_LEN,
		name:		"kernel",
		flags:		BLOB_PART_FLAG_LOAD | 
				BLOB_PART_FLAG_CRAMFS,
		mem_base:	KERNEL_RAM_BASE,
		entry_point:	KERNEL_RAM_BASE
	},
#if 0
	{
		/* ramdisk */
		magic:		BLOB_PART_VALID_MAGIC,
		next:		sizeof(blob_partition_t),
		offset:		RAMDISK_FLASH_BASE,
		size:		RAMDISK_FLASH_LEN,
		name:		"ramdisk",
		flags:		BLOB_PART_FLAG_LOAD,
		mem_base:	RAMDISK_RAM_BASE
	},
#endif
	{
		/* last entry */
		magic:	BLOB_PART_LAST_MAGIC
	}
};


static void smdk2500_init_hardware(void)
{
	/* setup flash stuff */
	flash_descriptors = smdk2500_flash_descriptors;
	flash_driver = &amd16_flash_driver;

	/* the default partition table */
	default_partition_table = smdk2500_default_partition_table;
	
	/* we don't know where the flash partition table will live, so
	 * put the pointer at 0x0000000 and let the partition table
	 * parser figure out.
	 */
	flash_partition_table = (blob_partition_t*)0x00000000;

	/* select drivers */
	reboot_driver = &s3c2500_reboot_driver;
	serial_driver = &s3c2500_serial_driver;
	led_driver = &s3c2500_gpio_led_driver;
	timer_driver = &s3c2500_timer_driver;
	ether_driver = &s3c2500_ether_driver;
	ether_driver->name = "eth0";
}

__initlist(smdk2500_init_hardware, INIT_LEVEL_DRIVER_SELECTION);
