/*
 * start-pxa: PXA (XScale) specific start code
 *
 * Copyright (C) 2002 Intel Corporation
 * Written by Rusty Geldmacher (russell.r.geldmacher@intel.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>

.text

/*
 * We start by implementing *all* exception vectors
 *
 * Reset vector: this initialises the machine
 * note here that this not yet taken sleep wakeup into account -- lets just
 * get something to work first.
 */

.macro blink, count
	mov		r3, \count
	b		endless_blink
.endm

.globl reset
reset:
	/* First, mask **ALL** interrupts */
	ldr		r0, =ICMR
	mov		r1, #0x00
	str		r1, [r0]

real_reset:
	bl		gpiosetup
	bl		ledsetup
	bl		memsetup
	bl		normal_boot

	blink	#1

.globl undefined_instruction
undefined_instruction:
	blink	#2

.globl software_interrupt
software_interrupt:
	blink	#3

.globl prefetch_abort
prefetch_abort:
	blink	#4

.globl data_abort
data_abort:
	blink	#5

.globl not_used
not_used:
	blink	#6

.globl irq
irq:
	blink	#7

.globl fiq
fiq:
	blink	#8

#define GPIO25_LED GPIO_bit(25)

ledset:
#ifdef CSIR_IMS
	ldr		r1, =GPSR0
	mov		r0, #GPIO25_LED
	str		r0, [r1]
#endif
	mov		pc, lr

ledclear:
#ifdef CSIR_IMS
	ldr		r1, =GPCR0
	mov		r0, #GPIO25_LED
	str		r0, [r1]
#endif
	mov		pc, lr

ledsetup:
	mov		r2, lr
#ifdef CSIR_IMS
	ldr		r1, =GPDR0
	ldr		r0, [r1]
	orr		r0, r0, #GPIO25_LED
	str		r0, [r1]
#endif
	bl		ledclear
	mov		pc, r2

.macro wait, count
	mov		r0, \count
0:
	subs	r0, r0, #1
	bne		0b
.endm

ledflash:
	mov		r2, lr
	bl		ledset
	wait	#0x400000
	bl		ledclear
	wait	#0x400000
	mov		pc, r2

endless_blink:
	mov		r4, r3
0:
	bl		ledflash
	subs	r4, r4, #1
	bne		0b
	wait	#0x2000000
	b		endless_blink

