/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#ifndef __ACODEC_H__
#define __ACODEC_H__
#include "xllp_defs.h"
#include "xllp_mfp.h"
#include "xllp_ost.h"


typedef unsigned int XLLP_ACODEC_DEVICE_ID_T;

#define AK_2440_ID 0x9e // 10011110 //this should be the I2C address
#define WM_8753_ID 0x34 //this should be the I2C address
#define WM_8734_ID 2 //this should be the I2C address

#define UCB_1400_ID 0x4304 //this should be the 7E register value
#define WM_9703_ID  0x4C03 //this should be the 7E register value
#define WM_9704_ID  0x4C04 //this should be the 7E register value
#define WM_9705_ID  0x4C05 //this should be the 7E register value
#define WM_9712_ID  0x4C12 //this should be the 7E register value
#define WM_9713_ID  0x4C13 //this should be the 7E register value


typedef enum _XLLP_ACODEC_ERROR_T {     
    XLLP_ACODEC_SUCCESS = 0,    //successful completion of a function
    XLLP_ACODEC_GENERAL_SW_ERR,    //null pointer to registers or other software error
    XLLP_ACODEC_CONTROLLER_INTERFACE_TIMEOUT, //time-out for waiting for respponse
    XLLP_ACODEC_SAMPLERATE_NOT_SUPPORTED, //the sample rate is not supported either in controller or codec
    XLLP_ACODEC_FEATURE_NO_SUPPORTED, //this codec feature is not supported 
    XLLP_ACODEC_GENERAL_HW_ERR ,//other hardware error besides time out
    XLLP_ACODEC_ROUTE_NO_SUPPORTED, //the codec can not set the route required
    XLLP_ACODEC_SLEEP //the codec is sleep
}XLLP_ACODEC_ERROR_T;

typedef enum _codec_state {
	XLLP_CODEC_SLEEP  = 0,
	XLLP_CODEC_WAKEUP = 1
} ACODEC_STATE_T;

//input route control

#define XLLP_INDEX_HIFI_NEAR_IN_CONTROL                      0

#define XLLP_INDEX_VOICE_NEAR_IN_CONTROL                    1

#define XLLP_INDEX_BT_HIFI_NEAR_IN_CONTROL                 2

#define XLLP_INDEX_BT_VOICE_NEAR_IN_CONTROL               3

#define XLLP_INDEX_BT_MIC_IN_CONTROL                             4

#define XLLP_INDEX_MIC1_IN_CONTROL                                 5

#define XLLP_INDEX_MIC2_IN_CONTROL                                 6

#define XLLP_INDEX_FAR_IN_CONTROL                                   7

#define XLLP_INDEX_MIDI_IN_CONTROL                                  8

#define XLLP_INDEX_FM_IN_CONTROL                                     9

//ouput route control

#define XLLP_INDEX_STEREO_HEAD_SET_OUT_CONTROL           0 

#define XLLP_INDEX_SPEAKER_OUT_CONTROL                            1

#define XLLP_INDEX_HAND_SET_OUT_CONTROL                         2

#define XLLP_INDEX_BT_STEREO_HEAD_SET_OUT_CONTROL     3

#define XLLP_INDEX_VOICE_NEAR_OUT_CONTROL                      4

#define XLLP_INDEX_HIFI_NEAR_OUT_CONTROL                        5

#define XLLP_INDEX_FAR_OUT_CONTROL                                    6

//ouput route control

#define XLLP_STEREO_HEAD_SET_OUT_ROUTE              ((XLLP_UINT16_T) 1<<XLLP_INDEX_STEREO_HEAD_SET_OUT_CONTROL) 

#define XLLP_SPEAKER_OUT_ROUTE                              ((XLLP_UINT16_T)1<<XLLP_INDEX_SPEAKER_OUT_CONTROL)

#define XLLP_HAND_SET_OUT_ROUTE                           ((XLLP_UINT16_T)1<<XLLP_INDEX_HAND_SET_OUT_CONTROL)

#define XLLP_BT_STEREO_HEAD_SET_OUT_ROUTE       ((XLLP_UINT16_T)1<<XLLP_INDEX_BT_STEREO_HEAD_SET_OUT_CONTROL)

#define XLLP_VOICE_NEAR_OUT_ROUTE                       ((XLLP_UINT16_T)1<<XLLP_INDEX_VOICE_NEAR_OUT_CONTROL)

#define XLLP_HIFI_NEAR_OUT_ROUTE                         ((XLLP_UINT16_T)1<<XLLP_INDEX_HIFI_NEAR_OUT_CONTROL)

#define XLLP_FAR_OUT_ROUTE                                    ((XLLP_UINT16_T)1<<XLLP_INDEX_FAR_OUT_CONTROL)

#define XLLP_SCENARIO_BYTES_SIZE    4

typedef enum {
       XLLP_FM = 0,
       XLLP_MIC1 = 1,
       XLLP_MIC2 = 2,
       XLLP_SPEAKER =3,
       XLLP_HEADSET =4,
       XLLP_HANDSET =5,
} VOL_PORT_TYPE_T;

typedef struct _CONTEXT_T {
	XLLP_ACODEC_DEVICE_ID_T ACodecId;		//  - an ID that uniquely identifies the codec to be used
	XLLP_UINT32_T initNumber; //used by driver to track whether it is inited or not
	
    XLLP_UINT32_T pMfpRegBase;	
    P_XLLP_MFP_RM_DB_ID_T pMfpRmDb;    
    void *pVoiceReg;       //pointer to Monahans registers that has PCM interface to codec
	void *pHifiReg;        //pointer to Monahans registers that has hifi interface to codec
    void *pCtrlReg;        //pointer to Monahans registers that has control interface to codec
	XLLP_OST_T *pOSTRegs;					// needed for time out
	void *pSaveMemory;        //pointer to a memory region to save context while suspend
    	void *pXllpScenario;        //pointer to the scenario data structure that used by XLLP. 
    	                                        //If XLLP uses this data, the data structure memory must be allocated in driver.
    	                                        //the size is XLLP_SCENARIO_BYTES_SIZE
	XLLP_INT32_T uMaxReadWriteTimeOutMs;	// - input the max time to wait in milliseconds before giving up on a read or write operation
	XLLP_INT32_T uMaxSetupTimeOutMs;		//  - input the maximum time in milliseconds to wait during initial setup of the ACODEC controller and codec

	//member functions //these pointers must be set by
	XLLP_ACODEC_ERROR_T (* g_pfnSetMasterVol)           (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T GainInDb);
	XLLP_ACODEC_ERROR_T (* g_pfnSetMasterInputGain)     (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T GainInDb);

//add for voice 
	XLLP_ACODEC_ERROR_T (* g_pfnGetVoiceOutSampleRate)        (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RateInHz);
	XLLP_ACODEC_ERROR_T (* g_pfnSetVoiceOutSampleRate)        (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RateInHz);
	XLLP_ACODEC_ERROR_T (* g_pfnGetVoiceInSampleRate)        (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RateInHz);
	XLLP_ACODEC_ERROR_T (* g_pfnSetVoiceInSampleRate)        (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RateInHz);

//it is used to set hifi sample rate
	XLLP_ACODEC_ERROR_T (* g_pfnGetInSampleRate)        (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RateInHz);
	XLLP_ACODEC_ERROR_T (* g_pfnGetOutSampleRate)       (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RateInHz);
	XLLP_ACODEC_ERROR_T (* g_pfnSetInSampleRate)        (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T RateInHz);
	XLLP_ACODEC_ERROR_T (* g_pfnSetOutSampleRate)       (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T RateInHz);

	XLLP_ACODEC_ERROR_T (* g_pfnCodecSpecificInit)      (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnCodecSpecificDeInit)    (struct _CONTEXT_T *pDeviceContext);
      XLLP_ACODEC_ERROR_T (* g_pfnACodecRead)             (struct _CONTEXT_T *pDevContext, XLLP_UINT16_T regAddr, XLLP_UINT16_T *pRegvalue);
      XLLP_ACODEC_ERROR_T (* g_pfnACodecWrite)            (struct _CONTEXT_T *pDevContext, XLLP_UINT16_T regAddr, XLLP_UINT16_T regvalue);
	XLLP_ACODEC_ERROR_T (* g_pfnHiFiStreamPathEnable)   (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnHiFiStreamPathDisable)  (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnRecordingPathEnable)    (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnRecordingPathDisable)   (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnVoicePathEnable)        (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnVoicePathDisable)       (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnSideToneEnable)         (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnSideToneDisable)        (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnHeadsetEnable)          (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnHeadsetDisable)         (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnEarSpeakerEnable)       (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnEarSpeakerDisable)      (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnLouderSpeakerEnable)    (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnLouderSpeakerDisable)   (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnMicEnable)              (struct _CONTEXT_T *pDeviceContext, XLLP_UINT8_T micType);
	XLLP_ACODEC_ERROR_T (* g_pfnMicDisable)             (struct _CONTEXT_T *pDeviceContext, XLLP_UINT8_T micType);
	XLLP_ACODEC_ERROR_T (* g_pfnSetVolumeTable)         (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnGetVolumeTable)         (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnSetToneTable)           (struct _CONTEXT_T *pDeviceContext);
	XLLP_ACODEC_ERROR_T (* g_pfnGetToneTable)           (struct _CONTEXT_T *pDeviceContext);
	
//add for route
	XLLP_ACODEC_ERROR_T (* g_pfnSetRoute)           (struct _CONTEXT_T *pDeviceContext, XLLP_UINT16_T * RoutMap ,XLLP_UINT16_T* CurrentMap);
//add for sleep the codec
	XLLP_ACODEC_ERROR_T (* g_pfnSleepCodec)           (struct _CONTEXT_T *pDeviceContext);
//add for Wake up the codec
	XLLP_ACODEC_ERROR_T (* g_pfnWakeCodec)           (struct _CONTEXT_T *pDeviceContext);
//add for get codec state
	XLLP_ACODEC_ERROR_T (* g_pfnGetState)           (struct _CONTEXT_T *pDeviceContext, ACODEC_STATE_T *pState);
//add for volume
	XLLP_ACODEC_ERROR_T (* g_pfnGetVol)(struct _CONTEXT_T *pDeviceContext, VOL_PORT_TYPE_T port,  XLLP_UINT16_T *GainInDb);
	XLLP_ACODEC_ERROR_T (* g_pfnSetVol)(struct _CONTEXT_T *pDeviceContext, VOL_PORT_TYPE_T port,  XLLP_UINT16_T GainInDb);
} XLLP_ACODEC_CONTEXT_T, *P_XLLP_ACODEC_CONTEXT_T;

#ifdef __cplusplus
extern "C" {
#endif

extern XLLP_ACODEC_ERROR_T XllpAcodecInit(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_BOOL_T hwInit);
extern XLLP_ACODEC_ERROR_T XllpAcodecDeInit(XLLP_ACODEC_CONTEXT_T *pDeviceContext);
extern XLLP_ACODEC_ERROR_T XllpAcodecPCMBusEnable(XLLP_ACODEC_CONTEXT_T *pDeviceContext);
extern XLLP_ACODEC_ERROR_T XllpAcodecPCMDisable(XLLP_ACODEC_CONTEXT_T *pDeviceContext);
extern XLLP_ACODEC_ERROR_T XllpAcodecEnableAmplifier(XLLP_ACODEC_CONTEXT_T *pDeviceContext);
extern XLLP_ACODEC_ERROR_T XllpAcodecSuspend(XLLP_ACODEC_CONTEXT_T *pDeviceContext);
extern XLLP_ACODEC_ERROR_T XllpAcodecResume(XLLP_ACODEC_CONTEXT_T *pDeviceContext);
#ifdef __cplusplus
}
#endif


#endif //__ACODEC_H__
