/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include    "xllp_serialization.h"
#include    "xllp_mfp.h"
#include    "xllp_ac97acodecplat.h"

static XLLP_UINT32_T xllp_ac97_mfp_list []={
								XLLP_MFP_AC97_SYSCLK_OFFSET,
								XLLP_MFP_AC97_AC97_BITCLK_OFFSET,
								XLLP_MFP_AC97_SYNC_OFFSET,
								XLLP_MFP_AC97_SDATA_OUT_OFFSET,
#ifdef	CONFIG_CPU_MONAHANS_PL
								XLLP_MFP_AC97_SDATA_IN_2_OFFSET,
#else
								XLLP_MFP_AC97_SDATA_IN_0_OFFSET,
#endif
								XLLP_MFP_AC97_nACRESET_OFFSET,
								XLLP_MFP_PIN_EOLIST_MARKER
								};

static XLLP_MFP_ALT_FN_T xllp_ac97_af_list []={
								XLLP_MFP_AC97_SYSCLK_AF,
								XLLP_MFP_AC97_AC97_BITCLK_AF,
								XLLP_MFP_AC97_SYNC_AF,
								XLLP_MFP_AC97_SDATA_OUT_AF,
#ifdef	CONFIG_CPU_MONAHANS_PL
								XLLP_MFP_AC97_SDATA_IN_2_AF,
#else
								XLLP_MFP_AC97_SDATA_IN_0_AF,
#endif
								XLLP_MFP_AC97_nACRESET_AF
								};

static XLLP_MFP_DRIVE_STRENGTH_T xllp_ac97_ds_list []={
								XLLP_MFP_DEFAULT_DS,
								XLLP_MFP_DEFAULT_DS,
								XLLP_MFP_DEFAULT_DS,
								XLLP_MFP_DEFAULT_DS,
								XLLP_MFP_DEFAULT_DS,
								XLLP_MFP_DEFAULT_DS
								};


/////////////////////////////////////////////////////////////////////////
//
//  The Low Power Mode output table
//
                          
static XLLP_MFP_LPM_OUTPUT_T xllp_ac97_lpm_output_list []={
								XLLP_MFP_LPMO_DRIVE_LOW,       
								XLLP_MFP_LPMO_FLOAT,       //PULL NEEDED?
								XLLP_MFP_LPMO_DRIVE_LOW,  
								XLLP_MFP_LPMO_DRIVE_LOW,  
								XLLP_MFP_LPMO_FLOAT,        //PULL NEEDED?
								XLLP_MFP_LPMO_DRIVE_HIGH,  
								};

XLLP_ACODEC_ERROR_T	XllpAc97ACodecMFPInit(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    XLLP_STATUS_T retval;
    P_XLLP_MFP_RM_DB_ID_T pMfpRmDb = pDeviceContext->pMfpRmDb;
    XLLP_UINT32_T pMfpRegBase = pDeviceContext->pMfpRegBase;


	retval =  XllpMfpResourceManager_List(pMfpRmDb, xllp_ac97_mfp_list, XLLP_MFP_RM_ID_XLLP_AC97, XLLP_SET);

	if ( XLLP_STATUS_SUCCESS != retval) {
		//check later: it is a temp workaround for resource manager bug.
	 	//return retval ;	
	}

    // Lock MFPR block until finished with all MFPRs.
    // Don't forget to unlock for early returns, too.
    XllpLock(XLLP_RESOURCE_MFPR);

    // Assign the Alternate Functions and Drive Strengths for each AC97 MFP.
	retval = XllpMfpSetAfDs_List(
	            (P_XLLP_VUINT32_T)pMfpRegBase, 
	            xllp_ac97_mfp_list, 
	            xllp_ac97_af_list, 
	            xllp_ac97_ds_list);

    if ( XLLP_STATUS_SUCCESS != retval) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
     	return retval ;	
  }

    // Configure the pin output levels for low power modes.

    retval = XllpMfpConfigureLpmOutputLevel_List (
			    (P_XLLP_VUINT32_T)pMfpRegBase,
                xllp_ac97_mfp_list,
                xllp_ac97_lpm_output_list);

	if ( XLLP_STATUS_SUCCESS != retval) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
		return retval ;	
	}

    // XLLI sets all pins to an activated pulled non-LPM level in order to
    //  establish a low-power default setting.  That is helpful if a driver is
    //  not loaded to control some pins, but here we are.  So deactivate
    //  that now in order to avoid conflicts with the Ac97 Interface device.

    retval = XllpMfpActivatePullUpDown_List (
                (P_XLLP_VUINT32_T)pMfpRegBase,
                xllp_ac97_mfp_list,
                XLLP_OFF);

	if ( XLLP_STATUS_SUCCESS != retval) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
		return retval ;	
	}

    // Do later to make sure that setting take effect
 
    XllpUnlock(XLLP_RESOURCE_MFPR);
    return retval;  
}

XLLP_ACODEC_ERROR_T	XllpAc97ACodecMFPDeInit(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//do later: free all MFP resources.
    return XLLP_ACODEC_SUCCESS;
}
