/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include "xllp_acodec.h"
#include "xllp_wm9713.h"
#include "xllp_ac97acodec.h"
#include "xllp_ac97acodecplat.h"


XLLP_ACODEC_ERROR_T XllpAcodecInit(XLLP_ACODEC_CONTEXT_T *pDeviceContext, XLLP_BOOL_T hwInit)
{
    //set codec specific functions
    //set mfp for Zylonite platform
    //call bus init function (AC97, I2S, I2C, SSP)
    //call specific init of codec
    XLLP_ACODEC_ERROR_T retval = XLLP_ACODEC_SUCCESS;

    if (pDeviceContext->ACodecId != WM_9713_ID)
    {//on Zylonite, it is Wolfson 9713 codec only
        return XLLP_ACODEC_GENERAL_SW_ERR;        
    }
    
    if (XLLP_TRUE == hwInit)
    {
    	XllpAc97ACodecMFPInit(pDeviceContext);
       XllpAc97ACodecInit(pDeviceContext);  //codec init common to ac97
    }
		    
	//wm9713-specific functions
	(pDeviceContext->g_pfnSetMasterVol)         = XllpWm9713SetMasterVol;
       (pDeviceContext->g_pfnSetMasterInputGain)   = XllpWm9713SetMasterInputGain;
	(pDeviceContext->g_pfnGetInSampleRate)      = XllpWm9713GetInSampleRate;
	(pDeviceContext->g_pfnGetOutSampleRate)     = XllpWm9713GetOutSampleRate;
	(pDeviceContext->g_pfnSetInSampleRate)      = XllpWm9713SetInSampleRate;
	(pDeviceContext->g_pfnSetOutSampleRate)     = XllpWm9713SetOutSampleRate;
	(pDeviceContext->g_pfnCodecSpecificInit)    = XllpWm9713SpecificInit;
	(pDeviceContext->g_pfnCodecSpecificDeInit)  = XllpWm9713SpecificDeInit;
	
	(pDeviceContext->g_pfnACodecRead)           = XllpAc97ACodecRead;
	(pDeviceContext->g_pfnACodecWrite)          = XllpAc97ACodecWrite;
	
	(pDeviceContext->g_pfnHiFiStreamPathEnable) = XllpWm9713HiFiStreamPathEnable;
	(pDeviceContext->g_pfnHiFiStreamPathDisable)= XllpWm9713HiFiStreamPathDisable;
	(pDeviceContext->g_pfnRecordingPathEnable)  = XllpWm9713RecordingPathEnable;
	(pDeviceContext->g_pfnRecordingPathDisable) = XllpWm9713RecordingPathDisable;
	(pDeviceContext->g_pfnVoicePathEnable)      = NULL;
	(pDeviceContext->g_pfnVoicePathDisable)     = NULL;
	(pDeviceContext->g_pfnSideToneEnable)       = NULL;
	(pDeviceContext->g_pfnSideToneDisable)      = NULL;
	
	(pDeviceContext->g_pfnHeadsetEnable)        = XllpWm9713HeadsetEnable;
	(pDeviceContext->g_pfnHeadsetDisable)       = XllpWm9713HeadsetDisable;
	(pDeviceContext->g_pfnEarSpeakerEnable)     = XllpWm9713EarSpeakerEnable;
	(pDeviceContext->g_pfnEarSpeakerDisable)    = XllpWm9713EarSpeakerDisable;
	(pDeviceContext->g_pfnLouderSpeakerEnable)  = XllpWm9713LouderSpeakerEnable;
	(pDeviceContext->g_pfnLouderSpeakerDisable) = XllpWm9713LouderSpeakerDisable;
	(pDeviceContext->g_pfnMicEnable)            = XllpWm9713MicEnable;
	(pDeviceContext->g_pfnMicDisable)           = XllpWm9713MicDisable;
	
	(pDeviceContext->g_pfnSetVolumeTable)       = NULL;
	(pDeviceContext->g_pfnGetVolumeTable)       = NULL;
	(pDeviceContext->g_pfnSetToneTable)         = NULL;
	(pDeviceContext->g_pfnGetToneTable)         = NULL;
    	(pDeviceContext->g_pfnSetRoute)  = XllpWm9713SetRoute;
    	(pDeviceContext->g_pfnSleepCodec) = NULL;
    	(pDeviceContext->g_pfnWakeCodec) = NULL;
    	(pDeviceContext->g_pfnGetState) = NULL;

    	(pDeviceContext->g_pfnGetVol) = XllpWm9713GetVol;
    	(pDeviceContext->g_pfnSetVol) = XllpWm9713SetVol;
    	(pDeviceContext->g_pfnGetVoiceInSampleRate) = NULL;
    	(pDeviceContext->g_pfnSetVoiceInSampleRate) = NULL;
    	
    if (XLLP_TRUE == hwInit)
    {
        retval = pDeviceContext->g_pfnCodecSpecificInit(pDeviceContext);
    }
			 
    return retval;
}

XLLP_ACODEC_ERROR_T XllpAcodecDeInit(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{
    //power down codec by codec specific power down function
    if (pDeviceContext->g_pfnCodecSpecificDeInit)
    {
	    pDeviceContext->g_pfnCodecSpecificDeInit(pDeviceContext);
	}
    //call bus deinit function
    XllpAc97ACodecDeInit(pDeviceContext);
    //restore MFP, set GPIO to suitable value
    XllpAc97ACodecMFPDeInit(pDeviceContext);
    
    return XLLP_ACODEC_SUCCESS;
}

XLLP_ACODEC_ERROR_T XllpAcodecPCMBusEnable(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//do later: PCM Bus is only used for BT audio
    return XLLP_ACODEC_SUCCESS;
}
	
XLLP_ACODEC_ERROR_T XllpAcodecPCMDisable(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//do later: PCM Bus is only used for BT audio
    return XLLP_ACODEC_SUCCESS;
}

XLLP_ACODEC_ERROR_T XllpAcodecEnableAmplifier(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//no implementation on Zylonite. WM9713 has internal amplifier, so no board amplifier needed
    return XLLP_ACODEC_SUCCESS;    
}

XLLP_ACODEC_ERROR_T XllpAcodecSuspend(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//this function is used in OS driver while suspend resume. 
    //Size of pSaveMemory should be AC97_SAVE_CONTEXT_SIZE + CODEC_CONTEXT_SIZE
    
    //Save WM9713 register and make wm9713 into sleep mode
    XllpWm9713Sleep(pDeviceContext);
    //Save AC97 related functions
    XllpAc97ACodecSaveContext (pDeviceContext, (XLLP_AC97_SAVE_CONTEXT_T *)((XLLP_UINT8_T *)(pDeviceContext->pSaveMemory) + CODEC_CONTEXT_SIZE));
    //do later: Save PCM Bus Status    
    //do later: Power down WM9713
    //Shutdown ACLINK
    XllpAc97ACodecDeInit (pDeviceContext);
    return XLLP_ACODEC_SUCCESS;    
}

XLLP_ACODEC_ERROR_T XllpAcodecResume(XLLP_ACODEC_CONTEXT_T *pDeviceContext)
{//This function is used in OS driver while suspend resume
    //Warm/Cold Reset ACLink
    XllpAc97ACodecInit(pDeviceContext);
    //Restore AC97 related functions
    XllpAc97ACodecRestoreContext(pDeviceContext, 
        (XLLP_AC97_SAVE_CONTEXT_T *) ((XLLP_UINT8_T *)(pDeviceContext->pSaveMemory) + CODEC_CONTEXT_SIZE) );
    //Restore WM9713 register wake up wm9713
    XllpWm9713Wakeup(pDeviceContext);
    //do later: Restore PCM Bus Status
    return XLLP_ACODEC_SUCCESS;    
}




