/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

 /* 
 * xllp_camera.c
 *
 * xllp_camera driver for Monahans 
 *
 */

/*++

Module Name:  $Workfile: xllp_camera.c $

Abstract:  
     contains all primitive functions for XLLP camera

Notes:Only valid for processor code named Monahans.

--*/
#include "xllp_defs.h"
#include "xllp_serialization.h"
#include "xllp_ost.h"
#include "xllp_mfp_plat.h"
#include "xllp_gpio_plat.h"
#include "xllp_mfp.h"
#include "xllp_camera.h"
#include "xllp_camera_os_depend.h"
#include "xllp_ci.h"

/***********************************************************************
 *
 * Declarations
 *
 ***********************************************************************/
#define SINGLE_DESCRIPTOR_TRANSFER_MAX  4096
#define SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX  ((1<<21) - 32)

// map of xllp camera image format (xllp_camera.h) ==> capture interface format (xllp_ci.h)
static const XLLP_CI_IMAGE_FORMAT FORMAT_MAPPINGS[] = {
        XLLP_CI_RAW8,                   //RAW
        XLLP_CI_RAW9,
        XLLP_CI_RAW10,

        XLLP_CI_RGB444,                 //RGB
        XLLP_CI_RGB555,
        XLLP_CI_RGB565,
        XLLP_CI_RGB666_PACKED,          //RGB Packed 
        XLLP_CI_RGB666,
        XLLP_CI_RGB888_PACKED,
        XLLP_CI_RGB888,
        XLLP_CI_RGBT555_0,              //RGB+Transparent bit 0
        XLLP_CI_RGBT888_0,
        XLLP_CI_RGBT555_1,              //RGB+Transparent bit 1  
        XLLP_CI_RGBT888_1,
    
        XLLP_CI_INVALID_FORMAT,
        XLLP_CI_YCBCR422,               //YCBCR
        XLLP_CI_YCBCR422_PLANAR,        //YCBCR Planar
        XLLP_CI_INVALID_FORMAT,
        XLLP_CI_INVALID_FORMAT
};

// MFP configuration
static XLLP_UINT32_T xllp_ci_mfp_list []={
                    XLLP_MFP_CIF_DD_0_OFFSET,           // CIF_DD<0>
                    XLLP_MFP_CIF_DD_1_OFFSET,           // CIF_DD<1>
                    XLLP_MFP_CIF_DD_2_OFFSET,           // CIF_DD<2>
                    XLLP_MFP_CIF_DD_3_OFFSET,           // CIF_DD<3>
                    XLLP_MFP_CIF_DD_4_OFFSET,           // CIF_DD<4>
                    XLLP_MFP_CIF_DD_5_OFFSET,           // CIF_DD<5>
                    XLLP_MFP_CIF_DD_6_OFFSET,           // CIF_DD<6>
                    XLLP_MFP_CIF_DD_7_OFFSET,           // CIF_DD<7>
                    XLLP_MFP_CIF_DD_8_OFFSET,           // CIF_DD<8>
                    XLLP_MFP_CIF_DD_9_OFFSET,           // CIF_DD<9>
                    XLLP_MFP_CIF_MCLK_OFFSET,           // CIF_MCLK
                    XLLP_MFP_CIF_PCLK_OFFSET,           // CIF_PCLK
                    XLLP_MFP_CIF_HSYNC_OFFSET,          // CIF_HSYNC
                    XLLP_MFP_CIF_VSYNC_OFFSET,          // CIF_VSYNC
                    XLLP_MFP_CIF_HI_PWDN_GPI0_OFFSET,   // CIF_HI_PWDN
                    XLLP_MFP_CIF_LO_PWDN_GPI0_OFFSET,   // CIF_LO_PWDN
                    XLLP_MFP_PIN_EOLIST_MARKER
                };//end of xllp_ci_mfp_list

static XLLP_MFP_ALT_FN_T xllp_ci_af_list []={
                    XLLP_MFP_CIF_DD_0_AF,           // CIF_DD<0>
                    XLLP_MFP_CIF_DD_1_AF,           // CIF_DD<1>
                    XLLP_MFP_CIF_DD_2_AF,           // CIF_DD<2>
                    XLLP_MFP_CIF_DD_3_AF,           // CIF_DD<3>
                    XLLP_MFP_CIF_DD_4_AF,           // CIF_DD<4>
                    XLLP_MFP_CIF_DD_5_AF,           // CIF_DD<5>
                    XLLP_MFP_CIF_DD_6_AF,           // CIF_DD<6>
                    XLLP_MFP_PIN_GPIO56_AF_GPIO_56, // XLLP_MFP_CIF_DD_6_AF, CIF_DD<7> //workaround of silicon issue
                    XLLP_MFP_CIF_DD_8_AF,           // CIF_DD<8>
                    XLLP_MFP_CIF_DD_9_AF,           // CIF_DD<9>
                    XLLP_MFP_CIF_MCLK_AF,           // CIF_MCLK
                    XLLP_MFP_CIF_PCLK_AF,           // CIF_PCLK
                    XLLP_MFP_CIF_HSYNC_AF,          // CIF_HSYNC
                    XLLP_MFP_CIF_VSYNC_AF,          // CIF_VSYNC
                    XLLP_MFP_CIF_HI_PWDN_GPI0_AF,   // CIF_HI_PWDN
                    XLLP_MFP_CIF_LO_PWDN_GPI0_AF,   // CIF_LO_PWDN
                };//end of xllp_ci_af_list

static XLLP_MFP_DRIVE_STRENGTH_T xllp_ci_ds_list []={
                    XLLP_MFP_CIF_DS,           // CIF_DD<0>
                    XLLP_MFP_CIF_DS,           // CIF_DD<1>
                    XLLP_MFP_CIF_DS,           // CIF_DD<2>
                    XLLP_MFP_CIF_DS,           // CIF_DD<3>
                    XLLP_MFP_CIF_DS,           // CIF_DD<4>
                    XLLP_MFP_CIF_DS,           // CIF_DD<5>
                    XLLP_MFP_CIF_DS,           // CIF_DD<6>
                    XLLP_MFP_CIF_DS,           // CIF_DD<7>
                    XLLP_MFP_CIF_DS,           // CIF_DD<8>
                    XLLP_MFP_CIF_DS,           // CIF_DD<9>
                    XLLP_MFP_CIF_DS,           // CIF_MCLK
                    XLLP_MFP_CIF_DS,           // CIF_PCLK
                    XLLP_MFP_CIF_DS,           // CIF_HSYNC
                    XLLP_MFP_CIF_DS,           // CIF_VSYNC
                    XLLP_MFP_CIF_DS,           // CIF_HI_PWDN
                    XLLP_MFP_CIF_DS,           // CIF_LO_PWDN
                };//end of xllp_ci_ds_list

static XLLP_MFP_LPM_OUTPUT_T xllp_ci_lpm_output_list []={
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<0>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<1>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<2>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<3>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<4>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<5>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<6>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<7>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<8>
                    XLLP_MFP_LPMO_FLOAT,           // CIF_DD<9>
                    XLLP_MFP_LPMO_PULL_LOW,        // CIF_MCLK
                    XLLP_MFP_LPMO_FLOAT,           // CIF_PCLK
                    XLLP_MFP_LPMO_FLOAT,           // CIF_HSYNC
                    XLLP_MFP_LPMO_FLOAT,           // CIF_VSYNC
                    XLLP_MFP_LPMO_DRIVE_HIGH,      // CIF_HI_PWDN
                    XLLP_MFP_LPMO_DRIVE_HIGH,      // CIF_LO_PWDN
                };//end of xllp_keypad_lpm_output_list
                                
static void PrvConfigureDMAC( P_XLLP_Camera_Context_T camera_context);
static XLLP_STATUS_T PrvStartCapture( P_XLLP_Camera_Context_T camera_context, unsigned int frames );
static void PrvSetStopState( P_XLLP_Camera_Context_T camera_context );

/***********************************************************************
 *
 * Private functions
 *
 ***********************************************************************/
void PrvConfigureDMAC( P_XLLP_Camera_Context_T camera_context)
{
    unsigned int des_physical;
    XLLP_Camera_Frame_Buffer_Queue_T *queue;

    queue = (camera_context->capture_mode == XLLP_CAMERA_MODE_VIDEO)?
            &(camera_context->video_capture_buffer_queue):
            &(camera_context->still_capture_buffer_queue);

    if (queue->head == NULL)
        return;
    
    des_physical = queue->head->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];       
    XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                            des_physical,
                            XLLP_CI_DMA_CHANNEL_0           
                            );

    if ( camera_context->fifo1_transfer_size ) {
        des_physical = queue->head->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];
        XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                                des_physical,
                                XLLP_CI_DMA_CHANNEL_1             
                                );
    }

    if ( camera_context->fifo2_transfer_size ) {
        des_physical = queue->head->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];
        XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                                des_physical,
                                XLLP_CI_DMA_CHANNEL_2             
                                );
    }

    camera_context->dma_running = XLLP_TRUE;
}

XLLP_STATUS_T PrvSyncCI( P_XLLP_Camera_Context_T camera_context )
{
    
    // If the sensor was put into single frame mode, it will perform one of
    // 2 action.
    //
    //    IF the command is entered before VSYNC goes active, the sensor
    //    will issue a VSYNC and 1 HREF.
    //
    //    IF the command is entered after VSYNC goes active, the sensor will
    //    complete the current frame and provide 1 more frame.
    //
    //    The first condition will require to restart the CI since the CI 
    //    will be waiting for HREF's before continuing to look for EOF.
    //
    //    The second condition needs to be handled by waiting for the 
    //    CI to go to an IDLE state.

    //    During the Stop capture routine, the camera interface was disabled.
    //    If the camera disable request has not been completed, force the disable
    //    and then enable it.
    if( camera_context->ci_disable_complete == XLLP_FALSE )
    {
        XllpCIDisable( camera_context->ci_reg_base, camera_context->ost_reg_base, XLLP_TRUE, XLLP_TRUE );
        XllpCIDisableComplete( camera_context->ci_reg_base );
    }

    // disable the CI dedicated DMAC by filling the DMA descriptor registers with NULL
#if 0
    XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                            0,
                            XLLP_CI_DMA_CHANNEL_0           
                            );

    XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                            0,
                            XLLP_CI_DMA_CHANNEL_1           
                            );

    XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                            0,
                            XLLP_CI_DMA_CHANNEL_2           
                            );
#endif
    // Enable the CI again.
    XllpCIEnable( camera_context->ci_reg_base/*, XLLP_TRUE */);

    return XLLP_STATUS_SUCCESS;
}



XLLP_STATUS_T PrvStartCapture( P_XLLP_Camera_Context_T camera_context, unsigned int frames )
{

    XLLP_STATUS_T   status;

    // Disable CI
    PrvSetStopState(camera_context);
    
    // clear ci fifo
    XllpCIResetFIFO(camera_context->ci_reg_base);
    XllpCIClearInterruptStatus(camera_context->ci_reg_base, 0xFFFFFFFF);

    if( camera_context->ci_disable_complete == XLLP_FALSE ) {
        XllpCIDisable( camera_context->ci_reg_base, camera_context->ost_reg_base, XLLP_TRUE, XLLP_TRUE );
        XllpCIDisableComplete( camera_context->ci_reg_base );
    }
	
    PrvConfigureDMAC(camera_context);
    
    XllpCIEnable( camera_context->ci_reg_base/*, XLLP_TRUE */);

    // start capture
    status = camera_context->camera_functions->start_capture(camera_context, frames);    
    return status;
}


/***********************************************************************
 *
 * Init/Deinit APIs
 *
 ***********************************************************************/
XLLP_STATUS_T XllpCameraInit( P_XLLP_Camera_Context_T camera_context )
{
    XLLP_STATUS_T   status = XLLP_STATUS_SUCCESS;
	XLLP_CI_DMAC_DESCRIPTOR_T       *init_descriptor;

#ifdef XLLP_DEBUG_PARAM_CHECK    

    //////////////////////////////////////////////
    // parameter check
    //////////////////////////////////////////////
    if (camera_context->sensor_type > XLLP_CAMERA_TYPE_MAX)
        return XLLP_STATUS_WRONG_PARAMETER; 
        
    // check the function dispatch table according to the sensor type
    if ( !camera_context->camera_functions )
        return XLLP_STATUS_WRONG_PARAMETER;
    
    if ( !camera_context->camera_functions->init ||         
         !camera_context->camera_functions->deinit ||
         !camera_context->camera_functions->set_capture_format ||
         !camera_context->camera_functions->start_capture ||
         !camera_context->camera_functions->stop_capture )         
        return XLLP_STATUS_WRONG_PARAMETER;
#endif

    //////////////////////////////////////////////
    // initialize some xllp camera used parameters
    //////////////////////////////////////////////
    camera_context->capture_input_width             = 0;
    camera_context->capture_input_height            = 0;
    camera_context->capture_output_width            = 0;
    camera_context->capture_output_height           = 0;
    camera_context->capture_input_format            = XLLP_CAMERA_IMAGE_FORMAT_MAX + 1;
    camera_context->capture_output_format           = XLLP_CAMERA_IMAGE_FORMAT_MAX + 1;
    camera_context->fifo0_transfer_size             = 0;
    camera_context->fifo1_transfer_size             = 0;
    camera_context->fifo2_transfer_size             = 0;
    camera_context->fifo3_transfer_size             = 0;
    camera_context->video_fifo0_transfer_size       = 0;
    camera_context->video_fifo1_transfer_size       = 0;
    camera_context->video_fifo2_transfer_size       = 0;
    camera_context->still_fifo0_transfer_size       = 0;
    camera_context->still_fifo1_transfer_size       = 0;
    camera_context->still_fifo2_transfer_size       = 0;
    camera_context->frame_buffer_number             = 0;
    camera_context->video_capture_buffer_queue.head = NULL;
    camera_context->video_capture_buffer_queue.tail = NULL;       
    camera_context->still_capture_buffer_queue.head = NULL;
    camera_context->still_capture_buffer_queue.tail = NULL;
    camera_context->psu_enable                      = XLLP_FALSE;
    camera_context->cgu_enable                      = XLLP_FALSE;
    camera_context->ssu_scale                       = XLLP_CI_SSU_SCALE_DISABLE;
    camera_context->cmu_usage                       = XLLP_CI_CMU_DISABLE;
    camera_context->dma_running                     = XLLP_FALSE;
    
    //////////////////////////////////////////////
    // mfp pins init
    //////////////////////////////////////////////
   status =  XllpMfpResourceManager_List(
                camera_context->mfp_rm_db, 
                xllp_ci_mfp_list, 
                XLLP_MFP_RM_ID_XLLP_CIF, 
                XLLP_SET);

    if ( XLLP_STATUS_SUCCESS != status) {
         return status ;    
    }

    // Lock MFPR block until finished with all MFPRs.
    // Don't forget to unlock for early returns, too.
    XllpLock(XLLP_RESOURCE_MFPR);

    // Assign the Alternate Functions and Drive Strengths for each CI MFP.
    status = XllpMfpSetAfDs_List(
                (P_XLLP_VUINT32_T)camera_context->mfp_reg_base, 
                xllp_ci_mfp_list, 
                xllp_ci_af_list, 
                xllp_ci_ds_list);

    if ( XLLP_STATUS_SUCCESS != status) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
        return status ;    
    }

    // Configure the pin output levels for low power modes.
    //  The pulled level settings also apply to functional mode (non-LPM) if
    //      activated for that use - but they are not so activated for the 
    //      CI.
    status = XllpMfpConfigureLpmOutputLevel_List (
                (P_XLLP_VUINT32_T)camera_context->mfp_reg_base,
                xllp_ci_mfp_list,
                xllp_ci_lpm_output_list);

    if ( XLLP_STATUS_SUCCESS != status) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
        return status ;    
    }

    // XLLI sets all pins to an activated pulled non-LPM level in order to
    //  establish a low-power default setting.  That is helpful if a driver is
    //  not loaded to control some pins, but here we are.  So deactivate
    //  that now in order to avoid conflicts with the CI Interface device.

    status = XllpMfpActivatePullUpDown_List (
                (P_XLLP_VUINT32_T)camera_context->mfp_reg_base,
                xllp_ci_mfp_list,
                XLLP_OFF);

    if ( XLLP_STATUS_SUCCESS != status) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
        return status ;    
    }

    //////////////////////////////////////////////        
    // gpio init
    //////////////////////////////////////////////

    // set two gpio pin direction as output to control the power of two sensors.
    XllpGpioSetDirection((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_HI_PWDN, XLLP_GPIO_DIRECTION_OUT);
    XllpGpioSetDirection((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_LO_PWDN, XLLP_GPIO_DIRECTION_OUT);

    // set two gpio pin output as HI to power off two sensors.
    XllpGpioSetOutputLevel((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_HI_PWDN, XLLP_HI);
    XllpGpioSetOutputLevel((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_LO_PWDN, XLLP_HI);

    // initialize the DMAC
	init_descriptor        = (XLLP_CI_DMAC_DESCRIPTOR_T*)((long)camera_context->phantom_buffer_virtual + 48);        
	init_descriptor->ddadr = camera_context->phantom_buffer_physical + 48;
	init_descriptor->dsadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR0; 
	init_descriptor->dtadr = camera_context->phantom_buffer_physical;
	init_descriptor->dcmd  = 76800;

    XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                            camera_context->phantom_buffer_physical + 48,
                            XLLP_CI_DMA_CHANNEL_0           
                            );
 
    XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                            camera_context->phantom_buffer_physical + 48,
                            XLLP_CI_DMA_CHANNEL_1           
                            );

    XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                            camera_context->phantom_buffer_physical + 48,
                            XLLP_CI_DMA_CHANNEL_2           
                            );
    
	XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                            camera_context->phantom_buffer_physical + 48,
                            XLLP_CI_DMA_CHANNEL_3           
                            );
	XllpUnlock(XLLP_RESOURCE_MFPR);

    //////////////////////////////////////////////        
    // capture interface init
    //////////////////////////////////////////////
    XllpCIInit(camera_context->ci_reg_base, camera_context->clk_reg_base);
    
    //////////////////////////////////////////////
    // sensor init
    //////////////////////////////////////////////
    status = camera_context->camera_functions->init(camera_context);    
    if (status){
        goto camera_init_err;
    }
    
    // set frame rate
    XllpCameraSetCaptureFrameRate(camera_context);

    return XLLP_STATUS_SUCCESS;
        
camera_init_err:
    XllpCameraDeInit(camera_context);
    return XLLP_STATUS_FAILURE; 
}

XLLP_STATUS_T XllpCameraDeInit( P_XLLP_Camera_Context_T camera_context )
{
    XLLP_STATUS_T status;

    // deinit sensor
    status = camera_context->camera_functions->deinit(camera_context);  
    
    // capture interface deinit
    XllpCIDeInit(camera_context->ci_reg_base, camera_context->clk_reg_base);

    return status;
}


/***********************************************************************
 *
 * Capture APIs
 *
 ***********************************************************************/
// Set the image format
XLLP_STATUS_T XllpCameraSetCaptureFormat( P_XLLP_Camera_Context_T camera_context )
{
    XLLP_STATUS_T status;
    unsigned int capture_input_width, capture_input_height;
    XLLP_CI_IMAGE_FORMAT ci_input_format, ci_output_format;
    unsigned int scale;

    ///////////////////////////////////////////////////////////////////       
    // caculate some XLLP parameters according to the driver parameters    
    ///////////////////////////////////////////////////////////////////        

    // get basic capture format and resolution info
    if (camera_context->capture_mode == XLLP_CAMERA_MODE_VIDEO){
        camera_context->capture_input_format  = camera_context->video_capture_input_format;
        camera_context->capture_output_format = camera_context->video_capture_output_format;
        camera_context->capture_output_width  = camera_context->video_capture_width; 
        camera_context->capture_output_height = camera_context->video_capture_height;
        camera_context->fifo0_transfer_size   = camera_context->video_fifo0_transfer_size;
        camera_context->fifo1_transfer_size   = camera_context->video_fifo1_transfer_size;
        camera_context->fifo2_transfer_size   = camera_context->video_fifo2_transfer_size;
    }
    else {
        camera_context->capture_input_format  = camera_context->still_capture_input_format;
        camera_context->capture_output_format = camera_context->still_capture_output_format;
        camera_context->capture_output_width  = camera_context->still_capture_width;       
        camera_context->capture_output_height = camera_context->still_capture_height; 
        camera_context->fifo0_transfer_size   = camera_context->still_fifo0_transfer_size;
        camera_context->fifo1_transfer_size   = camera_context->still_fifo1_transfer_size;
        camera_context->fifo2_transfer_size   = camera_context->still_fifo2_transfer_size;
    }

#ifdef XLLP_DEBUG_PARAM_CHECK        
    if (camera_context->capture_input_format >  XLLP_CAMERA_IMAGE_FORMAT_MAX ||
        camera_context->capture_output_format > XLLP_CAMERA_IMAGE_FORMAT_MAX )
        return XLLP_STATUS_WRONG_PARAMETER;
#endif

    // determine whether to enable CGU
    camera_context->cgu_enable = 
                ((camera_context->capture_input_format <= XLLP_CAMERA_IMAGE_FORMAT_RAW10) &&
                 (camera_context->capture_input_format != camera_context->capture_output_format))?
                 XLLP_TRUE : XLLP_FALSE;

    // determine how to use SSU
    camera_context->ssu_scale = (camera_context->capture_mode == XLLP_CAMERA_MODE_VIDEO)?
                                 (XLLP_CI_SSU_SCALE)camera_context->video_capture_scale:
                                 (XLLP_CI_SSU_SCALE)camera_context->still_capture_scale;

    // determine how to use CMU
    if ((camera_context->capture_input_format <= XLLP_CAMERA_IMAGE_FORMAT_RAW10) &&
        (camera_context->capture_output_format >= XLLP_CAMERA_IMAGE_FORMAT_RGB444) &&
        (camera_context->capture_output_format <= XLLP_CAMERA_IMAGE_FORMAT_RGBT888_1)
        ){
        
        camera_context->cmu_usage = XLLP_CI_CMU_OUTPUT_RGB;
        
    }else if ((camera_context->capture_input_format <= XLLP_CAMERA_IMAGE_FORMAT_RAW10) && 
              (camera_context->capture_output_format >= XLLP_CAMERA_IMAGE_FORMAT_YCBCR400) && 
              (camera_context->capture_output_format <= XLLP_CAMERA_IMAGE_FORMAT_YCBCR444_PLANAR)
             ){
             
        camera_context->cmu_usage = XLLP_CI_CMU_OUTPUT_YUV;
        
    }else{
    
        camera_context->cmu_usage = XLLP_CI_CMU_DISABLE;
    }

#ifdef XLLP_DEBUG_PARAM_CHECK        
    if ((camera_context->cmu_usage != XLLP_CI_CMU_DISABLE) &&
        (camera_context->ssu_scale == XLLP_CI_SSU_SCALE_DISABLE)){
        return XLLP_STATUS_WRONG_PARAMETER;
    }   
#endif

    // caculate the input resolution
    capture_input_width = camera_context->capture_output_width;
    capture_input_height = camera_context->capture_output_height;

    if (camera_context->cmu_usage != XLLP_CI_CMU_DISABLE)
    {
        capture_input_width  += 1;
        capture_input_height += 1;
    }

    if (camera_context->ssu_scale != XLLP_CI_SSU_SCALE_DISABLE)
    {
        scale = (camera_context->ssu_scale == XLLP_CAMERA_CAPTURE_SCALE_HALF)?2:4;
        capture_input_width = (camera_context->capture_input_format <= XLLP_CAMERA_IMAGE_FORMAT_RAW10)?
                              (capture_input_width*scale + 2):
                              (capture_input_width*scale + 1);
        capture_input_height = capture_input_height*scale + 1;
    }
        
    camera_context->capture_input_width = capture_input_width;
    camera_context->capture_input_height = capture_input_height;

    ///////////////////////////////////////////////////////////////////
    // set sensor setting
    ///////////////////////////////////////////////////////////////////
    
    status = camera_context->camera_functions->set_capture_format(camera_context);
    if (status){
        return status;
    }

    ///////////////////////////////////////////////////////////////////
    // set CI setting        
    ///////////////////////////////////////////////////////////////////

    // set scale to SSU
    XllpCISSUSetScale(camera_context->ci_reg_base, camera_context->ssu_scale);

    // enable/disable CMU
    XllpCICMUEnable(camera_context->ci_reg_base, camera_context->cmu_usage);

    // set CI image format
    ci_input_format = FORMAT_MAPPINGS[camera_context->capture_input_format];
    ci_output_format = FORMAT_MAPPINGS[camera_context->capture_output_format];

#ifdef XLLP_DEBUG_PARAM_CHECK        
    if (ci_input_format == XLLP_CI_INVALID_FORMAT || ci_output_format == XLLP_CI_INVALID_FORMAT)
        return XLLP_STATUS_WRONG_PARAMETER;
#endif

    XllpCISetImageFormat(camera_context->ci_reg_base, ci_input_format, ci_output_format);

    return XLLP_STATUS_SUCCESS;
}

// take a picture and copy it into the frame buffers
XLLP_STATUS_T XllpCameraCaptureStillImage( P_XLLP_Camera_Context_T camera_context )
{
    XLLP_STATUS_T status;
    status = PrvStartCapture( camera_context, 1 );
    return status;
}

// capture motion video and copy them to the frame buffers
XLLP_STATUS_T XllpCameraStartVideoCapture( P_XLLP_Camera_Context_T camera_context )
{
    XLLP_STATUS_T status;
    status = PrvStartCapture( camera_context, 0 );
    return status;
}

void PrvSetStopState( P_XLLP_Camera_Context_T camera_context )
{
    
    // Clear the counts and the current status.
    camera_context->ci_disable_complete = XLLP_FALSE;
    XllpCIClearInterruptStatus(camera_context->ci_reg_base, 0xFFFFFFFF);
    
    
    // Depending upon the timing of using the command to change to single
    // frame mode, the OV9640 may issue a VSYNC and 1 HSYNC signal and then
    // stop.  This would leave the CI in a non-IDLE state.  The camera
    // interface is now disabled before leaving the stop state.
    //XllpCIDisable( camera_context->ci_reg_base, camera_context->ost_reg_base, XLLP_FALSE, XLLP_FALSE );
    XllpCIDisable( camera_context->ci_reg_base, camera_context->ost_reg_base, XLLP_TRUE, XLLP_TRUE );
}

// disable motion video image capture
void XllpCameraStopVideoCapture( P_XLLP_Camera_Context_T camera_context )
{
    XLLP_STATUS_T status;
    
    // stop capture
    status = camera_context->camera_functions->stop_capture(camera_context);    

    // Set up to receive interrupts and track the sof, eof, and eol after the shutdown.
    PrvSetStopState(camera_context);
    
    // clear ci fifo
    XllpCIResetFIFO(camera_context->ci_reg_base);

    // empty the capture queue, 
    // WARNING: the driver must re-submit all of buffers into queue before it starts video/still capture next time!
    /* camera_context->video_capture_buffer_queue.head = NULL;
    camera_context->video_capture_buffer_queue.tail = NULL;       
    camera_context->still_capture_buffer_queue.head = NULL;
    camera_context->still_capture_buffer_queue.tail = NULL;  */
    camera_context->dma_running                    = XLLP_FALSE;

    return;
}


/***********************************************************************
 *
 * Flow Control APIs
 *
 ***********************************************************************/
// Get the buffer size and the DMA descriptors memory size for this buffer.
XLLP_STATUS_T XllpCameraGetBufferSize(
                P_XLLP_Camera_Context_T   camera_context,
                                    int   buffer_type,
                                    int  *buffer_size,
                                    int  *buffer_dma_desc_mem_size)
{
    unsigned int    capture_output_format;
    unsigned int    capture_output_width; 
    unsigned int    capture_output_height;
    unsigned int    frame_size;
    unsigned int    fifo0_transfer_size;
    unsigned int    fifo1_transfer_size;
    unsigned int    fifo2_transfer_size;
    unsigned int    fifo0_num_descriptors = 0;
    unsigned int    fifo1_num_descriptors = 0;
    unsigned int    fifo2_num_descriptors = 0;
    unsigned int    fifo0_phantom_num_descriptors = 0;
    unsigned int    fifo1_phantom_num_descriptors = 0;
    unsigned int    fifo2_phantom_num_descriptors = 0;
    unsigned int    num_descriptors_for_buffer;
    

    ///////////////////////////////////////////////////////////////////
    // caculate the fifo0~2 transfer size
    ///////////////////////////////////////////////////////////////////
    capture_output_format = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                            camera_context->video_capture_output_format:
                            camera_context->still_capture_output_format;
                                
    capture_output_width  = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                            camera_context->video_capture_width:
                            camera_context->still_capture_width;

    capture_output_height = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                            camera_context->video_capture_height:
                            camera_context->still_capture_height;

    switch(capture_output_format) {
        case XLLP_CAMERA_IMAGE_FORMAT_RAW10:
            frame_size = capture_output_width * capture_output_height * 2;
            fifo0_transfer_size = frame_size;
            fifo1_transfer_size = 0;
            fifo2_transfer_size = 0;
            break;
        case XLLP_CAMERA_IMAGE_FORMAT_RAW9:
            frame_size = capture_output_width * capture_output_height * 2;
            fifo0_transfer_size = frame_size;
            fifo1_transfer_size = 0;
            fifo2_transfer_size = 0;
            break;
        case XLLP_CAMERA_IMAGE_FORMAT_RAW8:
            frame_size = capture_output_width * capture_output_height;
            fifo0_transfer_size = frame_size;
            fifo1_transfer_size = 0;
            fifo2_transfer_size = 0;
            break;
        case XLLP_CAMERA_IMAGE_FORMAT_RGB565:
            frame_size = capture_output_width * capture_output_height * 2;
            fifo0_transfer_size = frame_size;
            fifo1_transfer_size = 0;
            fifo2_transfer_size = 0;
            break;
        case XLLP_CAMERA_IMAGE_FORMAT_YCBCR422_PACKED:
            frame_size = capture_output_width * capture_output_height * 2;
            fifo0_transfer_size = frame_size;
            fifo1_transfer_size = 0;
            fifo2_transfer_size = 0;
            break;
        case XLLP_CAMERA_IMAGE_FORMAT_YCBCR422_PLANAR:
            frame_size = capture_output_width * capture_output_height * 2;
            fifo0_transfer_size = frame_size / 2;
            fifo1_transfer_size = frame_size / 4;
            fifo2_transfer_size = frame_size / 4;
            break;
        default:
            return XLLP_STATUS_WRONG_PARAMETER;
            break;
    }

    if (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER){
        camera_context->video_fifo0_transfer_size = fifo0_transfer_size;
        camera_context->video_fifo1_transfer_size = fifo1_transfer_size;
        camera_context->video_fifo2_transfer_size = fifo2_transfer_size;
    }else{
        camera_context->still_fifo0_transfer_size = fifo0_transfer_size;
        camera_context->still_fifo1_transfer_size = fifo1_transfer_size;
        camera_context->still_fifo2_transfer_size = fifo2_transfer_size;    
    }

    ///////////////////////////////////////////////////////////////////
    // caculate the buffer size and descriptor memory size
    ///////////////////////////////////////////////////////////////////
    
    fifo0_num_descriptors = ( fifo0_transfer_size + SINGLE_DESCRIPTOR_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_TRANSFER_MAX;
        
    fifo1_num_descriptors = ( fifo1_transfer_size + SINGLE_DESCRIPTOR_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_TRANSFER_MAX;

    fifo2_num_descriptors = ( fifo2_transfer_size + SINGLE_DESCRIPTOR_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_TRANSFER_MAX;

    fifo0_phantom_num_descriptors = ( fifo0_transfer_size + SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX;
        
    fifo1_phantom_num_descriptors = ( fifo1_transfer_size + SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX;

    fifo2_phantom_num_descriptors = ( fifo2_transfer_size + SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX;

    num_descriptors_for_buffer = fifo0_num_descriptors + fifo1_num_descriptors + fifo2_num_descriptors;

    *buffer_size = num_descriptors_for_buffer * SINGLE_DESCRIPTOR_TRANSFER_MAX;
    *buffer_dma_desc_mem_size = sizeof(XLLP_CI_DMAC_DESCRIPTOR_T)*(      // descriptors memory for real frame buffer
                                fifo0_num_descriptors + fifo0_phantom_num_descriptors
                              + fifo1_num_descriptors + fifo1_phantom_num_descriptors
                              + fifo2_num_descriptors + fifo2_phantom_num_descriptors);

    return XLLP_STATUS_SUCCESS;
}

// Add the buffer into the buffer pool and generate the DMA chains for the buffer
XLLP_STATUS_T XllpCameraPrepareBuffer(
                P_XLLP_Camera_Context_T   camera_context,
                                  void   *buffer_virtual_address,
                                   int   *buffer_physical_address_array,
                                   int    buffer_physical_address_array_size,
                                   int    buffer_size,
                                   int    buffer_type,
                                  void   *dma_desc_mem_virtual_address,
                                  int    *dma_desc_mem_physical_address_array,
                                   int   *buffer_id,
                                  void   **pY,
                                  void   **pCb,
                                  void   **pCr)
{
    XLLP_Camera_Frame_Buffer_Info_T *buffer_info;
    int                              buffer_index;
    XLLP_CI_DMAC_DESCRIPTOR_T       *cur_des_virtual;
    int                              des_transfer_size, remain_size;
    unsigned int                     i, buffer_page_seq, dma_descriptor_seq;
    unsigned int                     fifo0_transfer_size;
    unsigned int                     fifo1_transfer_size;
    unsigned int                     fifo2_transfer_size;
    unsigned int                     fifo0_num_descriptors = 0;
    unsigned int                     fifo1_num_descriptors = 0;
    unsigned int                     fifo2_num_descriptors = 0;
    unsigned int                     fifo0_phantom_num_descriptors = 0;
    unsigned int                     fifo1_phantom_num_descriptors = 0;
    unsigned int                     fifo2_phantom_num_descriptors = 0;

    ///////////////////////////////////////////////////////////////////
    // compute descriptor number for fifo0~fifo2
    ///////////////////////////////////////////////////////////////////

    fifo0_transfer_size = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                           camera_context->video_fifo0_transfer_size:
                           camera_context->still_fifo0_transfer_size;

    fifo1_transfer_size = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                           camera_context->video_fifo1_transfer_size:
                           camera_context->still_fifo1_transfer_size;

    fifo2_transfer_size = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                           camera_context->video_fifo2_transfer_size:
                           camera_context->still_fifo2_transfer_size;

    // calculate how many descriptors are needed per frame

    fifo0_num_descriptors = (( fifo0_transfer_size + SINGLE_DESCRIPTOR_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_TRANSFER_MAX);
    
    fifo1_num_descriptors = (( fifo1_transfer_size + SINGLE_DESCRIPTOR_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_TRANSFER_MAX);
    
    fifo2_num_descriptors = (( fifo2_transfer_size + SINGLE_DESCRIPTOR_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_TRANSFER_MAX);

    fifo0_phantom_num_descriptors = (( fifo0_transfer_size + SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX);
    
    fifo1_phantom_num_descriptors = (( fifo1_transfer_size + SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX);
    
    fifo2_phantom_num_descriptors = (( fifo2_transfer_size + SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX - 1 )
         / SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX);

    if (((unsigned int)buffer_physical_address_array_size) < (fifo0_num_descriptors + fifo1_num_descriptors + fifo2_num_descriptors))
        return XLLP_STATUS_WRONG_PARAMETER;

    ///////////////////////////////////////////////////////////////////
    // add the buffer into the buffer pool
    ///////////////////////////////////////////////////////////////////

    buffer_index                          = (*buffer_id == -1)?camera_context->frame_buffer_number:*buffer_id;
    buffer_info                           = &(camera_context->master_frame_buffer_list[buffer_index]);
    buffer_info->frame_id                 = buffer_index;
    buffer_info->buffer_vir_addr          = (XLLP_VUINT32_T*)buffer_virtual_address;
    buffer_info->buffer_size              = buffer_size;
    buffer_info->dma_descriptors_virtual  = (XLLP_CI_DMAC_DESCRIPTOR_T*)dma_desc_mem_virtual_address;    
    buffer_info->pY                       = (void*)buffer_virtual_address;
    buffer_info->pCb                      = (fifo1_transfer_size)?(void*)((unsigned int)buffer_info->pY  + fifo0_num_descriptors*SINGLE_DESCRIPTOR_TRANSFER_MAX):0;
    buffer_info->pCr                      = (fifo2_transfer_size)?(void*)((unsigned int)buffer_info->pCb + fifo1_num_descriptors*SINGLE_DESCRIPTOR_TRANSFER_MAX):0;
    if (*buffer_id == -1) 
        camera_context->frame_buffer_number++;
   
    ///////////////////////////////////////////////////////////////////
    // generate the DMA chains
    ///////////////////////////////////////////////////////////////////
    dma_descriptor_seq = 0;
    buffer_page_seq = 0;
    cur_des_virtual = (XLLP_CI_DMAC_DESCRIPTOR_T*)dma_desc_mem_virtual_address;
    
    // 1. generate fifo0 DMA chain   
    if (fifo0_transfer_size){
        // 1.1 Build the primary DMA chain
        remain_size = fifo0_transfer_size;
        for(i = 0; i < fifo0_num_descriptors; i++) {
            // set descriptor
            if (remain_size > SINGLE_DESCRIPTOR_TRANSFER_MAX) 
                des_transfer_size = SINGLE_DESCRIPTOR_TRANSFER_MAX;
            else
                des_transfer_size = remain_size;
            
            cur_des_virtual->ddadr = dma_desc_mem_physical_address_array[++dma_descriptor_seq];
            cur_des_virtual->dsadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR0;            // FIFO0 physical address
            cur_des_virtual->dtadr = (unsigned)buffer_physical_address_array[buffer_page_seq++];
            cur_des_virtual->dcmd = des_transfer_size | XLLP_CI_DMAC_DCMD_INC_TRG_ADDR;

            if (i == 0){
                buffer_info->ch0_dma_desc_num = fifo0_num_descriptors;
                buffer_info->ch0_dma_desc_vir_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the first real descriptor
            }
            
            if (i == (fifo0_num_descriptors - 1)){
                buffer_info->ch0_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch0_dma_desc_phy_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the last real descriptor
            }
            
            // advance pointers
            remain_size -= des_transfer_size;
            cur_des_virtual++;
        }

        // 1.2 Build the phantom DMA chain
        remain_size = fifo0_transfer_size;
        for(i = 0; i < fifo0_phantom_num_descriptors; i++) {
            // set descriptor
            if (remain_size > SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX) 
                des_transfer_size = SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX;
            else
                des_transfer_size = remain_size;
            
            cur_des_virtual->ddadr = dma_desc_mem_physical_address_array[++dma_descriptor_seq];
            cur_des_virtual->dsadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR0;            // FIFO0 physical address
            cur_des_virtual->dtadr = camera_context->phantom_buffer_physical;
            cur_des_virtual->dcmd = des_transfer_size;

            if (i == 0){
                buffer_info->ch0_phantom_dma_desc_num = fifo0_phantom_num_descriptors;
                buffer_info->ch0_dma_desc_vir_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the first phantom descriptor
            }
            
            if (i == (fifo0_phantom_num_descriptors - 1)){
                buffer_info->ch0_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch0_dma_desc_phy_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the last phantom descriptor
                cur_des_virtual->ddadr = 
                    buffer_info->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
                        // loop the phantom buffer to itself
            }
            
            // advance pointers
            remain_size -= des_transfer_size;
            cur_des_virtual++;
        }
    }

    // 2. generate fifo1 DMA chain   
    if (fifo1_transfer_size){
        // 2.1 Build the primary DMA chain
        remain_size = fifo1_transfer_size;
        for(i = 0; i < fifo1_num_descriptors; i++) {
            // set descriptor
            if (remain_size > SINGLE_DESCRIPTOR_TRANSFER_MAX) 
                des_transfer_size = SINGLE_DESCRIPTOR_TRANSFER_MAX;
            else
                des_transfer_size = remain_size;
            
            cur_des_virtual->ddadr = dma_desc_mem_physical_address_array[++dma_descriptor_seq];
            cur_des_virtual->dsadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR1;            // FIFO1 physical address
            cur_des_virtual->dtadr = (unsigned)buffer_physical_address_array[buffer_page_seq++];
            cur_des_virtual->dcmd = des_transfer_size | XLLP_CI_DMAC_DCMD_INC_TRG_ADDR;

            if (i == 0){
                buffer_info->ch1_dma_desc_num = fifo1_num_descriptors;
                buffer_info->ch1_dma_desc_vir_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the first real descriptor
            }
            
            if (i == (fifo1_num_descriptors - 1)){
                buffer_info->ch1_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch1_dma_desc_phy_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the last real descriptor
            }
            
            // advance pointers
            remain_size -= des_transfer_size;
            cur_des_virtual++;
        }

        // 2.2 Build the phantom DMA chain
        remain_size = fifo1_transfer_size;
        for(i = 0; i < fifo1_phantom_num_descriptors; i++) {
            // set descriptor
            if (remain_size > SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX) 
                des_transfer_size = SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX;
            else
                des_transfer_size = remain_size;
            
            cur_des_virtual->ddadr = dma_desc_mem_physical_address_array[++dma_descriptor_seq];
            cur_des_virtual->dsadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR1;            // FIFO1 physical address
            cur_des_virtual->dtadr = camera_context->phantom_buffer_physical;
            cur_des_virtual->dcmd = des_transfer_size;

            if (i == 0){
                buffer_info->ch1_phantom_dma_desc_num = fifo1_phantom_num_descriptors;
                buffer_info->ch1_dma_desc_vir_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the first phantom descriptor
            }
            
            if (i == (fifo1_phantom_num_descriptors - 1)){
                buffer_info->ch1_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch1_dma_desc_phy_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the last phantom descriptor
                cur_des_virtual->ddadr = 
                    buffer_info->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
                        // loop the phantom buffer to itself
            }
            
            // advance pointers
            remain_size -= des_transfer_size;
            cur_des_virtual++;
        }    
    }
    
    // 3. generate fifo2 DMA chain   
    if (fifo2_transfer_size){
        // 3.1 Build the primary DMA chain
        remain_size = fifo2_transfer_size;
        for(i = 0; i < fifo2_num_descriptors; i++) {
            // set descriptor
            if (remain_size > SINGLE_DESCRIPTOR_TRANSFER_MAX) 
                des_transfer_size = SINGLE_DESCRIPTOR_TRANSFER_MAX;
            else
                des_transfer_size = remain_size;
            
            cur_des_virtual->ddadr = dma_desc_mem_physical_address_array[++dma_descriptor_seq];
            cur_des_virtual->dsadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR2;            // FIFO2 physical address
            cur_des_virtual->dtadr = (unsigned)buffer_physical_address_array[buffer_page_seq++];
            cur_des_virtual->dcmd = des_transfer_size | XLLP_CI_DMAC_DCMD_INC_TRG_ADDR;

            if (i == 0){
                buffer_info->ch2_dma_desc_num = fifo2_num_descriptors;
                buffer_info->ch2_dma_desc_vir_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the first real descriptor
            }
            
            if (i == (fifo2_num_descriptors - 1)){
                buffer_info->ch2_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch2_dma_desc_phy_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the last real descriptor
            }
            
            // advance pointers
            remain_size -= des_transfer_size;
            cur_des_virtual++;
        }

        // 3.2 Build the phantom DMA chain
        remain_size = fifo2_transfer_size;
        for(i = 0; i < fifo2_phantom_num_descriptors; i++) {
            // set descriptor
            if (remain_size > SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX) 
                des_transfer_size = SINGLE_DESCRIPTOR_PHANTOM_TRANSFER_MAX;
            else
                des_transfer_size = remain_size;
            
            cur_des_virtual->ddadr = dma_desc_mem_physical_address_array[++dma_descriptor_seq];
            cur_des_virtual->dsadr = XLLP_CI_REGBASE_PHY + XLLP_CIBR2;            // FIFO2 physical address
            cur_des_virtual->dtadr = camera_context->phantom_buffer_physical;
            cur_des_virtual->dcmd = des_transfer_size;

            if (i == 0){
                buffer_info->ch2_phantom_dma_desc_num = fifo2_phantom_num_descriptors;
                buffer_info->ch2_dma_desc_vir_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the first phantom descriptor
            }
            
            if (i == (fifo2_phantom_num_descriptors - 1)){
                buffer_info->ch2_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        cur_des_virtual;
                buffer_info->ch2_dma_desc_phy_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX] = 
                        (XLLP_UINT32_T)dma_desc_mem_physical_address_array[dma_descriptor_seq - 1];
                        // record address of the last phantom descriptor
                cur_des_virtual->ddadr = 
                    buffer_info->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
                        // loop the phantom buffer to itself
            }
            
            // advance pointers
            remain_size -= des_transfer_size;
            cur_des_virtual++;
        }
    }

    ///////////////////////////////////////////////////////////////////
    // return results
    ///////////////////////////////////////////////////////////////////
    *buffer_id  = buffer_info->frame_id;
    *pY         = buffer_info->pY;
    *pCb        = buffer_info->pCb;
    *pCr        = buffer_info->pCr;
    
    return XLLP_STATUS_SUCCESS;
}

// Submit a buffer into the capture queue.
XLLP_STATUS_T XllpCameraSubmitBuffer(
            P_XLLP_Camera_Context_T  camera_context,
                                int  buffer_id,
                                int  buffer_type)
{
    XLLP_Camera_Frame_Buffer_Info_T  *buffer_info;
    XLLP_Camera_Frame_Buffer_Queue_T *queue;
    unsigned int                      fifo0_transfer_size;
    unsigned int                      fifo1_transfer_size;
    unsigned int                      fifo2_transfer_size;
    unsigned int                      dma_need_reload = 0;

    // get the specified capture queue and buffer and other info
    queue = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
            &(camera_context->video_capture_buffer_queue):
            &(camera_context->still_capture_buffer_queue);

    buffer_info = &(camera_context->master_frame_buffer_list[buffer_id]);

    fifo0_transfer_size = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                       camera_context->video_fifo0_transfer_size:
                       camera_context->still_fifo0_transfer_size;

    fifo1_transfer_size = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                       camera_context->video_fifo1_transfer_size:
                       camera_context->still_fifo1_transfer_size;

    fifo2_transfer_size = (buffer_type == XLLP_VIDEO_CAPTURE_BUFFER)?
                       camera_context->video_fifo2_transfer_size:
                       camera_context->still_fifo2_transfer_size;

                       
    // attach the submitted buffer's DMA chain to the queue
    if ((queue->tail != NULL) && (queue->tail != buffer_info)){
        if (fifo0_transfer_size){        
            queue->tail->ch0_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX]->ddadr = 
                buffer_info->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];
            
            queue->tail->ch0_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX]->ddadr =
                buffer_info->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];
        }
        
        if (fifo1_transfer_size){
            queue->tail->ch1_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX]->ddadr = 
                buffer_info->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];
            
            queue->tail->ch1_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX]->ddadr =
                buffer_info->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];
        }

        if (fifo2_transfer_size){
            queue->tail->ch2_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX]->ddadr = 
                buffer_info->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];
            
            queue->tail->ch2_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX]->ddadr =
                buffer_info->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX];
        }
    }
    else if ((camera_context->dma_running && queue->tail == buffer_info) &&
             ((buffer_type == XLLP_VIDEO_CAPTURE_BUFFER && camera_context->capture_mode == XLLP_CAMERA_MODE_VIDEO) ||
              (buffer_type == XLLP_STILL_CAPTURE_BUFFER && camera_context->capture_mode == XLLP_CAMERA_MODE_STILL))) {

        dma_need_reload = 1;
    }

    // update the head of the queue, if necessary
    if (queue->head == NULL){
        queue->head = buffer_info;
    }

    // update the tail of the queue
    if (queue->tail == NULL){
        queue->tail = buffer_info;
    }else{
        queue->tail->next_buffer = buffer_info;
        queue->tail = buffer_info;
    }
    queue->tail->next_buffer = NULL;

    // ensure the tail's phantom buffer loop to itself
    if (fifo0_transfer_size){
        queue->tail->ch0_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX]->ddadr = 
            buffer_info->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
        
        queue->tail->ch0_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX]->ddadr =
            buffer_info->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
    }
    
    if (fifo1_transfer_size){
        queue->tail->ch1_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX]->ddadr = 
            buffer_info->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
        
        queue->tail->ch1_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX]->ddadr =
            buffer_info->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
    }

    if (fifo2_transfer_size){
        queue->tail->ch2_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX]->ddadr = 
            buffer_info->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
        
        queue->tail->ch2_dma_desc_vir_addr[XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX]->ddadr =
            buffer_info->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX];
    }
    
    // tag the buffer as clean
    *((unsigned int*)buffer_info->pY + fifo0_transfer_size/4 - 2) = XLLP_CAMERA_CLEAN_BUFFER_IDENTIFIER;
    *((unsigned int*)buffer_info->pY + fifo0_transfer_size/4 - 1) = XLLP_CAMERA_CLEAN_BUFFER_IDENTIFIER;

    // reload the dma descriptor if needed
    if (dma_need_reload){

        if (fifo0_transfer_size){
#if CI_DMA_BRANCH_IS_OK            
            XllpCIDMASetBranch(camera_context->ci_reg_base,
                               queue->head->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX],
                               XLLP_TRUE,
                               XLLP_TRUE,
                               XLLP_CI_DMA_CHANNEL_0);
#else
            XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                                    queue->head->ch0_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX],
                                    XLLP_CI_DMA_CHANNEL_0);
#endif
        }

        if (fifo1_transfer_size){
#if CI_DMA_BRANCH_IS_OK            
            XllpCIDMASetBranch(camera_context->ci_reg_base,
                               queue->head->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX], 
                               XLLP_TRUE,
                               XLLP_TRUE,
                               XLLP_CI_DMA_CHANNEL_1);
#else
            XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                                    queue->head->ch1_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX],
                                    XLLP_CI_DMA_CHANNEL_1);
#endif
        }

        if (fifo2_transfer_size){
#if CI_DMA_BRANCH_IS_OK            
            XllpCIDMASetBranch(camera_context->ci_reg_base,
                               queue->head->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX], 
                               XLLP_TRUE,
                               XLLP_TRUE,
                               XLLP_CI_DMA_CHANNEL_2);
#else
            XllpCIDMALoadDescriptor(camera_context->ci_reg_base,
                                    queue->head->ch2_dma_desc_phy_addr[XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX],
                                    XLLP_CI_DMA_CHANNEL_2);
#endif
        }
    }
        
    return XLLP_STATUS_SUCCESS;
}

// Get the buffer filled with valid frame data.
XLLP_STATUS_T XllpCameraGetFilledBuffer(
                P_XLLP_Camera_Context_T  camera_context,
                                    int *buffer_id)
{
    XLLP_Camera_Frame_Buffer_Queue_T *queue;
    unsigned int                      fifo0_transfer_size;

    // get the specified capture queue and other info
    queue = (camera_context->capture_mode == XLLP_CAMERA_MODE_VIDEO)?
           &(camera_context->video_capture_buffer_queue):
           &(camera_context->still_capture_buffer_queue);

    fifo0_transfer_size =  camera_context->fifo0_transfer_size;
    
    // check if there are any buffers filled with valid image data
    if (queue->head == NULL){
        return XLLP_STATUS_FAILURE;
    }else{
        if ((*((unsigned int*)queue->head->pY + fifo0_transfer_size/4 - 2) != XLLP_CAMERA_CLEAN_BUFFER_IDENTIFIER) &&
            (*((unsigned int*)queue->head->pY + fifo0_transfer_size/4 - 1) != XLLP_CAMERA_CLEAN_BUFFER_IDENTIFIER)){

            *buffer_id  = queue->head->frame_id;            
            queue->head = queue->head->next_buffer;
            
            return XLLP_STATUS_SUCCESS;
        }

        return XLLP_STATUS_FAILURE;
    }
}
/***********************************************************************
 *
 * Frame rate APIs
 *
 ***********************************************************************/
// Set desired frame rate
void XllpCameraSetCaptureFrameRate( P_XLLP_Camera_Context_T camera_context )
{
    XllpCISetFrameRate(camera_context->ci_reg_base, (XLLP_CI_FRAME_CAPTURE_RATE)camera_context->frame_rate);
    return;
} 

// return current setting
void XllpCameraGetCaptureFrameRate( P_XLLP_Camera_Context_T camera_context )
{
    camera_context->frame_rate = XllpCIGetFrameRate(camera_context->ci_reg_base);
    return;
} 


/***********************************************************************
 *
 * Interrupt APIs
 *
 ***********************************************************************/
// set interrupt mask 
void XllpCameraSetInterruptMask( P_XLLP_Camera_Context_T camera_context, unsigned int mask )
{
    // set CI interrupt
    XllpCISetInterruptMask( camera_context->ci_reg_base, mask );
}
 
// get interrupt mask 
unsigned int XllpCameraGetInterruptMask( P_XLLP_Camera_Context_T camera_context )
{
    // get CI mask
    return ( XllpCIGetInterruptMask( camera_context->ci_reg_base ));
} 

// get interrupt status
unsigned int XllpCameraGetInterruptStatus( P_XLLP_Camera_Context_T camera_context )
{
    return XllpCIGetInterruptStatus( camera_context->ci_reg_base );   
}

// clear interrupt status
void XllpCameraClearInterruptStatus( P_XLLP_Camera_Context_T camera_context, unsigned int status )
{
    XllpCIClearInterruptStatus( camera_context->ci_reg_base, status );   
}

/***********************************************************************
 *
 * Sensor Control APIs
 *
 ***********************************************************************/
XLLP_STATUS_T XllpCameraRead8Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T reg_addr, XLLP_UINT8_T *reg_val )
{
    XLLP_STATUS_T status;
    
    if (camera_context->camera_functions->read_8bit) {
        status = camera_context->camera_functions->read_8bit(camera_context, reg_addr, reg_val);
        return status;
    }
    return XLLP_STATUS_FAILURE;
}   

// CMOS sensor 8 bit register write
XLLP_STATUS_T XllpCameraWrite8Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T reg_addr, XLLP_UINT8_T reg_val )
{
    XLLP_STATUS_T status;
    
    if (camera_context->camera_functions->write_8bit) {
        status = camera_context->camera_functions->write_8bit(camera_context, reg_addr, reg_val);
        return status;
    }
    return XLLP_STATUS_FAILURE;
}

// CMOS sensor 16-bit register read
XLLP_STATUS_T XllpCameraRead16Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT16_T reg_addr, XLLP_UINT16_T *reg_val )
{
    XLLP_STATUS_T status;
    
    if (camera_context->camera_functions->read_16bit) {
        status = camera_context->camera_functions->read_16bit(camera_context, reg_addr, reg_val);
        return status;
    }
    return XLLP_STATUS_FAILURE;
}   

// CMOS sensor 16 bit register write
XLLP_STATUS_T XllpCameraWrite16Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT16_T reg_addr, XLLP_UINT16_T reg_val )
{
    XLLP_STATUS_T status;
    
    if (camera_context->camera_functions->write_16bit) {
        status = camera_context->camera_functions->write_16bit(camera_context, reg_addr, reg_val);
        return status;
    }
    return XLLP_STATUS_FAILURE;
}

// CMOS sensor 32 bit register read
XLLP_STATUS_T XllpCameraRead32Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT32_T reg_addr, XLLP_UINT32_T *reg_val )
{
    XLLP_STATUS_T status;
    
    if (camera_context->camera_functions->read_32bit) {
        status = camera_context->camera_functions->read_32bit(camera_context, reg_addr, reg_val);
        return status;
    }
    return XLLP_STATUS_FAILURE;
}   

// CMOS sensor 32 bit register write
XLLP_STATUS_T XllpCameraWrite32Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT32_T reg_addr, XLLP_UINT32_T reg_val )
{
    XLLP_STATUS_T status;
    
    if (camera_context->camera_functions->write_32bit) {
        status = camera_context->camera_functions->write_32bit(camera_context, reg_addr, reg_val);
        return status;
    }
    return XLLP_STATUS_FAILURE;
}

// CMOS sensor Power Mode read
void XllpCameraGetPowerMode ( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T *power_mode )
{
    *power_mode = camera_context->sensor_status.power_mode;
}

// CMOS sensor Power Mode write
XLLP_STATUS_T XllpCameraSetPowerMode ( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T power_mode )
{
    XLLP_STATUS_T status;
    
    status = XLLP_STATUS_SUCCESS;
    
    if (camera_context->sensor_status.power_mode != power_mode)    
        status = camera_context->camera_functions->set_power_mode(camera_context, power_mode);
    
    return status;        
}

// CMOS sensor Capability read
unsigned int XllpCameraGetCaps ( P_XLLP_Camera_Context_T camera_context)
{
    return camera_context->sensor_status.caps;    
}

// CMOS sensor contrast value read
void XllpCameraGetContrastValue ( P_XLLP_Camera_Context_T camera_context, unsigned char *mode, unsigned int *value)
{
    *mode = camera_context->sensor_status.contrast_mode;
    if (*mode == XLLP_SENSOR_MANUAL_CONTRAST)
        *value = camera_context->sensor_status.contrast_value;
    else 
        *value = 0;            
}

// CMOS sensor contrast value write
XLLP_STATUS_T XllpCameraSetContrastValue ( P_XLLP_Camera_Context_T camera_context, unsigned char mode, unsigned int value)
{    
    camera_context->sensor_status.contrast_mode = mode;
    camera_context->sensor_status.contrast_value = value;
    return camera_context->camera_functions->set_contrast(camera_context, mode, value);
}

// CMOS sensor white balance value read
void XllpCameraGetWhiteBalanceValue (P_XLLP_Camera_Context_T camera_context, unsigned char *mode, unsigned int *value)
{
    *mode = camera_context->sensor_status.whitebalance_mode;
    if (*mode == XLLP_SENSOR_MANUAL_WHITEBALANCE)
        *value = camera_context->sensor_status.whitebalance_value;
    else 
        *value = 0;
}

// CMOS sensor white balance value write
XLLP_STATUS_T XllpCameraSetWhiteBalanceValue (P_XLLP_Camera_Context_T camera_context, unsigned char mode, unsigned int value)
{
    camera_context->sensor_status.whitebalance_mode = mode;
    camera_context->sensor_status.whitebalance_value = value;    
    return camera_context->camera_functions->set_whitebalance(camera_context, mode, value);
}

// CMOS sensor exposure value read
void XllpCameraGetExposureValue (P_XLLP_Camera_Context_T camera_context, unsigned char *mode, unsigned int *value)
{
    *mode = camera_context->sensor_status.exposure_mode;
    if (*mode == XLLP_SENSOR_MANUAL_EXPOSURE)
        *value = camera_context->sensor_status.exposure_value;
    else 
        *value = 0;
}

// CMOS sensor exposure value write
XLLP_STATUS_T XllpCameraSetExposureValue (P_XLLP_Camera_Context_T camera_context, unsigned char mode, unsigned int value)
{
    camera_context->sensor_status.exposure_mode = mode;
    camera_context->sensor_status.exposure_value = value;    
    return camera_context->camera_functions->set_exposure(camera_context, mode, value);
}

// CMOS sensor zoom value read
void XllpCameraGetZoomValue (P_XLLP_Camera_Context_T camera_context, unsigned int *value)
{
    *value = camera_context->sensor_status.zoom_value;
}

// CMOS sensor zoom value write
XLLP_STATUS_T XllpCameraSetZoomValue (P_XLLP_Camera_Context_T camera_context, unsigned int value)
{
    camera_context->sensor_status.zoom_value = value;
    return camera_context->camera_functions->set_zoom(camera_context, value);
}

// get Histogram Info
XLLP_STATUS_T XllpCameraGetHistogramInfo(
            P_XLLP_Camera_Context_T     camera_context,
            unsigned int                color_type,
            unsigned int               *histogram_size,
            unsigned int               *histogram_sum)
{
    unsigned int mux_select;

    mux_select = (camera_context->capture_input_format == XLLP_CAMERA_IMAGE_FORMAT_RAW10)?XLLP_CI_HSU_MUX_1_TO_9:
                 (camera_context->capture_input_format == XLLP_CAMERA_IMAGE_FORMAT_RAW9) ?XLLP_CI_HSU_MUX_0_TO_8:
                                                                                          XLLP_CI_HSU_MUX_0_TO_7;
    camera_context->fifo3_transfer_size = (mux_select == XLLP_CI_HSU_MUX_0_TO_7)?512:1024;

    *histogram_size = camera_context->fifo3_transfer_size;
    return XllpCIHSUGetHistgram(
                                camera_context->ci_reg_base,
                                camera_context->ost_reg_base,
       (XLLP_CI_HSU_COLOR_TYPE) color_type,
     (XLLP_CI_HSU_MUX_SEL_TYPE) mux_select,
                (unsigned int*) camera_context->histogram_lut_buffer_virtual,
                                camera_context->histogram_lut_buffer_physical,
                (unsigned int*) camera_context->histogram_lut_dma_descriptors_virtual,
                                camera_context->histogram_lut_dma_descriptors_physical,
                                camera_context->fifo3_transfer_size,
                                histogram_sum);
}

// synchronize the CI with LCD
XLLP_STATUS_T XllpCameraSyncToLCD(
        P_XLLP_Camera_Context_T    camera_context,
        unsigned int               lcd_layer_type)
{
    return XllpCISyncToLCD(camera_context->ci_reg_base, (XLLP_CI_SYNC_TO_LCD_TYPE)lcd_layer_type);
}


/***********************************************************************
 *
 * sleep/wakeup APIs
 *
 ***********************************************************************/
// sleep Xllp camera
XLLP_STATUS_T XllpCameraSuspend(P_XLLP_Camera_Context_T    camera_context)
{
    // stop capture at first, if necessary
    if(camera_context->dma_running) {
	    if (camera_context->capture_mode == XLLP_CAMERA_MODE_VIDEO){
	        XllpCameraStopVideoCapture(camera_context);
	    }else if (camera_context->capture_mode == XLLP_CAMERA_MODE_STILL){
		    XllpCameraStopVideoCapture(camera_context);;
		}else {
	        return XLLP_STATUS_SUCCESS;
		}
		camera_context->dma_running = XLLP_TRUE;
    }

    // sleep sensor
    camera_context->camera_functions->sleep(camera_context);

    // deinit CI
    XllpCIDeInit(camera_context->ci_reg_base, camera_context->clk_reg_base);
    
    // set two gpio pin output as HI to power off two sensors.
    XllpGpioSetOutputLevel((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_HI_PWDN, XLLP_HI);
    XllpGpioSetOutputLevel((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_LO_PWDN, XLLP_HI);

    return XLLP_STATUS_SUCCESS;
}

// wake Xllp camera
XLLP_STATUS_T XllpCameraResume(P_XLLP_Camera_Context_T    camera_context)
{
    XLLP_STATUS_T status;

    // init MFP. FIX ME: need to check later if this is necessary!!
   status =  XllpMfpResourceManager_List(
                camera_context->mfp_rm_db, 
                xllp_ci_mfp_list, 
                XLLP_MFP_RM_ID_XLLP_CIF, 
                XLLP_SET);

    if ( XLLP_STATUS_SUCCESS != status) {
         return status ;    
    }

	// Lock MFPR block until finished with all MFPRs.
    // Don't forget to unlock for early returns, too.
    XllpLock(XLLP_RESOURCE_MFPR);
		
    status = XllpMfpSetAfDs_List(
                (P_XLLP_VUINT32_T)camera_context->mfp_reg_base, 
                xllp_ci_mfp_list, 
                xllp_ci_af_list, 
                xllp_ci_ds_list);

    if ( XLLP_STATUS_SUCCESS != status) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
        return status ;    
    }

    status = XllpMfpConfigureLpmOutputLevel_List (
                (P_XLLP_VUINT32_T)camera_context->mfp_reg_base,
                xllp_ci_mfp_list,
                xllp_ci_lpm_output_list);

    if ( XLLP_STATUS_SUCCESS != status) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
        return status ;    
    }

    status = XllpMfpActivatePullUpDown_List (
                (P_XLLP_VUINT32_T)camera_context->mfp_reg_base,
                xllp_ci_mfp_list,
                XLLP_OFF);

    if ( XLLP_STATUS_SUCCESS != status) {
        XllpUnlock(XLLP_RESOURCE_MFPR);
        return status ;    
    }
    
    // set two gpio pin direction as output to control the power of two sensors.
    XllpGpioSetDirection((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_HI_PWDN, XLLP_GPIO_DIRECTION_OUT);
    XllpGpioSetDirection((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_LO_PWDN, XLLP_GPIO_DIRECTION_OUT);

    // set two gpio pin output as HI to power off two sensors.
    XllpGpioSetOutputLevel((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_HI_PWDN, XLLP_HI);
    XllpGpioSetOutputLevel((P_XLLP_GPIO_T)camera_context->gpio_reg_base, XLLP_GPIO_CAMERA_LO_PWDN, XLLP_HI);

	XllpUnlock(XLLP_RESOURCE_MFPR);

    // capture interface init
    XllpCIInit(camera_context->ci_reg_base, camera_context->clk_reg_base);

    XllpCameraSetCaptureFrameRate(camera_context);
        
    // wake up sensor 
    status = camera_context->camera_functions->wakeup(camera_context);
    if (status != XLLP_STATUS_SUCCESS){
        goto camera_init_err;
    }

    // recovered to the pre-sleep status
    XllpCameraSetCaptureFormat(camera_context);
    if(camera_context->dma_running) {
	    if (camera_context->capture_mode == XLLP_CAMERA_MODE_VIDEO){
	        XllpCameraStartVideoCapture(camera_context);
	    } else if (camera_context->capture_mode == XLLP_CAMERA_MODE_STILL){
	        XllpCameraCaptureStillImage(camera_context);
	    }
    }

	
    
    return XLLP_STATUS_SUCCESS;

camera_init_err:
    XllpCameraDeInit(camera_context);
    
    return XLLP_STATUS_FAILURE; 
}

