/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**  FILENAME:       xllp_irda_plat.c
**
**  PURPOSE:        Definitions the low-level primitives that access the
**		            IrDA(SIR and FIR) registers
**
**
******************************************************************************/

#include <xllp_defs.h>
#include <xllp_serialization.h>
#include <xllp_mfp.h>
#include <xllp_gpio.h>

#include "xllp_irda_plat.h"


#define	XLLP_GPIO13	13

#ifdef	XLLP_MFP_SIR_TXD_OFFSET

// MFP definition for STUART
static XLLP_UINT32_T xllp_irda_mfp_list[] = {
	XLLP_MFP_SIR_TXD_OFFSET,
	XLLP_MFP_SIR_RXD_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER
};

static XLLP_MFP_ALT_FN_T xllp_irda_af_list[] = {
	XLLP_MFP_SIR_TXD_AF,
	XLLP_MFP_SIR_RXD_AF,
};

static XLLP_MFP_DRIVE_STRENGTH_T xllp_irda_ds_list[] = {
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
};

static XLLP_MFP_LPM_OUTPUT_T xllp_irda_lpm_output_list[] = {
	XLLP_MFP_LPMO_FLOAT,
	XLLP_MFP_LPMO_FLOAT
};

#endif

#ifdef	XLLP_MFP_IR_SHDN_N_GPIO_OFFSET
static XLLP_UINT32_T xllp_irda_shdn_mfp_list[] = {
	XLLP_MFP_IR_SHDN_N_GPIO_OFFSET,
	XLLP_MFP_CIR_ON_PWM_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER,
};

static XLLP_MFP_ALT_FN_T xllp_irda_shdn_af_list[] = {
	XLLP_MFP_IR_SHDN_N_GPIO_AF,
	XLLP_MFP_PIN_GPIO13_AF_GPIO_13,
};

static XLLP_MFP_DRIVE_STRENGTH_T xllp_irda_shdn_ds_list[] = {
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS,
};

#endif

/******************************************************************************
 * 	XllpIrDATransceiverOn
 * 	Turn on/off IrDA Transceiver LED
 * 	
 * 	Note: this functions just support SIR device.
 *	      The param Handle just need pMfpRegBase fields filled with
 *	      valid value.
 *
 * 	Param:
 * 		Handle: Pointer to the XLLP_IRDA_HANDLE_T
 * 		On: 	
 * 			XLLP_TRUE:  Turn on IrDA Transceiver LED
 * 			XLLP_FLASE: Turn off IrDA Transceiver LED
 * 	Return:
 * 		the status of operation
 ******************************************************************************
 */
XLLP_STATUS_T XllpIrDATransceiverOn(P_XLLP_IRDA_HANDLE_T Handle, XLLP_BOOL_T On)
{
	XLLP_STATUS_T	status = XLLP_STATUS_SUCCESS;

	// UART, MFP register, MFP resource DB ID.	
	P_XLLP_UART_T	pUARTRegs = Handle->pUARTHandle->pUARTRegs;
	P_XLLP_GPIO_T	pGPIORegs = Handle->pUARTHandle->pGPIORegs;
	P_XLLP_VUINT32_T	pMfpRegBase = Handle->pUARTHandle->pMfpRegBase;
	P_XLLP_MFP_RM_DB_ID_T	pMfpRmDb = Handle->pUARTHandle->pMfpRmDb;
	XLLP_MFP_RM_DB_ID_T	MfpRmDBID = Handle->pUARTHandle->MfpRmDBID;

#ifndef XLLP_MFP_IR_SHDN_N_GPIO_OFFSET
	return XLLP_STATUS_WRONG_PARAMETER;
#endif

	// This fucntion just support SIR device.
	if (XLLP_FIR == Handle->IrDAType)
		return XLLP_STATUS_WRONG_PARAMETER;
	
	// Make sure valid param	
	if ((NULL == pUARTRegs) || (NULL == pMfpRegBase) || (NULL == pGPIORegs))
		return XLLP_STATUS_WRONG_PARAMETER;
	
	//Get Resource Manager
	status = XllpMfpResourceManager_List(pMfpRmDb, 
			&xllp_irda_shdn_mfp_list[0],
			MfpRmDBID, XLLP_SET);	
	if (XLLP_STATUS_SUCCESS != status)
		return status;

	status = XllpMfpSetAfDs_List(pMfpRegBase,
			&xllp_irda_shdn_mfp_list[0],
			&xllp_irda_shdn_af_list[0],
			&xllp_irda_shdn_ds_list[0]);
	if (XLLP_STATUS_SUCCESS != status)
		return status;


	status = XllpGpioSetDirection(pGPIORegs, XLLP_GPIO13,
			XLLP_GPIO_DIRECTION_OUT);	
	if (XLLP_STATUS_SUCCESS != status)
		return status;

	//Note: Must disable IrDA unit remote control function
	//	or the IrDA diodes will burn out.
	status = XllpGpioSetOutputLevel(pGPIORegs, 
					XLLP_GPIO13, XLLP_LO);
	if (XLLP_STATUS_SUCCESS != status)
		return status;

	status = XllpGpioSetDirection(pGPIORegs, XLLP_GPIO_IR_SHDN_N, 
			XLLP_GPIO_DIRECTION_OUT);
	if (XLLP_STATUS_SUCCESS != status)
		return status;

	if (XLLP_TRUE == On) {		// turn on transceiver LED

		status = XllpGpioSetOutputLevel(pGPIORegs,
						XLLP_GPIO_IR_SHDN_N, XLLP_LO);
	} else { // turn off transceiver LED, just set output level to low.
		status = XllpGpioSetOutputLevel(pGPIORegs,
						XLLP_GPIO_IR_SHDN_N, XLLP_HI);
	}	// if (XLLP_TRUE == On)

	return status;
}	// end of XllpIrDATransceiverOn

/******************************************************************************
 * 	XllpIrDAMode
 * 	Set IrDA mode is FIR or SIR
 * 	
 * 	Note: this functions just support SIR device now.
 *	      The param Handle just need pMfpRegBase/MfpRmDBID 
 *	      fields filled with valid value.
 *
 * 	Param:
 * 		Handle: Pointer to the XLLP_IRDA_HANDLE_T
 * 		type: 	
 * 			XLLP_FIR:  Set IrDA mode FIR 
 * 			XLLP_SIR:  Set IrDA mode SIR
 * 	Return:
 * 		the status of operation
 ******************************************************************************
 */
XLLP_STATUS_T XllpIrDAMode(P_XLLP_IRDA_HANDLE_T Handle, XLLP_IRDATYPE_T type)
{
	// UART, MFP register, MFP resource DB ID.	
	P_XLLP_VUINT32_T	pMfpRegBase = Handle->pUARTHandle->pMfpRegBase;
	P_XLLP_MFP_RM_DB_ID_T	pMfpRmDb = Handle->pUARTHandle->pMfpRmDb;
	XLLP_MFP_RM_DB_ID_T	MfpRmDBID = Handle->pUARTHandle->MfpRmDBID;

	XLLP_STATUS_T	status = XLLP_STATUS_SUCCESS;

#ifndef	XLLP_MFP_SIR_TXD_OFFSET
	return XLLP_STATUS_WRONG_PARAMETER;
#endif

	if ( XLLP_FIR == type) {	// if type is FIR 
		// don't support FIR now
		return XLLP_STATUS_WRONG_PARAMETER;
	} else {		// if type is SIR
		status = XllpMfpResourceManager_List(pMfpRmDb, 
			&xllp_irda_mfp_list[0], 
			MfpRmDBID, XLLP_SET); 
		if (XLLP_STATUS_SUCCESS != status)
			return status;
        
		status = XllpMfpSetAfDs_List(pMfpRegBase,
			&xllp_irda_mfp_list[0],
			&xllp_irda_af_list[0],
			&xllp_irda_ds_list[0]);
		if (XLLP_STATUS_SUCCESS != status)
			return status;
        
		status = XllpMfpConfigureLpmOutputLevel_List(pMfpRegBase,
				&xllp_irda_mfp_list[0],
				&xllp_irda_lpm_output_list[0]);
		if (XLLP_STATUS_SUCCESS != status)
			return status;
	}

	return status;	
}

