/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/****************************************************************************
**   xllp_keypad.c
**
**   Valid for Zylonite Keypad
**
*****************************************************************************/

#include	"xllp_keypad.h"
#include	"xllp_keypad_proc.h"
#include "xllp_mfp.h"
#include "xllp_defs.h"
#include "xllp_serialization.h"

static XLLP_UINT32_T xllp_keypad_mfp_list []={
	XLLP_MFP_KP_DKIN_0_OFFSET,
	XLLP_MFP_KP_DKIN_1_OFFSET,
	XLLP_MFP_KP_MKIN_0_OFFSET,
	XLLP_MFP_KP_MKIN_1_OFFSET,
	XLLP_MFP_KP_MKIN_2_OFFSET,
	XLLP_MFP_KP_MKIN_3_OFFSET,
	XLLP_MFP_KP_MKIN_4_OFFSET,
	XLLP_MFP_KP_MKIN_5_OFFSET,
	XLLP_MFP_KP_MKIN_6_OFFSET,
	XLLP_MFP_KP_MKIN_7_OFFSET,
	XLLP_MFP_KP_MKOUT_0_OFFSET,
	XLLP_MFP_KP_MKOUT_1_OFFSET,
	XLLP_MFP_KP_MKOUT_2_OFFSET,
	XLLP_MFP_KP_MKOUT_3_OFFSET,
	XLLP_MFP_KP_MKOUT_4_OFFSET,
	XLLP_MFP_KP_MKOUT_5_OFFSET,
	XLLP_MFP_KP_MKOUT_6_OFFSET,
	XLLP_MFP_KP_MKOUT_7_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER
};/*end of xllp_keypad_mfp_listtypedef struct*/

static XLLP_MFP_ALT_FN_T xllp_keypad_af_list []={
	XLLP_MFP_KP_DKIN_0_AF,    /*DKIN_0*/
	XLLP_MFP_KP_DKIN_1_AF,    /*DKIN_1*/
	XLLP_MFP_KP_MKIN_0_AF,    /*MKIN_0*/											
	XLLP_MFP_KP_MKIN_1_AF,    /*MKIN_1*/
	XLLP_MFP_KP_MKIN_2_AF,    /*MKIN_2*/
	XLLP_MFP_KP_MKIN_3_AF,    /*MKIN_3*/
	XLLP_MFP_KP_MKIN_4_AF,    /*MKIN_4*/
	XLLP_MFP_KP_MKIN_5_AF,    /*MKIN_5*/
	XLLP_MFP_KP_MKIN_6_AF,    /*MKIN_6*/
	XLLP_MFP_KP_MKIN_7_AF,    /*MKIN_7*/
	XLLP_MFP_KP_MKOUT_0_AF,   /*MKOUT_0*/
	XLLP_MFP_KP_MKOUT_1_AF,   /*MKOUT_1*/
	XLLP_MFP_KP_MKOUT_2_AF,   /*MKOUT_2*/
	XLLP_MFP_KP_MKOUT_3_AF,   /*MKOUT_3*/
	XLLP_MFP_KP_MKOUT_4_AF,   /*MKOUT_4*/
	XLLP_MFP_KP_MKOUT_5_AF,   /*MKOUT_5*/
	XLLP_MFP_KP_MKOUT_6_AF,   /*MKOUT_6*/
	XLLP_MFP_KP_MKOUT_7_AF,   /*MKOUT_7*/
};/*end of xllp_keypad_af_list*/

static XLLP_MFP_DRIVE_STRENGTH_T xllp_keypad_ds_list []={
	/*
	 FIX ME
	 Note: The keypad is not in the list of devices that 
	           may require special drive strengths
	 */
	XLLP_MFP_DEFAULT_DS,   /*DKIN_0*/
	XLLP_MFP_DEFAULT_DS,   /*DKIN_1*/
	XLLP_MFP_DEFAULT_DS,   /*MKIN_0*/
	XLLP_MFP_DEFAULT_DS,   /*MKIN_1*/
	XLLP_MFP_DEFAULT_DS,   /*MKIN_2*/
	XLLP_MFP_DEFAULT_DS,   /*MKIN_3*/
	XLLP_MFP_DEFAULT_DS,   /*MKIN_4*/
	XLLP_MFP_DEFAULT_DS,   /*MKIN_5*/
	XLLP_MFP_DEFAULT_DS,   /*MKIN_6*/
	XLLP_MFP_DEFAULT_DS,   /*MKIN_7*/
	XLLP_MFP_DEFAULT_DS,   /*MKOUT_0*/
	XLLP_MFP_DEFAULT_DS,   /*MKOUT_1*/
	XLLP_MFP_DEFAULT_DS,   /*MKOUT_2*/
	XLLP_MFP_DEFAULT_DS,   /*MKOUT_3*/
	XLLP_MFP_DEFAULT_DS,   /*MKOUT_4*/
	XLLP_MFP_DEFAULT_DS,   /*MKOUT_5*/
	XLLP_MFP_DEFAULT_DS,   /*MKOUT_6*/
	XLLP_MFP_DEFAULT_DS    /*MKOUT_7*/
};/*end of xllp_keypad-ds_list*/


/*
The Low Power Mode output table

The following table provides this Low Power Mode signal out behavior 
by the keypad MFPs:
DKINs:  Float ("Z").  (They are all pulled low by the platform 
unless driven by the scroll wheel.)
MKINs:  Float ("Z").  (They are all pulled low by the platform unless
driven by connection with an MKOUT due to key press.)
MKOUTs: Driven high output to provide edge triggering 
capability on the MKINs if a matrix key is pressed.
Note that this is desirable only if wakeup from keypress
is intended.  Otherwise, they should probably be pulled 
low so that no power would be wasted if a key is pressed
and a connection is made with an MKIN.
*/
static XLLP_MFP_LPM_OUTPUT_T xllp_keypad_lpm_output_list []={
	XLLP_MFP_LPMO_FLOAT,         /*DKIN_0*/
	XLLP_MFP_LPMO_FLOAT,         /*DKIN_1*/
	XLLP_MFP_LPMO_FLOAT,         /*MKIN_0*/
	XLLP_MFP_LPMO_FLOAT,         /*MKIN_1*/
	XLLP_MFP_LPMO_FLOAT,         /*MKIN_2*/
	XLLP_MFP_LPMO_FLOAT,         /*MKIN_3*/
	XLLP_MFP_LPMO_FLOAT,         /*MKIN_4*/
	XLLP_MFP_LPMO_FLOAT,         /*MKIN_5*/
	XLLP_MFP_LPMO_FLOAT,         /*MKIN_6*/
	XLLP_MFP_LPMO_FLOAT,         /*MKIN_7*/
	XLLP_MFP_LPMO_DRIVE_HIGH,   /* MKOUT_0*/
	XLLP_MFP_LPMO_DRIVE_HIGH,    /*MKOUT_1*/
	XLLP_MFP_LPMO_DRIVE_HIGH,    /*MKOUT_2*/
	XLLP_MFP_LPMO_DRIVE_HIGH,    /*MKOUT_3*/
	XLLP_MFP_LPMO_DRIVE_HIGH,    /*MKOUT_4*/
	XLLP_MFP_LPMO_DRIVE_HIGH,    /*MKOUT_5*/
	XLLP_MFP_LPMO_DRIVE_HIGH,   /*MKOUT_6*/
	XLLP_MFP_LPMO_DRIVE_HIGH     /*MKOUT_7*/
};/*end of xllp_keypad_lpm_output_list*/

/*
The following tables combine to provide this configuration and 
enabling of wake up from Low Power Modes:

DKINs and MKINs: 
Configured to trigger a first-stage LPM wake up signal on 
either a rising or falling edge on the input of their 
respective MFPs.
All are currently in a separate list of MFPs that can be used  
to both enable and disable clear triggering of a   
first-stage LPM wake up signal.

MKOUTs: Neither configured nor enabled for wake up triggers.
(Note that the Monahans cannot be enabled to wake from transitions
on MFPs assigned by Alternate Function to any MKOUT.)
*/
static XLLP_UINT32_T xllp_keypad_wakeup_pins_candidates_list []={
	XLLP_MFP_KP_DKIN_0_OFFSET,
	XLLP_MFP_KP_DKIN_1_OFFSET,
	XLLP_MFP_KP_MKIN_0_OFFSET,
	XLLP_MFP_KP_MKIN_1_OFFSET,
	XLLP_MFP_KP_MKIN_2_OFFSET,
	XLLP_MFP_KP_MKIN_3_OFFSET,
	XLLP_MFP_KP_MKIN_4_OFFSET,
	XLLP_MFP_KP_MKIN_5_OFFSET,
	XLLP_MFP_KP_MKIN_6_OFFSET,
	XLLP_MFP_KP_MKIN_7_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER
};/*end of xllp_keypad_wakeup_pins_candidates_list*/

static XLLP_MFP_EDGE_T xllp_keypad_configure_wakeup_edges_list []={
	XLLP_MFP_RISING_FALLING_EDGES,  /*DKIN_0*/
	XLLP_MFP_RISING_FALLING_EDGES,  /*DKIN_1*/
	XLLP_MFP_RISING_FALLING_EDGES,  /*MKIN_0*/
	XLLP_MFP_RISING_FALLING_EDGES,  /*MKIN_1*/
	XLLP_MFP_RISING_FALLING_EDGES,  /*MKIN_2*/
	XLLP_MFP_RISING_FALLING_EDGES,  /*MKIN_3*/
	XLLP_MFP_RISING_FALLING_EDGES,  /*MKIN_4*/
	XLLP_MFP_RISING_FALLING_EDGES,  /*MKIN_5*/
	XLLP_MFP_RISING_FALLING_EDGES,  /*MKIN_6*/
	XLLP_MFP_RISING_FALLING_EDGES   /*MKIN_7*/
};/*end of xllp_keypad_configure_wakeup_edges_list*/

/*
It is useful for the keypad module to maintain a separate list of actual 
wake up trigger MFPs in the event that the scroll wheel unit will not 
be powered during low power modes.  In that case, it would be undesirable 
to arm the wake up trigger for the DKINs in case a transition occurs on 
entry to the LPM as power is removed.
This list is used to disable/clear the wake up triggers on the included
MFPs and also to enable / arm the wake up triggers on them.
*/

static XLLP_UINT32_T xllp_keypad_wakeup_pins_list []={
	XLLP_MFP_KP_DKIN_0_OFFSET,
	XLLP_MFP_KP_DKIN_1_OFFSET,
	XLLP_MFP_KP_MKIN_0_OFFSET,
	XLLP_MFP_KP_MKIN_1_OFFSET,
	XLLP_MFP_KP_MKIN_2_OFFSET,
	XLLP_MFP_KP_MKIN_3_OFFSET,
	XLLP_MFP_KP_MKIN_4_OFFSET,
	XLLP_MFP_KP_MKIN_5_OFFSET,
	XLLP_MFP_KP_MKIN_6_OFFSET,
	XLLP_MFP_KP_MKIN_7_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER
};/*end of xllp_keypad_wakeup_pins_list*/


/*
For the sake of speed and simplicity, this table is configured so that the  
single-closure matrix scan codes are placed using an MK_INn (row) by 
MK_OUTm (column) format.  That will permit direct use of the KPAS  
register's lower 8 bits, which yield that indexing. 
Multiclosure (chorded) scan codes are processed in-line by the multipress parser
*/
/* 
 Note: Zylonite-specific
 Unassigned cells filled with 0xFE (XLLP_KP_SC_INVALID_KEY)
 */
#define XLLP_KP_MAX_SINGLE_SCAN_CODES 64

/*
we treat 
key CONTACT as XLLP_KP_SC_AUX1; 
key CALENDAR as XLLP_KP_SC_AUX2; 
key EMAIL as XLLP_KP_SC_AUX3
*/
static XLLP_UINT8_T xllpKpScanCodeLut[XLLP_KP_MAX_SINGLE_SCAN_CODES] = 
{
		/*IN0*/ 
/*OUT0 */XLLP_KP_SC_A,   
/*OUT1 */XLLP_KP_SC_B, 
/*OUT2 */XLLP_KP_SC_C, 
/*OUT3 */XLLP_KP_SC_AUX1, 
/*OUT4 */XLLP_KP_SC_AUX3,  
/*OUT5 */XLLP_KP_SC_D,    
/*OUT6 */XLLP_KP_SC_SOFT1,    
/*OUT7 */XLLP_KP_SC_N_UP,

		/*IN1*/ 
/*OUT0 */XLLP_KP_SC_E,   
/*OUT1 */XLLP_KP_SC_F, 
/*OUT2 */XLLP_KP_SC_G, 
/*OUT3 */XLLP_KP_SC_1,    
/*OUT4 */XLLP_KP_SC_3,     
/*OUT5 */XLLP_KP_SC_H,    
/*OUT6 */XLLP_KP_SC_SOFT2,
/*OUT7*/XLLP_KP_SC_N_DOWN,

		/*IN2*/ 
/*OUT0 */XLLP_KP_SC_I,   
/*OUT1 */XLLP_KP_SC_J, 
/*OUT2 */XLLP_KP_SC_K, 
/*OUT3 */XLLP_KP_SC_4,    
/*OUT4 */XLLP_KP_SC_6,     
/*OUT5 */XLLP_KP_SC_L,    
/*OUT6 */XLLP_KP_SC_HOME ,    
/*OUT7 */XLLP_KP_SC_N_LEFT ,  

		/*IN3*/ 
/*OUT0 */XLLP_KP_SC_M,   
/*OUT1 */XLLP_KP_SC_N, 
/*OUT2 */XLLP_KP_SC_O, 
/*OUT3 */XLLP_KP_SC_7,    
/*OUT4 */XLLP_KP_SC_9,     
/*OUT5 */XLLP_KP_SC_P ,   
/*OUT6 */XLLP_KP_SC_END ,     
/*OUT7 */XLLP_KP_SC_N_RIGHT,  

		/*IN4*/ 
/*OUT0 */XLLP_KP_SC_AUX2,
/*OUT1 */XLLP_KP_SC_2, 
/*OUT2 */XLLP_KP_SC_5, 
/*OUT3 */XLLP_KP_SC_8,    
/*OUT4*/XLLP_KP_SC_0,   
/*OUT5 */XLLP_KP_SC_SPACE,
/*OUT6 */XLLP_KP_SC_SCROLL_PUSH,
/*OUT7 */0xFE,  

		/*IN5*/ 
/*OUT0 */XLLP_KP_SC_Q,    
/*OUT1 */XLLP_KP_SC_R,
/*OUT2 */XLLP_KP_SC_S, 
/*OUT3 */XLLP_KP_SC_STAR, 
/*OUT4 */XLLP_KP_SC_POUND, 
/*OUT5 */XLLP_KP_SC_T,    
/*OUT6 */XLLP_KP_SC_SEND,     
/*OUT7 */XLLP_KP_SC_N_ACTION,  

		/*IN6*/ 
/*OUT0 */XLLP_KP_SC_U,   
/*OUT1 */XLLP_KP_SC_V, 
/*OUT2 */XLLP_KP_SC_W, 
/*OUT3 */XLLP_KP_SC_SHIFT,
/*OUT4 */XLLP_KP_SC_DELETE,
/*OUT5 */XLLP_KP_SC_X,    
/*OUT6 */XLLP_KP_SC_BACK,     
/*OUT7 */XLLP_KP_SC_VOL_UP,  

		/*IN7*/ 
/*OUT0 */0xFE,           
/*OUT1 */XLLP_KP_SC_Y,
/*OUT2 */XLLP_KP_SC_Z, 
/*OUT3 */0xFE,              
/*OUT4 */0xFE,       
/*OUT5 */0xFE,            
/*OUT6 */XLLP_KP_SC_RECORD,   
/*OUT7 */XLLP_KP_SC_VOL_DN,  
};  /*xllpKpScanCodeLutLower*/

#define     XLLP_KEYPAD_COLS_NUM      8 
#define     XLLP_KEYPAD_ROWS_NUM     8

/*-----------------------------------------------------------------------
 Function:	xllp_keypad_config_mfp
 Purpose:	This function configures the MFP for keypad
 Returns:	XLLPSTATUS_T
 Type No:	I/F flat 1
--------------------------------------------------------------------------*/
XLLP_STATUS_T xllp_keypad_config_mfp(
			P_XLLP_VUINT32_T pMfpRegBase,
			P_XLLP_MFP_RM_DB_ID_T pMfpRmDb)
{
	XLLP_STATUS_T retval = XLLP_STATUS_SUCCESS;
	XLLP_VUINT32_T mfpLocker;
	retval =  XllpMfpResourceManager_List(
			pMfpRmDb, 
			xllp_keypad_mfp_list, 
			XLLP_MFP_RM_ID_XLLP_KEYPAD , 
			XLLP_SET);

	if ( XLLP_STATUS_SUCCESS != retval) {
		return retval ;	
	}

	 /*
	 Lock MFPR block until finished with all MFPRs.
	 Don't forget to unlock for early returns, too.*/

	 XllpLock(XLLP_RESOURCE_MFPR);
	 /*Assign the Alternate Functions and Drive Strengths for each keypad MFP.*/
	retval = XllpMfpSetAfDs_List(
			pMfpRegBase, 
			xllp_keypad_mfp_list, 
			xllp_keypad_af_list, 
			xllp_keypad_ds_list);

	if ( XLLP_STATUS_SUCCESS != retval) {
		XllpUnlock(XLLP_RESOURCE_MFPR);
	return retval ;	
	}
	/*
	 Configure the pin output levels for low power modes.
	  The pulled level settings also apply to functional mode (non-LPM) if
	      activated for that use - but they are not so activated for the 
	      keypad.
	 */
	retval = XllpMfpConfigureLpmOutputLevel_List (
			pMfpRegBase,
			xllp_keypad_mfp_list,
			xllp_keypad_lpm_output_list);

	if ( XLLP_STATUS_SUCCESS != retval) {
		XllpUnlock(XLLP_RESOURCE_MFPR);
		return retval ;	
	}
	/*
	 Configure the LPM wake up edge triggers for each interesting keypad MFP.
	 This does not enable or disable the edge triggering, which must be done
	  elsewhere.
	*/
	retval = XllpMfpConfigureWakeup_List (
			pMfpRegBase,
			xllp_keypad_wakeup_pins_candidates_list,
			xllp_keypad_configure_wakeup_edges_list);

	if ( XLLP_STATUS_SUCCESS != retval) {
		XllpUnlock(XLLP_RESOURCE_MFPR);
		return retval ;	
	}

	/*
	 XLLI sets all pins to an activated pulled non-LPM level in order to
	  establish a low-power default setting.  That is helpful if a driver is
	  not loaded to control some pins, but here we are.  So deactivate
	  that now in order to avoid conflicts with the Keypad Interface device.
	*/
	retval = XllpMfpActivatePullUpDown_List (
			pMfpRegBase,
			xllp_keypad_mfp_list,
			XLLP_OFF);

	if ( XLLP_STATUS_SUCCESS != retval) {
		XllpUnlock(XLLP_RESOURCE_MFPR);
		return retval ;	
	}

	/*
	 Do locking read
	 Similarly to "Split Transactions", this read guarantees that the writes
	  to all MFPRs have taken effect in the pins before proceeding.
	 */
	retval = XllpMfpGetPinConfig (
			pMfpRegBase,
			xllp_keypad_wakeup_pins_candidates_list[0],
			&mfpLocker);

	if ( XLLP_STATUS_SUCCESS != retval) {
		XllpUnlock(XLLP_RESOURCE_MFPR);
		return retval ;	
	}
	XllpUnlock(XLLP_RESOURCE_MFPR);
	return XLLP_STATUS_SUCCESS;
}/*end xllp_keypad_config_mfp*/

/*-----------------------------------------------------------------------------------------
 Function: XllpKeyPadEnableLpmWakeup
Purpose:  This function enables or disables waking up from Low Power Modes based on 
              the configured MFP edge triggers for the keypad. 
	XLLP_STATUS_SUCCESS		     - All parameters were valid.
	XLLP_STATUS_WRONG_PARAMETER	- Null pointer received or parameter
		                          is out of range.
--------------------------------------------------------------------------------------------*/

XLLP_STATUS_T XllpKeyPadEnableLpmWakeup (
                                         P_XLLP_VUINT32_T  pMfpRegBase,
                                         XLLP_CONTROL_T	   enable)
{
	XLLP_STATUS_T retval;
	XLLP_VUINT32_T mfpLocker;

	retval = XllpMfpEnableWakeup_List ( 
			pMfpRegBase,
			xllp_keypad_wakeup_pins_list,
			enable);
	if ( XLLP_STATUS_SUCCESS != retval) {
		return retval ;	
	}

	// Do locking read
	// Similarly to "Split Transactions", this read guarantees that the writes
	//  to all MFPRs have taken effect in the pins before proceeding.
	retval = XllpMfpGetPinConfig (
				pMfpRegBase,
				xllp_keypad_wakeup_pins_list[0],
				&mfpLocker);

	return(retval);

}  // end of XllpKeyPadEnableLpmWakeup ()



/*----------------------------------------------------------------------------
 Function: XllpReadScanCode
 Purpose:   
 Returns:  
-----------------------------------------------------------------------------*/
XLLP_STATUS_T XllpReadScanCode(
	XLLP_KEYPAD_REGS	*v_pkeypad_regs,
	XLLP_UINT8_T		*pui8_data)
{

	XLLP_UINT32_T	kp_status;
	XLLP_UINT8_T	key = NO_KEY;

	#ifdef XLLP_DEBUG_PARAM_CHECK
		if ( NULL == v_pkeypad_regs||NULL == pui8_data)
			return XLLP_STATUS_WRONG_PARAMETER;
	#endif

	kp_status = v_pkeypad_regs->kpControlReg;

	/*
	Process matrix first to capture scroll 
	wheel press rather than interpreting it
	as a rotate, which can happen unintentionally.
	*/

	if(kp_status & MATRIX_INTR_BIT) {
		read_scan_code_automatically(v_pkeypad_regs,&key);
		if(key == NO_KEY) {
			*pui8_data = NO_KEY;
			return(XLLP_STATUS_FAILURE);
		}
		key = xllpKpScanCodeLut[key];
	}else if(kp_status & DIRECT_INTR_BIT ) {
		read_direct_keys(v_pkeypad_regs,&key);
	}
	*pui8_data = key;

	if(key == NO_KEY){
		return(XLLP_STATUS_FAILURE);
	}
	else{
		return(XLLP_STATUS_SUCCESS);
	}
}/*end function XllpReadScanCode*/

/*------------------------------------------------------------
 Function: XllpKeyPadConfigure
 Purpose:  This function configures the KeyPad Controller on Zylonite with the 
           appropriate settings.
 Returns:  success/failure.
--------------------------------------------------------------*/

XLLP_STATUS_T XllpKeyPadConfigure (
	XLLP_KEYPAD_REGS *	pKeyPadRegs,
	P_XLLP_VUINT32_T	pMfpRegBase,
	P_XLLP_MFP_RM_DB_ID_T	pMfpRmDb,
	XLLP_BOOL_T		useDefaultDebounces, 
	XLLP_UINT8_T		matrixDebounceMs, 
	XLLP_UINT8_T		directDebounceMs)

{
	XLLP_STATUS_T re;
	re = xllp_keypad_config_mfp(pMfpRegBase, pMfpRmDb);

	xllp_keypad_config_controller(
		pKeyPadRegs, 
		useDefaultDebounces, 
		matrixDebounceMs, 
		directDebounceMs,
		XLLP_KEYPAD_ROWS_NUM,
		XLLP_KEYPAD_COLS_NUM);	

	return(re);
}/*end function XllpKeyPadConfigure*/

