/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/*++

Module Name:  $Workfile: xllp_ov2630.c $

Abstract:  
     contains all primitive functions for OV2630

Notes:Only valid for processor code named Monahans.

--*/

#include "xllp_defs.h"
#include "xllp_serialization.h"
#include "xllp_clkmgr.h"
#include "xllp_ost.h"
#include "xllp_gpio_plat.h"
#include "xllp_gpio_proc.h"
#include "xllp_camera.h"
#include "xllp_ci.h"
#include "xllp_ov2630.h"
#include "xllp_ov2630_hw.h"

// LUT Table, TBD
static unsigned char lut_table[] = {
    // RED LUT
    0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c,
    0x20, 0x24, 0x28, 0x2c, 0x30, 0x34, 0x38, 0x3c,
    0x40, 0x44, 0x48, 0x4c, 0x50, 0x54, 0x58, 0x5c,
    0x60, 0x64, 0x68, 0x6c, 0x70, 0x74, 0x78, 0x7c,
    0x80, 0x84, 0x88, 0x8c, 0x90, 0x94, 0x98, 0x9c,
    0xa0, 0xa4, 0xa8, 0xac, 0xb0, 0xb4, 0xb8, 0xbc,
    0xc0, 0xc4, 0xc8, 0xcc, 0xd0, 0xd4, 0xd8, 0xdc,
    0xe0, 0xe4, 0xe8, 0xec, 0xf0, 0xf4, 0xf8, 0xfc,

    // BLUE LUT
    0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c,
    0x20, 0x24, 0x28, 0x2c, 0x30, 0x34, 0x38, 0x3c,
    0x40, 0x44, 0x48, 0x4c, 0x50, 0x54, 0x58, 0x5c,
    0x60, 0x64, 0x68, 0x6c, 0x70, 0x74, 0x78, 0x7c,
    0x80, 0x84, 0x88, 0x8c, 0x90, 0x94, 0x98, 0x9c,
    0xa0, 0xa4, 0xa8, 0xac, 0xb0, 0xb4, 0xb8, 0xbc,
    0xc0, 0xc4, 0xc8, 0xcc, 0xd0, 0xd4, 0xd8, 0xdc,
    0xe0, 0xe4, 0xe8, 0xec, 0xf0, 0xf4, 0xf8, 0xfc,

    // GREEN LUT
    0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c,
    0x20, 0x24, 0x28, 0x2c, 0x30, 0x34, 0x38, 0x3c,
    0x40, 0x44, 0x48, 0x4c, 0x50, 0x54, 0x58, 0x5c,
    0x60, 0x64, 0x68, 0x6c, 0x70, 0x74, 0x78, 0x7c,
    0x80, 0x84, 0x88, 0x8c, 0x90, 0x94, 0x98, 0x9c,
    0xa0, 0xa4, 0xa8, 0xac, 0xb0, 0xb4, 0xb8, 0xbc,
    0xc0, 0xc4, 0xc8, 0xcc, 0xd0, 0xd4, 0xd8, 0xdc,
    0xe0, 0xe4, 0xe8, 0xec, 0xf0, 0xf4, 0xf8, 0xfc,
};

// CMU conversion matrixes, they're sensor specific
static XLLP_CI_CMU_COE_MATRIX ov2630_cRGB_sRGB_COE = {
    RGB_FLOAT_TO_INT(1.780214),  RGB_FLOAT_TO_INT(-0.96883), RGB_FLOAT_TO_INT(0.188617),
    RGB_FLOAT_TO_INT(-0.7987) ,  RGB_FLOAT_TO_INT(1.790752), RGB_FLOAT_TO_INT(0.007949),
    RGB_FLOAT_TO_INT(-0.67645),  RGB_FLOAT_TO_INT(-1.60901), RGB_FLOAT_TO_INT(3.285467)
} ;// This is not provided by OmniVision yet

static XLLP_CI_CMU_COE_MATRIX ov2630_cRGB_YUV_COE = {
    YUV_FLOAT_TO_INT(0.257) , YUV_FLOAT_TO_INT(0.504) , YUV_FLOAT_TO_INT(0.098),
    YUV_FLOAT_TO_INT(-0.148), YUV_FLOAT_TO_INT(0.291), YUV_FLOAT_TO_INT(0.439),
    YUV_FLOAT_TO_INT(0.439) , YUV_FLOAT_TO_INT(0.368) , YUV_FLOAT_TO_INT(0.071)
};// This is not provided by OmniVision yet

// Bad pixel table, it's sensor specific, TBD.
#define END_MARKER -1
static int ov2630_badpixel_table[] = {
//  column,        row
//    0,           0,
//    1,           1,
//    1615,        1207,
    END_MARKER, END_MARKER
};

/***********************************************************************
 *
 * OV2630 Functions
 *
 ***********************************************************************/
XLLP_STATUS_T CameraFuncOV2630Init( P_XLLP_Camera_Context_T camera_context )
{
    XLLP_UINT8_T cm_rev, cm_pid;
    XLLP_UINT32_T timeout;
    int status;

    // provide informat about the capabilities of the sensor
    camera_context->sensor_status.caps |= XLLP_SENSOR_CAP_MANUAL_CONTRAST |
                                          XLLP_SENSOR_CAP_MANUAL_WHITEBALANCE |
                                          XLLP_SENSOR_CAP_MANUAL_EXPOSURE;

    // Configure CI according to OV2630's hardware        
    // master parallel with 8 data pins
    XllpCISetMode(camera_context->ci_reg_base, XLLP_CI_MODE_MP, XLLP_CI_DATA_WIDTH10);
    
    // enable pixel clock(sensor will provide pclock) and master clock = 26MHZ
    XllpCISetClock(camera_context->ci_reg_base, camera_context->clk_reg_base, XLLP_TRUE, XLLP_TRUE, 2600);

    // data sample on rising and h,vsync active high
    XllpCISetPolarity(camera_context->ci_reg_base, XLLP_FALSE, XLLP_FALSE, XLLP_FALSE);
    
    // fifo control
    XllpCISetFIFO(camera_context->ci_reg_base, 0, XLLP_CI_FIFO_THL_32, XLLP_TRUE, XLLP_TRUE);

    // set black level
    XllpCICGUSetBlackLevel(camera_context->ci_reg_base, 0);

    // CGU Mux Select
    XllpCICGUSetAddrMuxSelect(camera_context->ci_reg_base, XLLP_CI_CGU_MUX_2_TO_9);

    // OV2630 Power on sequence
    // Take out of Power down mode (GPIO_57), PWRDWN=1, NORMAL=0
    // Assert Reset
    // Delay
    // Remove reset
    // Delay
    OV2630PowerDown( camera_context->gpio_reg_base, camera_context->ost_reg_base, XLLP_CAMERA_POWER_FULL );
    OV2630Reset();
    XllpOstDelayMilliSeconds( (P_XLLP_OST_T)camera_context->ost_reg_base, 1 );
    
    // 2 Turn on M_CLK using xx MHz and wait for 150 ms.
    XllpCIEnable(camera_context->ci_reg_base/*, XLLP_TRUE*/);
    XllpOstDelayMilliSeconds((P_XLLP_OST_T)camera_context->ost_reg_base, 150);

    // read out version    
    timeout = 50;
    do
    {
        cm_pid = cm_rev = 0;
        status = OV2630VersionRevision(&cm_pid, &cm_rev);
    
        // Check to make sure we are working with an OV2630
        if( cm_pid != PIDH_OV2630 || cm_rev != PIDL_OV2630 )
        {
            OV2630PowerDown( camera_context->gpio_reg_base, camera_context->ost_reg_base, XLLP_CAMERA_POWER_OFF );
            OV2630PowerDown( camera_context->gpio_reg_base, camera_context->ost_reg_base, XLLP_CAMERA_POWER_FULL );
            OV2630Reset();
            XllpOstDelayMilliSeconds( (P_XLLP_OST_T)camera_context->ost_reg_base, 1 );
        }
        if (--timeout == 0)
            return XLLP_STATUS_FAILURE;
    } while( cm_pid != PIDH_OV2630 );
    
    // turn sensor output off
    OV2630ViewfinderOff();

    
    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T CameraFuncOV2630DeInit(  P_XLLP_Camera_Context_T camera_context )
{
    // power off the external module
    OV2630PowerDown( camera_context->gpio_reg_base, camera_context->ost_reg_base, XLLP_CAMERA_POWER_OFF );

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T CameraFuncOV2630Sleep(  P_XLLP_Camera_Context_T camera_context )
{
    return CameraFuncOV2630DeInit( camera_context );    
}

XLLP_STATUS_T CameraFuncOV2630Wake(  P_XLLP_Camera_Context_T camera_context )
{
    return CameraFuncOV2630Init( camera_context );
}

XLLP_STATUS_T CameraFuncOV2630SetCaptureFormat(  P_XLLP_Camera_Context_T camera_context )
{
    XLLP_CI_MP_TIMING timing;
    XLLP_UINT32_T winStartX, winStartY;
    XLLP_UINT32_T winEndX, winEndY;
    int           *padPixelX, *padPixelY;
    int           badPixelNum;

    //
    // Set CMU Coe matrix, if necessary
    //
    if((camera_context->capture_input_format != XLLP_CAMERA_IMAGE_FORMAT_RAW10)&&
       (camera_context->capture_input_format != XLLP_CAMERA_IMAGE_FORMAT_RAW8) &&
       (camera_context->capture_input_format != XLLP_CAMERA_IMAGE_FORMAT_RAW9))
        return XLLP_STATUS_WRONG_PARAMETER;
        
    if (camera_context->cmu_usage == XLLP_CI_CMU_OUTPUT_YUV) {
        XllpCICMUSetColorCorrectionCoe(camera_context->ci_reg_base, &ov2630_cRGB_YUV_COE);
    }
    
    if (camera_context->cmu_usage == XLLP_CI_CMU_OUTPUT_RGB) {
        XllpCICMUSetColorCorrectionCoe(camera_context->ci_reg_base, &ov2630_cRGB_sRGB_COE);
    }

    XllpCICMUEnable(camera_context->ci_reg_base, camera_context->cmu_usage);

    //
    // Set OV2630 format
    //
    OV2630SetFormat(
        camera_context->capture_input_width, 
        camera_context->capture_input_height,
        &winStartX,
        &winStartY,
        &winEndX,
        &winEndY);

    // 
    // configure PSU, if necessary
    //
    if (camera_context->psu_enable == XLLP_TRUE){
        XllpCIPSUEnable(camera_context->ci_reg_base, XLLP_FALSE);
    }
    
    padPixelX = ov2630_badpixel_table;
    padPixelY = ov2630_badpixel_table + 1;
    badPixelNum = 0; 
    while ((*padPixelX != END_MARKER) && (*padPixelY != END_MARKER)){
        
        if (((XLLP_UINT32_T)(*padPixelX) >= winStartX) && ((XLLP_UINT32_T)(*padPixelX) <= winEndX) &&
            ((XLLP_UINT32_T)(*padPixelY) >= winStartY) && ((XLLP_UINT32_T)(*padPixelY) <= winEndY)){

            // the very first pixel of each color can't be tagged as bad!!!
            if ((*padPixelX - winStartX > 1) || (*padPixelY - winStartY > 1)){
                XllpCIPSUTagBadPixelAt(
                    camera_context->ci_reg_base, 
                    *padPixelX - winStartX, 
                    *padPixelY - winStartY);
                                
                badPixelNum++;
                if (badPixelNum == 128)
                    break;// PSU only can substitute 128 bad pixels!!
            }
        }
        
        padPixelX += 2;
        padPixelY += 2;
    }

    // fill the last PSU RAM location with all 0's,
    // signifying the end of the dead pixel addresses stored in the RAM
    if (badPixelNum < 128)
        XllpCIPSUTagBadPixelAt(camera_context->ci_reg_base, 0, 0);

    if (badPixelNum != 0){
        if (camera_context->psu_enable == XLLP_FALSE){
            XllpCIPSUEnable(camera_context->ci_reg_base, XLLP_TRUE);
            camera_context->psu_enable = XLLP_TRUE;
        }
    }

    //
    // set capture width/height and timing 
    //
    timing.BFW = 0x01;
    timing.BLW = 0x01;   /*GRBG to RGGB*/         
    XllpCIConfigureMP(camera_context->ci_reg_base, camera_context->capture_input_width-1, camera_context->capture_input_height-1, &timing);

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T CameraFuncOV2630StartCapture(  P_XLLP_Camera_Context_T camera_context, unsigned int frames )
{   
    (void) camera_context;
    
    XllpCICGULoadLutRam(camera_context->ci_reg_base,
                        camera_context->ost_reg_base,
         (unsigned int*)camera_context->histogram_lut_buffer_virtual,
                        camera_context->histogram_lut_buffer_physical,
         (unsigned int*)camera_context->histogram_lut_dma_descriptors_virtual,
                        camera_context->histogram_lut_dma_descriptors_physical,
                        lut_table);
    
    XllpCICGUEnable(camera_context->ci_reg_base, camera_context->cgu_enable);

    // turn auto function on only doing continues capture
    if (frames == 0) {
        OV2630AutoFunctionOn();
    }
    else {
        OV2630AutoFunctionOff();
    }

    // turn viewfinder on
    OV2630ViewfinderOn();
    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T CameraFuncOV2630StopCapture(  P_XLLP_Camera_Context_T camera_context )
{
    (void) camera_context;
    
    // turn auto function off
    OV2630AutoFunctionOff();    

    // turn viewfinder off
    OV2630ViewfinderOff();

    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T CameraFuncOV2630SetPowerMode(P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T mode)
{
    (void) camera_context;
    
    OV2630PowerDown( camera_context->gpio_reg_base, camera_context->ost_reg_base, mode );
    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T CameraFuncOV2630Read8bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T reg_addr,  XLLP_UINT8_T *reg_val)
{
    (void) camera_context;

    OV2630ReadSensorReg( reg_addr, reg_val );
    return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T CameraFuncOV2630Write8bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T reg_addr,  XLLP_UINT8_T reg_val)
{
    XLLP_UINT8_T buffer;
    XLLP_STATUS_T status;

    (void) camera_context;

    buffer = reg_val;
    status = (OV2630WriteSensorReg( reg_addr, &buffer ) == 0) ?
              XLLP_STATUS_SUCCESS : XLLP_STATUS_FAILURE;

    return status;
}

XLLP_STATUS_T CameraFuncOV2630SetContrast( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T mode, XLLP_UINT32_T value)
{
    (void) camera_context;
    
    if (mode == XLLP_SENSOR_MANUAL_CONTRAST)
        return OV2630SetContrast(value);
    else 
        return XLLP_STATUS_SUCCESS;        
}

XLLP_STATUS_T CameraFuncOV2630SetExposure( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T mode, XLLP_UINT32_T value)
{
    (void) camera_context;
    
    if (mode == XLLP_SENSOR_MANUAL_EXPOSURE)
        return OV2630SetExposure(value);
    else
        return XLLP_STATUS_SUCCESS;
}

XLLP_STATUS_T CameraFuncOV2630SetWhiteBalance( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T mode, XLLP_UINT32_T value)
{
    (void) camera_context;
    
    if (mode == XLLP_SENSOR_MANUAL_WHITEBALANCE)
        return OV2630SetWhiteBalance(value);
    else
        return XLLP_STATUS_SUCCESS;
}
