/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/*++


Module Name:  $Workfile: xllp_ov2630_hw.c $

Abstract:  
     contains all hardware related functions for OV2630

Notes:Only valid for processor code named Monahans.

--*/

#include "xllp_camera_os_depend.h"
#include "xllp_camera.h"
#include "xllp_gpio_plat.h"
#include "xllp_gpio_proc.h"
#include "xllp_ost.h"
#include "xllp_ov2630_hw.h"
#include "xllp_serialization.h"

/***********************************************************************
*  Register Settings
***********************************************************************/



const static XLLP_UINT8_T ov2630InitSetting[]=
{


    0x12,	0x80,
    0x0e,	0x0,
    0x0f,	0x42,
    0x13,	0xe7,
    0x14,	0x4e,
    0x24,	0x6c,
    0x25,	0x60,
    0x35,	0x90,
    0x36,	0x88,
    0x37,	0x44,
    0x3a,	0x90,
    0x3b,	0x14,
    0x3f,	0x0f,
    0x40,	0x0,
    0x41,	0x0,
    0x42,	0x0,
    0x43,	0x0,
    0x44,	0x80,
    0x4b,	0x00,
    0x4c,	0x28,
    0x50,	0xf4,
    0x58,	0x7,
    0x59,	0x20,
    0x5f,	0x40,
    0x75,	0x0f,
    0x78,	0x40,
    0x7a,	0x10,
    0x84,	0x4,
    0x86,	0x20,
    0x88,	0x0c,
    0x89,	0x08,
    0x8a,	0x2,
    OV2630_REGEND,   0x00
};

/***********************************************************************
*  Register Settings, Get from OmniVision
***********************************************************************/
static XLLP_UINT8_T UXGA[] =
{


    //0x11,	0x01,
    0x11,	0x02,   /* workaroud for issue 193451 */
    0x34,	0xf0,
    0x03,	0x48,
    0x17,	0x2d,
    0x18,	0x02,
    0x19,	0x01,
    0x1a,	0x97,
    0x1e,	0x40,
    0x32,	0x1b,
    0x4d,	0xc0,
    0x5a,	0x00,
    0x87,	0x10,
    
    /* for non-zoom */
    0x0c,	0x21,
    0x16,	0x00,
    0x12,	0x00,
    0x48,	0x80,
    0x4a,	0x00,
    0x4e,	0x18,
    0x4f,	0x08,
    OV2630_REGEND,   0x00

};

static XLLP_UINT8_T SVGA[] =
{


	0x11,	0x00,
    0x34,   0x70,
    0x3,	0x0e,
    0x17,	0x3f,
    0x18,	0x02,
    0x19,	0x0,
    0x1a,	0x4b,
    0x1e,	0x00,
    0x32,	0x1f,
    0x4d,	0xc0,
    0x5a,	0x00,
    0x87,	0x00,

    /* for non-zoom */
    0x0c,	0xa0,
    0x16,	0x00,
    0x12,	0x41,
    0x48,	0x00,
    0x4a,	0x00,
    0x4e,	0x08,
    0x4f,	0x00,
    OV2630_REGEND,   0x00



};

static XLLP_UINT8_T CIF[] =
{


    0x11,	0x01,
    0x34,	0x70,
    0x03,	0x0a,
    0x17,	0x3f,
    0x18,	0x01,
    0x19,	0x00,
    0x1a,	0x25,
    0x1e,	0x00,
    0x32,	0xbf,
    0x4d,	0xc0,
    0x5a,	0x80,
    0x87,	0x00,
    
    /* for non-zoom */
    0x0c,	0xa0,
    0x16,	0x00,
    0x12,	0x21,
    0x48,	0x00,
    0x4a,	0x00,
    0x4e,	0x08,
    0x4f,	0x00,
    OV2630_REGEND,   0x00

};

const static XLLP_UINT8_T gSensorSlaveAddr = 0x30;

/***********************************************************************
*  Private/helper api
***********************************************************************/
#ifdef XLLP_DEBUG_PARAM_CHECK
static XLLP_STATUS_T PrvGetRegValue( XLLP_UINT8_T *regP, XLLP_UINT8_T regAddr, XLLP_UINT8_T *regValueP )
{
    unsigned int index = 0;
    XLLP_UINT8_T curReg = 0;
    
    while( curReg < OV2630_REGEND )
    {
        curReg = regP[index << 1];
        if( curReg == regAddr )
        {
            *regValueP = regP[(index << 1) + 1];
            return XLLP_STATUS_SUCCESS;
        }    
        index ++;
    } 
        
    return XLLP_STATUS_FAILURE;

}

static XLLP_STATUS_T PrvSetRegValue( XLLP_UINT8_T *regP, XLLP_UINT8_T regAddr, XLLP_UINT8_T regValue )
{
    unsigned int index = 0;
    XLLP_UINT8_T curReg = 0;
    
    while( curReg < OV2630_REGEND )
    {
        curReg = regP[index << 1];
        if( curReg == regAddr )
        {
            regP[(index << 1) + 1] = regValue;
            return XLLP_STATUS_SUCCESS;
        }    
        index ++;
    } 
        
    return XLLP_STATUS_FAILURE;

}
#endif

/***********************************************************************
*  Sensor read/write 
***********************************************************************/
static int PrvReadSensorReg( const XLLP_UINT8_T subAddress, XLLP_UINT8_T *bufP )
{
    XLLP_UINT8_T buffer[1];
    int status;
    buffer[0] = subAddress;
    XllpLock(XLLP_RESOURCE_I2C);
    status = OS_I2CMasterWriteData( gSensorSlaveAddr, buffer, 1);
    if (!status) {
        status = OS_I2CMasterReadData( gSensorSlaveAddr, buffer, 1);
        *bufP = buffer[0];
    }
    XllpUnlock(XLLP_RESOURCE_I2C);
    return status;
}

static int PrvWriteSensorReg( const XLLP_UINT8_T subAddress, XLLP_UINT8_T *bufP )
{
    XLLP_UINT8_T buffer[2];
    int status;
    buffer[0] = subAddress;
    buffer[1] = *bufP;
    XllpLock(XLLP_RESOURCE_I2C);
    status = OS_I2CMasterWriteData( gSensorSlaveAddr, buffer, 2);
    XllpUnlock(XLLP_RESOURCE_I2C);
    return status;
}

static int PrvRMWSensorReg( const XLLP_UINT8_T subAddress, XLLP_UINT8_T *bufP, XLLP_UINT8_T andMask, XLLP_UINT8_T orMask ) 
{
    int status;
    status = PrvReadSensorReg( subAddress, bufP );
    if (!status) {
        *bufP &= andMask;
        *bufP |= orMask;
        status = PrvWriteSensorReg( subAddress, bufP );
    }
    return status;
}

int OV2630ReadSensorReg( const XLLP_UINT8_T subAddress, XLLP_UINT8_T *bufP )
{
    return PrvReadSensorReg(subAddress, bufP);
}

int OV2630WriteSensorReg( const XLLP_UINT8_T subAddress, XLLP_UINT8_T *bufP )
{
    return PrvWriteSensorReg(subAddress, bufP);
}

XLLP_STATUS_T OV2630SetRegs( const XLLP_UINT8_T *regP )
{
    XLLP_UINT32_T    curReg = 0;
    XLLP_STATUS_T    status = XLLP_STATUS_SUCCESS;
        
    // The list is a register number followed by the value.
    while( regP[curReg << 1] < OV2630_REGEND )
    {
        XLLP_UINT8_T regVal = regP[(curReg << 1) + 1];

        status = (PrvWriteSensorReg( regP[curReg << 1], &regVal ) == 0) ?
                 XLLP_STATUS_SUCCESS : XLLP_STATUS_FAILURE;
        
        if( curReg == 0 )
            OV2630Wait( 5 );
            
        curReg++;
    }
        
    return status;
}



XLLP_STATUS_T OV2630ReadAllRegs( XLLP_UINT8_T *bufP, XLLP_UINT32_T numRegs )
{
    XLLP_UINT32_T    curReg;
    
    for( curReg = 0; curReg < numRegs; curReg++, bufP++ )
        PrvReadSensorReg( (XLLP_UINT8_T)curReg, bufP );


    return XLLP_STATUS_SUCCESS;
}



/***********************************************************************
*  Power & Reset
***********************************************************************/
void OV2630PowerDown(unsigned int gpio_reg_base, unsigned int ost_reg_base, XLLP_UINT8_T powerMode )
{
    // OV2630 PWRDWN, 0 = NORMAL, 1=POWER DOWN
    if( powerMode == XLLP_CAMERA_POWER_OFF )
        XllpGpioSetOutputLevel( (P_XLLP_GPIO_T)gpio_reg_base, XLLP_GPIO_CAMERA_HI_PWDN, XLLP_HI );
    else
        XllpGpioSetOutputLevel( (P_XLLP_GPIO_T)gpio_reg_base, XLLP_GPIO_CAMERA_HI_PWDN, XLLP_LO );
    
   XllpOstDelayMilliSeconds( (P_XLLP_OST_T)ost_reg_base, 100 );
}

void OV2630Reset( )
{
    OV2630SetRegs(ov2630InitSetting);
    return;
}

void OV2630Wait( int ms )
{
    OS_Delay( ms );
}


/***********************************************************************
*  Settings
***********************************************************************/
int OV2630VersionRevision(XLLP_UINT8_T * pCmRevision, XLLP_UINT8_T *pSensorRevision)
{
    PrvReadSensorReg( OV2630_PIDH, pCmRevision );
    PrvReadSensorReg( OV2630_PIDL, pSensorRevision );
    return XLLP_STATUS_SUCCESS;
}

void OV2630SetHSYNC()
{
    XLLP_UINT8_T    val;
    
    // Makes HREF become HSYNC
    PrvReadSensorReg( OV2630_COMK, &val );
    val |= 0x40;
    PrvWriteSensorReg( OV2630_COMK, &val );
}

void OV2630AutoFunctionOn()
{

    XLLP_UINT8_T val;
    PrvReadSensorReg( OV2630_COMI, &val );
    val |= 0x07;    // don't disturb AWB
    PrvWriteSensorReg( OV2630_COMI, &val );
}

void OV2630AutoFunctionOff()
{

    XLLP_UINT8_T val;
    PrvReadSensorReg( OV2630_COMI, &val );
    val &= ~0x07;    // don't disturb AWB
    PrvWriteSensorReg( OV2630_COMI, &val );
}


/***********************************************************************
*  Viewfinder, still 
***********************************************************************/
int OV2630ViewfinderOn()
{
    XLLP_UINT8_T    com3;
    
    PrvReadSensorReg( OV2630_COMD, &com3 );
    com3 &= ~0x01;
    PrvWriteSensorReg( OV2630_COMD, &com3 );
        
    return OV_ERR_NONE;
}


int OV2630ViewfinderOff()
{
    XLLP_UINT8_T    com3;

    
    PrvReadSensorReg( OV2630_COMD, &com3 );
    com3 |= 0x01;
    PrvWriteSensorReg( OV2630_COMD, &com3 );
    
    return OV_ERR_NONE;
}


int OV2630HaltVideoOutput()
{
    XLLP_UINT8_T    com3;
    
    // Set the camera to only output 1 frame.
    PrvReadSensorReg( OV2630_COMD, &com3 );
    com3 |= 1;
    PrvWriteSensorReg( OV2630_COMD, &com3 );
    
    return OV_ERR_NONE;
}

int OV2630ResumetoFullOutputMode()
{
    XLLP_UINT8_T    mode;
    
    // Output still frames continuously
    // Turn off single capture mode COM3.
    PrvRMWSensorReg( OV2630_COMD, (&mode), ((XLLP_UINT8_T) ~1), 0 );
    return OV_ERR_NONE;
}

int OV2630GetSingleImage()
{
    XLLP_UINT8_T    mode;
    
    PrvRMWSensorReg( OV2630_COMD, &mode, (XLLP_UINT8_T) ~1, 1 );
    return OV_ERR_NONE;
}

/***********************************************************************
*  Format 
***********************************************************************/
XLLP_STATUS_T OV2630SetFormat(XLLP_UINT32_T captureWidth, 
                              XLLP_UINT32_T captureHeight, 
                              XLLP_UINT32_T *winStartX, 
                              XLLP_UINT32_T *winStartY, 
                              XLLP_UINT32_T *winEndX, 
                              XLLP_UINT32_T *winEndY)
{
    OV2630_MODE mode;
    unsigned short hStart;
    unsigned short vStart;
    unsigned char regVal;


	


    //
    // let the sensor work on proper mode
    //
	if((captureWidth <= 400) && (captureHeight <= 292)){
        mode = OV2630_CIF;
    } else if((captureWidth <= 800) && (captureHeight <= 600)) {
        mode = OV2630_SVGA;
    } else if((captureWidth <= 1600) && (captureHeight <= 1200)) {
        mode = OV2630_UXGA;
    } else {
        return XLLP_STATUS_WRONG_PARAMETER;
    }



 
    if (mode == OV2630_CIF){
        OV2630SetRegs(CIF);
    }else if (mode == OV2630_SVGA){
       OV2630SetRegs(SVGA);
    }else{
       OV2630SetRegs(UXGA);
    }

    //
    // set cropping window
    //
    if (mode == OV2630_CIF) {
        captureWidth *= 2;
    }
    
   
	if (mode == OV2630_CIF){

		hStart = (unsigned short)(511 + (800 -captureWidth)/2);




		vStart = (unsigned short)(2 + (292 - captureHeight)/4);





		
    }else if (mode == OV2630_SVGA){

	
		hStart = (unsigned short)(511 + (800 -captureWidth)/2);

		vStart = (unsigned short)(2 + (600 - captureHeight)/4);
    
    }else{

	    hStart = (unsigned short)(363 + (1600 -captureWidth)/2);
		vStart = (unsigned short)(4 + (1200 - captureHeight)/4);
    }

    
   
    // set Horizontal Window Start
    regVal = hStart>>3;
    PrvWriteSensorReg(OV2630_HREFST, &regVal);
    PrvReadSensorReg(OV2630_COMM, &regVal);
    regVal &= ~0x07;

    regVal |= hStart & 0x07;
    PrvWriteSensorReg(OV2630_COMM, &regVal);


    // set Vertical Window Start
    regVal = vStart>>2;
    PrvWriteSensorReg(OV2630_VSTRT, &regVal);
    PrvReadSensorReg(OV2630_COMA, &regVal);
    regVal &= ~0x03;
    regVal |= vStart & 0x03;
    PrvWriteSensorReg(OV2630_COMA, &regVal);






    //
    // return window region
    //
    


    *winStartX = hStart;
    *winStartY = vStart;
    *winEndX   = hStart + captureWidth;
    *winEndY   = vStart + captureHeight;
    
    return XLLP_STATUS_SUCCESS;
}

/***********************************************************************
 *
 * Contrast
 *
 ***********************************************************************/
// FIX ME: TBD
const static XLLP_UINT8_T ContrastLowestSettings[] = {

    /* need to follow up */

    OV2630_REGEND,     0x00        // End of list delimiter.
};

const static XLLP_UINT8_T ContrastLowSettings[] = {

    /* need to follow up */
	OV2630_REGEND,     0x00        // End of list delimiter.
};

const static XLLP_UINT8_T ContrastMiddleSettings[] = {
    /* need to follow up */
    OV2630_REGEND,     0x00        // End of list delimiter.
};

const static XLLP_UINT8_T ContrastHighSettings[] = {
    /* need to follow up */
    OV2630_REGEND,     0x00        // End of list delimiter.
};

const static XLLP_UINT8_T ContrastHighestSettings[] = {
    /* need to follow up */
    OV2630_REGEND,     0x00        // End of list delimiter.
};
    
XLLP_STATUS_T OV2630SetContrast(XLLP_UINT32_T value)
{
    const XLLP_UINT8_T *regP;

    regP = NULL;
    switch(value) {
        case XLLP_SENSOR_CONTRAST_LOWEST:
            regP = ContrastLowestSettings;
            break;
        case XLLP_SENSOR_CONTRAST_LOW:
            regP = ContrastLowSettings;
            break;
        case XLLP_SENSOR_CONTRAST_MIDDLE:
            regP = ContrastMiddleSettings;
            break;
        case XLLP_SENSOR_CONTRAST_HIGH:
            regP = ContrastHighSettings;
            break;
        case XLLP_SENSOR_CONTRAST_HIGHEST:
            regP = ContrastHighestSettings;
            break;
        default:
            regP = ContrastMiddleSettings;
            break;
    }
    
    // set hw
    if (regP)        
        OV2630SetRegs(regP);
    return XLLP_STATUS_SUCCESS;
}

/***********************************************************************
 *
 * Exposure
 *
 ***********************************************************************/
// FIX ME: TBD
const static XLLP_UINT8_T ExposureSettings[] = {
	OV2630_REGEND,     0x00        // End of list delimiter.
    /* need to follow up */
};

XLLP_STATUS_T OV2630SetExposure(XLLP_UINT32_T value)
{
    XLLP_UINT8_T aew, aeb, vv;
    int index = -1;

    switch(value) {
        case XLLP_SENSOR_EXPOSURE_LOWEST:
            index = 0;
            break;
        case XLLP_SENSOR_EXPOSURE_LOW:
            index = 3;
            break;
        case XLLP_SENSOR_EXPOSURE_MIDDLE:
            index = 6;
            break;
        case XLLP_SENSOR_EXPOSURE_HIGH:
            index = 9;
            break;
        case XLLP_SENSOR_EXPOSURE_HIGHEST:
            index = 12;
            break;
        default:
            break;
    }

    aew = aeb = vv = 0;

    if (index != -1){
        aew = ExposureSettings[index];
        aeb = ExposureSettings[index + 1];
        vv  = ExposureSettings[index + 2];
    }
    
    // set hw
    if( aew || aeb || vv){
        OV2630WriteSensorReg( OV2630_AEW, &aew);
        OV2630WriteSensorReg( OV2630_AEB, &aeb);
        OV2630WriteSensorReg( OV2630_VV, &vv);
    }
    
    return XLLP_STATUS_SUCCESS;
}

/***********************************************************************
 *
 * Auto White Balance
 *
 ***********************************************************************/
// FIX ME: TBD
const static XLLP_UINT8_T AWBAuto[] = {
    /* need to follow up */
    OV2630_REGEND,     0x00        // End of list delimiter.
};

const static XLLP_UINT8_T AWBFluorescent[] = {
    /* need to follow up */
    OV2630_REGEND,     0x00        // End of list delimiter.
};

const static XLLP_UINT8_T AWBOutdoor[] = {
    /* need to follow up */
    OV2630_REGEND,     0x00        // End of list delimiter.
};

const static XLLP_UINT8_T AWBIncandescent[] = {
    /* need to follow up */
    OV2630_REGEND,     0x00        // End of list delimiter.
};


XLLP_STATUS_T OV2630SetWhiteBalance(XLLP_UINT32_T value)
{
    const XLLP_UINT8_T *regP;

    regP = NULL;
    switch(value) {
        case XLLP_SENSOR_WHITEBALANCE_AUTO:                //Auto:
            regP = AWBAuto;
            break;
        case XLLP_SENSOR_WHITEBALANCE_INCANDESCENT:        //Incandescent:
            regP = AWBIncandescent;
            break;
        case XLLP_SENSOR_WHITEBALANCE_SUNNY:               //Sunny:
            regP = AWBOutdoor;
            break;
        case XLLP_SENSOR_WHITEBALANCE_FLUORESCENT:         //Fluorescent:
            regP = AWBFluorescent;
            break;
        default:
            break;
    }

    // set hw
    if (regP) {
        OV2630SetRegs(regP);
    }
    return XLLP_STATUS_SUCCESS;
}


