/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include "xllp_ssp_plat.h"
#include "xllp_mfp_proc.h"
#include "xllp_mfp_plat.h"

// Fix me: Add more pins here if necessary.
static const XLLP_UINT32_T ssp2_mfp_list []={
	XLLP_MFP_AC97_SDATA_OUT_OFFSET,
	XLLP_MFP_AC97_SYNC_OFFSET,
	XLLP_MFP_PIN_GPIO36_OFFSET,
	XLLP_MFP_AC97_SDATA_IN_0_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER 
};

static const XLLP_MFP_ALT_FN_T ssp2_af_list []={
	XLLP_MFP_PIN_GPIO37_AF_SSPTXD2,
	XLLP_MFP_PIN_GPIO38_AF_SSPRXD2,
	XLLP_MFP_PIN_GPIO36_AF_SSPSFRM2,
	XLLP_MFP_PIN_GPIO35_AF_SSPSCLK2 
};

static const XLLP_MFP_ALT_FN_T ssp2_af_z_list []={
	XLLP_MFP_ALT_FN_7,
	XLLP_MFP_ALT_FN_7,
	XLLP_MFP_ALT_FN_7,
	XLLP_MFP_ALT_FN_7 
};

static const XLLP_MFP_DRIVE_STRENGTH_T ssp2_ds_list []={
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X 
};

// Fix me: Add more pins here if necessary.
static const XLLP_UINT32_T ssp3_mfp_list []={
	XLLP_MFP_SSP_AUDIO_SCLK_OFFSET,
	XLLP_MFP_SSP_AUDIO_FRM_OFFSET,
	XLLP_MFP_SSP_AUDIO_TXD_OFFSET,
	XLLP_MFP_SSP_AUDIO_RXD_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER 
};

static const XLLP_MFP_ALT_FN_T ssp3_af_list []={
	XLLP_MFP_SSP_AUDIO_SCLK_AF,
	XLLP_MFP_SSP_AUDIO_FRM_AF,
	XLLP_MFP_SSP_AUDIO_TXD_AF,
	XLLP_MFP_SSP_AUDIO_RXD_AF
};

static const XLLP_MFP_ALT_FN_T ssp3_af_z_list []={
	XLLP_MFP_ALT_FN_7,
	XLLP_MFP_ALT_FN_7,
	XLLP_MFP_ALT_FN_7,
	XLLP_MFP_ALT_FN_7 
};

static const XLLP_MFP_DRIVE_STRENGTH_T ssp3_ds_list []={
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X 
};

// Fix me: Add more pins here if necessary.
static const XLLP_UINT32_T ssp4_mfp_list []={
	XLLP_MFP_RSVD_SSP_4_CLK_OFFSET,
	XLLP_MFP_RSVD_SSP_4_FRM_OFFSET,
	XLLP_MFP_RSVD_SSP_4_TXD_OFFSET,
	XLLP_MFP_RSVD_SSP_4_RXD_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER 
};

static const XLLP_MFP_ALT_FN_T ssp4_af_list []={
	XLLP_MFP_RSVD_SSP_4_CLK_AF,
	XLLP_MFP_RSVD_SSP_4_FRM_AF,
	XLLP_MFP_RSVD_SSP_4_TXD_AF,
	XLLP_MFP_RSVD_SSP_4_RXD_AF
};

static const XLLP_MFP_DRIVE_STRENGTH_T ssp4_ds_list []={
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X,
	XLLP_MFP_DS_08X 
};

XLLP_STATUS_T XllpSSPMFPConfigure (XLLP_SSP_PLAT_PORT_T sspPort, 
		XLLP_SSP_CONFIG_T sspConfig, XLLP_BOOL_T active,
		XLLP_UINT32_T mfpBaseAddress, 
		XLLP_SSP_REGISTER_T *pSSPReg)
{
	XLLP_UINT32_T *mfp_list;
	XLLP_MFP_ALT_FN_T *af_list, *af_z_list;
	XLLP_MFP_DRIVE_STRENGTH_T *ds_list;
	XLLP_STATUS_T status;
	switch(sspPort) {
		case XLLP_SSP_ZL_CEDAR_CREEK:
			mfp_list = (P_XLLP_UINT32_T)ssp2_mfp_list;
			af_list = (P_XLLP_MFP_ALT_FN_T)ssp2_af_list;
			af_z_list = (P_XLLP_MFP_ALT_FN_T)ssp2_af_z_list;
			ds_list = (P_XLLP_MFP_DRIVE_STRENGTH_T)ssp2_ds_list;
			// Do some peripheral-specific pin config here if necessary.		
			break;
		case XLLP_SSP_ZL_BT_BUS:
			mfp_list = (P_XLLP_UINT32_T)ssp3_mfp_list;
			af_list = (P_XLLP_MFP_ALT_FN_T)ssp3_af_list;
			af_z_list = (P_XLLP_MFP_ALT_FN_T)ssp3_af_z_list;
			ds_list = (P_XLLP_MFP_DRIVE_STRENGTH_T)ssp3_ds_list;
			// Do some peripheral-specific pin config here if necessary.		
			break;
		case XLLP_SSP_ZL_WM9713:
			mfp_list = (P_XLLP_UINT32_T)ssp3_mfp_list;
			af_list = (P_XLLP_MFP_ALT_FN_T)ssp3_af_list;
			af_z_list = (P_XLLP_MFP_ALT_FN_T)ssp3_af_z_list;
			ds_list = (P_XLLP_MFP_DRIVE_STRENGTH_T)ssp3_ds_list;
			// Do some peripheral-specific pin config here if necessary.		
			break;
		default:
			return XLLP_STATUS_WRONG_PARAMETER;
	}

	if (active == XLLP_TRUE) {
		status =  XllpMfpSetAfDs_List((P_XLLP_VUINT32_T)mfpBaseAddress, mfp_list, af_list, ds_list);
	} else {
		status =  XllpMfpSetAfDs_List((P_XLLP_VUINT32_T)mfpBaseAddress, mfp_list, af_z_list, ds_list);
		// FIXME: Need we activate pull-up/down manually here when we use non-existent af for tristate a pad ?
	}

	return XLLP_STATUS_SUCCESS;	
}

