/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include "xllp_true_ide_plat.h"

static XLLP_UINT32_T xllp_true_ide_mfp_list[]={
	XLLP_MFP_DREQ_GPIO_OFFSET,
	XLLP_MFP_NPWAIT_GPIO_OFFSET,
	XLLP_MFP_CFCD_GPIO_OFFSET,
	XLLP_MFP_CFNIRQ_GPIO_OFFSET,
	XLLP_MFP_PIN_EOLIST_MARKER
};/*end of XllpPccardMfpList*/

static XLLP_MFP_ALT_FN_T xllp_true_ide_af_list []={
	XLLP_MFP_DREQ_GPIO_AF,
	XLLP_MFP_NPWAIT_GPIO_AF,
	XLLP_MFP_CFCD_GPIO_AF,
	XLLP_MFP_CFNIRQ_GPIO_AF
};/*end of XllpPccardAfList*/

static XLLP_MFP_DRIVE_STRENGTH_T xllp_true_ide_ds_List[]={
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS, 
	XLLP_MFP_DEFAULT_DS,
	XLLP_MFP_DEFAULT_DS
};/*end of XllpPccardDsList*/

static XLLP_MFP_LPM_OUTPUT_T xllp_pccard_lpm_output_List[]={
	XLLP_MFP_LPMO_PULL_LOW,		/*  DREQ*/
	XLLP_MFP_LPMO_FLOAT,		/*  nPWAIT */
	XLLP_MFP_LPMO_PULL_HIGH,	/*  CFDC*/
	XLLP_MFP_LPMO_PULL_LOW,		/*  CFnIRQ*/
};/*end of xllp_keypad_lpm_output_list*/

/*------------------------------------------------------------
 Function:  xllp_micodisk_state
 Purpose:  get the micro disk state 
 Returns:  XLLP_MICRO_DISK_STATE_T
  I/F type: I/F PLAT 3
--------------------------------------------------------------*/
XLLP_MICRO_DISK_STATE_T xllp_microdisk_state(P_XLLP_TRUE_IDE_T true_ide_handle)
{
	XLLP_LEVEL_T return_level;
	XllpGpioGetLevel(true_ide_handle->gpio_regs, XLLP_GPIO_CFCD, &return_level);
	if(return_level == XLLP_HI)
	{
		return XLLP_MICRO_DISK_REMOVED;
	}
	else
	{
/*now, the micro disk always power on*/
		return XLLP_MICRO_DISK_ON;
	}
}
/*------------------------------------------------------------
 Function:  get_cf_socket_mode
 Purpose:  This function return the cf socket mode, if the Pin nSEL of CF socket 
 	is pull down, the CF socket is in XLLP_CF_TURE_IDE_MOD mode, 
 	at this mode micro disk that is CF true_ide interface can be used.
	Otherwise, the CF socket is in XLLP_CF_IO_MEM_MOD, at this 
	mode CF I/O and storage can be used. On zylonite board, we can 
	use register: 0x14200004 bit 4 to decide which mode the socket is in.  
	This function may also be used by XLLP PCCARD module. 
 Returns:  XLLP_CF_SOCKET_MODE
 parameter: hd_switch_addr:  the address of switch of true ide. 
 I/F type: E/F PLAT 2
-----------------------------------------------------------------*/
XLLP_CF_SOCKET_MODE_T xllp_get_cf_socket_mode(P_XLLP_VUINT32_T hd_switch_addr)
{
	if( (*hd_switch_addr) & 0x10 ){ 
		return XLLP_CF_IO_MEM_MOD ;
	}else {
		return XLLP_CF_TURE_IDE_MOD;
	}
}

/*------------------------------------------------------------
 Function:  xllp_ture_ide_init_plat
 Purpose:  configure the MFP 
 Returns:  XLLP_STATUS_T
 	XLLP_STATUS_INVALIED_STATE 
 	means that the SW1 is not at right position
 I/F type: I/F PLAT 1
--------------------------------------------------------------*/
XLLP_STATUS_T xllp_ture_ide_init_plat(P_XLLP_TRUE_IDE_T true_ide_handle)
{

	XLLP_STATUS_T status = XLLP_STATUS_SUCCESS;
	XLLP_CF_SOCKET_MODE_T mode= XLLP_CF_TURE_IDE_MOD;
	mode = xllp_get_cf_socket_mode(true_ide_handle->hd_switch_addr);

	if ( XLLP_CF_TURE_IDE_MOD != mode ) {
		/*SW1 is not in NON-DOT position*/
		return XLLP_STATUS_INVALID_STATE;
	}

	status = XllpMfpResourceManager_List(
			true_ide_handle->mfp_rs_db,
			xllp_true_ide_mfp_list,
			XLLP_MFP_RM_ID_XLLP_TRUE_IDE,
			XLLP_SET);
  	if (status != XLLP_STATUS_SUCCESS){
    		return status;
 	 }

	XllpLock(XLLP_RESOURCE_MFPR);

  	status = XllpMfpSetAfDs_List(
			true_ide_handle->mfp_base,
			xllp_true_ide_mfp_list,
			xllp_true_ide_af_list,
			xllp_true_ide_ds_List);

	 if (status != XLLP_STATUS_SUCCESS){
	 	 XllpUnlock(XLLP_RESOURCE_MFPR);
    		return status;
  	}

 	 status = XllpMfpConfigureLpmOutputLevel_List (
			true_ide_handle->mfp_base,
			xllp_true_ide_mfp_list,
			xllp_pccard_lpm_output_List);

	if ( XLLP_STATUS_SUCCESS != status) {
		XllpUnlock(XLLP_RESOURCE_MFPR);
		return status ;	
	}
	/*
	 The CF socket on zylonite board will not set the 
	 level on proper level, So below pins must be set
	 mannually.
	*/
	/*
	//pull up XLLP_MFP_CFCD_GPIO_OFFSET
 	//when there is a micro disk on the socket
 	//the micro disk will pull down it
 	*/
	XllpMfpConfigurePullUpDown(
		true_ide_handle->mfp_base, 
		XLLP_MFP_CFCD_GPIO_OFFSET,
		XLLP_MFP_PULL_UP);
	XllpMfpActivatePullUpDown(
		true_ide_handle->mfp_base,
		XLLP_MFP_CFCD_GPIO_OFFSET, 
		XLLP_ON);
	/*
	//we should pull down this pin, 
	//becuase if there are no card on the board,
	//the socket will request the irq to processor
	//fix me: 
	//need care about the drive power of the micro disk
	*/
	XllpMfpConfigurePullUpDown(
		true_ide_handle->mfp_base, 
		XLLP_MFP_CFNIRQ_GPIO_OFFSET,
		XLLP_MFP_PULL_DOWN);
 	XllpMfpActivatePullUpDown(
		true_ide_handle->mfp_base,
		XLLP_MFP_CFNIRQ_GPIO_OFFSET, 
		XLLP_ON);
		
	/*
	//pull down XLLP_MFP_DREQ_GPIO_OFFSET
	//the the pin is dive high when the micro disk
	//request DMA to processor
	
	//if the card is not in the socket pull down this pin
	//can make sure that there are not unneeded requset
	//to processor
	//fix me: need care about the drive power
	*/
	XllpMfpConfigurePullUpDown(
		true_ide_handle->mfp_base, 
		XLLP_MFP_DREQ_GPIO_OFFSET,
		XLLP_MFP_PULL_DOWN);
 	XllpMfpActivatePullUpDown(
		true_ide_handle->mfp_base,
		XLLP_MFP_DREQ_GPIO_OFFSET, 
		XLLP_ON);

	/*
 	//micro disk will pull down the the pin
 	//when it is busy and want processor to wait
 	*/
	XllpMfpConfigurePullUpDown(
		true_ide_handle->mfp_base, 
		XLLP_MFP_NPWAIT_GPIO_OFFSET,
		XLLP_MFP_PULL_UP);
 	XllpMfpActivatePullUpDown(
		true_ide_handle->mfp_base,
		XLLP_MFP_NPWAIT_GPIO_OFFSET, 
		XLLP_ON);

	XllpGpioSetDirection(
		true_ide_handle->gpio_regs, 
		XLLP_GPIO_CFCD, 
		XLLP_GPIO_DIRECTION_IN);

	XllpGpioSetDirection(
		true_ide_handle->gpio_regs, 
		XLLP_GPIO_NIRQ, 
		XLLP_GPIO_DIRECTION_IN);
	 
	XllpGpioSetRisingEdgeDetectEnable(
		true_ide_handle->gpio_regs,
		XLLP_GPIO_NIRQ, 
		XLLP_ON);

	XllpUnlock(XLLP_RESOURCE_MFPR);
	return XLLP_STATUS_SUCCESS;

}
/*------------------------------------------------------------
 Function:  xllp_ture_ide_deinit_plat
 Purpose:  deinitialize the plat
 Returns:  XLLP_STATUS_T
 	XLLP_STATUS_INVALIED_STATE 
 	means that the SW1 is not at right position
 I/F type: I/F PLAT 1
--------------------------------------------------------------*/
XLLP_STATUS_T xllp_true_ide_deinit_plat(P_XLLP_TRUE_IDE_T true_ide_handle)
{
	XLLP_STATUS_T status= XLLP_STATUS_SUCCESS;
	status = XllpMfpResourceManager_List(
			true_ide_handle->mfp_rs_db,
			xllp_true_ide_mfp_list,
			XLLP_MFP_RM_ID_XLLP_TRUE_IDE,
			XLLP_CLEAR);
    	return status;

}
/*------------------------------------------------------------
 Function:  xllp_power_on_true_ide
 Purpose:  Turn on the power supply of the mario disk
 Returns:  
 I/F type: I/F PLAT 2
 note:	after power on the micro disk, the function delay for a while to wait
 	the disk stable.
--------------------------------------------------------------*/
void xllp_power_on_true_ide(P_XLLP_TRUE_IDE_T true_ide_handle)
{
}

/*------------------------------------------------------------
 Function:  xllp_power_off_true_ide
 Purpose:  Turn off the power supply of the mario disk
 Returns:  
 I/F type: I/F PLAT 3
 note:	this function only close the power of micro disk. Before power off the 
 	disk, OS driver MUST flush the buffer of micro disk!!! 
--------------------------------------------------------------*/
void xllp_power_off_true_ide(P_XLLP_TRUE_IDE_T true_ide_handle)
{
}


