/*++
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
--*/


#include "xllp_defs.h"
#include "xllp_gpio.h"
#include "xllp_mfp.h"
#include "xllp_u2d.h"
#if defined(LITTLETON)
#include "xllp_gpio_plat.h"
#include "xllp_mfp_plat_L_LV.h"

static int platform_id = 0;
static void XllpDetectPlatform(P_XLLP_U2D_T pU2DHandle);
#endif

#define	XLLP_GPIO_ULPI_RESET		43

static XLLP_UINT32_T xllp_u2d_mfp_list[] =
{ 	
#if defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_L)
	XLLP_MFP_U2D_UTM_CLK_OFFSET,		

	XLLP_MFP_U2D_DATA0_OFFSET	,	

	XLLP_MFP_U2D_DATA1_OFFSET	,	

	XLLP_MFP_U2D_DATA2_OFFSET	,	

	XLLP_MFP_U2D_DATA3_OFFSET	,	

	XLLP_MFP_U2D_DATA4_OFFSET	,	

	XLLP_MFP_U2D_DATA5_OFFSET	,	

	XLLP_MFP_U2D_DATA6_OFFSET	,	

	XLLP_MFP_U2D_DATA7_OFFSET		,

	XLLP_MFP_U2D_RESET_OFFSET	,	

	XLLP_MFP_U2D_XCVR_SELECT_OFFSET,

	XLLP_MFP_U2D_TERM_SELECT_OFFSET,

	XLLP_MFP_U2D_SUSPENDM_X_OFFSET ,

	XLLP_MFP_U2D_LINESTATE0_OFFSET ,

	XLLP_MFP_U2D_LINESTATE1_OFFSET ,


	XLLP_MFP_U2D_TXVALID_OFFSET 	,

	XLLP_MFP_U2D_TXREADY_OFFSET		,

	XLLP_MFP_U2D_RXVALID_OFFSET 	,

	XLLP_MFP_U2D_RXACTIVE_OFFSET	,

	XLLP_MFP_U2D_RXERROR_OFFSET		,

	XLLP_MFP_U2D_OPMODE0_OFFSET 	,

	XLLP_MFP_U2D_OPMODE1_OFFSET 	,
#elif defined(CONFIG_CPU_MONAHANS_LV) || defined(CONFIG_CPU_TAVOR_P)

	XLLP_MFP_PIN_ULPI_STP_OFFSET,		
	XLLP_MFP_PIN_ULPI_DIR_OFFSET,		
	XLLP_MFP_PIN_ULPI_NXT_OFFSET,		

	XLLP_MFP_ULPI_CLK_OFFSET,		

	XLLP_MFP_ULPI_DATA0_OFFSET	,	

	XLLP_MFP_ULPI_DATA1_OFFSET	,	

	XLLP_MFP_ULPI_DATA2_OFFSET	,	

	XLLP_MFP_ULPI_DATA3_OFFSET	,	

	XLLP_MFP_ULPI_DATA4_OFFSET	,	

	XLLP_MFP_ULPI_DATA5_OFFSET	,	

	XLLP_MFP_ULPI_DATA6_OFFSET	,	

	XLLP_MFP_ULPI_DATA7_OFFSET		,

#ifdef CONFIG_CPU_TAVOR_P
	XLLP_MFP_ULPI_REFCLK_OFFSET,
	XLLP_MFP_ULPI_RESET_OFFSET	,
#endif

#endif	
	XLLP_MFP_PIN_EOLIST_MARKER,
};

static XLLP_MFP_ALT_FN_T xllp_u2d_af_list[] = 
{
#if defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_L)
	XLLP_MFP_U2D_UTM_CLK_AF			,

	XLLP_MFP_U2D_DATA0_AF			,

	XLLP_MFP_U2D_DATA1_AF			,

	XLLP_MFP_U2D_DATA2_AF			,

	XLLP_MFP_U2D_DATA3_AF			,

	XLLP_MFP_U2D_DATA4_AF			,

	XLLP_MFP_U2D_DATA5_AF			,

	XLLP_MFP_U2D_DATA6_AF			,

	XLLP_MFP_U2D_DATA7_AF				,

	XLLP_MFP_U2D_RESET_AF			,

	XLLP_MFP_U2D_XCVR_SELECT_AF		,

	XLLP_MFP_U2D_TERM_SELECT_AF		,

	XLLP_MFP_U2D_SUSPENDM_X_AF		,

	XLLP_MFP_U2D_LINESTATE0_AF		,

	XLLP_MFP_U2D_LINESTATE1_AF		,

	XLLP_MFP_U2D_TXVALID_AF			,

	XLLP_MFP_U2D_TXREADY_AF			,

	XLLP_MFP_U2D_RXVALID_AF			,

	XLLP_MFP_U2D_RXACTIVE_AF		,

	XLLP_MFP_U2D_RXERROR_AF			,

	XLLP_MFP_U2D_OPMODE0_AF			,

	XLLP_MFP_U2D_OPMODE1_AF			
#elif defined(CONFIG_CPU_MONAHANS_LV) || defined(CONFIG_CPU_TAVOR_P)
#ifdef CONFIG_CPU_MONAHANS_LV
	XLLP_MFP_ALT_FN_0,
	XLLP_MFP_ALT_FN_0,
	XLLP_MFP_ALT_FN_0,
#else
	XLLP_MFP_ULPI_STP_AF,
	XLLP_MFP_ULPI_DIR_AF,
	XLLP_MFP_ULPI_NXT_AF,
#endif

	XLLP_MFP_ULPI_CLK_AF,		

	XLLP_MFP_ULPI_DATA0_AF	,	

	XLLP_MFP_ULPI_DATA1_AF	,	

	XLLP_MFP_ULPI_DATA2_AF	,	

	XLLP_MFP_ULPI_DATA3_AF	,	

	XLLP_MFP_ULPI_DATA4_AF	,	

	XLLP_MFP_ULPI_DATA5_AF	,	

	XLLP_MFP_ULPI_DATA6_AF	,	

	XLLP_MFP_ULPI_DATA7_AF		,

#ifdef CONFIG_CPU_TAVOR_P
	XLLP_MFP_ULPI_REFCLK_AF,
	XLLP_MFP_ULPI_RESET_AF	,
#endif
#endif	

};

#define XLLP_MFP_USB2_DEFAULT_DS	XLLP_MFP_DS_12X

static XLLP_MFP_DRIVE_STRENGTH_T xllp_u2d_ds_list[] =
{	
#if defined(CONFIG_CPU_MONAHANS_P) || defined(CONFIG_CPU_MONAHANS_L)
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
	
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,


// 8


	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
	
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

// 16
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
	
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
#elif defined(CONFIG_CPU_MONAHANS_LV) || defined(CONFIG_CPU_TAVOR_P)
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,
	XLLP_MFP_USB2_DEFAULT_DS,

	XLLP_MFP_USB2_DEFAULT_DS,


#endif
};
#if defined(LITTLETON)
/* kv - Initialise platform detection pins to GPIO */
#define	XLLP_MFP_PLATFORM_DETECT_DEFAULT_DS	XLLP_MFP_DS_01X

static XLLP_UINT32_T xllp_platform_detect_mfp_list[] =
{
	XLLP_MFP_PLATFORM_DETECT_0,
	XLLP_MFP_PLATFORM_DETECT_1,
	XLLP_MFP_PLATFORM_DETECT_2,
	XLLP_MFP_PLATFORM_DETECT_3,
	XLLP_MFP_PLATFORM_DETECT_4,
	XLLP_MFP_PLATFORM_DETECT_5,
	XLLP_MFP_PLATFORM_DETECT_6,
	XLLP_MFP_PIN_EOLIST_MARKER
};

static XLLP_MFP_ALT_FN_T xllp_platform_detect_af_list[] =
{
	XLLP_MFP_PLATFORM_DETECT_0_AF,
	XLLP_MFP_PLATFORM_DETECT_1_AF,
	XLLP_MFP_PLATFORM_DETECT_2_AF,
	XLLP_MFP_PLATFORM_DETECT_3_AF,
	XLLP_MFP_PLATFORM_DETECT_4_AF,
	XLLP_MFP_PLATFORM_DETECT_5_AF,
	XLLP_MFP_PLATFORM_DETECT_6_AF,
};

static XLLP_MFP_DRIVE_STRENGTH_T xllp_platform_detect_ds_list[] =
{
	XLLP_MFP_PLATFORM_DETECT_DEFAULT_DS,
	XLLP_MFP_PLATFORM_DETECT_DEFAULT_DS,
	XLLP_MFP_PLATFORM_DETECT_DEFAULT_DS,
	XLLP_MFP_PLATFORM_DETECT_DEFAULT_DS,
	XLLP_MFP_PLATFORM_DETECT_DEFAULT_DS,
	XLLP_MFP_PLATFORM_DETECT_DEFAULT_DS,
	XLLP_MFP_PLATFORM_DETECT_DEFAULT_DS,
};
#endif
/*!
	Initialize platform 

@remarks
	This function related with platform
	
@param[in] 	  pU2DHandle		U2DHandle, which is initilizated in XllpU2DInitHandle
*/
void XllpU2DPlatformInit(P_XLLP_U2D_T pU2DHandle)
{

#if	!(defined(LITTLETON)) 
#if defined(CONFIG_CPU_MONAHANS_PL) || defined(CONFIG_CPU_MONAHANS_L)
	XllpGpioSetDirection (  pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_UTMI_SWITCH,
                                XLLP_GPIO_DIRECTION_OUT);

	XllpGpioSetOutputLevel (pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_UTMI_SWITCH,
                                XLLP_HI);


	XllpGpioSetDirection (  pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_UTMI_TEST_EN,
                                XLLP_GPIO_DIRECTION_OUT);

	XllpGpioSetOutputLevel (pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_UTMI_TEST_EN,
                                XLLP_LO);
#endif

#if defined(CONFIG_CPU_MONAHANS_LV) || defined(CONFIG_CPU_TAVOR_P)
#define ULPI_RESET_ECO
#ifndef ULPI_RESET_ECO
	XllpGpioSetDirection (  pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_ULPI_RESET,
                                XLLP_GPIO_DIRECTION_OUT);

	XllpGpioSetOutputLevel (pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_ULPI_RESET,
                                XLLP_HI);
#else
	int count;
	XllpGpioSetDirection (  pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_ULPI_RESET,
                                XLLP_GPIO_DIRECTION_OUT);

	XllpGpioSetOutputLevel (pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_ULPI_RESET,
                                XLLP_LO);

	count = 100000;
	while(count--);

	XllpGpioSetOutputLevel (pU2DHandle->pGPIORegs, 
                                XLLP_GPIO_ULPI_RESET,
                                XLLP_HI);

#endif

#endif
#else
	XllpDetectPlatform(pU2DHandle);
	if (0 == platform_id) {	/* Littleton Rev 1.x */
		xllp_max7320_read(0, &val);
		val |= 0x2;
		xllp_max7320_write(0, val);	
	} else { 		/* Littleton Rev 4.x */
		int count;
		XllpGpioSetDirection (  pU2DHandle->pGPIORegs, 
                	                XLLP_GPIO_ULPI_RESET,
                               		XLLP_GPIO_DIRECTION_OUT);
		XllpGpioSetOutputLevel (pU2DHandle->pGPIORegs, 
                	                XLLP_GPIO_ULPI_RESET,
                               		XLLP_LO);

		count = 100000;
		while(count--);
		XllpGpioSetOutputLevel (pU2DHandle->pGPIORegs,
		                        XLLP_GPIO_ULPI_RESET,											                   				     XLLP_HI);
	}

#endif

	//XLLP_STATUS_WRONG_PARAMETER
	XllpMfpSetAfDs_List(pU2DHandle->pMfpRegBase, 
				xllp_u2d_mfp_list,
				xllp_u2d_af_list, 
				xllp_u2d_ds_list);

	XllpMfpActivatePullUpDown_List(pU2DHandle->pMfpRegBase, 
				xllp_u2d_mfp_list,
				XLLP_OFF);
	
#ifdef CONFIG_CPU_TAVOR_P
#define CWSBR *(unsigned int*)(0x42404008)
	CWSBR |= 0x10000;	
#endif

}

/*!
	Deinitialize platform 

@remarks
	This function related with platform

@param[in] 	  pU2DHandle		U2DHandle, which is initilizated in XllpU2DInitHandle
*/
void XllpU2DPlatformUninit(P_XLLP_U2D_T __attribute__((unused)) pU2DHandle )
{

}

#if defined(LITTLETON)
static void XllpDetectPlatform(P_XLLP_U2D_T pU2DHandle)
{
	int i, board_id = 0;
	int board_detect_pins[7] = {
		XLLP_GPIO_PLATFORM_DETECT_0_N,
		XLLP_GPIO_PLATFORM_DETECT_1_N,
		XLLP_GPIO_PLATFORM_DETECT_2_N,
		XLLP_GPIO_PLATFORM_DETECT_3_N,
		XLLP_GPIO_PLATFORM_DETECT_4_N,
		XLLP_GPIO_PLATFORM_DETECT_5_N,
		XLLP_GPIO_PLATFORM_DETECT_6_N,
	};

	XLLP_LEVEL_T level = XLLP_LO;
	XllpMfpSetAfDs_List(pU2DHandle->pMfpRegBase,
			xllp_platform_detect_mfp_list,
			xllp_platform_detect_af_list,
			xllp_platform_detect_ds_list);

	XllpMfpActivatePullUpDown_List(pU2DHandle->pMfpRegBase,
			xllp_platform_detect_mfp_list,
			XLLP_OFF);

	/* set detect pins as gpio input */
	for (i = 6; i >= 0; i--) {
		XllpGpioGetLevel(pU2DHandle->pGPIORegs,
				board_detect_pins[i],
				&level);
		board_id = (level << i) | board_id;
	}

	platform_id = (board_id >> 4);
}
#endif

