/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/* 
 * xllp_camera.h
 *
 * xllp_camera driver for Monahans 
 *
 */

/*++

Module Name:  $Workfile: xllp_camera.h $

Abstract:  
      contains all XLLP camera specific macros, typedefs, and prototypes.
    Declares no storage.

Notes:Only valid for processor code named Monahans.

--*/

#ifndef _XLLP_CAMERA_H_
#define _XLLP_CAMERA_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "xllp_defs.h"
#include "xllp_ci.h"
#include "xllp_mfp_proc.h"

//////////////////////////////////////////////////////////////////////////////////////
//
//          Macros
//
//////////////////////////////////////////////////////////////////////////////////////
//
//  Sensor type
//
#define XLLP_CAMERA_TYPE_ADCM_2650               01
#define XLLP_CAMERA_TYPE_ADCM_2670               02
#define XLLP_CAMERA_TYPE_ADCM_2700               03
#define XLLP_CAMERA_TYPE_OMNIVISION_9640         04
#define XLLP_CAMERA_TYPE_OMNIVISION_2620         05
#define XLLP_CAMERA_TYPE_OMNIVISION_7660         06
#define XLLP_CAMERA_TYPE_OMNIVISION_2630         07
#define XLLP_CAMERA_TYPE_MAX                     XLLP_CAMERA_TYPE_OMNIVISION_2630

//
//  Image format definition
//
#define XLLP_CAMERA_IMAGE_FORMAT_RAW8                0
#define XLLP_CAMERA_IMAGE_FORMAT_RAW9                1
#define XLLP_CAMERA_IMAGE_FORMAT_RAW10               2
                                                     
#define XLLP_CAMERA_IMAGE_FORMAT_RGB444              3
#define XLLP_CAMERA_IMAGE_FORMAT_RGB555              4
#define XLLP_CAMERA_IMAGE_FORMAT_RGB565              5
#define XLLP_CAMERA_IMAGE_FORMAT_RGB666_PACKED       6
#define XLLP_CAMERA_IMAGE_FORMAT_RGB666_PLANAR       7
#define XLLP_CAMERA_IMAGE_FORMAT_RGB888_PACKED       8
#define XLLP_CAMERA_IMAGE_FORMAT_RGB888_PLANAR       9
#define XLLP_CAMERA_IMAGE_FORMAT_RGBT555_0          10  //RGB+Transparent bit 0
#define XLLP_CAMERA_IMAGE_FORMAT_RGBT888_0          11
#define XLLP_CAMERA_IMAGE_FORMAT_RGBT555_1          12  //RGB+Transparent bit 1  
#define XLLP_CAMERA_IMAGE_FORMAT_RGBT888_1          13

#define XLLP_CAMERA_IMAGE_FORMAT_YCBCR400           14
#define XLLP_CAMERA_IMAGE_FORMAT_YCBCR422_PACKED    15
#define XLLP_CAMERA_IMAGE_FORMAT_YCBCR422_PLANAR    16
#define XLLP_CAMERA_IMAGE_FORMAT_YCBCR444_PACKED    17
#define XLLP_CAMERA_IMAGE_FORMAT_YCBCR444_PLANAR    18
#define XLLP_CAMERA_IMAGE_FORMAT_MAX                XLLP_CAMERA_IMAGE_FORMAT_YCBCR444_PLANAR

//
//  Image Size definition
// FIX ME!
#define XLLP_CAMERA_IMAGE_SIZE_QQVGA                0x01
#define XLLP_CAMERA_IMAGE_SIZE_QVGA                 0x02
#define XLLP_CAMERA_IMAGE_SIZE_VGA                  0x04
#define XLLP_CAMERA_IMAGE_SIZE_SXGA                 0x08
#define XLLP_CAMERA_IMAGE_SIZE_QQCIF                0x10
#define XLLP_CAMERA_IMAGE_SIZE_QCIF                 0x20
#define XLLP_CAMERA_IMAGE_SIZE_CIF                  0x40
#define XLLP_CAMERA_IMAGE_SIZE_INVALID              0x80

// Interrupt mask
#define XLLP_CAMERA_INTMASK_FIFO_OVERRUN                XLLP_CI_INT_IFO            
#define XLLP_CAMERA_INTMASK_END_OF_FRAME                XLLP_CI_INT_EOF            
#define XLLP_CAMERA_INTMASK_START_OF_FRAME              XLLP_CI_INT_SOF          
#define XLLP_CAMERA_INTMASK_CI_DISABLE_DONE             XLLP_CI_INT_CDD         
#define XLLP_CAMERA_INTMASK_CI_QUICK_DISABLE            XLLP_CI_INT_CQD        
#define XLLP_CAMERA_INTMASK_PARITY_ERROR                XLLP_CI_INT_PAR_ERR            
#define XLLP_CAMERA_INTMASK_END_OF_LINE                 XLLP_CI_INT_EOL             
#define XLLP_CAMERA_INTMASK_FIFO_EMPTY                  XLLP_CI_INT_FEMPTY              
#define XLLP_CAMERA_INTMASK_TIME_OUT                    XLLP_CI_INT_FTO                
#define XLLP_CAMERA_INTMASK_FIFO3_UNDERRUN              XLLP_CI_INT_FU          
#define XLLP_CAMERA_INTMASK_BRANCH_STATUS               XLLP_CI_INT_BS           
#define XLLP_CAMERA_INTMASK_ENF_OF_FRAME_TRANSFER       XLLP_CI_INT_EOFX   
#define XLLP_CAMERA_INTMASK_DMA_CHANNEL0_STOP           XLLP_CI_INT_SC0       
#define XLLP_CAMERA_INTMASK_DMA_CHANNEL1_STOP           XLLP_CI_INT_SC1       
#define XLLP_CAMERA_INTMASK_DMA_CHANNEL2_STOP           XLLP_CI_INT_SC2       
#define XLLP_CAMERA_INTMASK_DMA_CHANNEL3_STOP           XLLP_CI_INT_SC3       
                                                                                                
// Interrupt status                               
#define XLLP_CAMERA_INTSTATUS_FIFO_OVERRUN_0            XLLP_CI_INTSTATUS_IFO_0        
#define XLLP_CAMERA_INTSTATUS_FIFO_OVERRUN_1            XLLP_CI_INTSTATUS_IFO_1        
#define XLLP_CAMERA_INTSTATUS_FIFO_OVERRUN_2            XLLP_CI_INTSTATUS_IFO_2        
#define XLLP_CAMERA_INTSTATUS_FIFO_OVERRUN_3            XLLP_CI_INTSTATUS_IFO_3        
#define XLLP_CAMERA_INTSTATUS_END_OF_FRAME              XLLP_CI_INTSTATUS_EOF          
#define XLLP_CAMERA_INTSTATUS_START_OF_FRAME            XLLP_CI_INTSTATUS_SOF        
#define XLLP_CAMERA_INTSTATUS_CI_DISABLE_DONE           XLLP_CI_INTSTATUS_CDD       
#define XLLP_CAMERA_INTSTATUS_CI_QUICK_DISABLE          XLLP_CI_INTSTATUS_CQD      
#define XLLP_CAMERA_INTSTATUS_PARITY_ERROR              XLLP_CI_INTSTATUS_PAR_ERR          
#define XLLP_CAMERA_INTSTATUS_END_OF_LINE               XLLP_CI_INTSTATUS_EOL           
#define XLLP_CAMERA_INTSTATUS_TIME_OUT                  XLLP_CI_INTSTATUS_FTO              
#define XLLP_CAMERA_INTSTATUS_FIFO_UNDERRUN_3           XLLP_CI_INTSTATUS_FU_3       
#define XLLP_CAMERA_INTSTATUS_BRANCH_LOADED_0           XLLP_CI_INTSTATUS_BS0       
#define XLLP_CAMERA_INTSTATUS_BRANCH_LOADED_1           XLLP_CI_INTSTATUS_BS1       
#define XLLP_CAMERA_INTSTATUS_BRANCH_LOADED_2           XLLP_CI_INTSTATUS_BS2       
#define XLLP_CAMERA_INTSTATUS_BRANCH_LOADED_3           XLLP_CI_INTSTATUS_BS3       
#define XLLP_CAMERA_INTSTATUS_END_OF_FRAME_TRANSFER     XLLP_CI_INTSTATUS_EOFX 
#define XLLP_CAMERA_INTSTATUS_DMA_CHANNEL0_STOP         XLLP_CI_INTSTATUS_SC0     
#define XLLP_CAMERA_INTSTATUS_DMA_CHANNEL1_STOP         XLLP_CI_INTSTATUS_SC1     
#define XLLP_CAMERA_INTSTATUS_DMA_CHANNEL2_STOP         XLLP_CI_INTSTATUS_SC2     
#define XLLP_CAMERA_INTSTATUS_DMA_CHANNEL3_STOP         XLLP_CI_INTSTATUS_SC3     
#define XLLP_CAMERA_INTSTATUS_EOF_IN_FIFO3              XLLP_CI_INTSTATUS_EOF3          
#define XLLP_CAMERA_INTSTATUS_SOF_IN_FIFO0              XLLP_CI_INTSTATUS_SOF0          
#define XLLP_CAMERA_INTSTATUS_SOF_IN_FIFO1              XLLP_CI_INTSTATUS_SOF1          
#define XLLP_CAMERA_INTSTATUS_SOF_IN_FIFO2              XLLP_CI_INTSTATUS_SOF2          
#define XLLP_CAMERA_INTSTATUS_SOF_IN_FIFO3              XLLP_CI_INTSTATUS_SOF3          
#define XLLP_CAMERA_INTSTATUS_HISTGRAM_INT              XLLP_CI_INTSTATUS_HST_INT          
#define XLLP_CAMERA_INTSTATUS_COMPANDER_INT             XLLP_CI_INTSTATUS_CGC_INT         
#define XLLP_CAMERA_INTSTATUS_SUBSEQUENT_INT            XLLP_CI_INTSTATUS_SINT        
      
// Capture mode
#define XLLP_CAMERA_MODE_VIDEO             0x0001
#define XLLP_CAMERA_MODE_STILL             0x0002

// Light
#define XLLP_CAMERA_LIGHT_NORMAL           0x0001
#define XLLP_CAMERA_LIGHT_LOW              0x0002

// buffer type
#define XLLP_VIDEO_CAPTURE_BUFFER             0
#define XLLP_STILL_CAPTURE_BUFFER             1

// clean buffer identifier
#define XLLP_CAMERA_CLEAN_BUFFER_IDENTIFIER   0x12345678

// XLLP_CAMERA_CAPTURE_SCALE_DISABLE
#define XLLP_CAMERA_CAPTURE_SCALE_DISABLE     XLLP_CI_SSU_SCALE_DISABLE
#define XLLP_CAMERA_CAPTURE_SCALE_HALF        XLLP_CI_SSU_SCALE_HALF
#define XLLP_CAMERA_CAPTURE_SCALE_QUATER      XLLP_CI_SSU_SCALE_QUARTER

// sensor capabilities
#define XLLP_SENSOR_CAP_MANUAL_CONTRAST       0x0001
#define XLLP_SENSOR_CAP_MANUAL_WHITEBALANCE   0x0002
#define XLLP_SENSOR_CAP_MANUAL_EXPOSURE       0x0004
#define XLLP_SENSOR_CAP_MANUAL_ZOOM           0x0008

// sensor power mode
#define XLLP_CAMERA_POWER_OFF                 0
#define XLLP_CAMERA_POWER_LOW                 1
#define XLLP_CAMERA_POWER_MID                 2
#define XLLP_CAMERA_POWER_FULL                3

// sensor contrast mode and value
#define XLLP_SENSOR_MANUAL_CONTRAST           0
#define XLLP_SENSOR_AUTO_CONTRAST             1

#define XLLP_SENSOR_CONTRAST_LOWEST           0
#define XLLP_SENSOR_CONTRAST_LOW              1
#define XLLP_SENSOR_CONTRAST_MIDDLE           2
#define XLLP_SENSOR_CONTRAST_HIGH             3
#define XLLP_SENSOR_CONTRAST_HIGHEST          4

// sensor white balance mode and value
#define XLLP_SENSOR_MANUAL_WHITEBALANCE       0
#define XLLP_SENSOR_AUTO_WHITEBALANCE         1

#define XLLP_SENSOR_WHITEBALANCE_AUTO         0
#define XLLP_SENSOR_WHITEBALANCE_INCANDESCENT 1
#define XLLP_SENSOR_WHITEBALANCE_SUNNY        2
#define XLLP_SENSOR_WHITEBALANCE_FLUORESCENT  3

// sensor exposure mode and value
#define XLLP_SENSOR_MANUAL_EXPOSURE           0
#define XLLP_SENSOR_AUTO_EXPOSURE             1

#define XLLP_SENSOR_EXPOSURE_LOWEST           0
#define XLLP_SENSOR_EXPOSURE_LOW              1
#define XLLP_SENSOR_EXPOSURE_MIDDLE           2
#define XLLP_SENSOR_EXPOSURE_HIGH             3
#define XLLP_SENSOR_EXPOSURE_HIGHEST          4

// Color type for Histogram info
#define XLLP_CAMERA_HISTO_RED               XLLP_CI_HISTO_RED
#define XLLP_CAMERA_HISTO_BULE              XLLP_CI_HISTO_BLUE
#define XLLP_CAMERA_HISTO_GREEN1            XLLP_CI_HISTO_GREEN1
#define XLLP_CAMERA_HISTO_GREEN2            XLLP_CI_HISTO_GREEN2

// LCD synchronization
#define XLLP_CAMERA_LCD_HANSHAKING_DISABLED XLLP_CI_LCD_HANSHAKING_DISABLED
#define XLLP_CAMERA_LCD_BASE_LAYER          XLLP_CI_LCD_BASE_LAYER
#define XLLP_CAMERA_LCD_OVERLAY1_LAYER      XLLP_CI_LCD_OVERLAY1_LAYER
#define XLLP_CAMERA_LCD_OVERLAY2_LAYER      XLLP_CI_LCD_OVERLAY2_LAYER

// Buffer Pool Information
#define XLLP_MAX_CAMERA_FRAME_BUFFERS                        32
                                                            // Max Buffer number that XLLP Camera Buffer Pool 
                                                            // has

#define XLLP_PHANTOM_BUFFER_SIZE                             64
                                                            // First 32bytes: phantom buffer
                                                            // Last 16 bytes: init dma descriptor for all channels
                                                            // 32 bytes are enough: clear DCMD[IncTrgAddr] of 
                                                            // phantom buffer's DMA descriptor so as to save 
                                                            // memory, since phantom buffer is useless.

// Frame Buffer Information
#define XLLP_SPECIAL_DMA_DESC_NUM_PER_CHANNEL_PER_FRAME      4
                                                            // each channel of frame buffer has 3 
                                                            // special DMA descriptors, two for real 
                                                            // buffer, another for phantom buffer.

#define XLLP_FIRST_DMA_DESC_FOR_REAL_BUFFER_INX              0 
                                                            // the first DMA descriptor of real 
                                                            // buffer
                            
#define XLLP_LAST_DMA_DESC_FOR_REAL_BUFFER_INX               1 
                                                            // the last DMA descriptor of real 
                                                            // buffer
                            
#define XLLP_FIRST_DMA_DESC_FOR_PHANTOM_BUFFER_INX           2 
                                                            // the first DMA descriptor of 
                                                            // phantom buffer

#define XLLP_LAST_DMA_DESC_FOR_PHANTOM_BUFFER_INX            3 
                                                            // the last DMA descriptor of 
                                                            // phantom buffer
                            
//////////////////////////////////////////////////////////////////////////////////////
//
//          Structures
//
//////////////////////////////////////////////////////////////////////////////////////

typedef struct XLLP_Camera_Context_S XLLP_Camera_Context_T, *P_XLLP_Camera_Context_T;

typedef struct {
    XLLP_STATUS_T (*init)(P_XLLP_Camera_Context_T camera_context);
    XLLP_STATUS_T (*deinit)(P_XLLP_Camera_Context_T camera_context);   
    XLLP_STATUS_T (*set_capture_format)( P_XLLP_Camera_Context_T camera_context );
    XLLP_STATUS_T (*start_capture)( P_XLLP_Camera_Context_T camera_context , unsigned int frames);
    XLLP_STATUS_T (*stop_capture)( P_XLLP_Camera_Context_T camera_context );
    XLLP_STATUS_T (*sleep)( P_XLLP_Camera_Context_T camera_context );
    XLLP_STATUS_T (*wakeup)( P_XLLP_Camera_Context_T camera_context );
    
    //  functions for sensor control.
    XLLP_STATUS_T (*read_8bit)( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T reg_addr, XLLP_UINT8_T *reg_val);
    XLLP_STATUS_T (*write_8bit)( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T reg_addr, XLLP_UINT8_T reg_val);
    XLLP_STATUS_T (*read_16bit)( P_XLLP_Camera_Context_T camera_context, XLLP_UINT16_T reg_addr, XLLP_UINT16_T *reg_val);
    XLLP_STATUS_T (*write_16bit)( P_XLLP_Camera_Context_T camera_context, XLLP_UINT16_T reg_addr, XLLP_UINT16_T reg_val);
    XLLP_STATUS_T (*read_32bit)( P_XLLP_Camera_Context_T camera_context, XLLP_UINT32_T reg_addr, XLLP_UINT32_T *reg_val);
    XLLP_STATUS_T (*write_32bit)( P_XLLP_Camera_Context_T camera_context, XLLP_UINT32_T reg_addr, XLLP_UINT32_T reg_val);
    XLLP_STATUS_T (*set_power_mode)(P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T power_mode);
    XLLP_STATUS_T (*set_contrast) ( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T mode, XLLP_UINT32_T value);
    XLLP_STATUS_T (*set_whitebalance) (P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T mode, XLLP_UINT32_T value);
    XLLP_STATUS_T (*set_exposure) (P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T mode, XLLP_UINT32_T value );
    XLLP_STATUS_T (*set_zoom) (P_XLLP_Camera_Context_T camera_context, XLLP_UINT32_T value);
} XLLP_Camera_Function_T, *P_XLLP_Camera_Function_T;


typedef struct {
    unsigned char power_mode;
    
    unsigned int caps;
    //  1 << 0:    Manual ContrastMode supported by sensor;
    //  1 << 1:    Manual whitebalance supported by sensor;
    //  1 << 2:    Manual exposure supported by sensor;
    //  1 << 3:    Manual Zoom supported by sensor;

    unsigned char contrast_mode;
    unsigned char whitebalance_mode;
    unsigned char exposure_mode;
    
    unsigned int contrast_value;
    unsigned int whitebalance_value;
    unsigned int exposure_value;
    unsigned int zoom_value;
} XLLP_Sensor_Status_T, *P_XLLP_Sensor_Status_T; 

typedef struct XLLP_Camera_Frame_Buffer_Info_S XLLP_Camera_Frame_Buffer_Info_T, *P_XLLP_Camera_Frame_Buffer_Info_T;
typedef struct XLLP_Camera_Frame_Buffer_Queue_S XLLP_Camera_Frame_Buffer_Queue_T, *P_XLLP_Camera_Frame_Buffer_Queue_T;

struct XLLP_Camera_Frame_Buffer_Info_S {
    //
    // Information for frame buffer itself
    //
    int                                 frame_id;
                                        // Frame ID, indicates this buffer's 
                                        // position in the buffer pool.

    XLLP_VUINT32_T                     *buffer_vir_addr;
                                        // the virtual address of this buffer, 
                                        // should be 1-page aligned. the physical 
                                        // address of the buffer is not defined here 
                                        // because the physical address may be an 
                                        // array and should be maintained by camera 
                                        // driver to save memory.
                
    int                                 buffer_size;       
                                        // the buffer size, can be obtained via 
                                        // XllpCameraGetBufferSize( ) before 
                                        // allocating the memory for the buffer.

    XLLP_CI_DMAC_DESCRIPTOR_T          *dma_descriptors_virtual;
                                        // the virtual address of the memory for DMA descriptors 
                                        // for this buffer, should be 16-bytes aligned.

    void                               *pY;
                                        //  the virtual address of the Y component
                 
    void                               *pCb;
                                        //  the virtual address of the Cb component
                
    void                               *pCr;
                                        //  the virtual address of the Cr component

    //
    // Information of the DMA descriptor Chains for this frame buffer
    //
    
    XLLP_VUINT32_T                      ch0_dma_desc_phy_addr[XLLP_SPECIAL_DMA_DESC_NUM_PER_CHANNEL_PER_FRAME]; 
                                        // pointer to an array of physical address of 
                                        // channel 0's DMA chain for this buffer;
    XLLP_CI_DMAC_DESCRIPTOR_T          *ch0_dma_desc_vir_addr[XLLP_SPECIAL_DMA_DESC_NUM_PER_CHANNEL_PER_FRAME]; 
                                        // pointer to an array of virtual address of 
                                        // channel 0's DMA chain for this buffer;
    int                                 ch0_dma_desc_num;     
                                        // DMA descriptors number for channel 0
    int                                 ch0_phantom_dma_desc_num;     
                                        // DMA descriptors number of phantom buffer for channel 0

    XLLP_VUINT32_T                      ch1_dma_desc_phy_addr[XLLP_SPECIAL_DMA_DESC_NUM_PER_CHANNEL_PER_FRAME]; 
                                        // pointer to an array of physical address of 
                                        // channel 1's DMA chain for this buffer;
    XLLP_CI_DMAC_DESCRIPTOR_T          *ch1_dma_desc_vir_addr[XLLP_SPECIAL_DMA_DESC_NUM_PER_CHANNEL_PER_FRAME]; 
                                        // pointer to an array of virtual address of 
                                        // channel 1's DMA chain for this buffer;
    int                                 ch1_dma_desc_num ;     
                                        // DMA descriptors number for channel 1
    int                                 ch1_phantom_dma_desc_num;     
                                        // DMA descriptors number of phantom buffer for channel 1

    XLLP_VUINT32_T                      ch2_dma_desc_phy_addr[XLLP_SPECIAL_DMA_DESC_NUM_PER_CHANNEL_PER_FRAME]; 
                                        // pointer to an array of physical address of 
                                        // channel 2's DMA chain for this buffer;
    XLLP_CI_DMAC_DESCRIPTOR_T          *ch2_dma_desc_vir_addr[XLLP_SPECIAL_DMA_DESC_NUM_PER_CHANNEL_PER_FRAME]; 
                                        // pointer to an array of virtual address of 
                                        // channel 2's DMA chain for this buffer;
    int                                 ch2_dma_desc_num;
                                        // DMA descriptors number for channel 2
    int                                 ch2_phantom_dma_desc_num;     
                                        // DMA descriptors number of phantom buffer for channel 2

    // 
    // Next buffer in the queue
    // 
    XLLP_Camera_Frame_Buffer_Info_T    *next_buffer;
                                        // next buffer in the queue, only used in 
                                        // video capture buffer queue or still 
                                        // capture buffer queue.
};

struct XLLP_Camera_Frame_Buffer_Queue_S {
     XLLP_Camera_Frame_Buffer_Info_T   *head;
                                        // first frame buffer owned by the queue. When 
                                        // this buffer will be reported the 
                                        // application when it is filled with frame 
                                        // data, then this pointer will point to the 
                                        // next buffer in the queue. If the queue runs 
                                        // out of its buffers, this pointer will be 
                                        // NULL until one new free buffer added into 
                                        // the queue when this pointer will point to 
                                        // this new buffer.
                
     XLLP_Camera_Frame_Buffer_Info_T   *tail;
                                        // last frame buffer in the queue, always 
                                        // followed by the phantom buffer. When there 
                                        // is no frame available in the queue, the DMA 
                                        // controller will transfer the frame data to 
                                        // its phantom buffer until the Camera driver 
                                        // submits new frames into the queue. The 
                                        // frame complete interrupt for phantom buffer 
                                        // should be ignored.
};

// context 
struct XLLP_Camera_Context_S {
    //
    // DRIVER FILLED PARAMTER
    //

    // sensor information.
    unsigned int                       sensor_type;
                                        // defines the imagine sensor type connected to the camera 
                                        // interface. For example, 
                                        // XLLP_CAMERA_TYPE_ OMNIVISION_2620 selected the Agilent 
                                        // 2620 sensor available on the Zylonite platform

    // capture image information.
    unsigned int                       capture_mode; 
                                        // video capture or still capture

    unsigned int                       video_capture_width;
    unsigned int                       video_capture_height;
                                        // video_capture_width and video_capture_height define the 
                                        // desired width and height of the motion video capture 
                                        // stream.  Knowledge of the particular sensor is required 
                                        // in order to specify a valid width and height.  Some 
                                        // sensors support arbitrary width and height up to the 
                                        // maximum sensor dimensions.  Other sensors support only 
                                        // subset of possible sizes.

    unsigned int                       video_capture_scale;
                                        // defines the scaling rate of sensor output vs QCI output. 
                                        // possible choices: 1:1(default) or 2:1 or 4:1.However, if there
                                        // is need to convert RAW data to YUV/RGB, then this field must be
                                        // set as 2:1 or 4:1

    unsigned int                       video_capture_input_format;
                                        // video_capture_input_format describes the data format for 
                                        // motion video that the sensor provides to the camera 
                                        // interface.  For example, 
                                        // XLLP_CAMERA_IMAGE_FORMAT_RAW10 is 
                                        // used to describe a 10-bits RAW data format, such as that 
                                        // used by the OV2620.

    unsigned int                       video_capture_output_format;
                                        // video_capture_output_format describes the data format 
                                        // for motion video that the camera interface will use to 
                                        // source data to the DMA controller.  The output format 
                                        // need not be the same as the input format.  The Monahans 
                                        // camera interface provides some limited conversion 
                                        // mechanisms.  For example, input data of 
                                        // XLLP_CAMERA_IMAGE_FORMAT_RAW10 may be converted to 
                                        // XLLP_CAMERA_IMAGE_FORMAT_YCBCR422_PLANAR by specifying 
                                        // that planar format.

    unsigned int                       still_capture_width;
    unsigned int                       still_capture_height;
                                        // still_capture_width and still_capture_height define the 
                                        // desired width and height of the still image capture. 
                                        // Knowledge of the particular sensor is required 
                                        // in order to specify a valid width and height.  Some 
                                        // sensors support arbitrary width and height up to the 
                                        // maximum sensor dimensions.  Other sensors support only 
                                        // subset of possible sizes.

    unsigned int                       still_capture_scale;
                                        // defines the scaling rate of sensor output vs QCI output. 
                                        // possible choices: 1:1(default) or 2:1 or 4:1.However, if there
                                        // is need to convert RAW data to YUV/RGB, then this field must be
                                        // set as 2:1 or 4:1

    unsigned int                       still_capture_input_format;
                                        // still_capture_input_format describes the data format for 
                                        // still image that the sensor provides to the camera 
                                        // interface.  For example, 
                                        // XLLP_CAMERA_IMAGE_FORMAT_RAW10 is 
                                        // used to describe a 10-bits RAW data format, such as that 
                                        // used by the OV2620.

    unsigned int                       still_capture_output_format;
                                        // still_capture_output_format describes the data format 
                                        // for still image that the camera interface will use to 
                                        // source data to the DMA controller.  The output format 
                                        // need not be the same as the input format.  The Monahans 
                                        // camera interface provides some limited conversion 
                                        // mechanisms.  For example, input data of 
                                        // XLLP_CAMERA_IMAGE_FORMAT_RAW10 may be converted to 
                                        // XLLP_CAMERA_IMAGE_FORMAT_YCBCR422_PLANAR by specifying 
                                        // that planar format.

    // frame rate control
    unsigned int                       frame_rate;
                                        // frame_rate specifies the desired frame rate control.  
                                        // Use a 0 to specify the maximum frame rate.  For n > 0, 
                                        // it specifies that the interface should capture 1 out of 
                                        // every n+1 frames generated.  A future implementation may 
                                        // change the interpretation of the frame rate in order to 
                                        // specify the frames per second.
    
    // memory for phantom buffer
    XLLP_VUINT32_T                    *phantom_buffer_virtual;
    XLLP_VUINT32_T                     phantom_buffer_physical;
                                        // buffer with XLLP_PHANTOM_BUFFER_SIZE bytes, 
                                        // allocated by camera driver, used as phantom buffer by all 
                                        // buffers, should be 16-bytes aligned. 

    // memory for getting Histogram or loading CGU LUT.
    XLLP_VUINT32_T                    *histogram_lut_buffer_virtual;
    XLLP_VUINT32_T                     histogram_lut_buffer_physical;
                                        // buffer with 1024 bytes, allocated by camera 
                                        // driver, used for getting Histogram(512 or 1024bytes) or 
                                        // loading CGU LUT(64*3bytes), should be 16-bytes aligned.

    // memory for DMA descriptors to get Histogram or load CGU LUT.
    XLLP_CI_DMAC_DESCRIPTOR_T         *histogram_lut_dma_descriptors_virtual;
    XLLP_VUINT32_T                     histogram_lut_dma_descriptors_physical;
                                        // memory for DMA descriptors allocated by Camera driver, 
                                        // 16-bytes aligned, should be 16-byte sized. This 
                                        // descriptor is required to get histogram data or load 
                                        // Compander look-up-table (CGU LUT).

    // os mapped register address       
    unsigned int                       ost_reg_base;
                                        // specifies the virtual address of the OST registers.

    unsigned int                       gpio_reg_base;
                                        // specifies the virtual address of the GPIO registers.

    unsigned int                       ci_reg_base;
                                        // specifies the virtual address of the CAMERA INTERFACE 
                                        // registers. 

    unsigned int                       clk_reg_base;
                                        // specifies the virtual address of the Clock registers 
                                        // registers.                                         

    unsigned int                       mfp_reg_base;
                                        // specifies the virtual address of the MFP registers.

    P_XLLP_MFP_RM_DB_ID_T              mfp_rm_db;
                                        // specifies the MFP Resource Manager Database

    // function dispatch table
    P_XLLP_Camera_Function_T           camera_functions;
                                        // specify pointers to the given camera sensor functions.

    //
    // XLLP INTERNALLY USED: DON'T TOUCH!
    //

    // Sensor status
    XLLP_Sensor_Status_T               sensor_status;
                                        // sensor related parameter or status will be put here.

    // capture parameters:
    //    those parameters are being used by XLLP Camera currently
    //    for video capture or still capture, generated from 
    //    driver-filled parameters according to the parameter 
    //    capture_mode.
    unsigned int                       capture_input_width; 
    unsigned int                       capture_input_height;
                                        // current image resolution of the sensor output or QCI 
                                        // input, may be the same as QCI output, may be not, 
                                        // depending on the capture scale.

    unsigned int                       capture_output_width; 
    unsigned int                       capture_output_height;
                                        // current image resolution of the QCI output.

    unsigned int                       capture_input_format;
                                        // current image format of the sensor output or QCI input.

    unsigned int                       capture_output_format;
                                        // current image format of the QCI output.

    unsigned int                       fifo0_transfer_size;
    unsigned int                       fifo1_transfer_size;
    unsigned int                       fifo2_transfer_size;
                                        // above three parameters indicate the current transfer size 
                                        // of channel 0, channel 1 and channel 2, respectively.
                                        // they would be calculated from the format and 
                                        // resolution of the current QCI output.


    unsigned int                       fifo3_transfer_size; 
                                        // transfer size of channel 3. This parameter is for 
                                        // reading Histogram only, may be 512 or 1024, depending on 
                                        // the CIHST.SCALE. For loading CGU LUT, the transfer size 
                                        // will be hard coded as 64*3 bytes, since the CGU LUT 
                                        // consists of three 64-element Look-Up-Tables.
                                        
    unsigned int                       video_fifo0_transfer_size;
    unsigned int                       video_fifo1_transfer_size;
    unsigned int                       video_fifo2_transfer_size;
                                        // above three parameters indicate the transfer size 
                                        // of channel 0, channel 1 and channel 2 for video capture, respectively.
                                        // they would be calculated from the format and 
                                        // resolution of the vide capture QCI output.

    unsigned int                       still_fifo0_transfer_size;
    unsigned int                       still_fifo1_transfer_size;
    unsigned int                       still_fifo2_transfer_size;
                                        // above three parameters indicate the transfer size 
                                        // of channel 0, channel 1 and channel 2 for still capture, respectively.
                                        // they would be calculated from the format and 
                                        // resolution of the still capture QCI output.

    // frame buffer pool
    XLLP_Camera_Frame_Buffer_Info_T    master_frame_buffer_list[XLLP_MAX_CAMERA_FRAME_BUFFERS];
                                        // frame buffer pool is maintained within the XLLP camera. 
                                        // When the application or camera driver is initialized, the 
                                        // frame buffer will be allocated, too. Those frame buffers 
                                        // will be added to this frame buffer pool. Afterward the 
                                        // application/camera driver will get buffers from this pool 
                                        // and submits them to the still capture buffer queue or the 
                                        // video capture buffer queue.
                                        
    int                                 frame_buffer_number;
                                        // indicates how many buffers the buffer pool currently has.
    
    // still capture buffer queue.
    XLLP_Camera_Frame_Buffer_Queue_T   still_capture_buffer_queue;
                                        // frame buffer queue used by still capture, its frame 
                                        // buffers come from frame buffer pool. Generally, this 
                                        // queue only has one buffer.

    // video capture buffer queue.
    XLLP_Camera_Frame_Buffer_Queue_T   video_capture_buffer_queue;
                                        // frame buffer queue used by video capture, its frame 
                                        // buffers come from frame buffer pool, too.

    // QCI status.
    unsigned int                       ci_disable_complete;
                                        // indicates whether QCI disabling is completed.

    XLLP_BOOL_T                        psu_enable;
                                        // indicates whether PSU is enabled.

    XLLP_BOOL_T                        cgu_enable;
                                        // indicates whether the CGU is enabled.

    XLLP_CI_SSU_SCALE                  ssu_scale;
                                        // indicates the SSU scale ratio.
                                        
    XLLP_CI_CMU_USAGE                  cmu_usage;
                                        // indicates how to use CMU. This varible will tell the sensor 
                                        // that it should choose which CMU matrix to load    

    XLLP_BOOL_T                        dma_running;
};


//////////////////////////////////////////////////////////////////////////////////////
//
//          Prototypes
//
//////////////////////////////////////////////////////////////////////////////////////

/***********************************************************************
 *
 * Init/Deinit APIs
 *
 ***********************************************************************/
// Setup the sensor type, configure image capture format (RGB, yuv 444, yuv 422, yuv 420, packed | planar, MJPEG) regardless
// of current operating mode (i.e. sets mode for both still capture and video capture)
XLLP_STATUS_T XllpCameraInit( P_XLLP_Camera_Context_T camera_context );

// Power off sensor
XLLP_STATUS_T XllpCameraDeInit( P_XLLP_Camera_Context_T camera_context );


/***********************************************************************
 *
 * Capture APIs
 *
 ***********************************************************************/
// Set the image format
XLLP_STATUS_T XllpCameraSetCaptureFormat( P_XLLP_Camera_Context_T camera_context );

// take a picture and copy it into the frame buffer
XLLP_STATUS_T XllpCameraCaptureStillImage( P_XLLP_Camera_Context_T camera_context );

// capture motion video and copy it the frame buffer
XLLP_STATUS_T XllpCameraStartVideoCapture( P_XLLP_Camera_Context_T camera_context );

// disable motion video image capture
void XllpCameraStopVideoCapture( P_XLLP_Camera_Context_T camera_context );


/***********************************************************************
 *
 * Flow Control APIs
 *
 ***********************************************************************/
// Get the buffer size and the DMA descriptors memory size for this buffer.
XLLP_STATUS_T XllpCameraGetBufferSize(
                P_XLLP_Camera_Context_T   camera_context,
                                    int   buffer_type,
                                    int  *buffer_size,
                                    int  *buffer_dma_desc_mem_size);

// Add the buffer into the buffer pool
XLLP_STATUS_T XllpCameraPrepareBuffer(
                P_XLLP_Camera_Context_T   camera_context,
                                  void   *buffer_virtual_address,
                                   int   *buffer_physical_address_array,
                                   int    buffer_physical_address_array_size,
                                   int    buffer_size,
                                   int    buffer_type,
                                  void   *dma_desc_mem_virtual_address,
                                   int   *dma_desc_mem_physical_address_array,
                                   int   *buffer_id,
                                  void   **pY,
                                  void   **pCb,
                                  void   **pCr);

// Submit a buffer into the capture queue.
XLLP_STATUS_T XllpCameraSubmitBuffer(
            P_XLLP_Camera_Context_T  camera_context,
                                int  buffer_id,
                                int  buffer_type);

// Get the buffer filled with valid frame data.
XLLP_STATUS_T XllpCameraGetFilledBuffer(
                P_XLLP_Camera_Context_T  camera_context,
                                    int *buffer_id);


/***********************************************************************
 *
 * Frame rate APIs
 *
 ***********************************************************************/
// Set desired frame rate
void XllpCameraSetCaptureFrameRate( P_XLLP_Camera_Context_T camera_context ); 

// return current setting
void XllpCameraGetCaptureFrameRate( P_XLLP_Camera_Context_T camera_context ); 

/***********************************************************************
 *
 * Interrupt APIs
 *
 ***********************************************************************/
// set interrupt mask 
void XllpCameraSetInterruptMask( P_XLLP_Camera_Context_T camera_context, unsigned int mask ); 

// get interrupt mask 
unsigned int XllpCameraGetInterruptMask( P_XLLP_Camera_Context_T camera_context ); 

// get interrupt status
unsigned int XllpCameraGetInterruptStatus( P_XLLP_Camera_Context_T camera_context );

// clear interrupt status
void XllpCameraClearInterruptStatus( P_XLLP_Camera_Context_T camera_context, unsigned int status );


/***********************************************************************
 *
 * Sensor Control APIs
 *
 ***********************************************************************/

// CMOS sensor 8 bit register read
XLLP_STATUS_T XllpCameraRead8Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T reg_addr, XLLP_UINT8_T *reg_val );

// CMOS sensor 8 bit register write
XLLP_STATUS_T XllpCameraWrite8Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T reg_addr, XLLP_UINT8_T reg_val );

// CMOS sensor 16-bit register read
XLLP_STATUS_T XllpCameraRead16Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT16_T reg_addr, XLLP_UINT16_T *reg_val );

// CMOS sensor 16 bit register write
XLLP_STATUS_T XllpCameraWrite16Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT16_T reg_addr, XLLP_UINT16_T reg_val );

// CMOS sensor 32 bit register read
XLLP_STATUS_T XllpCameraRead32Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT32_T reg_addr, XLLP_UINT32_T *reg_val );

// CMOS sensor 32 bit register write
XLLP_STATUS_T XllpCameraWrite32Bit( P_XLLP_Camera_Context_T camera_context, XLLP_UINT32_T reg_addr, XLLP_UINT32_T reg_val );

// CMOS sensor Power Mode read
void XllpCameraGetPowerMode ( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T *power_mode );

// CMOS sensor Power Mode write
XLLP_STATUS_T XllpCameraSetPowerMode ( P_XLLP_Camera_Context_T camera_context, XLLP_UINT8_T power_mode );

// CMOS sensor Capability read
unsigned int XllpCameraGetCaps ( P_XLLP_Camera_Context_T camera_context);

// CMOS sensor contrast value read
void XllpCameraGetContrastValue ( P_XLLP_Camera_Context_T camera_context, unsigned char *mode, unsigned int *value);

// CMOS sensor contrast value write
XLLP_STATUS_T XllpCameraSetContrastValue ( P_XLLP_Camera_Context_T camera_context, unsigned char mode, unsigned int value);

// CMOS sensor white balance value read
void XllpCameraGetWhiteBalanceValue (P_XLLP_Camera_Context_T camera_context, unsigned char *mode, unsigned int *value);

// CMOS sensor white balance value write
XLLP_STATUS_T XllpCameraSetWhiteBalanceValue (P_XLLP_Camera_Context_T camera_context, unsigned char mode, unsigned int value);

// CMOS sensor exposure value read
void XllpCameraGetExposureValue (P_XLLP_Camera_Context_T camera_context, unsigned char *mode, unsigned int *value);

// CMOS sensor exposure value write
XLLP_STATUS_T XllpCameraSetExposureValue (P_XLLP_Camera_Context_T camera_context, unsigned char mode, unsigned int value);

// CMOS sensor zoom value read
void XllpCameraGetZoomValue (P_XLLP_Camera_Context_T camera_context, unsigned int *value);

// CMOS sensor zoom value write
XLLP_STATUS_T XllpCameraSetZoomValue (P_XLLP_Camera_Context_T camera_context, unsigned int value);

// get Histogram Info
XLLP_STATUS_T XllpCameraGetHistogramInfo(
    P_XLLP_Camera_Context_T     camera_context,
    unsigned int                color_type,
    unsigned int               *histogram_size,
    unsigned int               *histogram_sum);

// synchronize the CI with LCD
XLLP_STATUS_T XllpCameraSyncToLCD(
    P_XLLP_Camera_Context_T    camera_context,
    unsigned int               lcd_layer_type);

// sleep Xllp camera
XLLP_STATUS_T XllpCameraSuspend(P_XLLP_Camera_Context_T    camera_context);

// wakeup Xllp camera
XLLP_STATUS_T XllpCameraResume(P_XLLP_Camera_Context_T    camera_context);

#ifdef __cplusplus
}
#endif

#endif


