/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/*++

Module Name:  $Workfile: xllp_dmac.h $

Abstract:  
    contains all DMA Controller specific macros, typedefs, and prototypes.
    Declares no storage.

Notes:Only valid for processor code named Monahans.

--*/
#ifdef __cplusplus
extern "C" {
#endif

#ifndef __XLLP_DMAC_H__
#define __XLLP_DMAC_H__

#include "xllp_defs.h"

/**
 * DCSR Register 
 **/
#define XLLP_DMAC_DCSR_BUS_ERR_INTR       (1U<<0)
#define XLLP_DMAC_DCSR_START_INTR         (1U<<1)
#define XLLP_DMAC_DCSR_END_INTR           (1U<<2)
#define XLLP_DMAC_DCSR_STOP_INTR          (1U<<3)
#define XLLP_DMAC_DCSR_RAS_INTR           (1U<<4)
#define XLLP_DMAC_DCSR_REQ_PEND           (1U<<8)
#define XLLP_DMAC_DCSR_EOR_INTR           (1U<<9)
#define XLLP_DMAC_DCSR_MASK_RUN           (1U<<22)
#define XLLP_DMAC_DCSR_RAS_EN             (1U<<23)
#define XLLP_DMAC_DCSR_EOR_STOP_EN        (1U<<26)
#define XLLP_DMAC_DCSR_EOR_JMP_EN         (1U<<27)
#define XLLP_DMAC_DCSR_EOR_IRQ_EN         (1U<<28)
#define XLLP_DMAC_DCSR_STOP_IRQ_EN        (1U<<29)
#define XLLP_DMAC_DCSR_NO_DESC_FETCH      (1U<<30)
#define XLLP_DMAC_DCSR_RUN                (1U<<31)

/**
 * DCMD Register
 **/
#define XLLP_DMAC_DCMD_LEN                  (1U<<0)
#define XLLP_DMAC_DCMD_WIDTH                (1U<<14)
#define XLLP_DMAC_DCMD_SIZE                 (1U<<16)
#define XLLP_DMAC_DCMD_FLY_BY_T             (1U<<19)
#define XLLP_DMAC_DCMD_FLY_BY_S             (1U<<20)
#define XLLP_DMAC_DCMD_END_IRQ_EN           (1U<<21)
#define XLLP_DMAC_DCMD_START_IRQ_EN         (1U<<22)
#define XLLP_DMAC_DCMD_ADDR_MODE            (1U<<23)
#define XLLP_DMAC_DCMD_CMP_EN               (1U<<25)
#define XLLP_DMAC_DCMD_FLOW_TRG             (1U<<28)
#define XLLP_DMAC_DCMD_FLOW_SRC             (1U<<29)
#define XLLP_DMAC_DCMD_INC_TRG_ADDR         (1U<<30)
#define XLLP_DMAC_DCMD_INC_SRC_ADDR         (1U<<31)


/**
 * BIT FIELDS
 **/
#define XLLP_BIT_FIELD_0    0
#define XLLP_BIT_FIELD_1    1
#define XLLP_BIT_FIELD_2    2
#define XLLP_BIT_FIELD_3    3
#define XLLP_BIT_FIELD_4    4
#define XLLP_BIT_FIELD_5    5
#define XLLP_BIT_FIELD_6    6
#define XLLP_BIT_FIELD_7    7
#define XLLP_BIT_FIELD_8    8
#define XLLP_BIT_FIELD_9    9
#define XLLP_BIT_FIELD_10   10
#define XLLP_BIT_FIELD_11   11
#define XLLP_BIT_FIELD_12   12
#define XLLP_BIT_FIELD_13   13
#define XLLP_BIT_FIELD_14   14
#define XLLP_BIT_FIELD_15   15
#define XLLP_BIT_FIELD_16   16
#define XLLP_BIT_FIELD_17   17
#define XLLP_BIT_FIELD_18   18
#define XLLP_BIT_FIELD_19   19
#define XLLP_BIT_FIELD_20   20
#define XLLP_BIT_FIELD_21   21
#define XLLP_BIT_FIELD_22   22
#define XLLP_BIT_FIELD_23   23
#define XLLP_BIT_FIELD_24   24
#define XLLP_BIT_FIELD_25   25
#define XLLP_BIT_FIELD_26   26
#define XLLP_BIT_FIELD_27   27
#define XLLP_BIT_FIELD_28   28
#define XLLP_BIT_FIELD_29   29
#define XLLP_BIT_FIELD_30   30
#define XLLP_BIT_FIELD_31   31

/**
 * DMAC DDADRx bit fields 
 **/
#define XLLP_DMAC_DDADR_STOP        (1U<<0)

/**
 * Used in DMA handler definition
 **/
#define XLLP_DMAC_CHANNEL_NUM       32
#define XLLP_DMAC_DRCMR1_NUM        64
#define XLLP_DMAC_DRCMR2_NUM        36

/**
 * Max length that can be transferred by a single DMA descriptor
 **/
#define XLLP_DCMD_LEN_MASK                  0x1FFFU    
#define XLLP_DMAC_ALIGN_MASK                0xFFFFFFF0U

#define XLLP_ALL_INTERRUPTS_STATUS (XLLP_DMAC_DCSR_BUS_ERR_INTR|\
                                    XLLP_DMAC_DCSR_START_INTR  |\
                                    XLLP_DMAC_DCSR_END_INTR    |\
                                    XLLP_DMAC_DCSR_STOP_INTR   |\
                                    XLLP_DMAC_DCSR_RAS_INTR    |\
                                    XLLP_DMAC_DCSR_EOR_INTR)

#define XLLP_ALL_INTERRUPTS_SOURCE (XLLP_DMAC_DCSR_STOP_IRQ_EN |\
                                    XLLP_DMAC_DCSR_EOR_IRQ_EN  |\
                                    XLLP_DMAC_DCSR_RAS_EN)


/**
 * Mask of all writable bits in DCSR; others must be written as 0
 **/
#define XLLP_DCSR_WRITABLES_MSK    (XLLP_DMAC_DCSR_BUS_ERR_INTR |\
                                    XLLP_DMAC_DCSR_START_INTR   |\
                                    XLLP_DMAC_DCSR_END_INTR     |\
                                    XLLP_DMAC_DCSR_RAS_INTR     |\
                                    XLLP_DMAC_DCSR_EOR_INTR     |\
                                    XLLP_DMAC_DCSR_RAS_EN       |\
                                    XLLP_DMAC_DCSR_EOR_STOP_EN  |\
                                    XLLP_DMAC_DCSR_EOR_JMP_EN   |\
                                    XLLP_DMAC_DCSR_EOR_IRQ_EN   |\
                                    XLLP_DMAC_DCSR_STOP_IRQ_EN  |\
                                    XLLP_DMAC_DCSR_NO_DESC_FETCH|\
                                    XLLP_DMAC_DCSR_RUN         )

/**
 * Mask of all write-1-clear bits in DCSR; those bits should be protected from operation of writing control bits
 **/
#define XLLP_DMAC_DCSR_WRITE_ONE_TO_CLEAR_BITS    (XLLP_DMAC_DCSR_BUS_ERR_INTR|\
                                                   XLLP_DMAC_DCSR_START_INTR  |\
                                                   XLLP_DMAC_DCSR_END_INTR    |\
                                                   XLLP_DMAC_DCSR_RAS_INTR    |\
                                                   XLLP_DMAC_DCSR_EOR_INTR)
/**
 * DMA Descriptor
 **/
typedef struct 
{
    XLLP_VUINT32_T   ddadr;  // descriptor address reg
    XLLP_VUINT32_T   dsadr;  // source address register
    XLLP_VUINT32_T   dtadr;  // target address register
    XLLP_VUINT32_T   dcmd;   // command address register
}XLLP_DMAC_DESCRIPTOR_T, *P_XLLP_DMAC_DESCRIPTOR_T;

/**
 * DMAC Register Definitions
 **/
typedef struct 
{
    XLLP_VUINT32_T dcsr[XLLP_DMAC_CHANNEL_NUM];         // DMA Control/Status Registers 0-31
    XLLP_VUINT32_T reserved0[0x9];                      // RESERVED0
    XLLP_VUINT32_T dpcsr;                               // DMA Programmed I/O Control status Regsiter
    XLLP_VUINT32_T reserved1[0xE];                      // RESERVED0
    XLLP_VUINT32_T drqsr0;                              // DMA DREQ(0) Status Register
    XLLP_VUINT32_T drqsr1;                              // DMA DREQ(1) Status Register
    XLLP_VUINT32_T drqsr2;                              // DMA DREQ(2) Status Register
    XLLP_VUINT32_T reserved2[0x1];                      // RESERVED1
    XLLP_VUINT32_T dint;                                // DMA Interrupt Register
    XLLP_VUINT32_T reserved3[0x3];                      // RESERVED2
    XLLP_VUINT32_T drcmr1[XLLP_DMAC_DRCMR1_NUM];        // Request to Channel Map for DREQ 0-63
    XLLP_DMAC_DESCRIPTOR_T ddg[XLLP_DMAC_CHANNEL_NUM];  // DMA Desc Group for channel 0-31
    XLLP_VUINT32_T reserved4[0x340];                    // RESERVED3
    XLLP_VUINT32_T drcmr2[XLLP_DMAC_DRCMR2_NUM];        // Request to Channel Map 64-99
} XLLP_DMAC_T, *P_XLLP_DMAC_T;


/**
 *  DMAC peripheral device width
 **/
typedef enum
{
    XLLP_DMAC_WIDTH_0 = 0,
    XLLP_DMAC_WIDTH_8,
    XLLP_DMAC_WIDTH_16,
    XLLP_DMAC_WIDTH_32
}XLLP_DMAC_DEVICE_WIDTH_T;

/**
 * DMAC peripheral burst size
 **/
typedef enum
{
    XLLP_DMAC_RESERVED_SIZE = 0,
    XLLP_DMAC_BURSTSIZE_8,
    XLLP_DMAC_BURSTSIZE_16,
    XLLP_DMAC_BURSTSIZE_32
}XLLP_DMAC_DEVICE_BURSTSIZE_T;

/**
 * DMAC channels
 **/
typedef enum
{
    XLLP_DMAC_CHANNEL_0 = 0,
    XLLP_DMAC_CHANNEL_1,
    XLLP_DMAC_CHANNEL_2,
    XLLP_DMAC_CHANNEL_3,
    XLLP_DMAC_CHANNEL_4,
    XLLP_DMAC_CHANNEL_5,
    XLLP_DMAC_CHANNEL_6,
    XLLP_DMAC_CHANNEL_7,
    XLLP_DMAC_CHANNEL_8,
    XLLP_DMAC_CHANNEL_9,
    XLLP_DMAC_CHANNEL_10,
    XLLP_DMAC_CHANNEL_11,
    XLLP_DMAC_CHANNEL_12,
    XLLP_DMAC_CHANNEL_13,
    XLLP_DMAC_CHANNEL_14,
    XLLP_DMAC_CHANNEL_15,
    XLLP_DMAC_CHANNEL_16,
    XLLP_DMAC_CHANNEL_17,
    XLLP_DMAC_CHANNEL_18,
    XLLP_DMAC_CHANNEL_19,
    XLLP_DMAC_CHANNEL_20,
    XLLP_DMAC_CHANNEL_21,
    XLLP_DMAC_CHANNEL_22,
    XLLP_DMAC_CHANNEL_23,
    XLLP_DMAC_CHANNEL_24,
    XLLP_DMAC_CHANNEL_25,
    XLLP_DMAC_CHANNEL_26,
    XLLP_DMAC_CHANNEL_27,
    XLLP_DMAC_CHANNEL_28,
    XLLP_DMAC_CHANNEL_29,
    XLLP_DMAC_CHANNEL_30,
    XLLP_DMAC_CHANNEL_31
}XLLP_DMAC_CHANNEL_T, *P_XLLP_DMAC_CHANNEL_T;

#define XLLP_INVALID_DMA_CHANNEL    0xFF

/**
 * DMAC DEVICE DRCMR ID's
 **/
typedef enum
{
    XLLP_DMAC_DREQ0 = 0,            // DREQ 0, Companion Chip request 0
    XLLP_DMAC_DREQ1 = 1,            // DREQ 1, Companion Chip request 1
    XLLP_DMAC_SSP_4_RX = 2,
    XLLP_DMAC_SSP_4_TX = 3,
    XLLP_DMAC_BTUART_RX = 4,
    XLLP_DMAC_BTUART_TX = 5,
    XLLP_DMAC_FFUART_RX = 6,
    XLLP_DMAC_FFUART_TX = 7,
    XLLP_DMAC_AC97_MIC = 8,
    XLLP_DMAC_AC97_MODEM_RX = 9,
    XLLP_DMAC_AC97_MODEM_TX = 10,
    XLLP_DMAC_AC97_AUDIO_RX = 11,
    XLLP_DMAC_AC97_AUDIO_TX = 12,
    XLLP_DMAC_SSP_1_RX = 13,
    XLLP_DMAC_SSP_1_TX = 14,
    XLLP_DMAC_SSP_2_RX = 15,
    XLLP_DMAC_SSP_2_TX = 16,
    XLLP_DMAC_STUART_RX = 19,
    XLLP_DMAC_STUART_TX= 20,
    XLLP_DMAC_MMC_RX = 21,
    XLLP_DMAC_MMC_TX = 22,
    XLLP_DMAC_USB_ENDPOINT_0 = 24,
    XLLP_DMAC_USB_ENDPOINT_A = 25,
    XLLP_DMAC_USB_ENDPOINT_B = 26,
    XLLP_DMAC_USB_ENDPOINT_C = 27,
    XLLP_DMAC_USB_ENDPOINT_D = 28,
    XLLP_DMAC_USB_ENDPOINT_E = 29,
    XLLP_DMAC_USB_ENDPOINT_F = 30,
    XLLP_DMAC_USB_ENDPOINT_G = 31,
    XLLP_DMAC_USB_ENDPOINT_H = 32,
    XLLP_DMAC_USB_ENDPOINT_I = 33,
    XLLP_DMAC_USB_ENDPOINT_J = 34,
    XLLP_DMAC_USB_ENDPOINT_K = 35,
    XLLP_DMAC_USB_ENDPOINT_L = 36,
    XLLP_DMAC_USB_ENDPOINT_M = 37,
    XLLP_DMAC_USB_ENDPOINT_N = 38,
    XLLP_DMAC_USB_ENDPOINT_P = 39,
    XLLP_DMAC_USB_ENDPOINT_Q = 40,
    XLLP_DMAC_USB_ENDPOINT_R = 41,
    XLLP_DMAC_USB_ENDPOINT_S = 42,
    XLLP_DMAC_USB_ENDPOINT_T = 43,
    XLLP_DMAC_USB_ENDPOINT_U = 44,
    XLLP_DMAC_USB_ENDPOINT_V = 45,
    XLLP_DMAC_USB_ENDPOINT_W = 46,
    XLLP_DMAC_USB_ENDPOINT_X = 47,
    XLLP_DMAC_BASEBAND_1_RX = 48,
    XLLP_DMAC_BASEBAND_1_TX = 49,
    XLLP_DMAC_BASEBAND_2_RX = 50,
    XLLP_DMAC_BASEBAND_2_TX = 51,
    XLLP_DMAC_BASEBAND_3_RX = 52,
    XLLP_DMAC_BASEBAND_3_TX = 53,
    XLLP_DMAC_BASEBAND_4_RX = 54,
    XLLP_DMAC_BASEBAND_4_TX = 55,
    XLLP_DMAC_BASEBAND_5_RX = 56,
    XLLP_DMAC_BASEBAND_5_TX = 57,
    XLLP_DMAC_BASEBAND_6_RX = 58,
    XLLP_DMAC_BASEBAND_6_TX = 59,
    XLLP_DMAC_BASEBAND_7_RX = 60,
    XLLP_DMAC_BASEBAND_7_TX = 61,
    XLLP_DMAC_USIM_RX = 62,
    XLLP_DMAC_USIM_TX = 63,
    XLLP_DMAC_SSP_3_RX = 66,
    XLLP_DMAC_SSP_3_TX = 67,
    XLLP_DMAC_TPM_RX = 71,
    XLLP_DMAC_TPM_TX_1 = 72,
    XLLP_DMAC_TPM_TX_2 = 73,
    XLLP_DMAC_DREQ2 = 74,
    XLLP_DMAC_USIM_2_RX = 91,
    XLLP_DMAC_USIM_2_TX = 92,
    XLLP_DMAC_MMC_2_RX = 93,
    XLLP_DMAC_MMC_2_TX = 94,
    XLLP_DMAC_AC97_SURROUND_TX = 95,
    XLLP_DMAC_CENTRE_LFE_TX = 96,
    XLLP_DMAC_NAND_DATA = 97,
    XLLP_DMAC_NAND_CMD = 99,
    XLLP_DMAC_MEM2MEM_MOVE = 255  // RESERVED for Memory to Memory moves
}XLLP_DMAC_DRCMR_T, XLLP_DMAC_DEVICE_T;


/**
 * DMAC Interrupts
 **/
typedef enum
{
    XLLP_DMAC_BUS_ERR_INT = 0,
    XLLP_DMAC_START_INT,
    XLLP_DMAC_END_INT,
    XLLP_DMAC_STOP_INT,
    XLLP_DMAC_RAS_INT,
    XLLP_DMAC_EOR_INT = 9
}XLLP_DMAC_INTERRUPT_T;

/**
 * DMAC Transfer Type
 **/
typedef enum
{
    XLLP_DMAC_TRANSFER_MEM_TO_MEM = 0,
    XLLP_DMAC_TRANSFER_IO_TO_MEM,
    XLLP_DMAC_TRANSFER_MEM_TO_IO
}XLLP_DMAC_TRANSFER_TYPE_T;

/**
 * DMAC Transfer Mode
 **/
typedef enum
{
    XLLP_DMAC_DESCRIPTOR_MODE = 0,
    XLLP_DMAC_NO_DESCRIPTOR_MODE
}XLLP_DMAC_TRANSFER_MODE_T;

/**
 * DMAC Descriptor Enable
 **/
typedef enum
{
    XLLP_DMAC_DESC_RUN_CHANNEL = 0,
    XLLP_DMAC_DESC_STOP_CHANNEL
}XLLP_DMAC_DESC_ENABLE_T;

/**
 * DMA Command
 **/
typedef struct 
{
    XLLP_INT16_T                 aLen;          // Length of transfer in bytes. Max 0x1FFF
    XLLP_DMAC_DEVICE_WIDTH_T     aWidth;        // Width of on-chip peripheral
    XLLP_DMAC_DEVICE_BURSTSIZE_T aSize;         // Max. burst size of each data transferred
    XLLP_BOOL_T                  aEndian;       // Device endianness. 0=Little, 1=Big endian
    XLLP_BOOL_T                  aEndIrqEn;     // End Interrupt Enable. When set, 
                                                // generate interrupt when aLen=0
    XLLP_BOOL_T                  aStartIrqEn;   // Start Int. enable. When set, generate
                                                // interrupt after loading descriptor.
                                                // Reserved for No Descriptor Mode Xfer
    XLLP_BOOL_T                  aFlowTrg;      // Flow control of the target.
    XLLP_BOOL_T                  aFlowSrc;      // Flow control of the source
    XLLP_BOOL_T                  aIncTrgAddr;   // Target address increment setting
    XLLP_BOOL_T                  aIncSrcAddr;   // Source address increment setting
}XLLP_DMAC_COMMAND_T, *P_XLLP_DMAC_COMMAND_T;

/**
 * DMAC Channel Status
 **/
typedef enum
{
 XLLP_DMAC_STATUS_BUSERRINTR = 0, // Bus error causing int.
 XLLP_DMAC_STATUS_STARTINTR,      // Successful Descriptor fetch int.
 XLLP_DMAC_STATUS_ENDINTR,        // Successful completion int.
 XLLP_DMAC_STATUS_STOPINTR,       // Channel state, 0=Running, 1=Stop
 XLLP_DMAC_STATUS_RASINTR,        // Request After Channel Stoped
 XLLP_DMAC_STATUS_REQPEND = 8,    // Channel Request pending state.
                                  // 1=Channel has pending req, 0=No Req
 XLLP_DMAC_STATUS_EORINT,         // Indicates status of peripheral Rx data
 XLLP_DMAC_STATUS_CMPST           // Descriptor compare status
}XLLP_DMAC_CHANNEL_STATUS_T, *P_XLLP_DMAC_CHANNEL_STATUS_T;


/**
 * DMAC External Request Pins
 **/
typedef enum
{
   XLLP_DMAC_EXTERNAL_PIN_0 = 0,
   XLLP_DMAC_EXTERNAL_PIN_1
}XLLP_DMAC_EXT_PIN_T;

#define XLLP_DMAC_DDADR_RESERVED_MASK     0xFFFFFFF0
#define XLLP_DMAC_DRCMR_ENABLE            0x80
#define XLLP_DMAC_DRCMR_DISABLE           0x00

/**
 * XLLP DMA Primitive Functions
 **/

XLLP_STATUS_T XllpDmacInit(P_XLLP_DMAC_T pDmacHandle);
 
void XllpDmacCfgChannelDescTransfer(
                                   P_XLLP_DMAC_T             pDmacHandle,
                                   P_XLLP_DMAC_DESCRIPTOR_T  pDesc,
                                   XLLP_DMAC_CHANNEL_T       aChannel,
                                   XLLP_DMAC_DEVICE_T        aDeviceDrcmr
                                   );

void XllpDmacCfgChannelNoDescTransfer(
                                     P_XLLP_DMAC_T             pDmacHandle,
                                     XLLP_UINT32_T             aSourceAddr,      
                                     XLLP_UINT32_T             aTargetAddr,      
                                     XLLP_DMAC_COMMAND_T*      pCmd,
                                     XLLP_DMAC_CHANNEL_T       aChannel,
                                     XLLP_DMAC_DEVICE_T        aDeviceDrcmr
                                     );

void XllpDmacStartTransfer(P_XLLP_DMAC_T  pDmacHandle, XLLP_DMAC_CHANNEL_T aChannel );

void XllpDmacStopTransfer(P_XLLP_DMAC_T  pDmacHandle,  XLLP_DMAC_CHANNEL_T aChannel );

void XllpDmacMapDeviceToChannel(
                               P_XLLP_DMAC_T       pDmacHandle, 
                               XLLP_DMAC_DRCMR_T   aDeviceDrcmr,
                               XLLP_DMAC_CHANNEL_T aChannel
                               );

void XllpDmacUnMapDeviceToChannel(
                                 P_XLLP_DMAC_T       pDmacHandle, 
                                 XLLP_DMAC_DRCMR_T   aDeviceDrcmr,
                                 XLLP_DMAC_CHANNEL_T aChannel
                                 );

void XllpDmacLoadDescriptor(P_XLLP_DMAC_T             pDmacHandle,
                             P_XLLP_DMAC_DESCRIPTOR_T  pDesc,
                             XLLP_DMAC_CHANNEL_T       aChannel
                            );

#endif //__DMAC_H__

 
#ifdef __cplusplus
}
#endif
