/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**  FILENAME:       xllp_gpio_proc.h
**
**  PURPOSE: Contains all processor-level GPIO definitions.  Hardware platform-
**              specific GPIO macros, typedefs, and prototypes are found
**              in xllp_gpio_plat.h
**           Declares no storage.
**                  
**  Validity Notes: Only valid for processor code named Monahans.
**
******************************************************************************/

#ifndef __XLLP_GPIO_PROC_H__
#define __XLLP_GPIO_PROC_H__

#include "xllp_defs.h"

// XLLP GPIO API typedefs 

// Note: The registers preceded by "do not use" are valid but are read-modify
//       -write.  The registers after offset 0x400 perform the same operations
//       in a bitwise write-only fashion.  That removes the need for software
//       serialization handling (lock() and unlock())

typedef struct 
{
    XLLP_VUINT32_T gplr0;             /* Level Detect Reg. Bank 0 */
    XLLP_VUINT32_T gplr1;             /* Level Detect Reg. Bank 1 */
    XLLP_VUINT32_T gplr2;             /* Level Detect Reg. Bank 2 */
/*do not use*/    XLLP_VUINT32_T gpdr0;            /* Data Direction Reg. Bank 0 */
/*do not use*/    XLLP_VUINT32_T gpdr1;            /* Data Direction Reg. Bank 1 */
/*do not use*/    XLLP_VUINT32_T gpdr2;            /* Data Direction Reg. Bank 2 */
    XLLP_VUINT32_T gpsr0;            /* Pin Output Set Reg. Bank 0 */
    XLLP_VUINT32_T gpsr1;            /* Pin Output Set Reg. Bank 1 */
    XLLP_VUINT32_T gpsr2;            /* Pin Output Set Reg. Bank 2 */
    XLLP_VUINT32_T gpcr0;            /* Pin Output Clr Reg. Bank 0 */
    XLLP_VUINT32_T gpcr1;            /* Pin Output Clr Reg. Bank 1 */
    XLLP_VUINT32_T gpcr2;            /* Pin Output Clr Reg. Bank 2 */
/*do not use*/    XLLP_VUINT32_T grer0;   /* Ris. Edge Detect Enable Reg. Bank 0 */
/*do not use*/    XLLP_VUINT32_T grer1;   /* Ris. Edge Detect Enable Reg. Bank 1 */
/*do not use*/    XLLP_VUINT32_T grer2;   /* Ris. Edge Detect Enable Reg. Bank 2 */
/*do not use*/    XLLP_VUINT32_T gfer0;   /* Fal. Edge Detect Enable Reg. Bank 0 */
/*do not use*/    XLLP_VUINT32_T gfer1;   /* Fal. Edge Detect Enable Reg. Bank 1 */
/*do not use*/    XLLP_VUINT32_T gfer2;   /* Fal. Edge Detect Enable Reg. Bank 2 */
    XLLP_VUINT32_T gedr0;       /* Edge Detect Status Reg. Bank 0 */
    XLLP_VUINT32_T gedr1;       /* Edge Detect Status Reg. Bank 1 */
    XLLP_VUINT32_T gedr2;       /* Edge Detect Status Reg. Bank 2 */

    XLLP_VUINT32_T  reserved01[43];    /* addr. offset 0x074-0x0fc */
    XLLP_VUINT32_T  gplr3;             /* Level Detect Reg. Bank 3 */
    XLLP_VUINT32_T  reserved02[2];      /* addr. offset 0x104-0x108 */
/*do not use*/    XLLP_VUINT32_T gpdr3;            /* Data Direction Reg. Bank 3 */
    XLLP_VUINT32_T  reserved03[2];      /* addr. offset 0x110-0x114 */
    XLLP_VUINT32_T gpsr3;            /* Pin Output Set Reg. Bank 3 */
    XLLP_VUINT32_T  reserved04[2];      /* addr. offset 0x11c-0x120 */
    XLLP_VUINT32_T gpcr3;            /* Pin Output Clr Reg. Bank 3 */
    XLLP_VUINT32_T  reserved05[2];      /* addr. offset 0x128-0x12c */
/*do not use*/    XLLP_VUINT32_T grer3;   /* Ris. Edge Detect Enable Reg. Bank 3 */
    XLLP_VUINT32_T  reserved06[2];      /* addr. offset 0x134-0x138 */
/*do not use*/    XLLP_VUINT32_T gfer3;   /* Fal. Edge Detect Enable Reg. Bank 3 */
    XLLP_VUINT32_T  reserved07[2];      /* addr. offset 0x140-0x144 */
    XLLP_VUINT32_T gedr3;       /* Edge Detect Status Reg. Bank 3 */
    XLLP_VUINT32_T  reserved08[173];      /* addr. offset  */

// Beginning of new write-only, bit-significant registers
    /* addr. offset 0x400 */
    XLLP_VUINT32_T gsdr0;
    XLLP_VUINT32_T gsdr1;
    XLLP_VUINT32_T gsdr2;
    XLLP_VUINT32_T gsdr3;
    XLLP_VUINT32_T  reserved09[4];        /* addr. offset  */
    XLLP_VUINT32_T gcdr0;
    XLLP_VUINT32_T gcdr1;
    XLLP_VUINT32_T gcdr2;
    XLLP_VUINT32_T gcdr3;
    XLLP_VUINT32_T  reserved10[4];        /* addr. offset  */
    XLLP_VUINT32_T gsrer0;
    XLLP_VUINT32_T gsrer1;
    XLLP_VUINT32_T gsrer2;
    XLLP_VUINT32_T gsrer3;
    XLLP_VUINT32_T  reserved11[4];        /* addr. offset  */
    XLLP_VUINT32_T gcrer0;
    XLLP_VUINT32_T gcrer1;
    XLLP_VUINT32_T gcrer2;
    XLLP_VUINT32_T gcrer3;
    XLLP_VUINT32_T  reserved12[4];        /* addr. offset  */
    XLLP_VUINT32_T gsfer0;
    XLLP_VUINT32_T gsfer1;
    XLLP_VUINT32_T gsfer2;
    XLLP_VUINT32_T gsfer3;
    XLLP_VUINT32_T  reserved13[4];        /* addr. offset  */
    XLLP_VUINT32_T gcfer0;
    XLLP_VUINT32_T gcfer1;
    XLLP_VUINT32_T gcfer2;
    XLLP_VUINT32_T gcfer3;
    XLLP_VUINT32_T  reserved14[4];        /* addr. offset  */

} XLLP_GPIO_T, *P_XLLP_GPIO_T;


typedef enum 
{
    XLLP_GPIO_DIRECTION_IN = 0,
    XLLP_GPIO_DIRECTION_OUT = 1
} XLLP_GPIO_DIRECTION_T;


///////////////////////////////////////////////
//
//  Processor-specific GPIO constants start here.
//

#if defined(CONFIG_CPU_MONAHANS_PL) || defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
#define XLLP_GPIO_EXPANDER_NUMBER               2 // decimal
#define XLLP_GPIO_EXPANDER_PIN_NUMBER           16 // decimal
#define XLLP_GPIO_EXP0_ADDR                     0x74 // decimal
#define XLLP_GPIO_EXP1_ADDR                     0x75 // decimal
                                                                                          
// Max ID and Reserved GPIO IDs
#define XLLP_GPIO_ID_END                        127 //decimal
                                                                                          
#define XLLP_EXP_GPIO_START                     (127 + 1) // decimal
#define XLLP_EXP_GPIO_END                       (127 + XLLP_GPIO_EXPANDER_PIN_NUMBER * XLLP_GPIO_EXPANDER_NUMBER)
                                                                                          
#define XLLP_EXP0_GPIO_START                    (XLLP_GPIO_ID_END + 1)
#define XLLP_EXP0_GPIO_HALF                     (XLLP_GPIO_ID_END + (XLLP_GPIO_EXPANDER_PIN_NUMBER / 2))
#define XLLP_EXP0_GPIO_END                      (127 + XLLP_GPIO_EXPANDER_PIN_NUMBER)
                                                                                          
#define XLLP_EXP1_GPIO_START                    (XLLP_EXP0_GPIO_END + 1)
#define XLLP_EXP1_GPIO_HALF                     (XLLP_EXP0_GPIO_END + (XLLP_GPIO_EXPANDER_PIN_NUMBER / 2))
#define XLLP_EXP1_GPIO_END                      (127 + XLLP_GPIO_EXPANDER_PIN_NUMBER * XLLP_GPIO_EXPANDER_NUMBER)
#endif

// Max ID and Reserved GPIO IDs
#define XLLP_GPIO_ID_MAX                       127 // decimal
#define XLLP_GPIO_ID_INVALID_56                 56 // decimal
#define XLLP_GPIO_ID_INVALID_59                 59 // decimal
#define XLLP_GPIO_ID_INVALID_60                 60 // decimal
#define XLLP_GPIO_ID_INVALID_61                 61 // decimal
#define XLLP_GPIO_ID_INVALID_62                 62 // decimal

//
//  End of Processor-level GPIO constants
//
///////////////////////////////////////////////

#ifdef __cplusplus
extern "C"
{
#endif


/* Processor-level XLLP GPIO Function Prototypes */
#if defined(CONFIG_CPU_MONAHANS_PL) || defined(CONFIG_CPU_MONAHANS_L) || defined(CONFIG_CPU_MONAHANS_LV)
XLLP_STATUS_T XllpGpioExpanderRead( XLLP_UINT8_T expAddr,
                        XLLP_UINT8_T reg,
                        XLLP_UINT8_T *pval);
                                                                                          
XLLP_STATUS_T XllpGpioExpanderWrite(XLLP_UINT8_T expAddr,
                        XLLP_UINT8_T reg,
                        XLLP_UINT8_T val);
#endif

XLLP_STATUS_T XllpGpioGetLevel (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_LEVEL_T* pLevel);

XLLP_STATUS_T XllpGpioSetOutputLevel (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_LEVEL_T level);

XLLP_STATUS_T XllpGpioGetDirection (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_GPIO_DIRECTION_T * pDirection);

XLLP_STATUS_T XllpGpioSetDirection (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_GPIO_DIRECTION_T direction);

XLLP_STATUS_T XllpGpioGetRisingEdgeDetectEnable (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_CONTROL_T* pEnabled);

XLLP_STATUS_T XllpGpioSetRisingEdgeDetectEnable  (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_CONTROL_T enable);

XLLP_STATUS_T XllpGpioGetFallingEdgeDetectEnable (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_CONTROL_T* pEnabled);

XLLP_STATUS_T XllpGpioSetFallingEdgeDetectEnable (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_CONTROL_T enable);

XLLP_STATUS_T XllpGpioGetEdgeDetectStatus (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId,
                                XLLP_UINT32_T* pDetectStatus);

XLLP_STATUS_T XllpGpioClearEdgeDetectStatus (
                                P_XLLP_GPIO_T pGPIO, 
                                XLLP_UINT32_T gpioId);

XLLP_STATUS_T XllpGpioSave (
                            P_XLLP_GPIO_T pGPIO, 
                            P_XLLP_GPIO_T pSaveContext);

XLLP_STATUS_T XllpGpioRestore(
                            P_XLLP_GPIO_T pGPIO, 
                            P_XLLP_GPIO_T pSaveContext);
#ifdef __cplusplus
}
#endif
#endif // __XLLP_GPIO_PROC_H__


