/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#ifndef __XLLP_INTC_H__
#define __XLLP_INTC_H__

/******************************************************************************
**  FILENAME:   xllp_intc.h
**
**  PURPOSE:    contains the XLLP INTC typedefs and bit definitions.
**
******************************************************************************/

#include "xllp_defs.h"


//
// INTC Register definitions
//
#define XLLP_INTC_IPR1_COUNT 32
#define XLLP_INTC_IPR2_COUNT 21
typedef struct
{
      // Registers for interrupt signals 0 C 31
 XLLP_VUINT32_T  reserved0[5];
 XLLP_VUINT32_T iccr;
 XLLP_VUINT32_T  reserved1[1];
 XLLP_VUINT32_T  ipr[XLLP_INTC_IPR1_COUNT];  // Interrupt Priority Registers [0-31]
 XLLP_VUINT32_T  reserved2[5];
 XLLP_VUINT32_T ipr2[XLLP_INTC_IPR2_COUNT];  // Interrupt Priority Registers [32-52]

} XLLP_INTC_T, *P_XLLP_INTC_T;

typedef struct
{
    XLLP_VUINT32_T iccr;
    XLLP_VUINT32_T ipr[53];  // Interrupt Priority Registers [0-52]
    
    XLLP_VUINT32_T  icmr;
    XLLP_VUINT32_T  icmr2;
    
    XLLP_VUINT32_T  iclr;
    XLLP_VUINT32_T  iclr2;
    
}XLLP_INTC_SAVE_T, *P_XLLP_INTC_SAVE_T;

//Do change seqeuncy because asm use fix number to access coprocessor registers 
typedef enum
{
 XLLP_INTC_ICIP, /*!< Interrupt Controller IRQ Pending register*/
 XLLP_INTC_ICMR, /*!< Interrupt Controller Mask register*/
 XLLP_INTC_ICLR, /*!< Interrupt Controller level register*/
 XLLP_INTC_ICFP, /*!< Interrupt Controller FIQ Pending register*/
 XLLP_INTC_ICPR, /*!< Interrupt Controller Pendingregister*/
 XLLP_INTC_ICHP, /*!< Interrupt Controller Highest Priority register*/
 XLLP_INTC_ICIP2,/*!< Interrupt Controller IRQ Pending register*/
 XLLP_INTC_ICMR2,/*!< Interrupt Controller Mask register*/
 XLLP_INTC_ICLR2,/*!< Interrupt Controller level register*/
 XLLP_INTC_ICFP2,/*!< Interrupt Controller FIQ Pending register*/
 XLLP_INTC_ICPR2 /*!< Interrupt Controller Pendingregister*/

}XLLP_INTC_REG;

typedef enum
{
 XLLP_XSCALE_IRQ=0x80,
 XLLP_XSCALE_FIQ=0x40,
 XLLP_XSCALE_FIQ_IRQ= XLLP_XSCALE_IRQ| XLLP_XSCALE_FIQ
} XLLP_INTC_CPSR_INT_MASK_T;

typedef enum
{
 // Values correspond to those in the 
 // "Summary of Bit Positions for Primary Sources"
 // Table in the INTC section of the DM.

 // Source IDs handled in the first of various paired
 // INTC registers
 XLLP_INTC_S_SSP3     =  0, //< SSP)2 service request
 XLLP_INTC_S_MSL1     =  1, //< MSL interface interrupt
 XLLP_INTC_S_USBH2    =  2, //< USB Host interrupt 2
 XLLP_INTC_S_USBH1    =  3, //< USB Host interrupt 1 (OHCI)
 XLLP_INTC_S_KEY      =  4, //< Keypad Controller interrupt
 // 5: not used
 XLLP_INTC_S_PI2C     =  6, //< Power I2C Interrupt
 XLLP_INTC_S_OST      =  7, //< OS timer matches register
 XLLP_INTC_S_GPIO0    =  8, //< GPIO[0] detects an edge
 XLLP_INTC_S_GPI01    =  9, //< GPIO[1] detects an edge
 XLLP_INTC_S_GPIOX    = 10, //< GPIO[x] or of the GPIO edge detect (except 0 and 1)
 XLLP_INTC_S_USBC     = 11, //< USB Client interrupt
 XLLP_INTC_S_PML      = 12, //< PMUL (Performace monitor) interrupt
 XLLP_INTC_S_SSP4     = 13, //< SSP_4 service request
 XLLP_INTC_S_AC97     = 14, //< AC97 interrupt
 XLLP_INTC_S_USIM     = 15, //< Smart Card Interface 1 status/error
 XLLP_INTC_S_SSP2     = 16, //< SSP_2 service request
 XLLP_INTC_S_LCD      = 17, //< LCD controller service request
 XLLP_INTC_S_I2C      = 18, //< I2C service request
 // 19: not used
 XLLP_INTC_S_STUART   = 20, //< Transmit or receive error in STUART
 XLLP_INTC_S_BTUART   = 21, //< Transmit or receive error in BTUART
 XLLP_INTC_S_FFUART   = 22, //< Transmit or receive error in FFUART
 XLLP_INTC_S_MMC1     = 23, //< MMC/SDIO/Flash Card 1 status/Error detection
 XLLP_INTC_S_SSP1     = 24, //< SSP_1 service request
 XLLP_INTC_S_DMA      = 25, //< DMA Channel service request
 XLLP_INTC_S_OST_M0   = 26, //< OS timer equals Match register 0
 XLLP_INTC_S_OST_M1   = 27, //< OS timer equals Match register 1
 XLLP_INTC_S_OST_M2   = 28, //< OS timer equals Match register 2
 XLLP_INTC_S_OST_M3   = 29, //< OS timer equals Match register 3
 XLLP_INTC_S_RTC_ONE  = 30, //< One-Hz clock TIC occurred
 XLLP_INTC_S_RTC_ALARM= 31, //< RTC equals Alarm register

 // Source IDs handled in the second of various paired
 // INTC registers
 XLLP_INTC_S_TPM      = 32, // 32+ 0< TPM interrupt
 XLLP_INTC_S_CAMERA   = 33, // 32+ 1< Quick Capture Interface Interrupt
 XLLP_INTC_S_CIR      = 34, // 32+ 2< Consumer Infrared
 // 35: not used            // 32+ 3
 XLLP_INTC_S_TSI      = 36, // 32+ 4< Touch screen
 // 37: not used            // 32+ 5
 XLLP_INTC_S_USIM2    = 38, // 32+ 6< USIM
 XLLP_INTC_S_GRAPHICS = 39, // 32+ 7< Graphics controller
 // 40: not used            // 32+ 8
 XLLP_INTC_S_MMC2     = 41, // 32+ 9< MultiMediaCard 2
 // 42,43: not used         // 32+10,32+11 
 XLLP_INTC_S_1WIRE    = 44, // 32+12< 1-Wire
 XLLP_INTC_S_NAND     = 45, // 32+13< NAND
 XLLP_INTC_S_USB2     = 46, // 32+14< USB 2.0 Client
 XLLP_INTC_S_SGP      = 47, // 32+15< SGP master PMU interface module
 // 48: not used            // 32+16
 XLLP_INTC_S_EXT1     = 49, // 32+17< External Wakeup source 1
 XLLP_INTC_S_EXT2     = 50, // 32+18< External Wakeup source 2
 XLLP_INTC_S_DMEM     = 51, // 32+19< Dmem controller
 XLLP_INTC_S_CLK      = 52, // 32+20< Processor CCU interrupts
 XLLP_INTC_S_MAX_ID   = 52  // 

}XLLP_INTC_SOURCE_T;

// Reserved, valid bit masks

#define XLLP_INTC_ICIP_RESERVED_BITS ((1u<<5)|(1u<<19))
#define XLLP_INTC_ICIP_MASK (~(XLLP_INTC_ICIP_RESERVED_BITS))
#define XLLP_INTC_ICMR_RESERVED_BITS (XLLP_INTC_ICIP_RESERVED_BITS)
#define XLLP_INTC_ICMR_MASK (~(XLLP_INTC_ICMR_RESERVED_BITS))
#define XLLP_INTC_ICLR_RESERVED_BITS (XLLP_INTC_ICIP_RESERVED_BITS)
#define XLLP_INTC_ICLR_MASK (~(XLLP_INTC_ICLR_RESERVED_BITS))
#define XLLP_INTC_ICFP_RESERVED_BITS (XLLP_INTC_ICIP_RESERVED_BITS)
#define XLLP_INTC_ICFP_MASK (~(XLLP_INTC_ICFP_RESERVED_BITS))
#define XLLP_INTC_ICPR_RESERVED_BITS (XLLP_INTC_ICIP_RESERVED_BITS)
#define XLLP_INTC_ICPR_MASK (~(XLLP_INTC_ICPR_RESERVED_BITS))
#define XLLP_INTC_ICCR_RESERVED_BITS 0xFFFFFFFEu
#define XLLP_INTC_ICCR_MASK (~(XLLP_INTC_ICCR_RESERVED_BITS))
#define XLLP_INTC_ICHP_RESERVED_BITS 0x7FC07FC0u
#define XLLP_INTC_ICHP_MASK (~(XLLP_INTC_ICHP_RESERVED_BITS))
#define XLLP_INTC_IPR_RESERVED_BITS 0x7FFFFFC0u
#define XLLP_INTC_IPR_MASK (~(XLLP_INTC_IPR_RESERVED_BITS))

#define XLLP_INTC_ICIP2_RESERVED_BITS 0xFFE10D28u
#define XLLP_INTC_ICIP2_MASK (~(XLLP_INTC_ICIP2_RESERVED_BITS))
#define XLLP_INTC_ICMR2_RESERVED_BITS (XLLP_INTC_ICIP2_RESERVED_BITS)
#define XLLP_INTC_ICMR2_MASK (~(XLLP_INTC_ICMR2_RESERVED_BITS))
#define XLLP_INTC_ICLR2_RESERVED_BITS (XLLP_INTC_ICIP2_RESERVED_BITS)
#define XLLP_INTC_ICLR2_MASK (~(XLLP_INTC_ICLR2_RESERVED_BITS))
#define XLLP_INTC_ICFP2_RESERVED_BITS (XLLP_INTC_ICIP2_RESERVED_BITS)
#define XLLP_INTC_ICFP2_MASK (~(XLLP_INTC_ICFP2_RESERVED_BITS))
#define XLLP_INTC_IPR2_RESERVED_BITS (XLLP_INTC_IPR_RESERVED_BITS)
#define XLLP_INTC_IPR2_MASK (~(XLLP_INTC_IPR2_RESERVED_BITS))

#ifdef __cplusplus
extern "C"
{
#endif

/**
 This routine reads the designated Interrupt Controller register via CoProcessor 6

@param  aReg  INTC register number to read define in XLLP_INTC_REG
@return    32-bit value read from register
*/
extern unsigned int XllpReadINTCReg(unsigned int aReg);
/**
 This routine writes the designated Interrupt Controller register via CoProcessor 6

@param  aReg  INTC register number to read define in XLLP_INTC_REG
   aValue  Value to write to INTC register
@return  
 0 success
 -1 error
*/
extern int XllpWriteINTCReg(unsigned int aReg, unsigned int aValue);

extern XLLP_BOOL_T XllpINTCEnableSource (XLLP_INTC_SOURCE_T source,XLLP_BOOL_T enable);
extern XLLP_BOOL_T XllpINTCSetSourceLevel (XLLP_INTC_SOURCE_T source,XLLP_BOOL_T irq);


/**
 XllpIntcDisableInterrupts disables the IRQ and FIQ interrupts by setting the IRQ and FIQ disable bits according MASK

@param
 mask disable IRQ and FIQ 

@return
 XLLP_UINT32_T savedInts C the previous settings of the I and F bits.  Bit settings are returned as they are positioned in the CPSR registers, with all other bits cleared. 

*/
extern XLLP_INT32_T XllpIntcDisableInterrupts(void);
extern void XllpIntcRestoreInts(XLLP_UINT32_T InterruptStatus);
extern XLLP_STATUS_T XllpINTCSave(P_XLLP_INTC_T pVirtual, P_XLLP_INTC_SAVE_T pSave);
extern XLLP_STATUS_T XllpINTCRestore(P_XLLP_INTC_T pVirtual, P_XLLP_INTC_SAVE_T pSave);

#ifdef __cplusplus
}
#endif



#endif
