/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/**
**  FILENAME:  xllp_memc.h
**
**  PURPOSE:   Header file for XLLP Memory Controller. 
**
*******************************************************************************/

#ifndef XLLP_MEMC_H
#define XLLP_MEMC_H

#include "xllp_defs.h"
#include "xllp_serialization.h"

typedef struct 
{
	XLLP_VUINT32_T mdcnfg;
	XLLP_VUINT32_T mdrefr;
	XLLP_VUINT32_T reserved0[6];	
	XLLP_VUINT32_T flycnfg;
	XLLP_VUINT32_T reserved1[7];	
	XLLP_VUINT32_T mdmrs;
	XLLP_VUINT32_T reserved2[3];		
	XLLP_VUINT32_T ddr_scal;
	XLLP_VUINT32_T reserved3[3];		
	XLLP_VUINT32_T ddr_hcal;
	XLLP_VUINT32_T reserved4;		
	XLLP_VUINT32_T ddr_wcal;
	XLLP_VUINT32_T reserved5;		
	XLLP_VUINT32_T dmcier;
	XLLP_VUINT32_T reserved6;		
	XLLP_VUINT32_T dmcisr;
	XLLP_VUINT32_T reserved7;	
	XLLP_VUINT32_T ddr_dls;
	XLLP_VUINT32_T reserved8[3];		
	XLLP_VUINT32_T empi;
	XLLP_VUINT32_T reserved9[3];		
	XLLP_VUINT32_T rcomp;
	XLLP_VUINT32_T reserved10[3];		
	XLLP_VUINT32_T pad_ma;
	XLLP_VUINT32_T pad_mdmsb;
	XLLP_VUINT32_T pad_mdlsb;
	XLLP_VUINT32_T pad_sdram;
	XLLP_VUINT32_T pad_sdclk;
	XLLP_VUINT32_T pad_sdcs;
	XLLP_VUINT32_T pad_smem;
	XLLP_VUINT32_T pad_sclk;							
} XLLP_DMEM_REGISTER_T, *P_XLLP_DMEM_REGISTER_T;

//
//static memory controller Registers structure
//
typedef struct
{
    XLLP_VUINT32_T    reserved0[2];
    XLLP_VUINT32_T    msc0;
    XLLP_VUINT32_T    msc1;
    XLLP_VUINT32_T    reserved1;
    XLLP_VUINT32_T    mecr;
    XLLP_VUINT32_T    reserved2;
    XLLP_VUINT32_T    sxcnfg;
    XLLP_VUINT32_T    reserved3[2];
    XLLP_VUINT32_T    mcmem0;
    XLLP_VUINT32_T    reserved4;
    XLLP_VUINT32_T    mcatt0;
    XLLP_VUINT32_T    reserved5;
    XLLP_VUINT32_T    mcio0;
    XLLP_VUINT32_T    reserved6[11];
    XLLP_VUINT32_T    memclkcfg;
    XLLP_VUINT32_T    reserved7[5];
    XLLP_VUINT32_T    casdrcfg0;
    XLLP_VUINT32_T    casdrcfg1;
    XLLP_VUINT32_T    casdrcfg2;
    XLLP_VUINT32_T    casdrcfg3;
    XLLP_VUINT32_T    casdrcfg_p;
    XLLP_VUINT32_T    reserved8[3];
    XLLP_VUINT32_T    csmsadrcfg;
    XLLP_VUINT32_T    reserved9[3];
    XLLP_VUINT32_T    clk_ret_del;
    XLLP_VUINT32_T    adv_ret_del;
} XLLP_SMEM_REGISTER_T, *P_XLLP_SMEM_REGISTER_T;


// bit definition for dmcier register.
#define XLLP_DMCIER_ERCI			(1<<31)
#define XLLP_DMCIER_EORF			(1<<30)
#define XLLP_DMCIER_EDLP			(1<<29)

// bit definition for dmcisr register.
#define XLLP_DMCISR_RCI				(1<<31)
#define XLLP_DMCISR_ORF				(1<<30)
#define XLLP_DMCISR_DLP				(1<<29)
#define XLLP_DMCISR_SLFREF			(1<<7)

// bit definition for rcomp register.
#define XLLP_RCOMP_SWEVAL			(1<<31)
#define XLLP_RCOMP_UPDATE			(1<<30)
#define XLLP_RCOMP_RCRNG_MASK		(0x1f<<25)
#define XLLP_RCOMP_RCNOP			(1<<24)
#define XLLP_RCOMP_REI_MASK			(0xfffff<<0)

typedef struct
{
	XLLP_UINT32_T dmem;
	XLLP_UINT32_T rcEvalInterval;
	XLLP_UINT32_T rcRange;
} XLLP_MEMC_T, *P_XLLP_MEMC_T;

void XllpMemcRcompInit(P_XLLP_MEMC_T pXllpMemc);
XLLP_STATUS_T XllpMemcRcompUpdate(P_XLLP_MEMC_T pXllpMemc);
void XllpMemcRcompEvalUpdate(P_XLLP_MEMC_T pXllpMemc);

extern void xlli_Rcomp(XLLP_UINT32_T dmemAddr);

#endif //XLLP_MEMC_H
