/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

/** FILENAME:	xllp_pm_suspend.h
**
** PURPOSE:	contain constant definition for Xllp Suspend/Resume.
**
******************************************************************************/

#ifndef __XLLP_PM_SUSPEND_H
#define __XLLP_PM_SUSPEND_H

#include "xllp_defs.h"
#include "xllp_pm.h"
#include "xllp_dfc_proc.h"

/* modeSaveFlags */
#define XLLP_PM_MODE_SAVE_FLAG_SYS	0x1
#define XLLP_PM_MODE_SAVE_FLAG_IRQ	0x2
#define XLLP_PM_MODE_SAVE_FLAG_FIQ	0x4
#define XLLP_PM_MODE_SAVE_FLAG_ABT	0x8
#define XLLP_PM_MODE_SAVE_FLAG_UND	0x10
#define XLLP_PM_MODE_SAVE_FLAG_SVC	0x20

#define WORD_SIZE	4

/* the position of each data memeber */
#define SleepState_checksum		0x0
#define SleepState_wordCount		SleepState_checksum + WORD_SIZE
#define SleepState_areaAddress		SleepState_wordCount + WORD_SIZE
#define SleepState_modeSaveFlags	SleepState_areaAddress + WORD_SIZE
/* save ARM registers */
#define	SleepState_ENTRY_REGS		SleepState_modeSaveFlags + WORD_SIZE
#define SleepState_ENTRY_CPSR		SleepState_ENTRY_REGS
#define SleepState_ENTRY_SPSR		SleepState_ENTRY_CPSR + WORD_SIZE
#define SleepState_ENTRY_R0		SleepState_ENTRY_SPSR + WORD_SIZE
#define SleepState_SYS_REGS		SleepState_ENTRY_REGS + 17*WORD_SIZE
#define SleepState_FIQ_REGS		SleepState_SYS_REGS + 2*WORD_SIZE 
#define SleepState_IRQ_REGS		SleepState_FIQ_REGS + 8*WORD_SIZE
#define SleepState_ABT_REGS		SleepState_IRQ_REGS + 3*WORD_SIZE 
#define SleepState_UND_REGS		SleepState_ABT_REGS + 3*WORD_SIZE
#define SleepState_SVC_REGS		SleepState_UND_REGS + 3*WORD_SIZE
/* save MMU settings */
#define SleepState_Cp15_ACR_MMU		SleepState_SVC_REGS + 3*WORD_SIZE
#define SleepState_Cp15_AUXCR_MMU	SleepState_Cp15_ACR_MMU + WORD_SIZE
#define SleepState_Cp15_TTBR_MMU	SleepState_Cp15_AUXCR_MMU + WORD_SIZE
#define SleepState_Cp15_DACR_MMU	SleepState_Cp15_TTBR_MMU + WORD_SIZE
#define SleepState_Cp15_PID_MMU		SleepState_Cp15_DACR_MMU + WORD_SIZE
#define SleepState_Cp15_CPAR		SleepState_Cp15_PID_MMU + WORD_SIZE

#define SleepState_extendedChecksumByteCount	SleepState_Cp15_CPAR + WORD_SIZE
//#define SleepState_sramAddress	SleepState_extendedChecksumByteCount + WORD_SIZE
#define SleepState_psprAddress	SleepState_extendedChecksumByteCount + WORD_SIZE
#define SleepState_flushFunc	SleepState_psprAddress + WORD_SIZE
//#define SleepState_copySramFunc	SleepState_flushFunc + WORD_SIZE

typedef void (* P_XLLP_VOID_FN_T) (void);
typedef XLLP_UINT32_T (* P_XLLP_INT_FN_T) (XLLP_UINT32_T );

typedef struct XLLP_PM_DATA_SAVE_S {
	XLLP_UINT32_T checksum;
	XLLP_UINT32_T wordCount;
	XLLP_UINT32_T areaAddress;
	XLLP_UINT32_T modeSaveFlags;
	/* current mode registers cpsr, sprsr, r0-r12, lr, sp */
	XLLP_UINT32_T ENTRY_REGS[17];
	/* SYS mode registers:sp, lr */
	XLLP_UINT32_T SYS_REGS[2];
	/* FIQ mode registers:spsr, r8-r12, sp, lr */
	XLLP_UINT32_T FIQ_REGS[8];
	/* IRQ mode registers:spsr, sp, lr */
	XLLP_UINT32_T IRQ_REGS[3];
	/* ABT mode registers:spsr, sp, lr */
	XLLP_UINT32_T ABT_REGS[3];
	/* UND mode registers:spsr, sp, lr */
	XLLP_UINT32_T UND_REGS[3];
	/* SVC mode registers:spsr, sp, lr */
	XLLP_UINT32_T SVC_REGS[3];
	/* MMU registers */
	XLLP_UINT32_T CP15_ACR_MMU;
	XLLP_UINT32_T CP15_AUXCR_MMU;
	XLLP_UINT32_T CP15_TTBR_MMU;
	XLLP_UINT32_T CP15_DACR_MMU;
	XLLP_UINT32_T CP15_PID_MMU;
	XLLP_UINT32_T CP15_CPAR;

	XLLP_UINT32_T extendedChecksumByteCount;
	//XLLP_UINT32_T sramAddress;
	XLLP_UINT32_T psprAddress;
	P_XLLP_VOID_FN_T flushFunc;
	/* the parameter is the reserved bytes by XLLP from 0x5c010000 */
	/* It returns the physical address of initlization code in SRAM */
	//P_XLLP_INT_FN_T copySramFunc;
}XLLP_PM_DATA_SAVE_T, *P_XLLP_PM_DATA_SAVE_T;

typedef enum{
	XLLP_PM_RUN = 0,
	XLLP_PM_IDLE = 1 ,
	XLLP_PM_LCDREFRESH = 2,
	XLLP_PM_STANDBY = 3,
	XLLP_PM_D0CS = 5,
	XLLP_PM_SLEEP = 6,
	XLLP_PM_DEEPSLEEP = 7,
} XLLP_PM_MODE_T;

typedef XLLP_UINT32_T    XLLP_PM_WAKEUP_SRC_T;
#define XLLP_PM_WE_EXTERNAL0 	(0x1UL << 0)
#define XLLP_PM_WE_EXTERNAL1 	(0x1UL << 1)
#define	XLLP_PM_WE_GENERIC0 	(0x1UL << 2)
#define	XLLP_PM_WE_GENERIC1 	(0x1UL << 3)
#define	XLLP_PM_WE_GENERIC2		(0x1UL << 4)
#define	XLLP_PM_WE_GENERIC3		(0x1UL << 5)
#define	XLLP_PM_WE_GENERIC4		(0x1UL << 6)
#define	XLLP_PM_WE_GENERIC5		(0x1UL << 7)
#define	XLLP_PM_WE_GENERIC6		(0x1UL << 8)
#define	XLLP_PM_WE_GENERIC7		(0x1UL << 9)
#define	XLLP_PM_WE_GENERIC8		(0x1UL << 10)
#define XLLP_PM_WE_GENERIC9		(0x1UL << 11)
#define	XLLP_PM_WE_GENERIC10	(0x1UL << 12)
#define	XLLP_PM_WE_GENERIC11	(0x1UL << 13)
#define	XLLP_PM_WE_GENERIC12	(0x1UL << 14)
#define	XLLP_PM_WE_GENERIC13	(0x1UL << 15)
#define	XLLP_PM_WE_OTG			(0x1UL << 16)
#define	XLLP_PM_WE_RESERVE0		(0x1UL << 17)
#define	XLLP_PM_WE_MLCD  		(0x1UL << 18)
#define	XLLP_PM_WE_USIM0		(0x1UL << 19)
#define	XLLP_PM_WE_USIM1		(0x1UL << 20)
#define	XLLP_PM_WE_KP			(0x1UL << 21)
#define	XLLP_PM_WE_MUX2			(0x1UL << 22)
#define	XLLP_PM_WE_MUX3			(0x1UL << 23)
#define	XLLP_PM_WE_MSL0			(0x1UL << 24)
#define	XLLP_PM_WE_RESERVE2		(0x1UL << 25)
#define	XLLP_PM_WE_USB2			(0x1UL << 26)
#define	XLLP_PM_WE_RESERVE3		(0x1UL << 27)
#define	XLLP_PM_WE_USBH			(0x1UL << 28)
#define	XLLP_PM_WE_TSI			(0x1UL << 29)
#define	XLLP_PM_WE_OST			(0x1UL << 30)
#define	XLLP_PM_WE_RTC 			(0x1UL << 31)

typedef struct {
	P_XLLP_PM_SPMU_T   spmu;
	P_XLLP_PM_BPMU_T   bpmu;
	P_XLLP_DFC_CONTEXT_T p_dfc_context;
} XLLP_PM_SUSPEND_CONTEXT_T, *P_XLLP_PM_SUSPEND_CONTEXT_T;


#ifdef __cplusplus
extern "C"
{
#endif

extern void XllpPmEnterSleep(XLLP_PM_DATA_SAVE_T *pm_save_data);
extern XLLP_UINT32_T XllpPmResumeFromSleep(XLLP_UINT32_T p_save_data_phy);
extern void XllpPmEnterDeepSleep(XLLP_PM_DATA_SAVE_T *pm_save_data);
extern XLLP_UINT32_T XllpPmResumeFromDeepSleep(XLLP_UINT32_T p_save_data_phy);
extern void XllpPmEnterStandby(void);
extern void XllpPmEnterLcdRefresh(void);
extern void XllpPmEnterIdle(void);
extern void XllpPmClearAllWakeupSource(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_PM_MODE_T to_mode,
		XLLP_PM_MODE_T from_mode
);
extern void XllpPmSelectWakeupSource(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_PM_MODE_T to_mode,
		XLLP_PM_MODE_T from_mode, 
		XLLP_PM_WAKEUP_SRC_T src
);
extern XLLP_STATUS_T XllpPmQueryWakeupSource(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_PM_MODE_T to_mode,
		XLLP_PM_MODE_T from_mode, 
		XLLP_PM_WAKEUP_SRC_T *psrc
);
extern XLLP_STATUS_T XllpPmCopyBootCode(
		P_XLLP_PM_SUSPEND_CONTEXT_T   p_pm_suspend_context,
		XLLP_UINT32_T boot_addr
);

#ifdef __cplusplus
}
#endif

#if 0
/* GPIO content save/restore */
extern void XllpPmGPIOSave(XLLP_GPIO_T *, XLLP_GPIO_T *);
extern void XllpPmGPIORestore(XLLP_GPIO_T *, XLLP_GPIO_T *);

/*Interrupt controller content save/restore */
extern void XllpPmINTCSave(XLLP_INTC_T *, XLLP_INTC_T *);
extern void XllpPmINTCRestore(XLLP_INTC_T *, XLLP_INTC_T *);

/* clock manager content save/restore */
extern void XllpPmCLKMGRSave(XLLP_CLKMGR_T *, XLLP_CLKMGR_T *);
extern void XllpPmCLKMGRRestore(XLLP_CLKMGR_T *, XLLP_CLKMGR_T *);

/* OST timer content save/restore */
extern void XllpPmOSTSave(XLLP_OST_T *, XLLP_OST_T *);
extern void XllpPmOSTRestore(XLLP_OST_T *, XLLP_OST_T *);
#endif
#endif
