/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#ifndef __XLLP_RTC_H__
#define __XLLP_RTC_H__

/******************************************************************************
**  FILENAME:   xllp_rtc.h
**
**  PURPOSE:    contains all XLLP RTC specific macros, typedefs, and prototypes.
**
******************************************************************************/

#include "xllp_defs.h"


//
// RTC Register Definitions
//
typedef struct
{
    XLLP_VUINT32_T    rcnr;     // RTC count register
    XLLP_VUINT32_T    rtar;     // RTC alarm register
    XLLP_VUINT32_T    rtsr;     // RTC status register
    XLLP_VUINT32_T    rttr;     // RTC timer trim register
    XLLP_VUINT32_T    rdcr;	    // RTC day counter register
    XLLP_VUINT32_T    rycr;	    // RTC year counter register
    XLLP_VUINT32_T    rdar1;    // RTC day alarm register 1
    XLLP_VUINT32_T    ryar1;    // RTC year alarm register 1
    XLLP_VUINT32_T    rdar2;    // RTC day alarm register 2
    XLLP_VUINT32_T    ryar2;    // RTC year alarm register 2
    XLLP_VUINT32_T    swcr;	    // RTC stopwatch counter
    XLLP_VUINT32_T    swar1;    // RTC stopwatch alarm register 1
    XLLP_VUINT32_T    swar2;    // RTC stopwatch alarm register 2
    XLLP_VUINT32_T    picr;	    // RTC periodic interrupt counter register
    XLLP_VUINT32_T    piar;	    // RTC periodic interrupt alarm register

} XLLP_RTC_T, *P_XLLP_RTC_T;





//
// RTC Bit definitions
//
//
// RTSR Bits
//
#define XLLP_RTSR_AL     (0x1 << 0)   // RTC Alarm detect
#define XLLP_RTSR_HZ     (0x1 << 1)   // HZ rising edge detect
#define XLLP_RTSR_ALE    (0x1 << 2)   // RTC Alarm enable
#define XLLP_RTSR_HZE    (0x1 << 3)   // HZ enable
#define XLLP_RTSR_RDAL1  (0x1 << 4)   // Wristwatch alarm 1 detect
#define XLLP_RTSR_RDALE1 (0x1 << 5)   // Wristwatch alarm 1 enable
#define XLLP_RTSR_RDAL2  (0x1 << 6)   // Wristwatch alarm 2 detect
#define XLLP_RTSR_RDALE2 (0x1 << 7)   // Wristwatch alarm 2 enable
#define XLLP_RTSR_SWAL1  (0x1 << 8)   // Stopwatch alarm 1 detect
#define XLLP_RTSR_SWALE1 (0x1 << 9)   // Stopwatch alarm 1 enable
#define XLLP_RTSR_SWAL2  (0x1 << 10)  // Stopwatch alarm 2 detect
#define XLLP_RTSR_SWALE2 (0x1 << 11)  // Stopwatch alarm 2 enable
#define XLLP_RTSR_SWCE   (0x1 << 12)  // Stopwatch counter enable
#define XLLP_RTSR_PIAL   (0x1 << 13)  // Periodic alarm detect
#define XLLP_RTSR_PIALE  (0x1 << 14)  // Periodic alarm enable
#define XLLP_RTSR_PICE   (0x1 << 15)  // Periodic interrupt count enable

#define XLLP_RTSR_RESERVED_BITS (0xFFFF0000)

// 
// RTSR has mixed bits: r/w and sticky
// Mask for use with write to clear bits
//
#define XLLP_RTSR_MASK (XLLP_RTSR_AL | XLLP_RTSR_HZ | XLLP_RTSR_RDAL1 | XLLP_RTSR_RDAL2 | XLLP_RTSR_SWAL1 | XLLP_RTSR_SWAL2 | XLLP_RTSR_PIAL)

#ifdef __cplusplus
extern "C"
{
#endif

//
// RTC XLLP Prototypes
//
extern void XllpRtcConfigureAlarm
	(P_XLLP_RTC_T pRTCReg , XLLP_UINT32_T alarmtime);

extern void XllpRtcConfigureIncreaseAlarm 
	(P_XLLP_RTC_T pRTCReg, XLLP_UINT32_T increase_time);
extern XLLP_STATUS_T XllpRTCSave(P_XLLP_RTC_T pVirtual, P_XLLP_RTC_T pSave);
extern XLLP_STATUS_T XllpRTCRestore(P_XLLP_RTC_T pVirtual, P_XLLP_RTC_T pSave);

#ifdef __cplusplus
}
#endif


#endif
