/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#ifndef __XLLP_SSP_CPU_H__
#define __XLLP_SSP_CPU_H__

#include "xllp_defs.h"
#include "xllp_serialization.h"
#include "xllp_clkmgr.h"

typedef struct
{
	XLLP_VUINT32_T sscr0;
	XLLP_VUINT32_T sscr1;
	XLLP_VUINT32_T sssr;
	XLLP_VUINT32_T ssitr;
	XLLP_VUINT32_T ssdr;
	XLLP_VUINT32_T reserved[5];
	XLLP_VUINT32_T ssto;
	XLLP_VUINT32_T sspsp;
	XLLP_VUINT32_T sstsa;
	XLLP_VUINT32_T ssrsa;
	XLLP_VUINT32_T sstss;
	XLLP_VUINT32_T ssacd;
	XLLP_VUINT32_T ssacdd;
} XLLP_SSP_REGISTER_T;

/* SSCR0  Macro definition for port 1/2/3/4 */

#define XLLP_SSCR0_MOD		(0x1 << 31)	/* 0 -- Normal SSP mode, 1 -- Network mode */
#define XLLP_SSCR0_ACS		(0x1 << 30)	/* 1 - Audio Clock, 0 -- decided by NCS and ECS */
#define XLLP_SSCR0_FPCKE	(0x1 << 29)	/* 1 - FIFO packing mode, 0 -- Normal mode */
#define XLLP_SSCR0_FRDC_MASK	(0x7 << 24)	/* The number of slots per frame in network mode */
#define XLLP_SSCR0_TIM		(0x1 << 23)	/* Transmit FIFO underrun Interrupt Mask, 0 -- TUR will generate SSP interrupt */
#define XLLP_SSCR0_RIM		(0x1 << 22)	/* Receive FIFO Over Run Interrupt Mask 0 -- will generate interrupt */
#define XLLP_SSCR0_NCS		(0x1 << 21)	/*Select network Clock 0-- ECS bit determine clock selection */
#define XLLP_SSCR0_EDSS		(0x1 << 20)	/*Extended data size select is used in conjunction with DSS to select the size of data transimitted. 1 --- one is pre-appended to the DSS value that sets DSS range from 17-32 bit.*/

#define XLLP_SSCR0_SCR_MASK	(0xFFF << 8)
#define XLLP_SSCR0_SCR_3M	(0x5 << 8)	/* 2.6MHZ */
#define XLLP_SSCR0_SCR_512K	(0x19<<8)	/* SCR value is 25 based on system SSP clock is 13MHZ */
#define XLLP_SSCR0_SCR_128K	(0x67 << 8)
#define XLLP_SSCR0_SCR_64K	(0xC8 << 8)
#define XLLP_SSCR0_SCR_17K	(0x2F1 << 8)
#define XLLP_SSCR0_SCR_8K	(0x65A << 8)
#define XLLP_SSCR0_SCR_48K	(0x10F)

#define XLLP_SSCR0_SSE		(0x1 << 7)	/* 1 -- SSP operation is enabled
*/
#define XLLP_SSCR0_ECS		(0x1 << 6)	/* External Clock Select, 0 - on_chip clock used to produce the SSP port's serial clock */
#define XLLP_SSCR0_FRF_TISSP	(0x1 << 4)	/* TI SSP */
#define XLLP_SSCR0_FRF_PSP	(0x3 << 4)	/* PSP -- programmable serial protocol */
#define XLLP_SSCR0_DSS_14BIT	0xD              // 14 bit data transfer
#define XLLP_SSCR0_DSS_32BIT	0xF              //32 bit data transfer
#define XLLP_SSCR0_DSS_8BIT	0x7              //8 bit data
#define XLLP_SSCR0_DSS_16BIT	0xF              //16 bit data EDSS = 0
#define XLLP_SSCR0_DSS_24BIT	0x7              //24 bit data EDSS = 1
#define XLLP_SSCR0_DSS_20BIT	0x3 
#define XLLP_SSCR0_DSS_25BIT	0x8              //24 bit data EDSS = 1
#define XLLP_SSCR0_DSS_26BIT	0x9              //24 bit data EDSS = 1
#define XLLP_SSCR0_DSS_28BIT	0xb              //28 bit data

//SSCR1 

#define XLLP_SSCR1_THLD_8		(0x8<<10)		//threshold is 8 bytes, written to 
#define XLLP_SSCR1_RFT(x)		((x)<<10)
#define XLLP_SSCR1_TFT(x)		((x)<<6)
#define XLLP_SSCR1_TTELP                (0x1<<31)       //1 -- TXD line will be tristated 1/2 clock after TXD is to be flopped
#define XLLP_SSCR1_TTE                  (0x1<<30)       //1 -- TXD line will not be tristated, 0 --- TXD line will be tristated when not transmitting data
#define XLLP_SSCR1_EBCE1                (0x1<<29)       //1 -- Interrupt due to a bit error is enabled. 00 -- Interrupt due to a bit count is disabled.
#define XLLP_SSCR1_SCFR                 (0x1<<28)       //1 -- clock iput to SSPSCLK is active only during trnasfers
#define XLLP_SSCR1_ECRA                 (0x1<<27)       //Enable Clock request A, 1 = Clcok request from other SSP port is enabled
#define XLLP_SSCR1_ECRB                 (0x1<<26)       //Enable Clock Request B
#define XLLP_SSCR1_SCLKDIR              (0x1<<25)       //0 -- Master Mode, the port generate SSPSCLK internally, acts as master and drive SSPSCLK
#define XLLP_SSCR1_SFRMDIR              (0x1<<24)       //SSP frame direction determine whether SSP port is the master or slave 0 -- Master
#define XLLP_SSCR1_RWOT                 (0x1<<23)       // 0 -- Transmit/Recieve mode, 1 -- Receive with out Transmit mode
#define XLLP_SSCR1_TRAIL                (0x1<<22)       //Trailing byte is used to configure how trailing byte are handled, 0 -- Processor Based, Trailing byte are handled by processor
#define XLLP_SSCR1_TSRE                 (0x1<<21)       //Transmit Service Request Enables the transmit FIFO DMA service request.1 -- DMA service request is enabled
#define XLLP_SSCR1_RSRE                 (0x1<<20)      //Receive Service Request enables the Receive FIFO DMa Service Request
#define XLLP_SSCR1_TINTE                (0x1<<19)       //Receiver time-out interrupt enables the receiver time-out interrupt, 1 -- receive time-out interrupts are enabled, 1 -- peripheral trailing byte interrupt are enabled
#define XLLP_SSCR1_PINTE                (0x1<<18)       //Peripheral trailing byte interrupt enables the peripheral trailing byte interrupt
#define XLLP_SSCR1_IFS					(0x1<<16)
#define XLLP_SSCR1_STRF                 (0x1<<15)       //Select whether the transmit FIFO or receive FIFO is enabled for writes and reads (test mode)
#define XLLP_SSCR1_EFWR                 (0x1<<14)       //Enable FIFO Write/Read (Test mode bit) for the SSP port
#define XLLP_SSCR1_MWDS                 (0x1<<5)        //Mircowire Transmit Data size.1 = 16 bit command word is transmitted
#define XLLP_SSCR1_SPH                  (0x1<<4)        //SPI SSPSCLK phase setting  0 = SSPSCLK is inactive one cycle at the start of frame and 1/2 cycle at the end of frame
#define XLLP_SSCR1_SPO                  (0x1<<3)        //Motoroloa SPI SSPSCLK polarity setting selects the polarity of the inactive state of the SSPSCLK pins
#define XLLP_SSCR1_LBM                  (0x1<<2)         //Loop-back mode
#define XLLP_SSCR1_TIE                  (0x1<<1)        //0 = Transmit FIFO level interrupt is disabled.
#define XLLP_SSCR1_RIE                  0x1              //0 = Receive FIFO level interrupt is disabled. The interrupt is masked.

//SSSR     SSP status Register
#define XLLP_SSSR_BCE                   (0x1<<23)       //Bit Count Error 1 -- the SSPSFRM signal has been asserted when the bit counter was not 0
#define XLLP_SSSR_CSS                   (0x1<<22)       //Clock Synchronous Status  1 -- The SSP is currently busy synchronizing slave mode operation.
#define XLLP_SSSR_TUR                   (0x1<<21)       //1 = Attempted read from the tramit FIFO when the FIFO was empty
#define XLLP_SSSR_EOC                   (0x1<<20)       // 1 -- DMA has singaled an end of chain condition, there is no more data to be processed
#define XLLP_SSSR_TINT                  (0x1<<19)       //1 = Receiver Time-out pending
#define XLLP_SSSR_PINT                  (0x1<<18)        // 1 = Peripheral Trailing Byte Interrupt Pending
#define XLLP_SSSR_RFL_MASK              (0xf<<12)        //Receive FIFO Level is the number of valid entries.
#define XLLP_SSSR_TFL_MASK              (0xf<<8)         //The Transmit FIFO level the number of valid entries currently in the transmit FIFO
#define XLLP_SSSR_ROR                   (0x1<<7)         //1 = attempted data write to full receive FIFO
#define XLLP_SSSR_RFS                   (0x1<<6)         // 1 = Receive FIFO level is at or above RFT tirgger 
#define XLLP_SSSR_TFS                   (0x1<<5)          //1 = Transmit FIFO level is at or below TFT tigger threshold, request interrupt
#define XLLP_SSSR_BSY                   (0x1<<4)          // 1 = SSP port is currently transmitting or receiving a frame
#define XLLP_SSSR_RNE                   (0x1<<3)          //1 = Receive FIFO is not empty
#define XLLP_SSSR_TNF                   (0x1<<2)          // 1 = Transmit FIFO is not full
        
//SSPSP Programmable Serial Protocol Register 
#define XLLP_SSPSP_SCMODE(x)	(x)
#define XLLP_SSPSP_SFRMP	(0x1 << 0x2)
//SSTO  SSP time-out register   0-23 bits R/W time-out interval = (TIMEOUT)/Peripheral Clock Frequency

//SSACD
#define XLLP_SSACD_SCDX8			(0x1<<7)
#define XLLP_SSACD_ACPS_MASK			(0x7<<4)
#define XLLP_SSACD_SCDB				(0x1<<3)
#define XLLP_SSACD_ACDS_MASK			(0x7)

typedef enum  
{
	MotSPI = 0,
	TISSP = 1,
	MicroWire = 2,
	PSP = 3,
	I2S = 4
} ssp_port_format_t;

typedef struct
{
	ssp_port_format_t	format;		//SSCR0.FRF
	XLLP_UINT32_T 		dataSize;	//SSCR0.EDSS,DSS
	XLLP_BOOL_T		fifoPacking;	//SSCR0.FPCKE
	XLLP_UINT32_T		formatAttrib;	//SSPSP
} XLLP_SSP_PORT_SETUP_FORMAT_T;

typedef enum 
{
	NORMAL,
	NETWORK
} ssp_port_mode_t;

#define XLLP_SSP_MODE_ATTR_CLK_SLAVE 	(1u<<25)
#define XLLP_SSP_MODE_ATTR_FRM_SLAVE	(1u<<24)
#define XLLP_SSP_MODE_ATTR_LOOPBACK	(1u<<2)

#define SSP_SSCR1_MODE_ATTR_MASK	\
			(XLLP_SSCR1_TTELP|XLLP_SSCR1_TTE \
			| XLLP_SSCR1_SCFR | XLLP_SSCR1_ECRA | XLLP_SSCR1_ECRB \
			| XLLP_SSCR1_SCLKDIR|XLLP_SSCR1_SFRMDIR \
			| XLLP_SSCR1_RWOT | XLLP_SSCR1_TRAIL \
			| XLLP_SSCR1_TSRE | XLLP_SSCR1_RSRE \
			| XLLP_SSCR1_RFT(0xF) | XLLP_SSCR1_TFT(0xF) \
			| XLLP_SSCR1_IFS | XLLP_SSCR1_MWDS \
			| XLLP_SSCR1_SPH | XLLP_SSCR1_SPO)

enum ssp_network_slot_sel_t {
	SSP_NETWORK_SLOT_0 = 0x1,
	SSP_NETWORK_SLOT_1 = 0x2,	
	SSP_NETWORK_SLOT_2 = 0x4,	
	SSP_NETWORK_SLOT_3 = 0x8,	
	SSP_NETWORK_SLOT_4 = 0x10,	
	SSP_NETWORK_SLOT_5 = 0x20,	
	SSP_NETWORK_SLOT_6 = 0x40,	
	SSP_NETWORK_SLOT_7 = 0x80	
};

typedef struct
{
	ssp_port_mode_t mode;		//SSCR0.MOD
	XLLP_UINT32_T	modeAttrib;	//SSCR1
	XLLP_UINT32_T	numSlot;	//SSCR0.FRDC
	XLLP_UINT32_T	activeTx;	//SSTSA
	XLLP_UINT32_T	activeRx;	//SSRSA
} XLLP_SSP_PORT_SETUP_MODE_T;

typedef enum 
{
	SSP_CLK_SEL_AUDIO_DIV_BY_4,
	SSP_CLK_SEL_AUDIO_DIV_BY_8,
	SSP_CLK_SEL_AUDIO_BYPASS_DIV,
	SSP_CLK_SEL_PLL13M,
	SSP_CLK_SEL_NETWORK,
	SSP_CLK_SEL_EXT
} ssp_clock_select_t;

typedef enum 
{
	SSP_CLK_AU_DIV_BY_1 = 0,
	SSP_CLK_AU_DIV_BY_2 = 1,
	SSP_CLK_AU_DIV_BY_4 = 2,
	SSP_CLK_AU_DIV_BY_8 = 3,
	SSP_CLK_AU_DIV_BY_16 = 4,
	SSP_CLK_AU_DIV_BY_32 = 5
} ssp_audio_div_t;

typedef enum
{
	SSP_CLK_AU_SEL_5_622M = 0,
	SSP_CLK_AU_SEL_11_345M = 1,
	SSP_CLK_AU_SEL_12_235M = 2,	
	SSP_CLK_AU_SEL_14_857M = 3,
	SSP_CLK_AU_SEL_32_842M = 4,
	SSP_CLK_AU_SEL_48_000M = 5,
	SSP_CLK_AU_SEL_DITHER = 6
} ssp_audio_sel_t;

typedef struct
{
	XLLP_UINT32_T		clockRate;	//SSCR0.SCR
	ssp_clock_select_t 	clockSelect;	//SSCR0.ACS,NCS,ECS,SCDB,SCDX8
	ssp_audio_div_t		audioClkDivider;	//SSACD.ACDS
	ssp_audio_sel_t		audioClkSelect; 	//SSACD.ACPS
	XLLP_UINT32_T		ditherNum;		//SSACDD.NUM
	XLLP_UINT32_T		ditherDen;		//SSACDD.DEN
} XLLP_SSP_PORT_SETUP_CLOCK_T;

typedef struct
{
	XLLP_SSP_PORT_SETUP_FORMAT_T	portFormat;
	XLLP_SSP_PORT_SETUP_MODE_T	portMode;
	XLLP_SSP_PORT_SETUP_CLOCK_T	portClock;
} XLLP_SSP_PORT_SETUP_T;

XLLP_STATUS_T XllpSSPFifoReadSample(XLLP_SSP_REGISTER_T *pReg, XLLP_UINT32_T *pData, XLLP_UINT32_T sampleWidth);
XLLP_STATUS_T XllpSSPFifoReadBurst(XLLP_SSP_REGISTER_T *pReg, XLLP_UINT32_T *pDataBuffer, XLLP_UINT32_T length, XLLP_UINT32_T sampleWidth);
XLLP_STATUS_T XllpSSPFifoWriteSample(XLLP_SSP_REGISTER_T *pReg, XLLP_UINT32_T data, XLLP_UINT32_T sampleWidth);
XLLP_STATUS_T XllpSSPFifoWriteBurst(XLLP_SSP_REGISTER_T *pReg, XLLP_UINT32_T *pDataBuffer, XLLP_UINT32_T length, XLLP_UINT32_T sampleWidth);
XLLP_STATUS_T XllpSSPPortSetup (XLLP_SSP_REGISTER_T *pReg, XLLP_SSP_PORT_SETUP_T *pPortPara);

#endif //XLLP_SSP_CPU_H
