/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#ifndef __XLLP_USBHOST_H__
#define __XLLP_USBHOST_H__

/**
**  FILENAME:   xllp_usbohci.h
**
**  PURPOSE:    contains all PXA27x specific XLLP USB OHCI typedefs and bit definitions..
**
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#include "xllp_defs.h"
#include "xllp_mfp.h"
#include "xllp_clkmgr.h"
#include "xllp_ost.h"
#include "xllp_usbohci_platform.h"
#include "xllp_udc.h"

//
// Current to PXA27x Processor Family Developer's Manual
//

// XLLP compliant declaration

//
// USB HOST OHCI (USBOHCI) Register Bank
//
typedef struct _XLLP_USBOHCI_S

{
    XLLP_VUINT32_T    uhcrev;		// HCI Spec Revision
    XLLP_VUINT32_T    uhchcon;		// Control register
    XLLP_VUINT32_T    uhccoms;		// Command Status
    XLLP_VUINT32_T    uhcints;		// Interrupt Status
    XLLP_VUINT32_T    uhcinte;		// Interrupt Enable Control register
    XLLP_VUINT32_T    uhcintd;		// Interrupt Disable Control register
    XLLP_VUINT32_T    uhchcca;		// Host controller Communication Area
    XLLP_VUINT32_T    uhcpced;		// Period Current Endpoint Descriptor
    XLLP_VUINT32_T    uhcched;		// Control Head Endpoint Descriptor register
    XLLP_VUINT32_T    uhccced;		// Control Current Endpoint Descriptor register
    XLLP_VUINT32_T    uhcbhed;		// Bulk Head Endpoint Descriptor register
    XLLP_VUINT32_T    uhcbced;		// Bulk Current Endpoint Descriptor register
    XLLP_VUINT32_T    uhcdhead;		// Done head register
    XLLP_VUINT32_T    uhcfmi;		// Frame Interval register
    XLLP_VUINT32_T    uhcfmr;		// Frame Remaining register
    XLLP_VUINT32_T    uhcfmn;		// Frame Number register
    XLLP_VUINT32_T    uhcpers;		// Periodic Start register
    XLLP_VUINT32_T    uhclst;		// Low Speed Threshold register
    XLLP_VUINT32_T    uhcrhda;		// Root Hub Descriptor A register
    XLLP_VUINT32_T    uhcrhdb;		// Root Hub Descriptor B register
    XLLP_VUINT32_T    uhcrhs;		// Root Hub Status register
    XLLP_VUINT32_T    uhcrhps1;		// Root Hub Port 1 Status register
    XLLP_VUINT32_T    uhcrhps2;		// Root Hub Port 2 Status register
    XLLP_VUINT32_T    uhcrhps3;		// Root Hub Port 3 Status register
    XLLP_VUINT32_T    uhcstat;		// USB Host Status
    XLLP_VUINT32_T    uhchr;		// USB Host Reset
    XLLP_VUINT32_T    uhchie;		// USB Host Interrupt Enable
    XLLP_VUINT32_T    uhchit;		// USB Host Interrupt Test

} XLLP_USBOHCI_T, *P_XLLP_USBOHCI_T;


// fields and bits for UHCREV:		HCI Spec Revision
#define XLLP_USBOHCI_UHCREV_OHCISPEC1_0_A		0x10

// fields and bits for UHCHCON:	 UHC Host Control register
// for the CBSR field
#define XLLP_USBOHCI_UHCHCON_CBSR_MASK			( 0x3u << 0 )
#define XLLP_USBOHCI_UHCHCON_CBSR_1_1			( 0x0u << 0 )
#define XLLP_USBOHCI_UHCHCON_CBSR_2_1			( 0x1u << 0 )
#define XLLP_USBOHCI_UHCHCON_CBSR_3_1			( 0x2u << 0 )
#define XLLP_USBOHCI_UHCHCON_CBSR_4_1			( 0x3u << 0 )

#define	XLLP_USBOHCI_UHCHCON_PLE			( 1u   << 2 )
#define	XLLP_USBOHCI_UHCHCON_IE				( 1u   << 3 )
#define	XLLP_USBOHCI_UHCHCON_CLE			( 1u   << 4 )
#define	XLLP_USBOHCI_UHCHCON_BLE			( 1u   << 5 )

// for the HCFS field
#define XLLP_USBOHCI_UHCHCON_HCFS_MASK		    ( 0x3u << 6 )
#define XLLP_USBOHCI_UHCHCON_HCFS_USBRESET		( 0u << 6 )
#define XLLP_USBOHCI_UHCHCON_HCFS_USBRESUME		( 1u << 6 )
#define XLLP_USBOHCI_UHCHCON_HCFS_USBOPERATIONAL    ( 2u << 6 )
#define XLLP_USBOHCI_UHCHCON_HCFS_USBSUSPEND		( 3u << 6 )

#define	XLLP_USBOHCI_UHCHCON_IR				( 1u  << 8 )
#define	XLLP_USBOHCI_UHCHCON_RWC			( 1u  << 9 )
#define	XLLP_USBOHCI_UHCHCON_RWE			( 1u  << 10 )



// fields and bits for UHCCOMS:	UHC Command Status register
#define	XLLP_USBOHCI_UHCCOMS_HCR				( 1u   << 0 )
#define	XLLP_USBOHCI_UHCCOMS_CLF				( 1u   << 1 )
#define	XLLP_USBOHCI_UHCCOMS_BLF				( 1u   << 2 )
#define	XLLP_USBOHCI_UHCCOMS_OCR				( 1u   << 3 )

#define XLLP_USBOHCI_UHCCOMS_SOC_MASK			( 0x3u << 16 )



// fields and bits for UHCINTS:		Interrupt Status register
// fields and bits for UHCINTE:		Interrupt Enable Control register
// fields and bits for UHCINTD:		Interrupt Disable Control register
// All the UHCINTS bits are Write 1 to clear bits
//
#define	XLLP_USBOHCI_UHCINT_SO					( 1u   << 0 )
#define	XLLP_USBOHCI_UHCINT_WDH					( 1u   << 1 )
#define	XLLP_USBOHCI_UHCINT_SF					( 1u   << 2 )
#define	XLLP_USBOHCI_UHCINT_RD					( 1u   << 3 )
#define	XLLP_USBOHCI_UHCINT_UE					( 1u   << 4 )
#define	XLLP_USBOHCI_UHCINT_FNO					( 1u   << 5 )
#define	XLLP_USBOHCI_UHCINT_RHSC				( 1u   << 6 )
#define	XLLP_USBOHCI_UHCINT_OC					( 1u   << 30 )
#define	XLLP_USBOHCI_UHCINT_MIE					( 1u   << 31 )

#define P_XLLP_USBOHCI_UHCINTS( pUsbhHandle ) ((pUsbhHandle)->pUSBHRegs->UHCINTS)

// Mask for write 1 to clear bits in UHCINTS
#define XLLP_USBOHCI_UHCINTS_MASK  (0xFFFFFFFF)
//#define XLLP_USBOHCI_UHCINTS_MWRITE( pUsbhHandle, y )  (P_XLLP_USBOHCI_UHCINTS( pUsbhHandle ) = (((P_XLLP_USBOHCI_UHCINTS( pUsbhHandle )) & (~XLLP_USBOHCI_UHCINTS_MASK)) | y))
//Optimized:
//#define XLLP_USBOHCI_UHCINTS_MWRITE( pUsbhHandle, y )    ((P_XLLP_USBOHCI_UHCINTS( pUsbhHandle )) = (y))
#define XLLP_USBOHCI_UHCINTS_MWRITE( pUsbhHandle, y )    (((pUsbhHandle)->pUSBHRegs->UHCINTS) = (y))
//To invoke, call:
//XLLP_USBOHCI_UHCINTS_MWRITE( pUsbhHandle, y)  

// fields and bits for UHCHCCA:		UHC Host controller Communication Area
#define XLLP_USBOHCI_UHCHCCA_MASK				( 0xffffffu << 8 )

// fields and bits for UHCPCED:		UHC Period Current Endpoint Descriptor
#define XLLP_USBOHCI_UHCPCED_MASK				( 0xfffffffu << 4 )

// fields and bits for UHCCHED:		UHC Control Head Endpoint Descriptor register
#define XLLP_USBOHCI_UHCCHED_MASK				( 0xfffffffu << 4 )

// fields and bits for UHCCCED:		UHC Control Current Endpoint Descriptor register
#define XLLP_USBOHCI_UHCCCED_MASK				( 0xfffffffu << 4 )

// fields and bits for UHCBHED:		UHC Bulk Head Endpoint Descriptor register
#define XLLP_USBOHCI_UHCBHED_MASK				( 0xfffffffu << 4 )

// fields and bits for UHCBCED:		UHC Bulk Current Endpoint Descriptor register
#define XLLP_USBOHCI_UHCBCED_MASK				( 0xfffffffu << 4 )

// fields and bits for UHCDHEAD:	UHC Done Head register
#define XLLP_USBOHCI_UHCDHED_MASK				( 0xfffffffu << 4 )	// should be "DHTD" because its a transfer descriptor

// fields and bits for UHCFMI:		UHC Frame Interval register
#define XLLP_USBOHCI_UHCFMI_FI_MASK				( 0x3fffu << 0 )
#define XLLP_USBOHCI_UHCFMI_FSMPS_MASK			( 0x7fffu << 16 )
#define	XLLP_USBOHCI_UHCFMI_FIT					( 1u   << 31 )

// fields and bits for UHCFMR:		UHC Frame Remaining register
#define XLLP_USBOHCI_UHCFMR_FR_MASK				( 0x3fffu << 0 )
#define	XLLP_USBOHCI_UHCFMI_FRT					( 1u   << 31 )

// fields and bits for UHCFMN:		UHC Frame Number register
#define XLLP_USBOHCI_UHCFMN_FN_MASK				( 0xffffu << 0 )

// fields and bits for UHCPERS:		UHC Periodic Start register
#define XLLP_USBOHCI_UHCPERS_PS_MASK			( 0x3fffu << 0 )

// fields and bits for UHCLST:		UHC Low Speed Threshold register
#define XLLP_USBOHCI_UHCPLST_LST_MASK			( 0xfffu << 0 )

// fields and bits for UHCRHDA:		UHC Root Hub Descriptor A register
#define XLLP_USBOHCI_UHCRHDA_NDP_MASK			( 0xffu << 0 )

#define	XLLP_USBOHCI_UHCRHDA_PSM				( 1u   << 8 )
#define	XLLP_USBOHCI_UHCRHDA_NPS				( 1u   << 9 )
#define	XLLP_USBOHCI_UHCRHDA_DT					( 1u   << 10 )
#define	XLLP_USBOHCI_UHCRHDA_OCPM				( 1u   << 11 )
#define	XLLP_USBOHCI_UHCRHDA_NOCP				( 1u   << 12 )

#define XLLP_USBOHCI_UHCRHDA_POTPGT_MASK		( 0xffu << 24 )

// fields and bits for UHCRHDB:		UHC Root Hub Descriptor B register
#define XLLP_USBOHCI_UHCRHDB_DR_MASK			( 0xffffu << 0 )
#define XLLP_USBOHCI_UHCRHDB_DR_PORT_1_REMOVABLE  ( 0x1u << 1 )
#define XLLP_USBOHCI_UHCRHDB_DR_PORT_2_REMOVABLE  ( 0x1u << 2 )
#define XLLP_USBOHCI_UHCRHDB_DR_PORT_3_REMOVABLE  ( 0x1u << 3 )

#define XLLP_USBOHCI_UHCRHDB_PPCM_MASK			( 0xffffu << 16 )
#define XLLP_USBOHCI_UHCRHDB_PPCM_PORT_1		( 0x1u << 17 )
#define XLLP_USBOHCI_UHCRHDB_PPCM_PORT_2		( 0x1u << 18 )
#define XLLP_USBOHCI_UHCRHDB_PPCM_PORT_3		( 0x1u << 19 )

// fields and bits for UHCRHS:		UHC Root Hub Status register
#define	XLLP_USBOHCI_UHCRHS_LPS					( 1u   << 0 )		// meaning on read
#define	XLLP_USBOHCI_UHCRHS_CGP					( 1u   << 0 )		// Write 1 to clear

#define	XLLP_USBOHCI_UHCRHS_OCI					( 1u   << 1 )

#define	XLLP_USBOHCI_UHCRHS_DRWE				( 1u   << 15 )		// meaning on read
#define	XLLP_USBOHCI_UHCRHS_SRWE				( 1u   << 15 )		// Write 1 to set

#define	XLLP_USBOHCI_UHCRHS_LPSC				( 1u   << 16 )		// meaning on read
#define	XLLP_USBOHCI_UHCRHS_SGP					( 1u   << 16 )		// meaning on write 1 to set

#define	XLLP_USBOHCI_UHCRHS_OCIC				( 1u   << 17 )      // Read/Write 1 to clear
#define	XLLP_USBOHCI_UHCRHS_CRWE				( 1u   << 31 )

#define P_XLLP_USBOHCI_UHCRHS( pUsbhHandle ) ((pUsbhHandle)->pUSBHRegs->uhcrhs)

// Mask for write 1 to clear bits in UHCRHS
#define XLLP_USBOHCI_UHCRHS_MASK  (XLLP_USBOHCI_UHCRHS_CGP | XLLP_USBOHCI_UHCRHS_OCIC)
//For sticky bits:
#define XLLP_USBOHCI_UHCRHS_MWRITE( pUsbhHandle, y )  (P_XLLP_USBOHCI_UHCRHS( pUsbhHandle ) = (((P_XLLP_USBOHCI_UHCRHS( pUsbhHandle )) & (~XLLP_USBOHCI_UHCRHS_MASK)) | y))
//For non-sticky bits:
//To set bits, above macro is enough.
//XLLP_USBOHCI_UHCRHS_MWRITE( pUsbhHandle, y) 
//To clear bit, the bit needs to be added to the mask.
#define XLLP_USBOHCI_UHCRHS_MWRITE_CLEAR_NON_STICKY(pUsbhHandle, y)     ( (P_XLLP_USBOHCI_UHCRHS( pUsbhHandle )) = ( (P_XLLP_USBOHCI_UHCRHS( pUsbhHandle )) &  (~(XLLP_USBOHCI_UHCRHS_MASK | y))) )

//To clear sticky bits and to set non-sticky bits, call:
//XLLP_USBOHCI_UHCRHS_MWRITE( pUsbhHandle, y) 
//To clear non-sticky bits, call: 
//#define XLLP_USBOHCI_UHCRHS_MWRITE_CLEAR_NON_STICKY(pUsbhHandle, y)


// fields and bits for UHCRHPS1:	UHC Root Hub Port 1 Status register
// fields and bits for UHCRHPS2:	UHC Root Hub Port 2 Status register
// fields and bits for UHCRHPS3:	UHC Root Hub Port 3 Status register
#define	XLLP_USBOHCI_UHCRHPS_CCS				( 1u   << 0 )
#define	XLLP_USBOHCI_UHCRHPS_PES				( 1u   << 1 )
#define	XLLP_USBOHCI_UHCRHPS_PSS				( 1u   << 2 )
#define	XLLP_USBOHCI_UHCRHPS_POCI				( 1u   << 3 )
#define	XLLP_USBOHCI_UHCRHPS_PRS				( 1u   << 4 )

#define	XLLP_USBOHCI_UHCRHPS_PPS				( 1u   << 8 )

#define	XLLP_USBOHCI_UHCRHPS_LSDA				( 1u   << 9 )		// meaning on read
#define	XLLP_USBOHCI_UHCRHPS_CPP				( 1u   << 9 )		// Write 1 to clear

#define	XLLP_USBOHCI_UHCRHPS_CSC				( 1u   << 16 )
#define	XLLP_USBOHCI_UHCRHPS_PESC				( 1u   << 17 )
#define	XLLP_USBOHCI_UHCRHPS_PSSC				( 1u   << 18 )
#define	XLLP_USBOHCI_UHCRHPS_POCIC				( 1u   << 19 )
#define	XLLP_USBOHCI_UHCRHPS_PRSC				( 1u   << 20 )

// fields and bits for UHCSTAT:		UHC Status register
// All UHCSTAT bits below are Write 1 to clear
#define	XLLP_USBOHCI_UHCSTAT_RWUE				( 1u   << 7 )
#define	XLLP_USBOHCI_UHCSTAT_HBA				( 1u   << 8 )
#define	XLLP_USBOHCI_UHCSTAT_HTA				( 1u   << 10 )
#define	XLLP_USBOHCI_UHCSTAT_UPS1				( 1u   << 11 )
#define	XLLP_USBOHCI_UHCSTAT_UPS2				( 1u   << 12 )
#define	XLLP_USBOHCI_UHCSTAT_UPRI				( 1u   << 13 )
#define	XLLP_USBOHCI_UHCSTAT_SBTAI				( 1u   << 14 )
#define	XLLP_USBOHCI_UHCSTAT_SBMAI				( 1u   << 15 )
#define	XLLP_USBOHCI_UHCSTAT_UPS3				( 1u   << 16 )

#define P_XLLP_USBOHCI_UHCSTAT( pUsbhHandle ) ((pUsbhHandle)->pUSBHRegs->UHCSTAT)

// Mask for write 1 to clear bits in UHCSTAT
#define XLLP_USBOHCI_UHCSTAT_MASK  (0xFFFFFFFF)
//#define XLLP_USBOHCI_UHCSTAT_MWRITE( pUsbhHandle, y )  (P_XLLP_USBOHCI_UHCSTAT( pUsbhHandle ) = (((P_XLLP_USBOHCI_UHCSTAT( pUsbhHandle )) & (~XLLP_USBOHCI_UHCSTAT_MASK)) | y))
////Optimized:
//#define XLLP_USBOHCI_UHCSTAT_MWRITE( pUsbhHandle, y )    ((P_XLLP_USBOHCI_UHCSTAT( pUsbhHandle )) = (y))
#define XLLP_USBOHCI_UHCSTAT_MWRITE( pUsbhHandle, y )    (((pUsbhHandle)->pUSBHRegs->UHCSTAT) = (y))
//To invoke, call:
//XLLP_USBOHCI_UHCSTAT_MWRITE( pUsbhHandle, y)  

// fields and bits for UHCHR:		UHC Reset register

#define XLLP_USBOHCI_UHCHR_FSBIR				( 1u <<  0)	// Force System Bus Interface Reset: When 1, resets the logic that interfaces to the system bus, DMA, etc. Auto clears after three system bus clocks.
#define	XLLP_USBOHCI_UHCHR_FHR					( 1u <<  1)	// Force Host controller Reset: When 1, resets OHCI core. Must be held high for 10 uSeconds, then cleared
#define XLLP_USBOHCI_UHCHR_CGR					( 1u <<  2)	// Clock Generation Reset: When 0, resets the OHCI Clock Generation block (DPLL). Used only in simulation
#define XLLP_USBOHCI_UHCHR_SSDC					( 1u <<  3)	// Simulation Scale Down Clock: When 1, internal 1 mSec timer changes to 1 uSec to speed up simulations
#define XLLP_USBOHCI_UHCHR_UIT					( 1u <<  4)	// USB Interrupt Test: Enable Interrupt Test Mode and UHCHIT register
#define XLLP_USBOHCI_UHCHR_SSE					( 1u <<  5)	// Sleep Standby Enable: enable/disable both ports SE receivers & power  supply
#define XLLP_USBOHCI_UHCHR_PSPL					( 1u <<  6)	// Power SENSE Polarity: Control polarity of Over-current Indicator signals input from the MAX1693EUB USB Power Switch
#define XLLP_USBOHCI_UHCHR_PCPL					( 1u <<  7)	// Power CONTROL Polarity: Control polarity of Power Enable signals output to the MAX1693EUB USB Power Switch
#define XLLP_USBOHCI_UHCHR_SSEP1				( 1u <<  9)	// Sleep Standby Enable: enable/disable port1 SE receivers & power supply
#define XLLP_USBOHCI_UHCHR_SSEP2				( 1u << 10)	// Sleep Standby Enable: enable/disable port2 SE receivers & power supply
#define XLLP_USBOHCI_UHCHR_SSEP3				( 1u << 11)	// Sleep Standby Enable: enable/disable port3 SE receivers & power supply


// fields and bits for UHCHIE:		UHC Interrupt Enable register
#define	XLLP_USBOHCI_UHCHIE_RWIE				( 1u   << 7 )
#define	XLLP_USBOHCI_UHCHIE_HBAIE				( 1u   << 8 )
#define	XLLP_USBOHCI_UHCHIE_TAIE				( 1u   << 10 )
#define	XLLP_USBOHCI_UHCHIE_UPS1IE				( 1u   << 11 )
#define	XLLP_USBOHCI_UHCHIE_UPS2IE				( 1u   << 12 )
#define	XLLP_USBOHCI_UHCHIE_UPRIE				( 1u   << 13 )
#define	XLLP_USBOHCI_UHCHIE_UPS3IE				( 1u   << 14 )


// fields and bits for UHCHIT:		USB Host Interrupt Test register
#define	XLLP_USBOHCI_UHCHIT_RWUT				( 1u   << 7 )
#define	XLLP_USBOHCI_UHCHIT_BAT					( 1u   << 8 )
#define	XLLP_USBOHCI_UHCHIT_IRQT				( 1u   << 9 )
#define	XLLP_USBOHCI_UHCHIT_TAT					( 1u   << 10 )
#define	XLLP_USBOHCI_UHCHIT_UPS1T				( 1u   << 11 )
#define	XLLP_USBOHCI_UHCHIT_UPS2T				( 1u   << 12 )
#define	XLLP_USBOHCI_UHCHIT_UPRT				( 1u   << 13 )
#define	XLLP_USBOHCI_UHCHIT_STAT				( 1u   << 14 )
#define	XLLP_USBOHCI_UHCHIT_SMAT				( 1u   << 15 )
#define	XLLP_USBOHCI_UHCHIT_UPS3T				( 1u   << 16 )


// Port Power Managment Modes
enum {
	XLLP_USBOHCI_PPM_NPS,
	XLLP_USBOHCI_PPM_GLOBAL,
	XLLP_USBOHCI_PPM_PERPORT,
	XLLP_USBOHCI_PPM_MIXED
};


typedef struct XLLP_USBH_HANDLE_S 
{
   P_XLLP_USBOHCI_T  pUSBHRegs;    // Pointer to USB Host registers
   P_XLLP_UDC_REGISTERS_T pUdcRegs;// Pointer to USB Client registers
   P_XLLP_VUINT32_T	pMfpRegs;  // Pointer to MFP registers
   P_XLLP_CLKMGR_T pCLKMGRRegs;    // Pointer to the Clock Manager registers
   P_XLLP_OST_T pOSTRegs;          // Pointer to OST registers
   P_XLLP_PLATFORM_USBH_T pPlatformUSBHConfig ; //Pointer to Platform Specific USBH Configuration

} XLLP_USBH_HANDLE_T, *P_XLLP_USBH_HANDLE_T;


typedef enum _XLLP_USBH_ERROR_T {
     
    XLLP_USBH_SUCCESS = 0,
	XLLP_USBH_GPIO_POINTER_NULL,
	XLLP_USBH_CONTROLLER_POINTER_NULL,
	XLLP_USBH_BLR_POINTER_NULL,
	XLLP_USBH_CLKMGR_POINTER_NULL,
	XLLP_USBH_OST_POINTER_NULL, 
	XLLP_USBH_FEATURE_NOT_SUPPORTED,
	XLLP_USBH_PORT3_MODE_UNVALID

}XLLP_USBH_ERROR_T;

typedef enum _XLLP_USBH_PORT_T {
     
    XLLP_USBH_PORT_1 = 1,
	XLLP_USBH_PORT_2,
	XLLP_USBH_PORT_3

}XLLP_USBH_PORT_T;

typedef enum _XLLP_USBH_PORT3_MODE_T {
	XLLP_USBH_PORT3_EXT_TRANSCEIVER=0,
	XLLP_USBH_PORT3_EXT_DEVICE=2
} XLLP_USBH_PORT3_MODE_T;


/*
************************************************************************************
*                            FUNCTION PROTOTYPES 
************************************************************************************
*/



//Chip specific:

XLLP_USBH_ERROR_T XllpUsbhReset(P_XLLP_USBH_HANDLE_T pUsbhHandle);

XLLP_USBH_ERROR_T XllpUsbhSelectPowerManagementMode (
								   P_XLLP_USBH_HANDLE_T pUsbhHandle , 
								   XLLP_UINT32_T  PowerMode,                           
                                   XLLP_UINT32_T  NumPorts, 
                                   P_XLLP_UINT32_T  pPortMode) ;

XLLP_USBH_ERROR_T XllpUsbhPowerUp(P_XLLP_USBH_HANDLE_T pUsbhHandle);

XLLP_USBH_ERROR_T XllpUsbhPowerDown(P_XLLP_USBH_HANDLE_T pUsbhHandle);

XLLP_USBH_ERROR_T XllpUsbhConfigPort3(P_XLLP_USBH_HANDLE_T pUsbHandle, XLLP_USBH_PORT3_MODE_T mode);

//Platform Specific:


extern XLLP_USBH_ERROR_T XllpUsbhPortEnableOverCurrentNotification(P_XLLP_USBH_HANDLE_T pUsbhHandle,XLLP_USBH_PORT_T Port);

extern XLLP_USBH_ERROR_T XllpUsbhPortEnablePower(P_XLLP_USBH_HANDLE_T pUsbhHandle, XLLP_USBH_PORT_T Port); 

extern XLLP_USBH_ERROR_T XllpUsbhInitPlatformSpecificConfig(P_XLLP_USBH_HANDLE_T pUsbhHandle);

extern XLLP_USBH_ERROR_T XllpUsbhConfigPlatformSpecificUSBHReset(P_XLLP_USBH_HANDLE_T pUsbhHandle);

XLLP_USBH_ERROR_T XllpUsbhPortEnableSleepStandbyEnable(P_XLLP_USBH_HANDLE_T pUsbhHandle, 
													   XLLP_USBH_PORT_T Port, 
													   XLLP_BOOL_T enable);


XLLP_USBH_ERROR_T XllpUsbhConfigureGlobalSleepStandbyEnable(P_XLLP_USBH_HANDLE_T pUsbhHandle, 
				  XLLP_BOOL_T enable);

XLLP_USBH_ERROR_T XllpUsbhConfigurePowerSensePolarity(P_XLLP_USBH_HANDLE_T pUsbhHandle, 
					  XLLP_LEVEL_T level);

XLLP_USBH_ERROR_T XllpUsbhConfigurePowerControlPolarity(
			P_XLLP_USBH_HANDLE_T pUsbhHandle, XLLP_LEVEL_T level) ;

#ifdef __cplusplus
}
#endif


#endif //__XLLP_USBHOST_H__
