@ This program is free software; you can redistribute it and/or modify
@ it under the terms of the GNU General Public License as published by
@ the Free Software Foundation; either version 2 of the License, or
@ (at your option) any later version.
@ 
@ This program is distributed in the hope that it will be useful,
@ but WITHOUT ANY WARRANTY; without even the implied warranty of
@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@ GNU General Public License for more details.
@ 
@ You should have received a copy of the GNU General Public License
@ along with this program; if not, write to the Free Software
@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@

@ (C) Copyright 2006 Marvell International Ltd.  
@  All Rights Reserved 
@  FILENAME:       xllp_intc_gnu.s
@
@  PURPOSE:        Provides low level INTC primitive functions written specifically 
@                  for the Monahans/Zylonite processor/platform.  
@
@******************************************************************************


 .equ       xlli_CPSR_I_Bit,(0x80)
.equ       xlli_CPSR_F_Bit,(0x40)
@
@ List of primitive functions in this module:
@
    .global XllpIntcDisableInterrupts  
    .global XllpIntcRestoreInts  
    .global XllpReadINTCReg 
    .global XllpWriteINTCReg 
 
    .text

 


@;;;;;;;;;;;;;;;;;;;;;;;;;;
@;
@; UINT32  XllpIntcDisableInterrupts
@;
@;  Disable IRQ and FIQ interrupts by setting the I+F bits in the CPSR.
@;  
@;  Returns (R0): the state of the I+F bits before they were set 
@;
@;  Assumes caller is running in privileged mode.
@;

XllpIntcDisableInterrupts:
	mov     r1, #(xlli_CPSR_I_Bit|xlli_CPSR_F_Bit)
	mrs      r2, cpsr 
  	and      r0, r2, r1 	@This is the return with the current int flags
	orr       r2, r2, r1 	@mask the I F flag according to the inupt parameter
	msr     cpsr_c, r2 	@ Only interested in control fields
       	bx  lr

@;;;;;;;;;;;;;;;;;;;;;;;;;;
@;
@; void  XllpIntcRestoreInts (UINT32)
@;
@;  Set the I+F bits in the CPSR to the values received in R0.
@;  - Protects against changing any other bits the CPSR.
@;  - If the value returned by pmDisableInts() is used as the parameter, 
@;    this function restores the interrupt enable/disable state to
@;    that which existed at the time pmDisableInts() was invoked.
@;
@;  Assumes caller is running in privileged mode.
@;

XllpIntcRestoreInts:

    mov	r1, #(xlli_CPSR_I_Bit|xlli_CPSR_F_Bit)
    and	r0, r0, r1  @ Isolate the I+F bits
    mrs	r2, cpsr    @ Get existing settings
    bic	r2, r2, r1  @ Clear out the flag locations
    orr	r2, r2, r0  @ Set the restored flags in place
    msr	cpsr_c, r2  @ Only interested in control fields
    bx	lr

 
@
@ XllpReadINTCReg - Read the INTC Register
@
@ Description:
@ 	This routine reads the designated INTC register via CoProcesser 6.
@
@ Input Parameters:
@       r0 - arg1, INTC register number to read.  
@	if r0 contains:
@		0 ->XLLP_INTC_ICIP,	/*!< Interrupt Controller IRQ Pending register*/
@		1 ->XLLP_INTC_ICMR,	/*!< Interrupt Controller Mask register*/
@		2 ->XLLP_INTC_ICLR,	/*!< Interrupt Controller level register*/
@		3 ->XLLP_INTC_ICFP,	/*!< Interrupt Controller FIQ Pending register*/
@		4 ->XLLP_INTC_ICPR,	/*!< Interrupt Controller Pendingregister*/
@		5 ->XLLP_INTC_ICHP,	/*!< Interrupt Controller Highest Priority register*/
@		6 ->XLLP_INTC_ICIP2,/*!< Interrupt Controller IRQ Pending register*/
@		7 ->XLLP_INTC_ICMR2,/*!< Interrupt Controller Mask register*/
@		8 ->XLLP_INTC_ICLR2,/*!< Interrupt Controller level register*/
@		9 ->XLLP_INTC_ICFP2,/*!< Interrupt Controller FIQ Pending register*/
@		10->XLLP_INTC_ICPR2	/*!< Interrupt Controller Pendingregister*/
@ Returns:	
@	r0 - 32-bit value read from CoProcessor
@             if the input parameter is great than 10 return 0xffffffff
@    
@ Registers Modified:
@	CoProcessor Register Modified: None
@	General Purpose Registers Modified: r0	
@
@ NOTE:  
@	
@

XllpReadINTCReg:

    cmp	r0, #10
    ldrgt	r0, =0xffffffff  @ r0 > 10 renturn 0xffffffff
    bgt	RRet
    addls	pc, pc, r0, lsl #2    @ do not use thumb instruction the instructionis 32 bit so left shit 2 bits
    mov	r0, r0	@The pc value in the last instrction"addls	pc, pc, r0, lsl #2" 
    				@is (current address + 8). So add this instruction here.
    b		RdICIP
    b		RdICMR
    b		RdICLR
    b		RdICFP
    b		RdICPR
    b		RdICHP
    b		RdICIP2
    b		RdICMR2  
    b		RdICLR2
    b		RdICFP2
    b		RdICPR2
			
RdICIP:  
    mrc     p6, 0, r0, c0, c0, 0		@ Read ICIP
    b       RRet
RdICMR:  
    mrc     p6, 0, r0, c1, c0, 0		@ Read ICMR
    b		RRet
RdICLR:  
    mrc     p6, 0, r0, c2, c0, 0		@ Read ICLR
    b		RRet
RdICFP:  
    mrc     p6, 0, r0, c3, c0, 0		@ Read ICFP
    b		RRet
RdICPR:  
    mrc     p6, 0, r0, c4, c0, 0		@ Read ICPR
    b		RRet
RdICHP:  
    mrc     p6, 0, r0, c5, c0, 0		@ Read ICHP
    b		RRet
RdICIP2:  
    mrc     p6, 0, r0, c6, c0, 0		@ Read ICIP2
    b		RRet
RdICMR2:  
    mrc     p6, 0, r0, c7, c0, 0		@ Read ICMR2
    b		RRet
RdICLR2:
    mrc     p6, 0, r0, c8, c0, 0		@ Read ICLR2
    b		RRet 
RdICFP2:  
    mrc     p6, 0, r0, c9, c0, 0		@ Read ICFR2
    b		RRet
RdICPR2:
    mrc     p6, 0, r0, c10, c0, 0		@ Read ICPR2
    
RRet:
        bx  lr
	 

@
@ XllpWriteINTCReg - Writes to the INTC Register
@
@ Description:
@ 	This routine writes to the designated INTC register via CoProcesser 6.
@
@ Input Parameters:   
@       r0 - arg1 - INTC register number to write
@       r1 - arg2 - Value to write to INTC register
@
@	if r0 contains:
@	if r0 contains:
@		1 ->XLLP_INTC_ICMR,	/*!< Interrupt Controller Mask register*/
@		2 ->XLLP_INTC_ICLR,	/*!< Interrupt Controller level register*/
@		7 ->XLLP_INTC_ICMR2,/*!< Interrupt Controller Mask register*/
@		8 ->XLLP_INTC_ICLR2,/*!< Interrupt Controller level register*/
@
@ Returns:
@	None
@      r0 ==0xffffffff if the input out put is not right
@
@ Registers Modified:
@	CoProcessor Register Modified: INTC Register
@	General Purpose Registers Modified: None
@
@ NOTE   
@	
@     r3 is changed
@

XllpWriteINTCReg:
    mov	r3, r0
    ldr	r0, =0xffffffff          @r0 is not in 1,2,7,8 renturn 0xffffffff
    cmp    r3, #8
    bgt     WRet
    addls	pc, pc, r3, lsl #2    @ do not use thumb instruction the instructionis 32 bit so left shit 2 bits 
    mov	r0, r0	@The pc value in the last instrction"addls	pc, pc, r3, lsl #2" 
    				@is (current address + 8). So add this instruction here.
    b		WrICIP
    b		WrICMR
    b		WrICLR
    b		WrICFP
    b		WrICPR
    b		WrICHP
    b		WrICIP2
    b		WrICMR2
    b		WrICLR2
WrICIP:  
@    mcr     p6, 0, r1, c0, c0, 0		; Write ICIP Read only
    b       WRet                               
WrICMR:                                         
    mcr     p6, 0, r1, c1, c0, 0		@ Write ICMR
    mov    r0, #0				@success return 0
    b		WRet                               
WrICLR:                                         
    mcr     p6, 0, r1, c2, c0, 0		@ Write ICLR 
    mov    r0, #0				@success return 0
    b		WRet                               
WrICFP:                                         
@    mcr     p6, 0, r1, c3, c0, 0		; Write ICFP Read only
    b		WRet                               
WrICPR:                                         
@    mcr     p6, 0, r1, c4, c0, 0		; Write ICPR Read only
    b		WRet                               
WrICHP:                                         
@    mcr     p6, 0, r1, c5, c0, 0		; Write ICHP Read only
    b		WRet                               
WrICIP2:                                        
@   mcr     p6, 0, r1, c6, c0, 0		; Write ICIP2 Read only
    b		WRet                               
WrICMR2:                                        
    mcr     p6, 0, r1, c7, c0, 0		@ Write ICMR2
    mov    r0, #0				@ success return 0
    b		WRet                               
WrICLR2:                                        
    mcr     p6, 0, r1, c8, c0, 0		@ Write ICLR2 
    mov    r0, #0				@;success return 0
WRet:
        bx  lr
	 

