@ This program is free software; you can redistribute it and/or modify
@ it under the terms of the GNU General Public License as published by
@ the Free Software Foundation; either version 2 of the License, or
@ (at your option) any later version.
@ 
@ This program is distributed in the hope that it will be useful,
@ but WITHOUT ANY WARRANTY; without even the implied warranty of
@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@ GNU General Public License for more details.
@ 
@ You should have received a copy of the GNU General Public License
@ along with this program; if not, write to the Free Software
@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@

@ (C) Copyright 2006 Marvell International Ltd.  
@  All Rights Reserved 
@ FILENAME:	xllp_pm_suspend_a.s
@
@ PURPOSE:	 Provides assembly level code to support context saving,
@                  processor sleep and context restoration after sleep.
@
@****************************************************************************


@ include the files
#include "xllp_pm_suspend_gnu.inc"
#include "xlli_monahans_defs_gnu.inc"

@ the global function that may be exported
	.global XllpPmEnterSleep
	.global XllpPmResumeFromSleep
	.global XllpPmEnterStandby
	.global XllpPmEnterDeepSleep
	.global XllpPmResumeFromDeepSleep
	.global XllpPmEnterLcdRefresh
	.global XllpPmEnterIdle

@ external function that needed by this file

@*****************************************************************************
@ XllpPmChecksumCalculate
@ 
@ Calculate checksum
@
@ Inputs:
@	r0: the virutal address of the data area which will be calculated the
@		checksum
@	r1: the toltal word of the data area. Checksum is done on 4-byte word
@
@ Output: 
@	checksum
@
@ Registers used
@	r0, r1, r2, r3
@

XllpPmChecksumCalculate:
	
	@ pick a non-zero seed
	ldr	r2, =(0x5A72) 
calculate:
	@ get value and increment pointer
	ldr	r3, [r0], #4             
	add	r2, r2, r3
	@ rotate left by one bit position
	mov	r2, r2, ROR #31
	subs	r1, r1, #1
	bne	calculate
	mov	r0, r2
	mov	pc, lr


@*****************************************************************************
@ XllpPmEnterSleepOrDeepSleep
@
@ Put the system into S2D3C4 or S3D4C4 state
@ 
@ Inputs:
@	r0: the virutal address of the data area to save the content of core
@	r1: sleep type, 6(sleep), 7(deep sleep)
@	
@ Outputs:
@	None
@
@ Notes:
@      r1 should be saved previously
@

XllpPmEnterSleepOrDeepSleep:
	
	@ Step 1
	@ store registers(r0-r12), sp, lr of current mode in the data array 
	@ ENTRY_REGS
	@ the r0 changes to be virutal address of ENTRY_REGS
	add	r0, r0, #SleepState_ENTRY_R0
	@ skip r0 and r1 save
	add     r0, r0, #4
	stmib	r0, {r2 - r12, sp, lr}
	sub     r0, r0, #4
	mov     r11, r1 @save the sleep type
	@ save r0 register
	sub	r5, r0, #SleepState_ENTRY_R0
	str	r5, [r0]

	@ store cpsr of current mode in the data array ENTRY_REGS.
	mrs	r3, cpsr
	str	r3, [r5, #SleepState_ENTRY_CPSR]

	@ store spsr(if not SYS mode) of current mode in the content area
	ldr	r2, =xlli_CPSR_Mode_SYS
	and	r1, r3, r2
	cmp	r1, r2        
	mrsne	r2, spsr
	strne	r2, [r5, #SleepState_ENTRY_SPSR]

	@ Step 2
	@ compare "modeSaveFlag" to decide which mode will be saved
	@ the private registers are saved in an array. the consequence should
	@ be "spsr", "r8-r12", sp, lr
	@ the data array stores registers from low address to high address.
	ldr	r6, [r5, #SleepState_modeSaveFlags]

1:
	@ save SYS mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_SYS)
	beq	2f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
	orr	r3, r3, #(xlli_CPSR_Mode_SYS | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr	cpsr_c, r3
	add	r7, r5, #SleepState_SYS_REGS
	stmia	r7, {sp, lr}

2:
	@ save FIQ mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_FIQ)
	beq	3f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
	orr	r3, r3, #(xlli_CPSR_Mode_FIQ | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr	cpsr_c, r3 
	mrs     r2, spsr
	add	r7, r5, #SleepState_FIQ_REGS
	stmia   r7, {r2, r8 - r12, sp, lr}            

3:
	@ save IRQ mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_IRQ)
	beq	4f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
	orr	r3, r3, #(xlli_CPSR_Mode_IRQ | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr	cpsr_c, r3 
	mrs	r2, spsr
	add	r7, r5, #SleepState_IRQ_REGS
	stmia	r7, {r2, sp, lr}                   

4:
	@ save ABT mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_ABT)
	beq	5f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
	orr	r3, r3, #(xlli_CPSR_Mode_ABT | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr     cpsr_c, r3 
	mrs     r2, spsr
	add	r7, r5, #SleepState_ABT_REGS
	stmia	r7, {r2, sp, lr}                   

5:	
	@ save UND mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_UND)
	beq	6f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
    	orr	r3, r3, #(xlli_CPSR_Mode_UND | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr	cpsr_c, r3
	mrs     r2, spsr
	add	r7, r5, #SleepState_UND_REGS
	stmia	r7, {r2, sp, lr}                   

6:
	@ save SVC mode content?
	ands    r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_SVC)
	beq	7f
	bic     r3, r3, #(xlli_CPSR_Mode_MASK)
        orr     r3, r3, #(xlli_CPSR_Mode_SVC | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit)
        msr     cpsr_c, r3
	mrs     r2, spsr
	add	r7, r5, #SleepState_SVC_REGS
	stmia	r7, {r2, sp, lr}                   

7:
	@ Step 3
	@ save MMU settings
	@ r5 is pointer to sleep save data area

	@ Cp15_ACR_MMU
	mrc	p15, 0, r0, c1, c0, 0
	str	r0, [r5, #SleepState_Cp15_ACR_MMU]

	@ Cp15_AUXCR_MMU;   
	mrc	p15, 0, r0, c1, c0, 1
	str	r0, [r5, #SleepState_Cp15_AUXCR_MMU]

	@ Cp15_TTBR_MMU;
	mrc	p15, 0, r0, c2, c0, 0
	str	r0, [r5, #SleepState_Cp15_TTBR_MMU]

        @ Cp15_DACR_MMU; 
	mrc	p15, 0, r0, c3, c0, 0
	str	r0, [r5, #SleepState_Cp15_DACR_MMU]

        @ Cp15_PID_MMU;
	mrc	p15, 0, r0, c13, c0, 0
	str	r0, [r5, #SleepState_Cp15_PID_MMU] 

	@ Cp15_CPAR; 
	mrc	p15, 0, r0, c15, c1, 0
	str	r0, [r5, #SleepState_Cp15_CPAR]

	@ Now enable access to all valid coprocessors
	mcr	p15, 0, r1, c15, c1, 0

	@ cp_wait	
	mrc	p15, 0, r0, c2, c0, 0
	mov	r0, r0
	sub	pc, pc, #4

	@ Step 4
	@ The block 0 of nand flash should be copied to SRAM 0x5c014000
	@ The OS should save the resume back address and the content save area address
	@ load current pspr to r12 register 
	ldr	r12, [r5, #SleepState_psprAddress]
	@ Store 0x5c014000 to PSPR
	ldr	r1, =0x5c014000
	str	r1, [r12]	

	@ Step 5
	@ calculate checksum
	@ get total word count for ckecksum and should not include "checksum" 
	mov	r1, #SleepState_size - 4
	ldr	r2, [r5, #SleepState_extendedChecksumByteCount]
	add	r1, r1, r2
	@ get the word count by /4
	mov	r1, r1, lsr #2
	mov	r0, r5
	str	r1, [r0, #SleepState_wordCount]!
	bl	XllpPmChecksumCalculate	
	str	r0, [r5, #SleepState_checksum]
	@ Step 6
	@ invoke user flush function
	ldr	r0, [r5, #SleepState_flushFunc]
	cmp	r0, #0
	movne	lr, pc
	movne	pc, r0

       	b	1f
	.align	5
1:
     	@ Step 7
	@ enter sleep or deep sleep
	mcr     p14, 0, r11, c7, c0, 0

	@ wait for sleep
20:     
	nop
        b       20b 
        
@*****************************************************************************
@ XllpPmEnterSleep
@
@ Put the system into S2D3C4 state
@ 
@ Inputs:
@	r0: the virutal address of the data area to save the content of core
@	
@ Outputs:
@	None
@

XllpPmEnterSleep:
	str     r1, [r0, #SleepState_ENTRY_R1]
	mov	r1, #XLLP_PM_S2D3C4
	b       XllpPmEnterSleepOrDeepSleep

@*****************************************************************************
@ XllpPmEnterDeepSleep
@
@ Put the system into S3D4C4 state
@ 
@ Inputs:
@	r0: the virutal address of the data area to save the content of core
@	
@ Outputs:
@	None
@

XllpPmEnterDeepSleep:
	str     r1, [r0, #SleepState_ENTRY_R1]
	mov	r1, #XLLP_PM_S3D4C4
	b       XllpPmEnterSleepOrDeepSleep
	
@******************************************************************************	
@
@ XllpPmResumeFromDeepSleep
@
@ Restore saved content and return back
@ 
@ Inputs:
@	r0: The physical address of the saved data area
@
@ Outputs:
@	None
@
XllpPmResumeFromDeepSleep:

@******************************************************************************	
@
@ XllpPmResumeFromSleep
@
@ Restore saved content and return back
@ 
@ Inputs:
@	r0: The physical address of the saved data area
@
@ Outputs:
@	None
@

XllpPmResumeFromSleep:
	@ make sure that we are in SVC mode with irq and fiq off
	mov     r1, #(xlli_CPSR_Mode_SVC | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit)
	msr     cpsr_c, r1

	@ Step 1
	@ validate checksum
	@ get the address of the first word that is checksumable 
	mov	r9, r0
	ldr	r1, [r9, #SleepState_wordCount]
	mov	r8, lr
	add	r0, r9, #4
	bl	XllpPmChecksumCalculate
	ldr	r3, [r9, #SleepState_checksum]
	subs	r1, r3, r0
	mov	r0, #0
	movne	r0, #1
	@ return if checksum is wrong
	movne	pc, r8	
	mov	r0, r9

	@ Step 2
	ldr	r9, [r0, #SleepState_Cp15_ACR_MMU]
	ldr	r8, [r0, #SleepState_Cp15_AUXCR_MMU]
	ldr	r7, [r0, #SleepState_Cp15_TTBR_MMU]
	ldr	r6, [r0, #SleepState_Cp15_DACR_MMU]
	ldr	r5, [r0, #SleepState_Cp15_PID_MMU]
	ldr	r4, [r0, #SleepState_Cp15_CPAR]
	ldr	r0, [r0, #SleepState_areaAddress]

	@ invalidate I, D caches & BTB
	mcr     p15, 0, ip, c7, c7, 0            
	@ Drain Write (& Fill) Buffer
	mcr     p15, 0, ip, c7, c10, 4
	@ Prefetch Flush
	mcr     p15, 0, ip, c7, c5, 4           
	@ invalidate I, D TLBs
	mcr     p15, 0, ip, c8, c7, 0           
	
	@ Step 3
	@ Rrestore MMU settings and turn on MMU
	mcr	p15, 0, r4, c15, c1, 0		
	mcr	p15, 0, r5, c13, c0, 0
	mcr	p15, 0, r6, c3, c0, 0
	mcr	p15, 0, r7, c2, c0, 0
	mcr	p15, 0, r8, c1, c0, 1

	@ Get page table address
	mrc     p15, 0, r1, c2, c0, 0
	bic     r1, r1, #0xff
	bic     r1, r1, #0x3f00
	ldr	r2, =0x542e
	
	@ Mapping resume_turn_on_mmu in the pagetable
	adr	r3, resume_turn_on_mmu	
	mov	r3, r3, lsr #20
	orr	r4, r2, r3, lsl #20 
	ldr	r5, [r1, r3, lsl #2]
	str     r4, [r1, r3, lsl #2]

	@ Mapping page table address in the page table
	mov	r6, r1, lsr #20
	orr	r7, r2, r6, lsl #20
	ldr	r8, [r1, r6, lsl #2]
	str	r7, [r1, r6, lsl #2]

	ldr	r10, =resume_after_turn_on_mmu
	mov	r10, r10
        b	resume_turn_on_mmu
	
	.align	5

resume_turn_on_mmu:
	mcr	p15, 0, r9, c1, c0, 0

	@ cp_wait	
	mrc	p15, 0, r2, c2, c0, 0
	mov	r2, r2
	mov	r2, r2
	mov	pc, r10
	nop
	nop
	nop
	nop

resume_after_turn_on_mmu:
	@ Restore the Mappings in page table
	str	r5, [r1, r3, lsl #2]
	str	r8, [r1, r6, lsl #2]

	@ Step 4
	@ r0 stores the virtual address of the content save area
	@ compare "modeSaveFlag" to decide which mode will be saved
	ldr	r6, [r0, #SleepState_modeSaveFlags]
1:	
	@ restore SVC content?
	ands     r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_SVC)
	beq	2f
	add	r7, r0, #SleepState_SVC_REGS
        ldmia   r7, {r2, sp, lr}
	msr     spsr, r2

2:
	@ restore UND mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_UND)
	beq	3f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
    	orr	r3, r3, #(xlli_CPSR_Mode_UND | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr	cpsr_c, r3
	add	r7, r0, #SleepState_UND_REGS
        ldmia   r7, {r2, sp, lr}
	msr     spsr, r2

3:
	@ restore ABT mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_ABT)
	beq	4f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
	orr	r3, r3, #(xlli_CPSR_Mode_ABT | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr     cpsr_c, r3 
	add	r7, r0, #SleepState_ABT_REGS
        ldmia   r7, {r2, sp, lr}
	msr     spsr, r2

4:
	@ restore IRQ mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_IRQ)
	beq	5f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
	orr	r3, r3, #(xlli_CPSR_Mode_IRQ | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr	cpsr_c, r3 
	add	r7, r0, #SleepState_IRQ_REGS
        ldmia   r7, {r2, sp, lr}
	msr	spsr, r2

5:
	@ restore FIQ mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_FIQ)
	beq	6f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
	orr	r3, r3, #(xlli_CPSR_Mode_FIQ | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr	cpsr_c, r3 
	add	r7, r0, #SleepState_FIQ_REGS
        ldmia   r7, {r2, r8-r12, sp, lr}
	msr     spsr, r2

6:
	@ restore SYS mode content?
	ands	r1, r6, #(XLLP_PM_MODE_SAVE_FLAG_SYS)
	beq	7f
	bic	r3, r3, #(xlli_CPSR_Mode_MASK)
	orr	r3, r3, #(xlli_CPSR_Mode_SYS | xlli_CPSR_I_Bit | xlli_CPSR_F_Bit) 
	msr	cpsr_c, r3
	add	r7, r0, #SleepState_SYS_REGS
        ldmia   r7, {sp, lr}

7:
	@ Step 5
	@ Re-establish whatever mode was in use at the time XllpPmEnterSleep() 
	@ was invoked and restore complete register context.  Before restoring 
	@ the SPSR, make sure that the entry mode was not SYS mode, which has 
	@ no SPSR.

	@  Load CPSR, sp and (if not SYS mode) SPSR
	ldr	r3, [r0, #SleepState_ENTRY_CPSR]
	msr	cpsr, r3 
	ldr	r2, =xlli_CPSR_Mode_SYS 
	and	r3, r3, r2
	cmp	r3, r2 
	ldrne	r2, [r0, #SleepState_ENTRY_SPSR]
	msrne	spsr, r2
	add	r0, r0, #SleepState_ENTRY_R0
	@ use "increase after" to skip r0 register restore, 
	ldmib	r0, {r1 - r12, sp, lr}
	@ restore r0 reigster
	ldr	r0, [r0]
	
	@ return to next instruction after XllpPmEnterSleep
	mov	pc, lr
	

@******************************************************************************	
@
@ XllpPmEnterStandby
@
@ Put the system into S0D2C2 state
@ 
@ Inputs:
@	None
@
@ Outputs:
@	None
@

XllpPmEnterStandby:
	
	@save registers on stack
	stmfd	sp!, {r0 - r12, lr}           	
	b	1f

	.align  5

1:
	@ enter S0D2C2 state
	mov	r0, #XLLP_PM_S0D2C2
	mcr     p14, 0, r0, c7, c0, 0

	@ wait for standby
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

        ldmfd   sp!, {r0 - r12, pc}

@******************************************************************************	
@
@ XllpPmEnterLcdRefresh
@
@ Put the system into S0D1C2 state
@ 
@ Inputs:
@	None
@
@ Outputs:
@	None
@

XllpPmEnterLcdRefresh:
	
	@save registers on stack
	stmfd	sp!, {r0 - r12, lr}           	
	b	1f

	.align  5

1:
	@ enter S0D1C2 state
	mov	r0, #XLLP_PM_S0D1C2
	mcr     p14, 0, r0, c7, c0, 0

	@ wait for entering LCD Refresh mode
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

        ldmfd   sp!, {r0 - r12, pc}

@******************************************************************************	
@
@ XllpPmEnterIdle
@
@ Put the system into S0D0C1 state
@ 
@ Inputs:
@	None
@
@ Outputs:
@	None
@

XllpPmEnterIdle:
	
	@save registers on stack
	stmfd	sp!, {r0 - r12, lr}           	
	b	1f

	.align  5

1:
	@ enter S0D0C1 state
	mov	r0, #XLLP_PM_S0D0C1
	mcr     p14, 0, r0, c7, c0, 0
	
        ldmfd   sp!, {r0 - r12, pc}
	
