/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include "xllp_ssp_cpu.h"

XLLP_STATUS_T XllpSSPFifoReadSample(XLLP_SSP_REGISTER_T *pReg, 
			XLLP_UINT32_T *pData, XLLP_UINT32_T sampleWidth)
{
	if (sampleWidth > 16) {
		*pData = pReg->ssdr;
	} 
	else if (sampleWidth > 8) {
		*((P_XLLP_VUINT16_T)pData) = *(P_XLLP_VUINT16_T)&pReg->ssdr; //Warning C4197
	} 
	else {
		*((P_XLLP_VUINT8_T)pData) = *(P_XLLP_VUINT8_T)&pReg->ssdr;//Warning C4197
	}
	
	return XLLP_STATUS_SUCCESS;
}
									
XLLP_STATUS_T XllpSSPFifoReadBurst(XLLP_SSP_REGISTER_T *pReg, 
			XLLP_UINT32_T *pDataBuffer, XLLP_UINT32_T length, 
			XLLP_UINT32_T sampleWidth)
{
	if (sampleWidth > 16) {
		while (length > 0) {
			*(pDataBuffer++) = pReg->ssdr;
			length--;
		}
	} 
	else if (sampleWidth > 8) {
		while (length > 0) {
			*((P_XLLP_VUINT16_T)pDataBuffer) = *(P_XLLP_VUINT16_T)&pReg->ssdr; //Warning C4197
			pDataBuffer = (P_XLLP_UINT32_T)((XLLP_UINT32_T)pDataBuffer+2);
			length--;
		}
	} 
	else {
		while (length > 0) {
			*((P_XLLP_VUINT8_T)pDataBuffer) = *(P_XLLP_VUINT8_T)&pReg->ssdr;//Warning C4197
			pDataBuffer = (P_XLLP_UINT32_T)((XLLP_UINT32_T)pDataBuffer+1);
			length--;
		}		
	}
	
	return XLLP_STATUS_SUCCESS;
}
									
XLLP_STATUS_T XllpSSPFifoWriteSample(XLLP_SSP_REGISTER_T *pReg,
			XLLP_UINT32_T data, XLLP_UINT32_T sampleWidth)
{
	pReg->ssdr = data;
	return XLLP_STATUS_SUCCESS;	
}
					
XLLP_STATUS_T XllpSSPFifoWriteBurst(XLLP_SSP_REGISTER_T *pReg, 
			XLLP_UINT32_T *pDataBuffer, XLLP_UINT32_T length, 
			XLLP_UINT32_T sampleWidth)
{
	if (sampleWidth > 16) {
		while (length > 0) {
			pReg->ssdr = *(pDataBuffer++); 
			length--;
		}
	} 
	else if (sampleWidth > 8) {
		while (length > 0) {
			pReg->ssdr = (XLLP_UINT32_T)(*((P_XLLP_VUINT16_T)pDataBuffer));
			pDataBuffer = (P_XLLP_UINT32_T)((XLLP_UINT32_T)pDataBuffer+2);
			length--;
		}
	} 
	else {
		while (length > 0) {
			pReg->ssdr = (XLLP_UINT32_T)(*((P_XLLP_VUINT8_T)pDataBuffer));
			pDataBuffer = (P_XLLP_UINT32_T)((XLLP_UINT32_T)pDataBuffer+1);
			length--;
		}		
	}
	
	return XLLP_STATUS_SUCCESS;	
}
									
XLLP_STATUS_T XllpSSPPortSetup(XLLP_SSP_REGISTER_T *pReg, 
			XLLP_SSP_PORT_SETUP_T *pPortPara)
{
	XLLP_UINT32_T regCR0, regCR1, regACD;	
	XLLP_BOOL_T	configAudio = XLLP_FALSE;
	
	// Disable the port during setup.
	regCR0 = pReg->sscr0;
	pReg->sscr0 = regCR0 & (~XLLP_SSCR0_SSE);

	if (pPortPara)
	{
		// portFormat.format
		regCR0 &= ~XLLP_SSCR0_FRF_PSP;
		// For Monahans A stepping, all formats are emulated via PSP.
		//regCR0 |= ((pPortPara->portFormat.format)<<4) & XLLP_SSCR0_FRF_PSP;
		regCR0 |= XLLP_SSCR0_FRF_PSP;
		if (pPortPara->portFormat.format == PSP) {
			// portFormat.pspSetting
			pReg->sspsp = pPortPara->portFormat.formatAttrib;	
		} else {
			// FIXME: emulation settings for Mot SPI, TI SSP and NS MicroWire.
			//pReg->sspsp = ;	
		}
		
		// portFormat.dataSize
		if (pPortPara->portFormat.dataSize > 16)
		{
			regCR0 |= XLLP_SSCR0_EDSS;
			//m_dma_width = (0x11)<<14; // 32-bit width
			//m_dma_burst = (0x11)<<16; // 32bytes can fill 8 entries.(kFifoThreshold+1)
		}
		else
		{
			regCR0 &= ~XLLP_SSCR0_EDSS;
			//if (pPortPara->portFormat.dataSize > 8)
			//{
				//m_dma_width = (0x10)<<14; // 16-bit width
				//m_dma_burst = (0x10)<<16; // 16bytes can fill 8 entries.(kFifoThreshold+1)
			//}				
			//else
			//{
				//m_dma_width = (0x01)<<14; // 8-bit width
				//m_dma_burst = (0x01)<<16; // 8bytes can fill 8 entries.(kFifoThreshold+1)
			//}			
		}
		regCR0 &= ~(0xf);
		regCR0 |= (pPortPara->portFormat.dataSize - 1) & 0xf;
		
		// portFormat.fifoPacking
		if (pPortPara->portFormat.fifoPacking)
		{
			regCR0 |= XLLP_SSCR0_FPCKE;
			//m_dma_width = (0x11)<<14;	// 32-bit width
			//m_dma_burst = (0x11)<<16;	// 32bytes can fill 8 entries.(kFifoThreshold+1)			
		}
		else
			regCR0 &= ~XLLP_SSCR0_FPCKE;
			
		// portMode.mode
		if (pPortPara->portMode.mode == NORMAL)
			regCR0 &= ~XLLP_SSCR0_MOD;
		else
		{
			// for network mode only
			regCR0 |= XLLP_SSCR0_MOD;
			regCR0 &= ~XLLP_SSCR0_FRDC_MASK;
			regCR0 |= (pPortPara->portMode.numSlot<<24) & XLLP_SSCR0_FRDC_MASK ;
			pReg->sstsa = pPortPara->portMode.activeTx;
			pReg->ssrsa = pPortPara->portMode.activeRx;
		}	
			
		// portMode.modeAttrib
		//regCR1 = /*(kFifoThreshold<<6)|(kFifoThreshold<<10)
		//		|*/XLLP_SSCR1_TSRE|XLLP_SSCR1_RSRE
		//		|XLLP_SSCR1_TRAIL;
		
		// Fix me: We need to determine which bits should be set by inside XLLP
		//			and which could be configurable.
		regCR1 = 0;
		
		regCR1 &= ~SSP_SSCR1_MODE_ATTR_MASK;
		regCR1 |= (pPortPara->portMode.modeAttrib) & SSP_SSCR1_MODE_ATTR_MASK;		
		pReg->sscr1 = regCR1;

		// portClock.clockRate
		regCR0 &= ~XLLP_SSCR0_SCR_MASK;
		regCR0 |= (pPortPara->portClock.clockRate<<8) & XLLP_SSCR0_SCR_MASK;
		
		// portClock.clockSelect
		regCR0 &= ~(XLLP_SSCR0_ACS|XLLP_SSCR0_NCS|XLLP_SSCR0_ECS);
		
		regACD = 0;
		switch(pPortPara->portClock.clockSelect)
		{
			case SSP_CLK_SEL_PLL13M:
				break;
			case SSP_CLK_SEL_EXT:
				regCR0 |= XLLP_SSCR0_ECS;
				break;
			case SSP_CLK_SEL_NETWORK:
				regCR0 |= XLLP_SSCR0_NCS;
				break;
			case SSP_CLK_SEL_AUDIO_DIV_BY_4:
				regCR0 |= XLLP_SSCR0_ACS;
				configAudio = XLLP_TRUE;
				break;
			case SSP_CLK_SEL_AUDIO_DIV_BY_8:
				regCR0 |= XLLP_SSCR0_ACS;
				regACD |= XLLP_SSACD_SCDX8;
				configAudio = XLLP_TRUE;
				break;
			case SSP_CLK_SEL_AUDIO_BYPASS_DIV:
				regCR0 |= XLLP_SSCR0_ACS;
				regACD |= XLLP_SSACD_SCDB;
				configAudio = XLLP_TRUE;
				break;
			default:
				return XLLP_STATUS_WRONG_PARAMETER;
		}
		
		if (configAudio == XLLP_TRUE)
		{
			// portClock.audioClkDivider
			regACD |= ((pPortPara->portClock.audioClkDivider)& XLLP_SSACD_ACDS_MASK)
						|((pPortPara->portClock.audioClkSelect<<4)& XLLP_SSACD_ACPS_MASK);
			pReg->ssacd = regACD;
			if (pPortPara->portClock.audioClkSelect == SSP_CLK_AU_SEL_DITHER)
				pReg->ssacdd = (pPortPara->portClock.ditherNum<<16)|(pPortPara->portClock.ditherDen);					
		}
	}
	
	pReg->sscr0 = regCR0;
	return XLLP_STATUS_SUCCESS;	
}								
