/*
 * dump.c: hexdump command
 *
 * Copyright (C) 2001 Tim Riker <tim@rikers.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: dump.c,v 1.3 2002/04/26 18:14:01 seletz Exp $"

/**********************************************************************
 * Includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/errno.h>
#include <blob/serial.h>
#include <blob/types.h>
#include <blob/util.h>


/**********************************************************************
 * defines
 */

/* this will send a cold shiver through erik's spine ... */
#define	ERR( x )		{ ret = x; goto DONE; }

/*********************************************************************
 * dump
 *
 * AUTHOR: Tim Riker
 * REVISED: seletz
 *
 * dumps memory
 *
 */
int dump_cmd( int argc, char *argv[] )
{
	int ret = 0;
	static u32 address = 0L;
	u32 endaddress, tmpaddress, value;

	if ( argc < 1 )
		ERR( -EINVAL );

	if ( argc > 1 ) {
		ret = strtou32(argv[1], &address);
		if ( ret < 0 )
			ERR( -EINVAL );
	}

	if ( argc == 3 ) {
		ret = strtou32(argv[2], &endaddress);
		if ( ret < 0 )
			ERR( -EINVAL );
	} else
		endaddress = address + 0x80;


	/* check alignment of address and endaddress */
	if((address & 0x03) || (endaddress & 0x03))
		ERR(-EALIGN);

	/* print it */
	for ( ; address < endaddress; address += 0x10) {
		printf( "0x%08x: ", address );
		for (tmpaddress = address; tmpaddress < address + 0x10; tmpaddress += 4) {
			value = (*((u32 *)tmpaddress));
			printf( "0x%08x ", value );
		}
		for (tmpaddress = address; tmpaddress < address + 0x10; tmpaddress++) {
			value = (*((u8 *)tmpaddress)) & 0xff;
			if ((value >= ' ') && (value <= '~'))
				serial_write(value);
			else
				serial_write('.');
		}
		serial_write('\n');
	}

	ret = 0;

DONE:
	return ret;
}

char dump_help[] = "dump [address [endAddress]]\n";
