/*
 * setip.c: set client/server IP
 *
 * Client and server IP are used for TFTP downloads.
 *
 * Copyright (C) 2003 Stefan Eletzhofer <stefan.eletzhofer@eletztrick.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: setip.c,v 1.3 2003/08/21 00:28:03 seletz Exp $"

/**********************************************************************
 * Includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#if defined( CONFIG_NETWORK_SUPPORT ) 

#include <blob/errno.h>
#include <blob/debug.h>
#include <blob/types.h>
#include <blob/util.h>

#include <net/net.h>

/**********************************************************************
 * defines
 */
#undef SETIP_DEBUG
#ifdef SETIP_DEBUG
static int dbg = 1;
#	define DBG( x, args... )	if ( dbg>=x ) printf( args )
#else
#	define DBG( x, args... )
#endif

/**********************************************************************
 * command
 */
int setip_cmd( int argc, char *argv[] )
{
	u8 *ip;
	char *ptr, *num;
	int i, fini;

	if ( argc < 3 ) {
		printf("Our server IP : %d.%d.%d.%d \n",
			serveripaddress[0], serveripaddress[1], 
			serveripaddress[2], serveripaddress[3] );
		printf("Our client IP : %d.%d.%d.%d \n",
			clientipaddress[0], clientipaddress[1], 
			clientipaddress[2], clientipaddress[3] );
		return 0;
	}

	switch ( argv[1][0] ) {
		case 'c':
		case 'C':
			ip = clientipaddress;
			break;
		case 's':
		case 'S':
			ip = serveripaddress;
			break;
		default:
			return -EINVAL;
			break;
	}

	num = ptr = argv[2];
	fini=0;
	for ( i=0; i<4 || !fini; i++ ) {
		u32 n;

		while ( *ptr != '\0' && *ptr != '.' )
			ptr++;

		if ( *ptr == '\0' )
			fini = 1;

		*ptr = 0;

		strtou32( num, &n );
		ip[i]=n;

		num = ++ptr;
	}

	if ( fini && i<4 ) {
		printf( "error parsing ip.\n" );
		return -EINVAL;
	}

	printf( "Set IP to %d.%d.%d.%d\n",
			ip[0], ip[1], ip[2], ip[3] );

	return 0;
}
char setip_help[] = "setip command. Set IP addresses for tftp.\n"
"usage: setip {client|server} ip\n"
"\tip in usual dotted-quad format please.\n";;

#endif

