/*
 * tdownload.c: tftp download command (ala download and xdownload)
 *
 * Copyright (C) 2003 Stefan Eletzhofer <stefan.eletzhofer@eletztrick.de>
 * Copyright (C) 2003 Russ Dill <Russ.Dill@asu.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: tdownload.c,v 1.1 2003/09/04 17:47:00 russd Exp $"

/**********************************************************************
 * Includes
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/errno.h>
#include <blob/debug.h>
#include <blob/types.h>
#include <blob/download.h>
#include <blob/util.h>

#include <net/net.h>

extern int do_tftp(char *file, unsigned long addr, unsigned long *size);

/**********************************************************************
 * defines
 */
#undef TFTP_DEBUG
#ifdef TFTP_DEBUG
static int dbg = 1;
#	define DBG( x, args... )	if ( dbg>=x ) printf( args )
#else
#	define DBG( x, args... )
#endif

/**********************************************************************
 * command
 */
int tdownload_cmd(int argc, char *argv[])
{
	int ret = 0;
	int size = 0;
	char *tftp_file;
	u32 base, *digest;
	int *numread;

	if (argc < 2) argv[1] = "blob";
	if (argc > 2) tftp_file = argv[2];	
	else tftp_file = argv[1];

	ret = set_download_parameters(argv[1], &base,
		&size, &numread, &digest);
	if (ret < 0) return 0;

	ret = do_tftp(tftp_file, base, (unsigned long *) &size);
	if (ret < 0) {
		do_reload(argv[1]);
		return -EINVAL;
	}
	*numread = size;
	do_md5sum((u32 *)base, size, digest);

	printf( "%s: file '%s' loaded via tftp to address 0x%08x.\n",
			__FUNCTION__, tftp_file, base);
	return 0;
}
char tdownload_help[] = "tdownload command.\n"
"usage: tdownload {blob|param|kernel|ramdisk} [filename]\n"
"\tdownloads file over TFTP to RAM.\n"
"\tUse setip or bootp to set client and server IP addresses\n";


