/*
 *  bootp.c
 *
 *  Copyright 1996-2001 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <blob/util.h>
#include <blob/serial.h>
#include <net/net.h>
#include <net/ip_bits.h>

static const unsigned char bootp_txpacket[] =
	{					/* destination - broadcast		*/
						/*+source address			*/
	 					/* protocol - ip			*/
	 /* IP (14) */
	 0x45, 0x00, 0x01, 0x48,		/* ver/ihl, TOS, length			*/
	 0x00, 0x00,				/*+id					*/
	 0x00, 0x00, 0x40, 0x11,		/* frag off, ttl, proto			*/
	 0x00, 0x00,				/*+checksum				*/
	 0x00, 0x00, 0x00, 0x00,		/* source address (unknown)		*/
	 0xff, 0xff, 0xff, 0xff,		/* destination address (broadcast)	*/
	 /* UDP (34) */
	 0x00, 0x44,				/* source port				*/
	 0x00, 0x43,				/* destination port			*/
	 0x01, 0x34,				/* length (inc header)			*/
	 0x00, 0x00,				/*+checksum				*/
	 /* BOOTP (42) */
	 0x01,					/* Bootp REQUEST			*/
	 0x01,					/* Hardware type - ethernet		*/
	 0x06,					/* Hardware address length		*/
#if 0
	 0x00,					/* HOPS ???				*/
	 0x00, 0x00, 0x00, 0x00,		/*+random ID				*/
	 0x00, 0x00, 0x00, 0x00,		/*+time					*/
	 0x00, 0x00, 0x00, 0x00,		/* Client IP				*/
	 0x00, 0x00, 0x00, 0x00,		/* Your IP				*/
	 0x00, 0x00, 0x00, 0x00,		/* Server IP				*/
	 0x00, 0x00, 0x00, 0x00,		/* Gateway IP				*/
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	/*+Hardware address we're asking for	*/
#endif
	};


static unsigned int bootp_xid = 0xd1030000;	/* bootp_xid is net byte order */
static unsigned long zero = 0;
static unsigned char bootp_tx[42 + 300];
static unsigned char bootp_rx[42 + 400];

unsigned char clientipaddress[4];
unsigned char serveripaddress[4];
unsigned char tftpfilename[128];

int bootp_rcv (void)
{
	unsigned int bootp_off, bootp_sz;
	unsigned char *bootp;
	int i;

	if ((bootp_off = ipbits_rcv (bootp_rx, 360, &bootp_sz, 0x43, 0x44)) == 0)
		return 0;

	bootp = bootp_rx + bootp_off;

	if (bootp[0] != 2 || bootp[1] != 1 || bootp[2] != 6)
		return 0;			/* not reply or wrong hardware type (error?) */

	if (!memeq (bootp + 4, &bootp_xid, 4))
		return 0;			/* wrong xid */

	if (!memeq (bootp + 12, &zero, 4))
		return 0;			/* not our client IP (error?) */

	if (!memeq (bootp + 28, hwaddress, 6))
		return 0;			/* not our hardware address (error?) */

	memcpy (clientipaddress, bootp + 16, 4);	/* grab our & server ip addresses */
	memcpy (serveripaddress, bootp + 20, 4);

	for (i = 0; i < 127; i++)
		if ((tftpfilename[i] = bootp[108 + i]) == 0)
			break;
	tftpfilename[i] = '\0';

	printip ("\nOur IP   : ", clientipaddress);
	printip ("\nServer IP: ", serveripaddress);
	printascii ("\nTFTP file: ");
	printascii ((const char *)tftpfilename);
	printascii ("\n");
	return 1;
}

char *do_bootp (void)
{
	unsigned int request_time = 0;
	unsigned int packet_length;
	int i;

	memzero (bootp_tx, sizeof (bootp_tx));
	memcpy (bootp_tx + 14, bootp_txpacket, sizeof (bootp_txpacket));
	memset (bootp_tx + OFF_ETHER, 0xff, 6);
	memcpy (bootp_tx + OFF_BOOTP + 28, hwaddress, 6);
	memcpy (bootp_tx + OFF_BOOTP + 4, &bootp_xid, 4);
	packet_length = (bootp_tx[OFF_UDP + 4] << 8) + bootp_tx[OFF_UDP + 5];
	printascii ("Sending bootp request");

	do {
		printch ('.');
		bootp_tx[OFF_BOOTP + 8] = request_time >> 8;
		bootp_tx[OFF_BOOTP + 9] = request_time;
		request_time ++;
		ipbits_tx (bootp_tx, packet_length);

		for (i = 0; i < 0x00030000; i++)
			if (bootp_rcv())
				return (char *)tftpfilename;
	} while (1);
}
