/*-------------------------------------------------------------------------
 * Filename:      ether.c
 * Version:       $Id: ether.c,v 1.4 2003/09/04 18:55:53 russd Exp $
 * Copyright:     Copyright (C) 1999, Erik Mouw
 * Author:        Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Description:   Serial utilities for blob
 * Created at:    Tue Aug 24 20:25:00 1999
 * Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 *                Abraham van der Merwe <abraham@2d3d.co.za>
 *		  Russ Dill <Russ.Dill@asu.edu>
 *-----------------------------------------------------------------------*/
/*
 * ether.c: Ethernet interfacing for blob
 *
 * Copyright (C) 1999 2000 2001 Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 * Copyright (C) 2003 Russ Dill (Russ.Dill@asu.edu)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: ether.c,v 1.4 2003/09/04 18:55:53 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/command.h>
#include <blob/arch.h>
#include <blob/errno.h>
#include <blob/debug.h>
#include <net/ether.h>
#include <blob/init.h>
#include <blob/util.h>

ether_driver_t *ether_driver;
u8 hwaddress[6] = {2, 3, 4, 5, 6, 7};

/* initialise an ethernet interface with the given base address. Return 
 * zero success, or a negative error number on failure
 */

void ether_init(void)
{
	if(ether_driver == NULL) {
		deprintf("No ethernet driver defined in arch "
			 "specific file\n");
		return;
	}

	if (ether_driver->init(ether_driver->base) < 0){
		deprintf("Ethernet driver failed to init\n");
	}
}

__initlist(ether_init, INIT_LEVEL_OTHER_HARDWARE);

int ether_rcv(u8 *data, int size)
{
	if (!ether_driver) return -EINVAL;
	return ether_driver->rcv(data, size);
}


int ether_snd(u8 *data, int size)
{
	if (!ether_driver) return -EINVAL;
	return ether_driver->snd(data, size);
}


int ether_get_addr(u8 hw_addr[6])
{
	if (!ether_driver) return -EINVAL;
	return ether_driver->get_addr(hw_addr);
}


int ether_set_addr(u8 hw_addr[6])
{
	if (!ether_driver) return -EINVAL;
	memcpy(hwaddress, hw_addr, 6);
	return ether_driver->set_addr(hw_addr);
}


char *ether_get_name(void)
{
	if (!ether_driver) return NULL;//-EINVAL;
	return ether_driver->name;
}

static int xdigitvalue(char isdigit)
{
	if (isdigit >= '0' && isdigit <= '9' )
		return isdigit - '0';
	if (isdigit >= 'a' && isdigit <= 'f')
		return 10 + isdigit - 'a';
	if (isdigit >= 'A' && isdigit <= 'F')
		return 10 + isdigit - 'A';
	return -1;
}


static void strtomac(char* str, u8 addr[6])
{
	int i;
	
	for(i=0; (i<6) && str[0] && str[1]; i++) {
		addr[i] = ((xdigitvalue(str[0]) << 4) | xdigitvalue(str[1]));
		str += 3;
	}
}

static int setmac(int argc, char *argv[]) 
{ 
	u8 hw_addr[6];

	if (argc < 2) {
		ether_driver->get_addr(hw_addr);
		printf("Usage: setmac xx-xx-xx-xx-xx-xx\n"
				"current MAC:%02x-%02x-%02x-%02x-%02x-%02x\n", 
				hw_addr[0], hw_addr[1], hw_addr[2], hw_addr[3], 
				hw_addr[4], hw_addr[5]);
	} else {
		strtomac(argv[1], hw_addr);
#if 0
		printf("MAC:%02x-%02x-%02x-%02x-%02x-%02x\n", hw_addr[0], 
				hw_addr[1], hw_addr[2], hw_addr[3], 
				hw_addr[4], hw_addr[5]);
#endif
		ether_driver->set_addr(hw_addr);
	}
	return 0;
}

static char setmachelp[] = "Get or set ethernet mac address in format:\n"
"xx-xx-xx-xx-xx-xx\n";

__commandlist(setmac, "setmac", setmachelp);
