/*
 * Generic receive layer for the PXA USB client function
 *
 * This code was loosely inspired by the original version which was
 * Copyright (c) Compaq Computer Corporation, 1998-1999
 * Copyright (c) 2001 by Nicolas Pitre
 *
 * (C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * 02-May-2002
 *   Frank Becker (Intrinsyc) - derived from sa1100 usb_recv.c
 * 
 * TODO: Add support for DMA.
 *
 */

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/arch.h>
#include <blob/types.h>
#include <blob/init.h>
#include <blob/pxa_usb.h>

#include <linux/errno.h>
#include "mhn_u2d_ctl.h"

#include "xllp_u2d.h"


#undef DEBUG
//#define DEBUG 3
#if DEBUG
static unsigned int usb_debug = DEBUG;
#else
#define usb_debug 0     /* gcc will remove all the debug code for us */
#endif

extern XLLP_U2D_T	xllp_u2d;
void dump_info(int ep_num);
extern int arp_t;
int req_pending = 0;

#ifndef MMU_ENABLE
static char *ep2_buf;
#else
static char *callback_buf;
extern char *ep2_buf;
#endif
static int   ep2_len;
static int   ep2_remain;
static usb_callback_t ep2_callback;
static int rx_pktsize;
P_XLLP_DMAC_DESCRIPTOR_T	ep2_dma_desc;

static P_XLLP_U2D_EP_CONFIG ep2_config = &xllp_u2d.ConfigTable[2];
static P_XLLP_U2D_TRANSFER ep2_txfr = &xllp_u2d.u2dEpXferTable[2];


static void
ep2_start(void)
{
	u32 dcsr = U2DMACSR(2) & U2DMACSR_MASK;

	XllpU2DEnableInterrupt(&xllp_u2d, 2, U2D_DMA_EORSTOP|U2D_DMA_STOP);
  	XllpU2DDisableInterrupt(&xllp_u2d, 2, U2D_PACKET_COMPL_INT);
	
	dcsr &= ~U2DMACSR_RUN;
	U2DMACSR(2) = dcsr;

	/* init the params for transfer */
	ep2_txfr->direct = 0;
	ep2_txfr->physicalAddress = (XLLP_UINT32_T)ep2_buf;
	ep2_txfr->EnableEndIrq = 1;
	ep2_txfr->isZeroPackage = (ep2_len==0)?1:0;
	ep2_txfr->len = ep2_len;
	ep2_txfr->udcEndpointNum = 2;

	ep2_config->vDmaDescriptor = ep2_dma_desc;
	ep2_config->pDmaDescriptor = ep2_dma_desc;
	ep2_config->nDescriptors = DMA_DESC_NUM/2;

	XllpU2DStartTransfer(&xllp_u2d, ep2_txfr);
}

void
ep2_done(int flag)
{
	int size = ep2_len - ep2_remain;

	if (!ep2_len)
		return;

	ep2_len = 0;
	if (ep2_callback) {
#ifdef MMU_ENABLE
		//printf("ep2_buf %p callback_buf %p size %d\n", 
		//	ep2_buf, callback_buf, size);
		memcpy(callback_buf, ep2_buf, size);
#endif
		ep2_callback(flag, size);
	}
}

void
ep2_state_change_notify( int new_state )
{
}

void
ep2_stall( void )
{
	/* SET_FEATURE force stall at UDC */
	/* UDCCS2 |= UDCCS_BO_FST; */
}

#if 0
int
ep2_init(int chn)
{
	/*
	int i;
	desc_t * pdesc = pxa_usb_get_descriptor_ptr();

	for ( i = 0; i < pdesc->ep_num; i++ ) {
		if( BULK_OUT1 == ( 0xF & pdesc->ep[i].bEndpointAddress) ) {
			rx_pktsize = __le16_to_cpu( pdesc->ep[i].wMaxPacketSize );
		}
	}
	*/

	/* FIXME */
	rx_pktsize = 64;
	ep2_done(-EAGAIN);
	return 0;
}
#endif

void
ep2_reset(void)
{
	/*
	int i;
	desc_t * pdesc = pxa_usb_get_descriptor_ptr();

	for ( i = 0; i < pdesc->ep_num; i++ ) {
		if( BULK_OUT1 == ( 0xF & pdesc->ep[i].bEndpointAddress) ) {
			rx_pktsize = __le16_to_cpu( pdesc->ep[i].wMaxPacketSize );
		}
	}
	*/
	/* FIXME */
	rx_pktsize = 64;
	/* UDCCS2 &= ~UDCCS_BO_FST; */
	ep2_done(-EINTR);
}

extern int usb_connected;

void
ep2_int_hndlr(int udcsr)
{
	int status = U2DCSRB;
	int dcsr = U2DMACSR(2);
	int dcmd = U2DMACMDR(2);
	int dadr = U2DMADADR(2), dadr_old = U2DMADADR(2);
	unsigned int desc_num, length, i, count;

	//U2DMACSR(2) &= ~U2DMACSR_RUN;
	//dcsr &= ~U2DMACSR_RUN;

	//if( usb_debug) 
		//printf("ep2_int_hndlr: csr=%x, dcsr=%x, dcmd=%x\n",	status, dcsr, U2DMACMDR(2));

#if 0
	if( (status & (U2DCSR_PC | U2DCSR_SP)) == U2DCSR_SP)
	{
		/* zero-length packet */
		if (usb_debug) printf("%s:zero-length packet\n", __FUNCTION__);
		U2DCSRB |= U2DCSR_PC;
	}

	if( status & U2DCSR_PC)
	{
		/* ack RPC - FIXME: '|=' we may ack SST here, too */
		U2DCSRB |= U2DCSR_PC;
		ep2_remain = 0;
		ep2_done(0);
	}
#endif

	/* dma */
	if (dcsr & U2DMACSR_BUSERRINTR) {
		if (usb_debug) 
		if (usb_debug) printf("%s: bus error, u2dmacsr1:0x%x\n", __FUNCTION__, dcsr);
		if (usb_debug) printf("dcsr:%x, ddadr:%x, dsadr:%x, dtadr:%x, dcmd:%x\n",
			U2DMACSR(2), U2DMADADR(2), U2DMASADR(2), U2DMATADR(2), U2DMACMDR(2));
		U2DMACSR(2) = U2DMACSR_BUSERRINTR;
	/*} else if ((dcsr & U2DMACSR_RASINTR) || (dcsr & U2DMACSR_REQPEND)){
		U2DMACSR(2) |= U2DMACSR_RUN | U2DMACSR_RASINTR;

		while(U2DMACSR(2) & U2DMACSR_REQPEND);

		U2DMACSR(2) &= ~U2DMACSR_RUN;
		
		XllpU2DDisableInterrupt(&xllp_u2d, 2, U2D_DMA_STOP);
		U2DMACSR(2) = dcsr & (U2DMACSR_ENDINTR|U2DMACSR_EORINTR);
		U2DCSRB = U2DCSR_PC;

		dadr = U2DMADADR(2);
		desc_num = ((unsigned)(dadr&0xfffffff0) - (unsigned)&ep2_dma_desc[0])/16;
		length = ep2_config->MPS*desc_num - (dcmd & U2DMACMDR_LEN);

		ep2_remain = ep2_len - length;
		printf("pending dadr %x start %p num %d dcsr %x dadr %x ep2_remain %d, length %d cmd %x\n", 
			U2DMADADR(2), &ep2_dma_desc[0], desc_num, dcsr, U2DMADADR(2), ep2_remain, length, U2DMACMDR(2));

		ep2_done(0);*/
	} else if ((dcsr & (/*U2DMACSR_REQPEND|U2DMACSR_RASINTR|U2DMACSR_ENDINTR|U2DMACSR_EORINTR|*/U2DMACSR_STOPINTR)))
	/*(dcsr & (U2DMACSR_ENDINTR|U2DMACSR_EORINTR|U2DMACSR_STOPINTR))*/ {

		/*if(dcsr & (U2DMACSR_REQPEND|U2DMACSR_RASINTR)){
			dcsr &= U2DMACSR_MASK;
			dcsr |= U2DMACSR_RUN;
			U2DMACSR(2) = dcsr; 
			count = 1000;
			while((!(U2DMACSR(2) & U2DMACSR_REQPEND)) && count--);
 			if(count <= 0) 
				printf("reqpending??? csr %x dcsr %x dadr %x dcmd %x\n", U2DCSRB, U2DMACSR(2),
						U2DMADADR(2), U2DMACMDR(2));

			count = 1000;
			while((!(U2DMACSR(2) & U2DMACSR_STOPINTR)) && count--);
			if(count <= 0) 
				printf("no stop??? csr %x dcsr %x dadr %x dcmd %x\n", U2DCSRB, U2DMACSR(2),
					U2DMADADR(2), U2DMACMDR(2));

		}*/
		
		dcmd = U2DMACMDR(2);
		while(!dcmd){

			if((dcsr & U2DMACSR_ENDINTR))	break;

			XllpU2DEnableInterrupt(&xllp_u2d, 2, U2D_DMA_EORSTOP|U2D_DMA_STOP);
			dadr = U2DMADADR(2);
			U2DMADADR(2) = dadr;

			count = 1000;
			while((!(U2DMACSR(2) & U2DMACSR_REQPEND)) && count--);
			if(count <= 0) 
				printf("no reqpending, csr %x dcsr %x dadr %x dcmd %x\n", U2DCSRB, U2DMACSR(2),
						U2DMADADR(2), U2DMACMDR(2));

			dcsr = U2DMACSR(2);
			dcsr &= U2DMACSR_MASK;
			dcsr |= U2DMACSR_RUN;
			U2DMACSR(2) = dcsr; 
			
			count = 1000;
			while((!(U2DMACSR(2) & U2DMACSR_STOPINTR)) && count--);
			if(count <= 0) 
				printf("no stopintr, csr %x dcsr %x dadr %x dcmd %x\n", U2DCSRB, U2DMACSR(2),
						U2DMADADR(2), U2DMACMDR(2));
			
			dcmd = U2DMACMDR(2);
			dcsr = U2DMACSR(2);
		}
		
		dcsr &= ~U2DMACSR_RUN;
		U2DMACSR(2) = dcsr & U2DMACSR_MASK;
		if(dcsr & U2DMACSR_STOPINTR)
			XllpU2DDisableInterrupt(&xllp_u2d, 2, U2D_DMA_STOP);
			
		if(dcmd || (dcsr & U2DMACSR_ENDINTR)){
				
			/* caculate the length */
			dcmd = U2DMACMDR(2);
			dadr = U2DMADADR(2);
			desc_num = ((unsigned)(dadr&0xfffffff0) - (unsigned)&ep2_dma_desc[0])/16;
			length = ep2_config->MPS*desc_num - (dcmd & U2DMACMDR_LEN);
				
			U2DCSRB = U2DCSR_PC|U2DCSR_DME;

			ep2_remain = ep2_len - length;
			//printf("\tnum %d dcsr %x csr %x ep2_remain %d, length %d cmd %x\n", 
				//desc_num, dcsr, status, ep2_remain, length, U2DMACMDR(2));
		}

		ep2_done(0);
	} else if (dcsr & U2DMACSR_STARTINTR){
		dcsr &= U2DMACSR_MASK;
		dcsr |= U2DMACSR_RUN;
		U2DMACSR(2)	= dcsr;
		printf("???, U2DMACSR_STARTINTR\n");

	} else if (dcsr & U2DMACSR_EORINTR){
	} else
		if (usb_debug) 
		printf("%s: Others dma:2 DCSR:0x%x DCMD:0x%x\n",
				__FUNCTION__, U2DMACSR(2), U2DMACMDR(2));	

	if( usb_debug) 
		printf("ep2_int_hndlr end: csr=%x, dcsr=%x, dcmd=%x, req_pending %d, len %d\n",
			U2DCSRB, U2DMACSR(2), U2DMACMDR(2), req_pending, length);

	return;
}

int
ep2_recv(char *buf, int len, usb_callback_t callback)
{
	int flags;

	if (ep2_len){
		printf("%s, busy, ep2_len %d\n", __FUNCTION__, ep2_len);
		return -EBUSY;
	}

#ifndef MMU_ENABLE
	ep2_buf = buf;
#else
	callback_buf = buf;
#endif

	ep2_len = len;
	ep2_callback = callback;
	ep2_remain = len;
	ep2_start();
	
	return 0;
}

int ep2_begin(int flag)
{
	int dcsr = U2DMACSR(2);

#if 0
	if(!req_pending){
		dcsr &= ~U2DMACSR_RUN;
		XllpU2DDisableInterrupt(&xllp_u2d, 2, U2D_DMA_STOP|U2D_DMA_RAS);

		U2DMACSR(2) = dcsr;

		while(U2DISR & (U2DINT_SPACKETCMP <<6)){
			U2DISR = U2DINT_SPACKETCMP <<6;
			U2DCSRB = U2DCSR_FEF | U2DCSR_PC;
			//printf("%s, isr %x csr %x, dcsr %x\n", __FUNCTION__, U2DISR, U2DCSRB, U2DMACSR(2));
		}
		U2DCSRB = 0;
	  	XllpU2DDisableInterrupt(&xllp_u2d, 2, U2D_DMA_STOP);

		//printf("%s isr %x u2dmacsr2 %x u2dcsr2 %x\n", __FUNCTION__, U2DISR, U2DMACSR(2), U2DCSRB);
		U2DCSRB = U2DCSR_DME;
	}
#endif
	XllpU2DEnableInterrupt(&xllp_u2d, 2, U2D_DMA_STOP/*|U2D_SHORT_PACKET_COMPL_INT*/|U2D_DMA_EORSTOP);

	dcsr = U2DMACSR(2);
	dcsr &= U2DMACSR_MASK;
	dcsr |= U2DMACSR_RUN;

	U2DMACSR(2) = dcsr;
	dcsr = U2DMACSR(2);
	dcsr = U2DCSRB;

	//printf("%s isr %x u2dmacsr2 %x u2dcsr2 %x\n", __FUNCTION__, U2DISR, U2DMACSR(2), U2DCSRB);

}

int ep2_end(int flag)
{

	int dcsr = U2DMACSR(2);

	//printf("%s u2dmacsr2 %x u2dcsr2 %x\n", __FUNCTION__, U2DMACSR(2), U2DCSRB);
	dcsr &= ~U2DMACSR_RUN;
	dcsr &= U2DMACSR_MASK;
	//XllpU2DDisableInterrupt(&xllp_u2d, 2, U2D_DMA_STOP|U2D_DMA_RAS);
	U2DMACSR(2) = dcsr;
	//U2DCSRB = U2DCSR_PC|U2DCSR_TRN;
	//while(U2DISR & (U2DINT_SPACKETCMP <<6))U2DISR = U2DINT_SPACKETCMP <<6;

	//if(flag)U2DCSRB = 0;
//	}
	//dump_info(2);

}

#include <blob/command.h>
char ep2_begin_help[] = "\n";

__commandlist(ep2_begin, "epb", ep2_begin_help);

char ep2_end_help[] = "\n";

__commandlist(ep2_end, "epe", ep2_end_help);


