/*-------------------------------------------------------------------------
 * Filename:      time-intelarm.c
 * Version:       $Id: time-intelarm.c,v 1.1 2003/09/04 17:39:01 russd Exp $
 * Copyright:     Copyright (C) 1999, Erik Mouw
 * Author:        Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Description:   Some easy timer functions for blob
 * Created at:    Tue Aug 24 21:08:25 1999
 * Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Modified at:   Sun Oct  3 21:10:21 1999
 *-----------------------------------------------------------------------*/
/*
 * timer.c: Timer functions for blob
 *
 * Copyright (C) 1999 2000 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: time-intelarm.c,v 1.1 2003/09/04 17:39:01 russd Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/init.h>
#include <blob/led.h>
#include <blob/arch.h>
#include <blob/time.h>

static void intelarm_timer_init(void)
{
	/* clear counter */
	OSCR = 0;

	/* we don't want to be interrupted */
	OIER = 0;

	/* wait until OSCR > 0 */
	while(OSCR == 0);

	/* clear match register 0 */
	OSMR0 = 0;

	/* clear match bit for OSMR0 */
	OSSR = OSSR_M0;
}

/* returns the time in 1/TICKS_PER_SECOND seconds */
static u32 intelarm_timer_read(void)
{
	return((u32) OSCR);
}

static int intelarm_timer_overflow(void)
{
	return(OSSR & OSSR_M0);
}

static void intelarm_timer_overflow_clear(void)
{
	OSSR = OSSR_M0;
}

/* export time driver */
timer_driver_t intelarm_timer_driver = {
	init:		intelarm_timer_init,
	read:		intelarm_timer_read,
	overflow:	intelarm_timer_overflow,
	overflow_clear:	intelarm_timer_overflow_clear,
	tps:		TICKS_PER_SECOND,
};

